#!/usr/bin/perl 

################################################################################
#
# PROGRAM:  Cart.pl - from closedShop - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: Shopping cart part
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-03   cbf    Created
# 2001-06-07   cbf    Added attribute support for items
# 2001-06-19   cbf    Added credit card validation
# 2001-06-24   cbf    If you try and retrieve the value of a nonexistent cookie
#          	            netscape doesn't like it. netscape sucks.
# 2001-08-04   cbf    Continue Shopping Link now goes to last item added to cart
# 2001-08-05   cbf    Was missing Header call for an error message
#                     If you're logged in already, checkout process doesn't
#                        ask for username/password
# 2001-08-12   cbf    All input validated
# 2001-10-28   cbf    fixed shipping,billing errors
# 2002-05-14   cbf    now saves billing address info
# 2002-05-14   cbf    Continue Shopping and Final Order link variables added
# 2003-04-01   cbf    Added ShowTax option
#                     Added new taxing options (state,national,internat)
# 2003-04-02   cbf    Added gift certificates, country listings, pay by check
# 2004-07-07   cbf    Added update quantity ability, fixed inventory tracking
# 2005-05-18   cbf    displays MaxMind information for orders if enabled
# 2005-05-22   cbf    allows for a force Cart.pl URL to force SSL
################################################################################

use CGI::Carp(fatalsToBrowser);
use closedShop;
use CGI qw/:standard/;
use CGI::Cookie;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query        = new CGI;
my $program_name = "$closedShop::FileNames{cart}";

my %actions = (
    AddProduct         => $closedShop::TEXT{"ADD_PROD"},
    DeleteProduct      => $closedShop::TEXT{"DEL_PROD"},
    ViewCart           => $closedShop::TEXT{"VIEW_CRT"},
    Checkout           => $closedShop::TEXT{"CHKOUT"},
    AddWishList        => $closedShop::TEXT{"ADD_WISH"},
    AddGiftCertificate => $closedShop::TEXT{"ADD_GC"},
    UpdateQuantities   => $closedShop::TEXT{"UPDATE_QTY"},
    SetCookie          => "Sets the cookie for redirect"
);
################################################################################

Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main {
    my $action = $query->param('action');
    $action = closedShop::ValidateInputCustomer($action);
    if ( !$actions{$action} ) {
        ViewCart("");
    }
    elsif ( $action eq "Checkout" ) {
        if ( $closedShop::ForceSSL eq "Y" && $ENV{'HTTPS'} ne "on" ) {
            RequireSSL();
        }
        else {
            Checkout();
        }
    }
    else {
        eval($action);
    }
}

###############################################################################
# FUNCTION:  GetForceSSLURL
# DESCRIPTION: gets the SSL URL
################################################################################
# MODIFICATION HISTORY
# 2005-07-24   cbf    Courtesy of gypsy rodgers gypsy@freeq.com
##############################################################################
sub GetForceSSLURL {
    my $force = "https://$ENV{'SERVER_NAME'}$ENV{'SCRIPT_NAME'}";

    if ($closedShop::ForceCartURL) {
        $force = $closedShop::ForceCartURL;
    }

    return $force . "?action=Checkout";
}

###############################################################################
# FUNCTION:  RequireSSL
# DESCRIPTION: changes to the SSL
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   gyp    Courtesy of gypsy rodgers gypsy@freeq.com
##############################################################################
sub RequireSSL {

    my $force = GetForceSSLURL();
    my $cart = closedShop::GetCookie("cart");
    my $uri .= '?' . 'action=SetCookie&redirect=' . CGI::escape($force) . '&SetCart=' . CGI::escape($cart);
    return RefreshCheckout($uri);
}

###############################################################################
# FUNCTION:  SetCookie
# DESCRIPTION: sets the cookie
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub SetCookie {
    my $uri = closedShop::ValidateInputCustomer( $query->param("redirect") );

    my $cart = closedShop::ValidateInputCustomer( $query->param('SetCart') );
    my $cookie = new CGI::Cookie( -name => 'cart', -value => "$cart" );
    closedShop::SaveCookie( "cart", "$cart" );
    print redirect(
        -Cookie => [$cookie],
        -URL    => "$uri"
    );
}

###############################################################################
# FUNCTION:  RequireSSL
# DESCRIPTION: changes to the SSL
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub RefreshCheckout {
    my ($uri) = @_;
    print header( -Location => "$uri" );
    return 0;
}

###############################################################################
# FUNCTION:  AddGiftCertificate
# DESCRIPTION: add gift certificate
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AddGiftCertificate {
    my $recipient_name  = closedShop::ValidateInputCustomer( $query->param("recipient_name") );
    my $recipient_email = closedShop::ValidateInputCustomer( $query->param("recipient_email") );
    my $gc_amount       = $query->param("gc_amount");
    $gc_amount =~ tr/0-9_.-//cd;

    if ( !$gc_amount || !$recipient_name || !$recipient_email ) {
        print header( -Location => "$closedShop::FileNames{item}?action=ShowGiftCerticates&message=Please%20Enter%20All%20Information" );
    }

    my $cart = closedShop::GetCookie("cart");

    $cart .= $closedShop::CookieItemDelimiter . "gc" . $closedShop::CookieSectionDelimiter . $recipient_name . $closedShop::CookieSectionDelimiter . $recipient_email . $closedShop::CookieSectionDelimiter . $gc_amount . $closedShop::CookieItemDelimiter;

    my $cookie  = new CGI::Cookie( -name => 'cart',     -value => "$cart" );
    my $cookie2 = new CGI::Cookie( -name => 'lastitem', -value => "gc" );
    closedShop::SaveCookie( "cart",     "$cart" );
    closedShop::SaveCookie( "lastitem", "gc" );

    print redirect(
        -Cookie => [ $cookie, $cookie2 ],
        -URL    => "$program_name"
    );
}

###############################################################################
# FUNCTION:  UpdateQuantities
# DESCRIPTION: add to wishlist
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub UpdateQuantities {

    my @ary  = ();
    my $cart = closedShop::GetCookie("cart");
    if ($cart) {
        @ary = split( /$closedShop::CookieItemDelimiter/, $cart )
          if ( defined($cart) );
    }

    my $nucookie = "";
    foreach (@ary) {
        if ( !$_ ) { next; }
        my ( $id, $qty, $attr ) =
          split( /$closedShop::CookieSectionDelimiter/, $_ );

        my $uniq = closedShop::URLEncode($_);

        my $nuqty = closedShop::ValidateInputCustomer( closedShop::URLDecode( $query->param("qty$uniq") ) );
        $attr  =~ s/NULL//g;
        $nuqty =~ tr/0-9//cd;
        $id    =~ tr/0-9//cd;

        if ( $nuqty == 0 ) {
            next;
        }

        my $SQL             = qq| select quantity from $closedShop::sql_tables{"item"} where id="$id" |;
        my $sth             = closedShop::DatabaseQuery($SQL);
        my ($num_remaining) = $sth->fetchrow_array();

        if ( $closedShop::EnableInventory eq "Y"
            && ( $num_remaining - $nuqty ) < 0 )
        {
            $nuqty = $num_remaining;
        }

        if ($nuqty) {
            if ( $id eq "gc" ) {
                $nucookie .= $closedShop::CookieItemDelimiter . $_ . $closedShop::CookieItemDelimiter;
            }
            else {
                $nucookie .= $closedShop::CookieItemDelimiter . $id . $closedShop::CookieSectionDelimiter . $nuqty . $closedShop::CookieSectionDelimiter . $attr . $closedShop::CookieItemDelimiter;
            }
        }
        else {
            $nucookie .= $_;
        }
    }

    my $cookie = new CGI::Cookie( -name => 'cart', -value => "$nucookie" );
    closedShop::SaveCookie( "cart", "$nucookie" );

    print redirect(
        -cookie => [$cookie],
        -URL    => "$program_name"
    );
}

###############################################################################
# FUNCTION:  AddWishList
# DESCRIPTION: add to wishlist
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AddWishList {
    my $id = closedShop::ValidateInputCustomer( $query->param("id") );
    $id =~ tr/0-9//cd;

    my ( $session, $uid ) = closedShop::GetSessionCookie();
    if ( !closedShop::Authenticate( $session, $uid, "user" ) ) {
        my $mesg    = $closedShop::TEXT{"PLS_LOGIN"};
        my $cookie  = new CGI::Cookie( -name => 'cs_lastaction', -value => "AddWishList" );
        my $cookie2 = new CGI::Cookie( -name => 'cs_lastaction_item', -value => "$id" );
        closedShop::SaveCookie( "cs_lastaction",      "AddWishList" );
        closedShop::SaveCookie( "cs_lastaction_item", "$id" );
        print redirect(
            -cookie => [ $cookie, $cookie2 ],
            -URL    => "$closedShop::FileNames{user}?message=$mesg"
        );
        return;
    }

    closedShop::AddItemToWishList( $uid, $id );
    print redirect("$closedShop::FileNames{'user'}");
}

###############################################################################
# FUNCTION:  AddProduct
# DESCRIPTION: add a product to users shopping cart
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AddProduct {
    my $id = $query->param("id");
    $id =~ tr/0-9//cd;
    my $quantity = $query->param("quantity");
    $quantity =~ tr/0-9//cd;

    my $SQL = qq| select quantity from $closedShop::sql_tables{"item"} where
				  id="$id" |;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ($num_remaining) = $sth->fetchrow_array();

    my $t         = 1;
    my $attribute = "";
    while ( my $at = closedShop::ValidateInputCustomer( $query->param("attribute$t") ) ) {
        $attribute .= $at . "$closedShop::CookieAttributeDelimiter";
        $t++;
    }

    if (
           !$id
        || !$quantity
        || ( $closedShop::EnableInventory eq "Y"
            && ( $num_remaining - $quantity ) < 0 )
      )
    {
        if ( $num_remaining == 0 ) {
            $quantity = 0;
        }
        else {
            $quantity = $num_remaining;
        }
    }

    my $cart = closedShop::GetCookie("cart");

    $cart .= $closedShop::CookieItemDelimiter . $id . $closedShop::CookieSectionDelimiter . $quantity . $closedShop::CookieSectionDelimiter . $attribute . $closedShop::CookieItemDelimiter;

    my $cookie  = new CGI::Cookie( -name => 'cart',     -value => "$cart" );
    my $cookie2 = new CGI::Cookie( -name => 'lastitem', -value => "$id" );

    closedShop::SaveCookie( "cart",     "$cart" );
    closedShop::SaveCookie( "lastitem", "$id" );

    print redirect(
        -Cookie => [ $cookie, $cookie2 ],
        -URL    => "$program_name"
    );

}

###############################################################################
# FUNCTION:  CheckoutStep4
# DESCRIPTION: checkout process
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub CheckoutStep4 {

    my ( $session, $uid ) = closedShop::GetSessionCookie();
    if ( !closedShop::Authenticate( $session, $uid, "user" ) ) {
        closedShop::Header();
        print qq|<font class="main"><font color="red">$closedShop::TEXT{"PLS_LOGIN"}</font>|;
        return DISPLAY();
    }

    my $oid     = closedShop::GetCookie("oid");
    my $gc_code = closedShop::GetCookie("gc_code");

    my $SQL = qq|select fname,email from $closedShop::sql_tables{"user"} where id="$uid"|;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $un, $ue ) = $sth->fetchrow_array();

    # insert new authorization

    $SQL = qq|select b_fname,b_lname,b_address,b_zip,b_state,b_phone,b_city,
    b_country,total,cc_num,cc_year,cc_month,cc_vcode,
    s_fname,s_lname,s_address,s_zip,s_state,s_city,
    s_country from $closedShop::sql_tables{"order"} where id="$oid" 
       and user_id = '$uid'|;
    $sth = closedShop::DatabaseQuery($SQL);

    my ( $b_fname, $b_lname, $b_address, $b_zip, $b_state, $b_phone, 
    $b_city, $b_country, $tot, $cc_num, $cc_year, $cc_month, $cc_vcode, 
    $s_fname, $s_lname, $s_address, $s_zip, $s_state, $s_city, $s_country ) 
    = $sth->fetchrow_array();

    if (   $closedShop::EnableAuthorizeNET eq "Y"
        && $cc_num
        && $cc_month
        && $cc_year )
    {

        my %inputdata = (
            "x_login"              => "$closedShop::AuthorizeNETLoginID",
            "x_tran_key"           => "$closedShop::AuthorizeNETTransactionKey",
            "x_amount"             => "$tot",
            "x_card_num"           => "$cc_num",
            "x_exp_date"           => "$cc_month/$cc_year",
            "x_card_code"          => "$cc_vcode",
            "x_type"               => "AUTH_CAPTURE",
            "x_customer_ip"        => $ENV{'REMOTE_ADDR'},
            "x_email"              => "$ue",
            "x_first_name"         => "$b_fname",
            "x_last_name"          => "$b_lname",
            "x_address"            => "$b_address",
            "x_city"               => "$b_city",
            "x_state"              => "$b_state",
            "x_zip"                => "$b_zip",
            "x_country"            => "$b_country",
            "x_phone"              => "$b_phone",
            "x_ship_to_first_name" => "$s_fname",
            "x_ship_to_last_name"  => "$s_lname",
            "x_ship_to_address"    => "$s_address",
            "x_ship_to_city"       => "$s_city",
            "x_ship_to_state"      => "$s_state",
            "x_ship_to_zip"        => "$s_zip",
            "x_ship_to_country"    => "$s_country"
        );

        my @auth_results = AuthorizeNET( \%inputdata );

        $SQL = qq|update $closedShop::sql_tables{"order"} set AuthorizeNETResult='$auth_results[3]'
	        where id = "$oid" and user_id="$uid"|;
        print $SQL;
        $sth = closedShop::DatabaseQuery($SQL);

    }

    $SQL = qq|update $closedShop::sql_tables{"order"} set
        cc_vcode='' where id = "$oid" and user_id="$uid"|;
    $sth = closedShop::DatabaseQuery($SQL);

    $SQL = qq|select amount_left from $closedShop::sql_tables{"gc"} where
		gc_code="$gc_code"|;
    $sth = closedShop::DatabaseQuery($SQL);
    my ($amount_left) = $sth->fetchrow_array();

    my $receiptbody = qq|$closedShop::TEXT{"NAME"}\t$closedShop::TEXT{"QTY"}\t$closedShop::TEXT{"ITEM"} $closedShop::TEXT{'PRC'}\t$closedShop::TEXT{"SHP_PRC"}\t$closedShop::TEXT{"TOT"}\n
-------------------------------------------------
\n|;

    $SQL = qq|select item_ids,b_state,b_country,cc,discount,taxExemptID from
	     $closedShop::sql_tables{"order"}
	     where id = "$oid" and user_id="$uid"|;
    $sth = closedShop::DatabaseQuery($SQL);
    my ( $item_ids, $b_state, $b_country, $cc, $gdiscount, $taxExemptID ) = $sth->fetchrow_array();

    my @ary   = split( /$closedShop::CookieItemDelimiter/, $item_ids );
    my $k     = 1;
    my $grand = 0;
    my $totalweight = 0;
    my $shipsub = 0;
    
    foreach (@ary) {
        if ( !$_ ) { next; }
        my ( $id, $qty, $attr ) =
          split( /$closedShop::CookieSectionDelimiter/, $_ );
        if ( !$id || $id eq "NULL" ) { next; }
        $attr =~ s/NULL//g;
        if ( $id eq "gc" ) {
            my ( $id, $recipient_name, $recipient_email, $gc_amount ) =
              split( /$closedShop::CookieSectionDelimiter/, $_ );
            my $total = sprintf( "%.2f", $gc_amount );
            my $gc_code = closedShop::GetGiftCertificateCode();
            $SQL = qq| insert into $closedShop::sql_tables{"gc"} set customer_name="$un",
		 		 customer_id="$uid",recipient_name="$recipient_name",
		 		 recipient_email="$recipient_email",closed="N",initial_amount="$gc_amount",
		 		 amount_left="$gc_amount",date_bought=CURDATE(),oid="$oid",gc_code="$gc_code",
		 		 active="N"|;
            $sth = closedShop::DatabaseQuery($SQL);

            my $body = qq|
$closedShop::TEXT{"HELLO"} $recipient_name,

$un $closedShop::TEXT{"SNT_GC"} $closedShop::CurrencyMarker $gc_amount

$closedShop::TEXT{"CAN_RED"}
$closedShop::website

$closedShop::TEXT{"SEL_ADD"}

GC Code: $gc_code

$closedShop::TEXT{"DON_FOR"}

$closedShop::TEXT{"THANK"},
$closedShop::CompanyName
$closedShop::website
$closedShop::CompanyPhone
|;
            closedShop::SendEmail( $recipient_email, "$closedShop::TEXT{'REC_GC'} $closedShop::CompanyName", $body );

            $grand += $total;
            $receiptbody .= qq|$closedShop::TEXT{'GC'}\t1 x $closedShop::CurrencyMarker $gc_amount\t1 x $closedShop::CurrencyMarker 0.00\t$closedShop::CurrencyMarker  $total\n|;

            next;
        }

        my @attr = ();
        if ($attr) {
            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
        }
        $attr = "";
        my $attr_cost = 0.00;
        foreach my $at (@attr) {
            my ( $prod_id, $attr_id, $data_id ) =
              split( /$closedShop::GiftCertDelimiter/, $at );
            my $SQL = qq|select name,data,price from $closedShop::sql_tables{"attributes"},
	   $closedShop::sql_tables{"attribute_values"} where product_id='$prod_id' and
	   $closedShop::sql_tables{"attributes"}.id = '$attr_id' and
	   $closedShop::sql_tables{"attribute_values"}.id = '$data_id'|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $atname, $atdata, $atprice ) = $sth->fetchrow_array();
            $attr .= "$atname: $atdata";
            if ( $atprice > 0 ) {
                $attr .= " (+$closedShop::CurrencyMarker$atprice)";
                $attr_cost += $atprice;
            }
            $attr .= "   ";
        }
        if ( !$attr ) { $attr = " "; }

        $SQL = qq| select name,price,shipping_price,description,product_code,
	              category,reduced_shipping_price,quantity,weight from
                $closedShop::sql_tables{"item"} where id="$id" |;
        $sth = closedShop::DatabaseQuery($SQL);
        my ( $name, $price, $ship, $desc, $code, $cat, $rship, $num_rem, $weight ) = $sth->fetchrow_array();
		$totalweight += $qty*$weight;

        if ( $closedShop::EnableInventory eq "Y" ) {
            my $newqty = $num_rem - $qty;
            $SQL = qq|update $closedShop::sql_tables{"item"} set quantity="$newqty" where id="$id"|;
            $sth = closedShop::DatabaseQuery($SQL);
            if ( $newqty <= 0 ) {
                closedShop::SendEmail( $closedShop::CompanyEmail, $closedShop::TEXT{'ITEMOUT'}, "The Item $name (Code: $code) is out of stock" );
            }
        }

        my $total = sprintf( "%.2f", $qty * ( $price + $attr_cost ) );
        my ( $shiptotal, $shipstring ) = closedShop::GetShippingCostsItem( $ship, $qty, $rship, $weight);

		$shipsub += $shiptotal;
        $total += $shiptotal;
        $grand += $total;
        if ( length($name) > 18 ) { $name = substr( $name, 0, 18 ) . "..."; }

        $attr =~ s/NULL//g;
        $receiptbody .= qq|$name\t$qty\t$qty x $closedShop::CurrencyMarker $price\t$shipstring \t$closedShop::CurrencyMarker  $total\n|;
        if ($attr) {
            $receiptbody .= qq|\tAttributes: $attr\n|;
        }
    }

	my ( $shipclass, $shipstring1 ) = closedShop::GetShippingCostsClass( $totalweight, $grand);
	if ($shipsub == 0) {
		$shipsub = $shipclass;
		$grand += $shipsub;
	}
	
    my ( $stax, $ctax, $itax ) = closedShop::GetTaxCosts( $grand, $taxExemptID, $b_state, $b_country );
    my $tax = $stax + $ctax + $itax;

    my $total_cost = $tax + $grand;
    $total_cost = sprintf( "%.2f", $total_cost );
    $grand      = sprintf( "%.2f", $grand );

    my $gift_certificate = "";
    my $amount_used      = 0;

    my $pre_t = $total_cost;
    if ( $amount_left > 0 ) {
        $gift_certificate = qq|$closedShop::TEXT{'GC'}: $closedShop::CurrencyMarker -($amount_left)|;
        $amount_used      = $amount_left;
        $total_cost -= $amount_left;
        if ( $total_cost <= 0 ) {
            $amount_used = $pre_t;
            $amount_left = -($total_cost);
            $total_cost  = 0;
        }
        else {
            $amount_left = 0;
        }

        my $closed = "N";
        if ( $amount_left == 0 ) { $closed = "Y"; }
        $SQL = qq|update $closedShop::sql_tables{"gc"} set amount_left="$amount_left",
			   closed="$closed" where gc_code="$gc_code"|;
        $sth = closedShop::DatabaseQuery($SQL);
    }

    my $group = "";
    if ( $gdiscount > 0 && $gdiscount ) {
        my $SQL = qq|select $closedShop::sql_tables{"groups"}.name
		 from $closedShop::sql_tables{"user"},$closedShop::sql_tables{"groups"}
		where $closedShop::sql_tables{"user"}.id = '$uid' and 
		$closedShop::sql_tables{"user"}.groupid = $closedShop::sql_tables{"groups"}.id |;
        my $sth     = closedShop::DatabaseQuery($SQL);
        my ($gname) = $sth->fetchrow_array();
        my $disc    = sprintf( "%.2f", ( $gdiscount / 100 ) * $total_cost );
        $total_cost = $total_cost - $disc;

        $group = "$gname Discount ($gdiscount\%): - $closedShop::CurrencyMarker$disc";
    }

    my $userip = $ENV{'REMOTE_ADDR'} . " " . $ENV{'REMOTE_HOST'};
    $SQL = qq|update $closedShop::sql_tables{"order"} set date_bought=CURDATE(),
	    complete="Y",amount_gc_used="$amount_used",ip_address="$userip"
            where id = "$oid" and user_id="$uid"|;
    $sth = closedShop::DatabaseQuery($SQL);

    # update commissions if any
    my $aff_id = closedShop::GetAffiliateID();

    if ($aff_id) {
        $SQL = qq|select commission from $closedShop::sql_tables{"affiliates"} 
	        where id = "$aff_id" |;
        $sth = closedShop::DatabaseQuery($SQL);
        my ($comm) = $sth->fetchrow_array();
        my $owed = $total_cost * $comm / 100;
        $SQL = qq|insert into $closedShop::sql_tables{"affiliate_sales"} 
	    set date_sold=CURDATE(),
	    oid='$oid',sale='$total_cost',commission='$comm',
	    status='NOTPAID',owed='$owed',aid='$aff_id'|;
        $sth = closedShop::DatabaseQuery($SQL);
    }

    $receiptbody .= qq|\nSub $closedShop::TEXT{'TOT'}: $closedShop::CurrencyMarker $grand
$closedShop::TEXT{'TOT'} Tax: $closedShop::CurrencyMarker $tax
$gift_certificate
$group
$closedShop::TEXT{'TOT'}: $closedShop::CurrencyMarker $total_cost\n|;

    my $exmesg = "";
    if ( $cc eq "check" ) {
        $exmesg = $closedShop::TEXT{'PAY_CHK'};
    }

    my $body = qq|<pre>
$un,

$closedShop::TEXT{'THANK'} $closedShop::TEXT{'FOR_PUR'} $closedShop::CompanyName

$closedShop::TEXT{'OR_DET'}
-------------------------------------------------
$receiptbody
-------------------------------------------------

$closedShop::TEXT{'VW_STAT'},
$closedShop::website
$closedShop::TEXT{'AND_LOG'}

$closedShop::TEXT{'WILL_CON'}

$exmesg
$closedShop::FinalMessage

$closedShop::TEXT{'THANK'},
$closedShop::CompanyName
$closedShop::CompanyAddress
$closedShop::CompanyCity, $closedShop::CompanyState $closedShop::CompanyZip
$closedShop::CompanyCountry
$closedShop::CompanyEmail
Ph: $closedShop::CompanyPhone
Fax: $closedShop::CompanyFax
</pre>
|;

    my $adminurl = $closedShop::FileNames{"admin"};
    if ( $adminurl !~ /^http/ ) {
        $adminurl = $closedShop::urltofolder . "/" . $adminurl;
    }

    # send email to customer
    closedShop::SendEmail( $ue, $closedShop::TEXT{'TNK_ORD'}, $body );

    my $gcmesg     = "";
    my $paypalmesg = "";
    if ($gift_certificate) {
        $gcmesg = $closedShop::TEXT{'GC_MESG'};
    }
    if ( $cc eq "paypal" ) {
        $paypalmesg = $closedShop::TEXT{'PP_MESG'};
    }

    $body = qq|<pre>
$closedShop::TEXT{'OR_PC_BY'} $un - $ue

$closedShop::TEXT{'OR_DET'}
-------------------------------------------------
$receiptbody
-------------------------------------------------

$gcmesg
$paypalmesg
$closedShop::TEXT{'PLS_PROC'}
$adminurl
</pre>
|;

    # send email to merchant
    closedShop::SendEmail( $closedShop::CompanyEmail, "$closedShop::TEXT{'OR_PC_BY'} $un", $body );

    my $checkmesg = "";
    if ( $cc eq "check" ) {
        $checkmesg = qq|$closedShop::TEXT{'PAY_CHK'}
<BR><BR>$closedShop::CompanyName<BR>
$closedShop::CompanyAddress<BR>
$closedShop::CompanyCity, $closedShop::CompanyState $closedShop::CompanyZip<BR>
$closedShop::CompanyCountry<BR>
$closedShop::CompanyEmail<BR>
ph: $closedShop::CompanyPhone<BR>
fax: $closedShop::CompanyFax
<BR><BR>|;
    }

    # 3rd party credit card fraud modules
    if ( $cc ne "check" && $cc ne "paypal" && $cc ne "cod" ) {

        $SQL = qq| select b_fname,b_lname,b_address,b_city,b_state,
                  b_zip,b_phone,b_country,s_fname,s_lname,s_address,s_city,s_state,
                  s_zip,s_phone,s_country,item_ids,cc_name,cc,cc_num,cc_month,cc_year,
		     total,binName,binPhone from $closedShop::sql_tables{"order"} 
	              	where id="$oid" and user_id="$uid" |;
        $sth = closedShop::DatabaseQuery($SQL);
        my ( $b_fname, $b_lname, $b_address, $b_city, $b_state, $b_zip, $b_phone, $b_country, $s_fname, $s_lname, $s_address, $s_city, $s_state, $s_zip, $s_phone, $s_country, $item_ids, $cc_name, $card, $cc_num, $cc_month, $cc_year, $total, $binName, $binPhone ) = $sth->fetchrow_array();

        if ( length($b_phone) < 9 ) {
            $b_phone .= "0" x eval( 9 - length($b_phone) );
        }
        my ( $badd1, $badd2 ) = split( /\n/, $b_address );
        $badd2 =~ s/\n/ /g;

        if ( $closedShop::EnablePreCharge eq "Y" ) {
            eval("use Net::SSLeay");
            my $err = $@;
            if ( !$err ) {
                require Net::SSLeay;
                my ($page) = Net::SSLeay::post_https(
                    'api.precharge.net',
                    443,
                    '/charge',
                    '',
                    Net::SSLeay::make_form(
                        "Merchant_ID"                      => "$closedShop::PreChargeMerchantID",
                        "Security_1"                       => "$closedShop::PreChargeSecurityKey1",
                        "Security_2"                       => "$closedShop::PreChargeSecurityKey2",
                        "Ecom_BillTo_Postal_Name_First"    => "$b_fname",
                        "Ecom_BillTo_Postal_Name_Last"     => "$b_lname",
                        "Ecom_BillTo_Postal_PostalCode"    => "$b_zip",
                        "Ecom_BillTo_Postal_CountryCode"   => "$b_country",
                        "Ecom_BillTo_Postal_State"         => "$b_state",
                        "Ecom_BillTo_Postal_Street_Line1"  => "$badd1",
                        "Ecom_BillTo_Postal_Street_Line2"  => "$badd2",
                        "Ecom_BillTo_Postal_City"          => "$b_city",
                        "Ecom_BillTo_Telecom_Phone_Number" => "$b_phone",
                        "Ecom_BillTo_Online_Email"         => "$ue",
                        "Ecom_BillTo_Online_IP"            => $ENV{'REMOTE_ADDR'},
                        "Ecom_Payment_Card_Number"         => "$cc_num",
                        "Ecom_Payment_Card_ExpDate_Month"  => "$cc_month",
                        "Ecom_Payment_Card_ExpDate_Year"   => "$cc_year",
                        "Ecom_Transaction_Amount"          => "$total",
                        "Invoice_Number"                   => "$oid"
                    )
                );
                my @response = split( /,/, $page );
                my %output = ();
                foreach my $response (@response) {
                    my ( $name, $value ) = split( /=/, $response );
                    $output{$name} = $value;
                }

                my $score    = $output{'score'};
                my $response = $output{'response'};
                my $error    = $output{'error'};
                if ( $response == 3 ) { $response .= ":$error"; }
                $SQL = qq|update $closedShop::sql_tables{"order"} set precharge_score='$score',
	       precharge_response='$response' where id = "$oid" and user_id="$uid"|;
                $sth = closedShop::DatabaseQuery($SQL);
            }
        }

        if ( $closedShop::EnableMaxMind eq "Y" ) {

            my $license_key = $closedShop::MaxMindLicenseKey;
            my $emdomain    = substr( $ue, index( $ue, '@' ) + 1, length($ue) );
            my $emailhash   = lc($ue);

            eval("use Digest::MD5");
            my $err = $@;

            if ( !$err ) {
                require Digest::MD5;
                $emailhash = Digest::MD5::md5_hex( lc($ue) ),;
            }

            my $binnum = substr( $cc_num, 0, 6 );

            my %inputdata = (
                "i"           => $ENV{'REMOTE_ADDR'},
                "city"        => "$b_city",
                "region"      => "$b_state",
                "postal"      => "$b_zip",
                "country"     => "$b_country",
                "domain"      => "$emdomain",
                "bin"         => "$binnum",
                "binName"     => "$binName",
                "binPhone"    => "$binPhone",
                "custPhone"   => "$b_phone",
                "forwardedIP" => $ENV{'HTTP_X_FORWARDED_FOR'},
                "license_key" => "$license_key",
                "emailMD5"    => "$emailhash",
                "shipAddr"    => "$s_address",
                "shipCity"    => "$s_city",
                "shipRegion"  => "$s_state",
                "shipPostal"  => "$s_zip",
                "shipCountry" => "$s_country",
                "txnID"       => "$oid"
            );

            my %max_results = MaxMindCreditFraud( \%inputdata );

            %inputdata = (
                "i"           => $ENV{'REMOTE_ADDR'},
                "city"        => "$b_city",
                "region"      => "$b_state",
                "postal"      => "$b_zip",
                "country"     => "$b_country",
                "license_key" => "$license_key"
            );

            my %max_location = MaxMindLocationVerification( \%inputdata );

            $SQL = qq|insert into $closedShop::sql_tables{'maxmind'} set
	  oid='$oid', score="$max_results{'score'}",binMatch="$max_results{'binMatch'}",
  		binName="$max_results{'binName'}",binCountry="$max_results{'binCountry'}",
  		binNameMatch="$max_results{'binNameMatch'}",binPhoneMatch="$max_results{'binPhoneMatch'}",
  		binPhone="$max_results{'binPhone'}",freeMail="$max_results{'freeMail'}",
  		carderEmail="$max_results{'carderEmail'}",anonymousProxy="$max_results{'anonymousProxy'}",
  		ip_region="$max_location{'ip_region'}",ip_city="$max_location{'ip_city'}",
		ip_isp="$max_location{'ip_isp'}",countryMatch="$max_location{'countryMatch'}",
  		highRiskCountry="$max_location{'highRiskCountry'}",
		distance="$max_location{'distance'}",shipForward="$max_results{'shipForward'}",
  		custPhoneInBillingLoc="$max_results{'custPhoneInBillingLoc'}" |;
            $sth = closedShop::DatabaseQuery($SQL);
        }
    }

    if ( $cc eq "paypal" ) {
        my $pemail = $closedShop::PayPalEmail;
        if (!$pemail) { $pemail = $closedShop::CompanyEmail; }
        
        my $at     = "@";
        my $per    = "%40";
        my $per2   = "%3A";
        $pemail =~ s/$at/$per/g;
        my $returnpage = "$closedShop::FileNames{cart}?action=Checkout&selected=5";
        if ( $returnpage !~ /^http/ ) {
            $returnpage = $closedShop::urltofolder . "/" . $returnpage;
        }
        $returnpage =~ s/:/$per2/i;
        my $currency = "USD";
        my $pound    = pack( "c", 163 );
        my $yen      = pack( "c", 165 );
        my $euro     = pack( "c", 128 );
        if    ( $closedShop::CurrencyMarker eq $pound )   { $currency = "GBP"; }
        elsif ( $closedShop::CurrencyMarker eq $yen )     { $currency = "YEN"; }
        elsif ( $closedShop::CurrencyMarker eq $euro )    { $currency = "EUR"; }
        elsif ( $closedShop::CompanyCountry eq "Canada" ) { $currency = "CAD"; }

        print header( -Location => "https://www.paypal.com/cgi-bin/webscr?_xclick=1&cmd=_xclick&business=$pemail&item_name=Shopping+from+$closedShop::CompanyName&item_number=item$oid&amount=$total_cost&no_note=1&currency_code=$currency&return=$returnpage" );
        return;
    }
    else {
        return FINALDISPLAY($checkmesg);
    }
}

###############################################################################
# FUNCTION:  Authorize.NET
# DESCRIPTION: process max mind
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AuthorizeNET() {
    my %data = %{ $_[0] };
    require LWP::UserAgent;

    my $VERSION      = '1.3';
    my $API_VERSION  = join( '/', 'Perl', $VERSION );
    my $query_string = join( '&', map { "$_=" . $data{$_} } keys %data );
    $query_string .= "&x_version=3.1&x_delim_data=true&x_relay_response=false";
    my $ua  = LWP::UserAgent->new;
    my $url = "https://secure.authorize.net/gateway/transact.dll";

    my $request = HTTP::Request->new( 'POST', join( '?', $url, $query_string ) );

    #  print "Content-type: text/html \n\n";
    # print  "sending HTTP::Request: " . $request->as_string . " with query_string=" . $query_string;

    my @con;
    my $response = $ua->request($request);
    if ( $response->is_success ) {
        my $content = $response->content;
        @con = split( /,/, $content );
    }

    # print join("",@con);
    return @con;

}

###############################################################################
# FUNCTION:  MaxMindCreditFraud
# DESCRIPTION: process max mind
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub MaxMindCreditFraud() {
    my %data = %{ $_[0] };
    require LWP::UserAgent;

    my $VERSION      = '1.3';
    my $API_VERSION  = join( '/', 'Perl', $VERSION );
    my $query_string = join( '&', map { "$_=" . $data{$_} } keys %data );
    $query_string .= "&clientAPI=$API_VERSION";
    my $ua  = LWP::UserAgent->new;
    my $url = "https://www.maxmind.com/app/ccv2r";

    my $request = HTTP::Request->new( 'GET', join( '?', $url, $query_string ) );

    #   print  "sending HTTP::Request: " . $request->as_string . " with query_string=" . $query_string;

    my %output;
    my $response = $ua->request($request);
    if ( $response->is_success ) {
        my $content = $response->content;
        my @kvpair  = split( ';', $content );

        for my $kvp (@kvpair) {
            my ( $key, $value ) = split( '=', $kvp, 2 );
            $output{$key} = $value;
        }
    }
    return %output;

}

###############################################################################
# FUNCTION:  MaxMindLocationVerification
# DESCRIPTION: process max mind
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub MaxMindLocationVerification() {
    my %data = %{ $_[0] };
    require LWP::UserAgent;

    my $query_string = join( '&', map { "$_=" . $data{$_} } keys %data );
    my $ua           = LWP::UserAgent->new;
    my $url          = "https://www.maxmind.com/app/locvr";

    my $request = HTTP::Request->new( 'GET', join( '?', $url, $query_string ) );

    #  print  "sending HTTP::Request: " . $request->as_string . " with query_string=" . $query_string;

    my %output;
    my $response = $ua->request($request);
    if ( $response->is_success ) {
        my $content = $response->content;
        my @kvpair  = split( ';', $content );

        for my $kvp (@kvpair) {
            my ( $key, $value ) = split( '=', $kvp, 2 );
            $output{$key} = $value;
        }
    }
    return %output;

}

###############################################################################
# FUNCTION:  CheckoutStep3
# DESCRIPTION: checkout process
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub CheckoutStep3 {
    my $cc    = closedShop::ValidateInputCustomer( $query->param("cc") );
    my $ccnum = $query->param("ccnum");
    $ccnum =~ tr/0-9//cd;
    my $ccname = closedShop::ValidateInputCustomer( $query->param("ccname") );

    my $binName     = closedShop::ValidateInputCustomer( $query->param("binName") );
    my $binPhone    = closedShop::ValidateInputCustomer( $query->param("binPhone") );
    my $taxExemptID = closedShop::ValidateInputCustomer( $query->param("taxExemptID") );

    my $ccmonth = $query->param("ccmonth");
    $ccmonth =~ tr/0-9//cd;
    my $ccyear = $query->param("ccyear");
    $ccyear =~ tr/0-9//cd;
    my $ccvcode = $query->param("ccvcode");
    $ccvcode =~ tr/0-9//cd;
    my $gc_code = closedShop::ValidateInputCustomer( $query->param("gc_code") );
    $gc_code =~ tr/a-zA-Z0-9//cd;

    my $SQL = qq|select amount_left,active from $closedShop::sql_tables{"gc"} where
		gc_code="$gc_code"|;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $amount_left, $active ) = $sth->fetchrow_array();

    $ccnum = ValidateCard($ccnum);

    my $cart = closedShop::GetCookie("cart");
    my $oid  = closedShop::GetCookie("oid");

    if (   !$ccnum
        && !$amount_left
        && $cc ne "check"
        && $cc ne "paypal"
        && $cc ne "cod" )
    {
        closedShop::Header();
        print qq|<font class="main"> $closedShop::TEXT{'YR_CC'}|;
        return CREDITCARD();
    }
    if (   ( !$cc || !$ccname || !$ccmonth || !$ccyear )
        && !$amount_left
        && $cc ne "check"
        && $cc ne "paypal"
        && $cc ne "cod" )
    {
        closedShop::Header();
        print qq|<font class="main"> $closedShop::TEXT{'FLL_ALL'}|;
        return CREDITCARD();
    }
    if ( $active ne "Y" && $gc_code ) {
        closedShop::Header();
        print qq|<font class="main"><font color="red"> $closedShop::TEXT{'GC_INV'}
		 <a href="mailto:$closedShop::CompanyEmail">$closedShop::CompanyEmail</a>|;
        return CREDITCARD();
    }

    my ( $session, $uid ) = closedShop::GetSessionCookie();
    if ( !closedShop::Authenticate( $session, $uid, "user" ) ) {
        closedShop::Header();
        print qq|<font class="main"><font color="red">$closedShop::TEXT{'PLS_LOGIN'}</font>|;
        return DISPLAY();
    }

    my @ary = split( /$closedShop::CookieItemDelimiter/, $cart );

    $SQL = qq|select $closedShop::sql_tables{"groups"}.name,
		discount from $closedShop::sql_tables{"user"},$closedShop::sql_tables{"groups"}
		where $closedShop::sql_tables{"user"}.id = '$uid' and 
		$closedShop::sql_tables{"user"}.groupid = $closedShop::sql_tables{"groups"}.id |;
    $sth = closedShop::DatabaseQuery($SQL);
    my ( $gname, $gdiscount ) = $sth->fetchrow_array();

    my $k         = 1;
    my $grand     = 0;
    my $shiptotal = 0;
    my $totalweight = 0;
    foreach (@ary) {
        if ( !$_ ) { next; }
        my ( $id, $qty, $attr ) =
          split( /$closedShop::CookieSectionDelimiter/, $_ );

        if ( $id eq "gc" ) {
            my ( $id, $recipient_name, $recipient_email, $gc_amount ) =
              split( /$closedShop::CookieSectionDelimiter/, $_ );
            my $total = sprintf( "%.2f", $gc_amount );
            $grand += $total;
            next;
        }

        my @attr = ();
        if ($attr) {
            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
        }

        my $attr_cost = 0.00;
        foreach my $at (@attr) {
            my ( $prod_id, $attr_id, $data_id ) =
              split( /$closedShop::GiftCertDelimiter/, $at );
            my $SQL = qq|select name,data,price from $closedShop::sql_tables{"attributes"},
	      $closedShop::sql_tables{"attribute_values"} where product_id='$prod_id' and
	      $closedShop::sql_tables{"attributes"}.id = '$attr_id' and
	      $closedShop::sql_tables{"attribute_values"}.id = '$data_id'|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $atname, $atdata, $atprice ) = $sth->fetchrow_array();
            if ( $atprice > 0 ) {
                $attr_cost += $atprice;
            }
        }

        my $SQL = qq| select name,price,shipping_price,description,
	              category,reduced_shipping_price,weight from
			$closedShop::sql_tables{"item"} where id="$id" |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ( $name, $price, $ship, $desc, $cat, $rship, $weight ) = $sth->fetchrow_array();

        my $total = sprintf( "%.2f", $qty * ( $price + $attr_cost ) );

        my ( $shiptot, $shipstring ) = closedShop::GetShippingCostsItem( $ship, $qty, $rship, $weight );

        $total     += $shiptot;
        $grand     += $total;
        $shiptotal += $shiptot;
        $totalweight += $qty*$weight;
    }
    
	my ( $shipclass, $shipstring1 ) = closedShop::GetShippingCostsClass( $totalweight, $grand );

	if ($shiptotal == 0) {
		$shiptotal = $shipclass;
		$grand += $shiptotal;
	}
	

    $SQL = qq|update $closedShop::sql_tables{"order"} set cc="$cc",
	   cc_name="$ccname",cc_year="$ccyear",cc_month="$ccmonth",
           cc_num="$ccnum",cc_vcode="$ccvcode",binName="$binName",
	   binPhone="$binPhone",item_ids="$cart",ship_total="$shiptotal",sub_total="$grand",
	   taxExemptID="$taxExemptID" where id = "$oid" and user_id="$uid"|;
    $sth = closedShop::DatabaseQuery($SQL);

    closedShop::Header();
    print qq|<form action="|;

    if ( $closedShop::ForceSSL eq "Y" && $closedShop::ForceCartURL ) {
        print $closedShop::ForceCartURL;
    }
    else {
        print $program_name;
    }

    print qq|" method="POST"><font class="main">
	    <input type="hidden" name="action" value="Checkout">
			<input type="hidden" name="gc_code" value="$gc_code">
	    <input type="hidden" name="selected" value="4">
		<input type="hidden" name="oid" value="$oid">
		<b>$closedShop::TEXT{'PLS_REV'}</b><br>
		<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	    <font class="head">$closedShop::TEXT{'CHK_PROC'}
		- Step 4</font></th></tr>

        <tr class="mainrow">
        <td colspan="2"><div align="center">
		<font class="main"><strong>$closedShop::TEXT{'REV_ORD'}</strong></div></td></tr>

		<tr class="mainrow">
        <td ><font class="main">
		<strong>$closedShop::TEXT{'BILL_NFO'}</strong><br>
		|;
    $SQL = qq| select b_fname,b_lname,b_address,b_city,b_state,
                      b_zip,b_phone,b_country,s_fname,s_lname,s_address,s_city,s_state,
                      s_zip,s_phone,s_country,item_ids,cc_name,cc,cc_num,cc_month,cc_year
	    	      from $closedShop::sql_tables{"order"} 
	              where id="$oid" and user_id="$uid" |;
    $sth = closedShop::DatabaseQuery($SQL);
    my ( $b_fname, $b_lname, $b_address, $b_city, $b_state, $b_zip, $b_phone, $b_country, $s_fname, $s_lname, $s_address, $s_city, $s_state, $s_zip, $s_phone, $s_country, $item_ids, $cc_name, $card, $cc_num, $cc_month, $cc_year ) = $sth->fetchrow_array();
    my $cc_num2 = $cc_num;
    $cc_num = substr( $cc_num, 0, 4 ) . "**** **** **** ";

    print qq|
		$b_fname $b_lname<br>
		$b_address<br>
		$b_city, $b_state $b_zip<br>
		$b_country<BR>
		$b_phone
		</td>
		<td ><font class="main">
		<strong>$closedShop::TEXT{'SHIP'} $closedShop::TEXT{'NFO'} </strong><br>
		$s_fname $s_lname<br>
		$s_address<br>
		$s_city, $s_state $s_zip<br>
		$s_country<BR>
		$s_phone
		</td></tr>
		<tr class="mainrow">
        <td colspan="2"><font class="main">
		<strong>$closedShop::TEXT{'PAY_NFO'}</strong><br>
		|;
    if ( $card eq "check" || $card eq "paypal" || $card eq "cod" ) {
        print qq|&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'PAY_METH'}</u>: $closedShop::AcceptedCards{$card}<BR>|;
    }
    else {
        print qq|
		&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'CC'}</u>: $closedShop::AcceptedCards{$card}<br>
		&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'CC_NAME'}</u>: $cc_name<br>
		&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'CC_NUM'}</u>: $cc_num<br>
		&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'EXP_DATE'}</u>: $cc_month / $cc_year<BR>
		  |;
    }

    if ( $closedShop::EnableTaxExempt eq "Y" && $taxExemptID != "" ) {
        print qq|<BR>&nbsp;&nbsp;&nbsp;<u>Tax Exempt</u>: $taxExemptID|;
    }

    print qq|		
		</td></tr>
		<tr class="mainrow"><td colspan="2">
		<font class="main">
		<b>$closedShop::TEXT{'PROD'} $closedShop::TEXT{'NFO'}</b>
		<center>
		<table width="100%"  class="main" cellpadding="0" cellspacing="2">
		<tr class="mainrow">
		<td><font class="main"><strong>$closedShop::TEXT{'NAME'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'QTY'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'ATT'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'PRC'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'SHIP'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'TOT'}</strong></td>
		</tr>
		|;

    @ary   = split( /$closedShop::CookieItemDelimiter/, $item_ids );
    $k     = 1;
    $grand = 0;
    my $shipsub = 0;
    my $itemsub = 0;
    my $totalweight = 0;
    foreach (@ary) {
        if ( !$_ ) { next; }
        my ( $id, $qty, $attr ) =
          split( /$closedShop::CookieSectionDelimiter/, $_ );

        my $rowcolor = "style='background-color: white;'";

        if ( $id eq "gc" ) {
            my ( $id, $recipient_name, $recipient_email, $gc_amount ) =
              split( /$closedShop::CookieSectionDelimiter/, $_ );

            my $total = sprintf( "%.2f", $gc_amount );
            $grand += $total;

            print qq|
	   	    <tr class="mainrow" $rowcolor>
		      <td colspan="3"><font class="main"> $closedShop::TEXT{'GC'}
					($closedShop::CurrencyMarker $gc_amount)</td>
		     <td><font class="main"> 1 x $closedShop::CurrencyMarker$gc_amount</td>
		     <td><font class="main"> 1 x $closedShop::CurrencyMarker 0.00</td>
		     <td><font class="main"> $closedShop::CurrencyMarker$total</td>
		     </tr>  |;
            $k++;
            next;
        }

        if ( $qty <= 0 ) {
            next;
        }

        my @attr = ();
        if ($attr) {
            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
        }
        $attr = "";

        my $attr_cost = 0.00;
        foreach my $at (@attr) {
            my ( $prod_id, $attr_id, $data_id ) =
              split( /$closedShop::GiftCertDelimiter/, $at );
            my $SQL = qq|select name,data,price from $closedShop::sql_tables{"attributes"},
	   $closedShop::sql_tables{"attribute_values"} where product_id='$prod_id' and
	   $closedShop::sql_tables{"attributes"}.id = '$attr_id' and
	   $closedShop::sql_tables{"attribute_values"}.id = '$data_id'|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $atname, $atdata, $atprice ) = $sth->fetchrow_array();
            $attr .= "<nobr><u>$atname</u>: $atdata";
            if ( $atprice > 0 ) {
                $attr .= " (+$closedShop::CurrencyMarker$atprice)";
                $attr_cost += $atprice;
            }
            $attr .= "</nobr><BR>";
        }

        if ( !$attr ) { $attr = "&nbsp;"; }

        $SQL = qq| select name,price,shipping_price,description,
	           category,reduced_shipping_price,weight from $closedShop::sql_tables{"item"} where
		   id="$id" |;
        $sth = closedShop::DatabaseQuery($SQL);
        my ( $name, $price, $ship, $desc, $cat, $rship, $weight ) = $sth->fetchrow_array();

        if ( $weight && $closedShop::ShowShipping eq "WEIGHT" ) {
            $attr .= sprintf( "%.2f", $weight ) . " $closedShop::WeightIdentifier";
        }

		$totalweight += $qty*$weight;
        my $total = $qty * ( $price + $attr_cost );
        $itemsub += $total;

        my ( $shiptotal, $shipstring ) = closedShop::GetShippingCostsItem( $ship, $qty, $rship, $weight );

        $total   += $shiptotal;
        $shipsub += $shiptotal;

        $grand += $total;
        $total = sprintf( "%.2f", $total );
        print qq|
	   	  <tr class="mainrow" $rowcolor>
		  <td><font class="main"> <a 
		  href="$closedShop::FileNames{item}?action=ShowItem&id=$id">$name</a></td>
		  <td><font class="main"> $qty</td>
		  <td><font class="main"> $attr</td>
		  <td><font class="main"> $qty x $closedShop::CurrencyMarker$price|;
        if ( $attr_cost > 0 ) {
            print qq|<BR></nobr>+($qty x $closedShop::CurrencyMarker$attr_cost)</nobr>|;
        }
        print qq| </td>
		<td><font class="main"> $shipstring </td>
		<td><font class="main"> $closedShop::CurrencyMarker$total</td> </tr>  |;
        $k++;
    }

	my ( $shipclass, $shipstring1 ) = closedShop::GetShippingCostsClass( $totalweight, $grand );
	if ($shipsub == 0) { 
		$shipsub = $shipclass; 
		$grand += $shipsub;
	}
	
	
    my ( $stax, $ctax, $itax ) = closedShop::GetTaxCosts( $grand, $taxExemptID, $b_state, $b_country );
    my $tax = $stax + $ctax + $itax;

    my $total_cost = $tax + $grand;
    $total_cost = sprintf( "%.2f", $total_cost );
    $grand      = sprintf( "%.2f", $grand );
    $shipsub    = sprintf( "%.2f", $shipsub );

    my $vrate = ( $closedShop::TaxRate * 100 );
    print qq|<tr class="mainrow">
	<td colspan="3" align="right">&nbsp;</td><td colspan='2'><font class="main">
     <strong>Item $closedShop::TEXT{'TOT'}: </td>
     <td><font class="main"> $closedShop::CurrencyMarker|
      . sprintf( "%.2f", $itemsub ) . qq|</strong></td></tr>
	
	<td colspan="3" align="right">&nbsp;</td><td colspan='2'><font class="main">
    <strong>$closedShop::TEXT{'SHIP'} $closedShop::TEXT{'TOT'}</strong>:  $shipstring1</td>
    <td><font class="main"> $closedShop::CurrencyMarker$shipsub</strong></td></tr>|;

    if ( $closedShop::ShowTax eq "Y" ) {

        if ( $closedShop::TaxRate > 0 ) {
            print qq|
		  <tr class="mainrow"><td colspan="3" align="right">&nbsp;</td>
		  <td colspan='2'><font class="main"> &nbsp;&nbsp;&nbsp;&nbsp;$closedShop::TEXT{'TAX'} $closedShop::TEXT{'TOT'}(for $closedShop::CompanyState at $vrate%):</td>
      <td><font class="main"> $closedShop::CurrencyMarker$stax </td></tr>
			|;
        }
        if ( $closedShop::TaxRateCountry > 0 ) {
            my $crate = ( $closedShop::TaxRateCountry * 100 );
            print qq|
		     <tr class="mainrow"><td colspan="3" align="right">&nbsp;</td>
		     <td colspan='2'><font class="main">&nbsp;&nbsp;&nbsp;&nbsp;$closedShop::TEXT{'TAX'} $closedShop::TEXT{'TOT'} (for $closedShop::CompanyCountry at $crate%):</td>
         <td><font class="main"> $closedShop::CurrencyMarker$ctax </td></tr>
			   |;
        }
        if (   $closedShop::TaxRateInt > 0
            && $closedShop::CompanyCountry ne $b_country )
        {
            my $irate = ( $closedShop::TaxRateInt * 100 );
            print qq|
		     <tr class="mainrow"><td colspan="3" align="right">&nbsp;</td>
		     <td colspan='2'><font class="main"> &nbsp;&nbsp;&nbsp;&nbsp;$closedShop::TEXT{'TAX'} $closedShop::TEXT{'TOT'} (for $closedShop::TEXT{'INTNATL'} at $irate%):</td>
         <td><font class="main"> $closedShop::CurrencyMarker$itax </td></tr>
			   |;
        }
    }

    my $amount_deduct = $amount_left;
    my $pre_t         = $total_cost;
    if ( $amount_left > 0 ) {
        $total_cost -= $amount_left;
        if ( $total_cost <= 0 ) {
            $amount_deduct = $pre_t;
            $amount_left   = -($total_cost);
            $total_cost    = 0;
        }
        else {
            $amount_left = 0;
        }
        print qq|		
		   <tr class="mainrow"><td colspan="3" align="right">&nbsp;</td><td colspan='2'><font class="main">
     	 <strong><font color='red'>$closedShop::TEXT{'GC'} $closedShop::TEXT{'DED'}</font>:</td>
			 <td><font class="main"><font color='red'>-$closedShop::CurrencyMarker$amount_deduct</strong></td></tr>
			 |;
    }

    if ( $gname || $gdiscount ) {
        my $disc = sprintf( "%.2f", $total_cost * ( $gdiscount / 100 ) );
        $total_cost = $total_cost - $disc;

        print qq|		
	<tr class="mainrow"><td colspan="3" align="right">&nbsp;</td><td colspan='2'><font class="main">
     	 <font color='red'>$gname Discount ($gdiscount\%)</font>:</td>
	 <td><font class="main"><font color='red'>-$closedShop::CurrencyMarker$disc</strong></td></tr>
			 |;

    }

    $SQL = qq|update $closedShop::sql_tables{"order"} set tax="$tax",itax='$closedShop::TaxRateInt',
    ctax='$closedShop::TaxRateCountry',stax='$closedShop::TaxRate',
    discount='$gdiscount',total="$total_cost" where id = "$oid" and user_id="$uid"|;
    $sth = closedShop::DatabaseQuery($SQL);

    print qq|
		<tr class="mainrow"><td colspan="3" align="right">&nbsp;</td><td colspan='2'><font class="main">
     	<strong>Grand $closedShop::TEXT{'TOT'}:</td><td><font class="main"><b> $closedShop::CurrencyMarker$total_cost</b></strong></td></tr>
		</table></td></tr>|;

    if ( $total_cost < $closedShop::MinimumOrder && $pre_t == $total_cost ) {
        print qq|
		<tr class="mainrow">
        <td align="" colspan="2"><font class="main"><div align="center">
		<b>$closedShop::TEXT{'BEL_MIN'} ($closedShop::CurrencyMarker $closedShop::MinimumOrder) amount</b></div>
		</td></tr>	|;
    }
    elsif ( $cc_num2 eq "1111111111111111" ) {
        print qq|
		<tr class="mainrow">
        <td align="" colspan="2"><font class="main"><div align="center">
		<b>$closedShop::TEXT{'CC_NOTREAL'}</b></div>
		</td></tr>|;
    }
    elsif (( !$cc || !$ccname || !$ccmonth || !$ccyear || !$ccnum )
        && $total_cost > 0
        && $cc ne "check"
        && $cc ne "paypal"
        && $cc ne "cod" )
    {
        print qq|
		<tr class="mainrow">
        <td align="" colspan="2"><font class="main"><div align="center">
		<b>$closedShop::TEXT{'NOT_GCFULL'}</b><BR>
                <a href="javascript:history.go(-1);">$closedShop::TEXT{'GBACK'}</a></div>
		</td></tr>	|;
    }
    else {
        print qq|
		<tr class="mainrow">
        <td align="" colspan="2"><font class="main"><div align="center">
	<input type="button" value="<-- $closedShop::TEXT{'GBACK'}" onClick="javascript:history.go(-1);">&nbsp;&nbsp;&nbsp;
		<input type="submit" value="$closedShop::TEXT{'COMP_ORD'}"></div>
		</td></tr>	|;
    }

    print qq|	</table>	|;

}

###############################################################################
# FUNCTION:  CheckoutStep2
# DESCRIPTION: checkout process
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub CheckoutStep2 {
    my ( $session, $uid ) = closedShop::GetSessionCookie();

    my %d = ();
    foreach (
        qw/shipfname shiplname shipaddress shipcity shipstate shipzip
        shipphone shipcountry billfname billlname billaddress
        billcity billstate billzip billphone billcountry/
      )
    {
        if ( $closedShop::RestrictState eq "Y"
            && ( $_ eq "shipstate" || $_ eq "billstate" ) )
        {
            next;
        }

        $d{$_} = closedShop::ValidateInputCustomer( $query->param("$_") );
        if ( $_ =~ /bill/ ) {
            if ( !$d{$_} || ( $_ eq "billstate" && $d{$_} eq "NONE" ) ) {
                my $tmp = $_;
                $tmp =~ s/bill/ship/;
                $d{$_} = $d{$tmp};
            }
        }
	  }
    
     foreach (
        qw/shipfname shiplname shipaddress shipcity shipstate shipzip
        shipphone shipcountry billfname billlname billaddress
        billcity billstate billzip billphone billcountry/ )  {
	
	  	  if ( !$d{$_} ) {
            closedShop::Header();
            SHIPBILL($uid,\%d);
            exit();
        }
  	}


    if ( !closedShop::Authenticate( $session, $uid, "user" ) ) {
        closedShop::Header();
        print qq|<font class="main"><font color="red">$closedShop::TEXT{'PLS_LOGIN'}</font>|;
        return DISPLAY();
    }

    my $SQL = qq|insert into $closedShop::sql_tables{"order"} set user_id="$uid",
	  s_fname="$d{'shipfname'}",s_lname="$d{'shiplname'}",
	  s_address="$d{'shipaddress'}",s_city="$d{'shipcity'}",s_state="$d{'shipstate'}",
	  s_zip="$d{'shipzip'}",s_phone="$d{'shipphone'}",s_country="$d{'shipcountry'}",
	  b_fname="$d{'billfname'}",b_lname="$d{'billlname'}",b_country="$d{'billcountry'}",
	  b_address="$d{'billaddress'}",b_city="$d{'billcity'}",b_state="$d{'billstate'}",
	  b_zip="$d{'billzip'}",b_phone="$d{'billphone'}" |;
    my $sth      = closedShop::DatabaseQuery($SQL);
    my $order_id = $sth->{'mysql_insertid'};

    $SQL = qq|update $closedShop::sql_tables{"user"} set 
	  address="$d{'billaddress'}",city="$d{'billcity'}",state="$d{'billstate'}",
	  zip="$d{'billzip'}",phone="$d{'billphone'}",country="$d{'billcountry'}" where id="$uid" |;
    $sth = closedShop::DatabaseQuery($SQL);

    my $cookie = new CGI::Cookie( -name => 'oid', -value => $order_id );
    closedShop::SaveCookie( "oid", "$order_id" );

    print header(
        -Cookie  => [$cookie],
        -expires => '+1d',
        -type    => "text/html"
    );
    closedShop::Header("nohttp");
    CREDITCARD();

}

###############################################################################
# FUNCTION:  CheckoutStep1
# DESCRIPTION: checkout process
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub CheckoutStep1 {
    if ( $query->param("new") ) {

        my $fname    = closedShop::ValidateInputCustomer( $query->param("fname") );
        my $lname    = closedShop::ValidateInputCustomer( $query->param("lname") );
        my $email    = closedShop::ValidateInputCustomer( $query->param("email2") );
        my $password = closedShop::ValidateInputCustomer( $query->param("password2") );
        my $confirm  = closedShop::ValidateInputCustomer( $query->param("confirm") );

        if ( $password ne $confirm ) {
            closedShop::Header();
            print qq|<font class="main"><font color="red">$closedShop::TEXT{'NO_MATCH'}</font>|;
            return DISPLAY();
        }
        if ( !$fname || !$lname || !$email || !$password ) {
            closedShop::Header();
            print qq|<font class="main"><font color="red">$closedShop::TEXT{'FLL_ALL'}</font>|;
            return DISPLAY();
        }
        my $SQL = qq| select email from $closedShop::sql_tables{"user"} 
		              where email="$email" |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ($temail) = $sth->fetchrow_array();
        if ($temail) {
            closedShop::Header();
            print qq|<font class="main"><font color="red">$closedShop::TEXT{'EM_USED'}</font>|;
            return DISPLAY();
        }
        $SQL = qq| insert into $closedShop::sql_tables{"user"} 
		           set email="$email",fname="$fname",lname='$lname',password="$password" |;
        $sth = closedShop::DatabaseQuery($SQL);
        my $new_id = $sth->{'mysql_insertid'};
        LoginUser($new_id);

        my $force = "$program_name?action=Checkout&selected=2";
        if ( $closedShop::ForceSSL eq "Y" ) {
            my $force = "https://$ENV{'SERVER_NAME'}$ENV{'SCRIPT_NAME'}";

            if ($closedShop::ForceCartURL) {
                $force = $closedShop::ForceCartURL;
            }
            $force .= "?action=Checkout&selected=2";
        
        print qq|
			<html><head><meta http-equiv="Content-Type" 
			content="text/html; charset=utf-8"">
		    <meta http-equiv="refresh" content="0; URL="$force">
   			 <title></title></head><body
		    <font style='font-family: $closedShop::fontmain_family;'>
		      $closedShop::CompanyName requires you to use Secure
    	 	 Checkout in order to protect your personal information.<BR>
	      	<BR>If you are not automatically forwarded please go to 
    		  <a href="$force">here</a> instead.
		    </body></html> |;
  	      return;
        }
        else {
       		closedShop::Header("nohttp");
       		SHIPBILL($new_id);
       }
    }
    else {
        my $email = $query->param("email");
        $email = closedShop::ValidateInputCustomer($email);
        my $password = $query->param("password1");
        $password = closedShop::ValidateInputCustomer($password);
        my $SQL = qq| select id from $closedShop::sql_tables{"user"}
		              where email="$email" and password = "$password" |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ($uid) = $sth->fetchrow_array();

        if ( !$uid || !$email || !$password ) {
            closedShop::Header();
            print qq|<font class="main"><font color="red">$closedShop::TEXT{'BAD_EM'}</font>|;
            DISPLAY();
        }
        else {
            LoginUser($uid);
            closedShop::Header("nohttp");
            SHIPBILL($uid);
        }
    }
}

###############################################################################
# FUNCTION:  Checkout
# DESCRIPTION: checkout process
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Checkout {
    my $message = shift;

    my $cart = closedShop::GetCookie("cart");
    if ( !$cart ) {
        return ViewCart();
    }

    if ( $query->param("selected") == 5 ) {
        FINALDISPLAY( $closedShop::TEXT{"SUCC_PP"} );
    }
    elsif ( $query->param("selected") == 4 ) {
        CheckoutStep4();
    }
    elsif ( $query->param("selected") == 3 ) {
        CheckoutStep3();
    }
    elsif ( $query->param("selected") == 2 ) {
        CheckoutStep2();
    }
    elsif ( $query->param("selected") == 1 ) {
        CheckoutStep1();
    }
    else {

        my ( $session, $uid ) = ();
        ( $session, $uid ) = closedShop::GetSessionCookie();

        closedShop::Header();

        if ( closedShop::Authenticate( $session, $uid, "user" ) ) {
            SHIPBILL($uid);
        }
        else {
            DISPLAY();
        }
    }
}

###############################################################################
# FUNCTION:  FINALDISPLAY
# DESCRIPTION: text for final display
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub FINALDISPLAY {
    my $mesg      = "<B>" . $_[0] . "</b><BR><BR>" if ( $_[0] );
    my $finalpage = $closedShop::website;
    if (   $closedShop::CompletedOrderURL ne ""
        && $closedShop::CompletedOrderURL ne "http://" )
    {
        $finalpage = $closedShop::CompletedOrderURL;
    }

    my $cookie1 = new CGI::Cookie( -name => 'cart', -value => '' );
    my $cookie2 = new CGI::Cookie( -name => 'oid',  -value => '' );
    closedShop::SaveCookie( "cart", "" );
    closedShop::SaveCookie( "oid",  "" );

    print header( -Cookie => [ $cookie1, $cookie2 ], -type => "text/html" );
    closedShop::Header("nohttp");
    print qq|
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	    <font class="head">$closedShop::TEXT{'CHK_PROC'}
		- $closedShop::TEXT{'COMP_ORD'}</font></th></tr>
		<tr class="mainrow"><td colspan="2"><font class="main"> $mesg
		$closedShop::TEXT{'THNK_PURC'}
		<br><BR>
		$closedShop::FinalMessage<br>
		<br>
		<div align="center"><a href="$finalpage">$closedShop::TEXT{'RET_HOME'}</a></div>
		</td></tr></table>
		|;
    closedShop::Footer();
}

###############################################################################
# FUNCTION:  CREDITCARD
# DESCRIPTION: text for getting CC info
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub CREDITCARD {
    print qq|<font class="main">
	
	<script language='JavaScript'>
	<!--
		function checktoggle(item,div) {
			if (item == "check" \|\| item == "paypal" \|\|
			item == "cod") {
				hide(div);
			}
			else {
				show(div);
			}
		}
	-->
	</script>
	
    <form name="form1" action="|;

    if ( $closedShop::ForceSSL eq "Y" && $closedShop::ForceCartURL ) {
        print $closedShop::ForceCartURL;
    }
    else {
        print $program_name;
    }

    print qq|" method="POST">
    <input type="hidden" name="action" value="Checkout">
    <input type="hidden" name="selected" value="3">
	<br>$closedShop::TEXT{'FLL_ALL'}<br>
	<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
    <font class="head"> $closedShop::TEXT{'CHK_PROC'}
	- Step 3</font></th></tr>
    <tr class="mainrow">
    <td colspan="2"><div align="center">
	<font class="main"><strong>$closedShop::TEXT{'PAY_NFO'}</strong></div></td></tr>
   <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'PAY_METH'}: </td>
 	<td><font class="main"> 
	<select name="cc" onChange="checktoggle(document.form1.cc.value,'ccinfo');" >|;
    my $paypal = 0;
    my $accept_cc = 0;
    my ($vis,$dis) = ("hidden","none");
    my $firstitem = 0;
    foreach ( sort keys %closedShop::AcceptedCards ) {
    	
        if ( $_ eq "paypal" ) { $paypal = 1; }
        print qq|<option value="$_">$closedShop::AcceptedCards{$_}</option>\n|;
       
		if ($_ eq "mastercard" || $_ eq "discover" || $_ eq "visa" ||
        	$_ eq "amex" || $_ eq "diner") { 
        	$accept_cc = 1; 
        	if ($firstitem == 0) { 
        		($vis,$dis) = ("visible","block");
        	}
        }
        $firstitem++;
    }
    print qq|
	</select>
	</td></tr>
	|;
	
	if ($accept_cc) {
		print qq|
		<tr><td colspan='2'>
		<div style="visibility: collapse;">
		
		<div name='ccinfo' id='ccinfo' style="visibility: $vis; 
		display: $dis;	position: relative; z-index: 1;">
		
		
		<table width=100% cellspacing=1 cellpadding=0>
   		 <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'CC_NUM'}: </td>
  		<td><font class="main"> <input type="text" name="ccnum" value="" 
		size="16" maxlength="16"></td></tr>
		<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'CC_NAME'}: </td>
  		<td><font class="main"> <input type="text" name="ccname" value="" 
		size="25" ></td></tr>
		<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'EXP_DATE'}: </td>
  		<td><font class="main"> <input type="text" name="ccmonth" value=""
		size="2" maxlength="2"> / <input type="text" name="ccyear" value="" 
		size="4" maxlength="4"></td></tr>
        <tr class="mainrow"><td valign=top><font class="main">
         $closedShop::TEXT{'CC_VCODE'}:<BR><i>($closedShop::TEXT{'CC_VCODE_EXPLANATION'})</i></td>
       <td valign=top><font 
       class="main"><input type="text" name="ccvcode" maxlength='3'
        value="" size="3" ></td></tr>
		|;

	    if ( $closedShop::EnableMaxMind eq "Y" ) {
        	print qq|<tr class="mainrow"><td valign=top><font class="main">
	         Name of Bank that issued the credit card<BR><i>(may be found on back of credit card)</i></td>
    	   <td valign=top><font 
	       class="main"><input type="text" name="binName" value="" size="25" ></td></tr>
			<tr class="mainrow"><td valign=top><font class="main">
    	     Customer Service Phone<BR> <i>(may be found on back of credit card)</i></td>
	       <td valign=top><font 
	       class="main"><input type="text" name="binPhone" value="" size="12" ></td></tr>  |;
    	}
    	print qq|
	</table>
	</td></tr>
	</div></div>
	|;
	}
	
	
	
	
    if ( $closedShop::EnableTaxExempt eq "Y" ) {
        print qq|
	  <tr class="mainrow"><td valign=top><font class="main">
         Tax exempt ID: </td>
       <td valign=top><font 
       class="main"><input type="text" name="taxExemptID" value="" size="25" ></td></tr>
	  |;
    }

    print qq|
        <tr class="mainrow"><td colspan=2>&nbsp;</td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'GC'} Code: <BR>
	$closedShop::TEXT{'GC_NOT'}</td>
  	<td><font class="main"> <input type="text" name="gc_code" value="" 
	size="30"> </td></tr>

	<tr class="mainrow"><td colspan="2"><div align="center"><font class="main">
	|;
    if ($paypal) {
        print qq|<B><font size="3">$closedShop::TEXT{'PP_MESG2'}<BR>|;
    }
    print qq|
	<input type="button" value="<-- $closedShop::TEXT{'GBACK'} " onClick="javascript:history.go(-1);">&nbsp;&nbsp;&nbsp;
  	<input type="submit" name="new" value="$closedShop::TEXT{'REV_ORD'} -->"></div></td></tr>
	</table>
    |;
    closedShop::Footer();
}

###############################################################################
# FUNCTION:  SHIPBILL
# DESCRIPTION: text for shipping and billing
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub SHIPBILL {
    my ($uid,$d) = @_;
    
    my %currdata = ();
    if (defined($d)) {
    	%currdata = %{$d};
    }
    
    my $SQL = qq| select fname,lname,address,city,state,zip,phone,country
		          from $closedShop::sql_tables{"user"} where id = "$uid" |;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $fname, $lname, $address, $city, $state, $zip, $phone, $country ) = $sth->fetchrow_array();
    
	if (!$currdata{"shipfname"}) { $currdata{"shipfname"} = $fname; }
	if (!$currdata{"shiplname"}) { $currdata{"shiplname"} = $lname; }
	if (!$currdata{"shipaddress"}) { $currdata{"shipaddress"} = $address; }
	if (!$currdata{"shipcity"}) { $currdata{"shipcity"} = $city; }
	if (!$currdata{"shipzip"}) { $currdata{"shipzip"} = $zip; }
	if (!$currdata{"shipstate"}) { $currdata{"shipstate"} = $state; }	 
	if (!$currdata{"shipphone"}) { $currdata{"shipphone"} = $phone; }	 
	if (!$currdata{"shipcountry"}) { $currdata{"shipcountry"} = $country; }	 
	
	if (!$currdata{"billfname"}) { $currdata{"billfname"} = $fname; }
	if (!$currdata{"billlname"}) { $currdata{"billlname"} = $lname; }
	if (!$currdata{"billaddress"}) { $currdata{"billaddress"} = $address; }
	if (!$currdata{"billcity"}) { $currdata{"billcity"} = $city; }
	if (!$currdata{"billzip"}) { $currdata{"billzip"} = $zip; }
	if (!$currdata{"billstate"}) { $currdata{"billstate"} = $state; }	 
	if (!$currdata{"billphone"}) { $currdata{"billphone"} = $phone; }	 
	if (!$currdata{"billcountry"}) { $currdata{"billcountry"} = $country; }	 
		 
    print qq|<font class="main">
	<form action="|;

    if ( $closedShop::ForceSSL eq "Y" && $closedShop::ForceCartURL ) {
        print $closedShop::ForceCartURL;
    }
    else {
        print $program_name;
    }

    print qq|" method="POST">
	<input type="hidden" name="action" value="Checkout">
	<input type="hidden" name="selected" value="2">
	<br>$closedShop::TEXT{'FLL_ALL'}<br>
	<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	<font class="head"> $closedShop::TEXT{'CHK_PROC'}
	- Step 2</font></th></tr>
     <tr class="mainrow">
	 <td colspan="2"><div align="center"><font class="main">
	 <strong>$closedShop::TEXT{'SHIP'} $closedShop::TEXT{'NFO'}</strong></div></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'FNAME'} </td>
	<td><font class="main"> <input type="text" name="shipfname" value="$currdata{shipfname}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'LNAME'} </td>
	<td><font class="main"> <input type="text" name="shiplname" value="$currdata{shiplname}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'ADDRESS'} </td>
	<td><font class="main"> <textarea rows="2" cols="30" name="shipaddress">$currdata{shipaddress}</textarea></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'CITY'} </td>
	<td><font class="main"> <input type="text" name="shipcity" value="$currdata{shipcity}" size="20"></td></tr>
	|;

    if ( $closedShop::RestrictState ne "Y" ) {
        print qq|<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'STATE'} </td>
	<td><font class="main"> |;
        closedShop::ListStates( "shipstate", $currdata{"shipstate"} );
        print qq|
	</td></tr>
	|;
    }
    print qq|
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'ZIP'} </td>
	<td><font class="main"> <input type="text" name="shipzip" value="$currdata{shipzip}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'COUNTRY'} </td>
	<td><font class="main"> |;
    closedShop::ListCountries( "shipcountry", $currdata{"shipcountry"}, "restrict" );
    print qq|
	</td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'PHONE'} </td>
	<td><font class="main"> <input type="text" name="shipphone" value="$currdata{shipphone}" size="20"></td></tr>
	<tr class="mainrow">
	<td colspan="2"><div align="center"><font class="main"><strong>$closedShop::TEXT{'BILL_NFO'}</strong>
	<br>$closedShop::TEXT{'ON_DIFF'}</div></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'FNAME'} </td>
	<td><font class="main"> <input type="text" name="billfname" value="$currdata{billfname}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'LNAME'} </td>
	<td><font class="main"> <input type="text" name="billlname" value="$currdata{billlname}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'ADDRESS'} </td>
	<td><font class="main"> <textarea rows="2" cols="30" name="billaddress">$currdata{billaddress}</textarea></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'CITY'} </td>
	<td><font class="main"> <input type="text" name="billcity" value="$currdata{billcity}" size="20"></td></tr>
	|;
    if ( $closedShop::RestrictState ne "Y" ) {
        print qq|
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'STATE'} </td>
	<td><font class="main"> |;

        closedShop::ListStates( "billstate", $currdata{"billstate"} );
        print qq|
	</td></tr>
	|;
    }
    print qq|
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'ZIP'} </td>
	<td><font class="main"> <input type="text" name="billzip" value="$currdata{billzip}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'COUNTRY'} </td>
	<td><font class="main"> |;
    closedShop::ListCountries( "billcountry", $currdata{"billcountry"}, "restrict" );
    print qq|
	</td></tr>
	<tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'PHONE'} </td>
	<td><font class="main"> <input type="text" name="billphone" value="$currdata{billphone}" size="20"></td></tr>
	<tr class="mainrow"><td colspan="2"><div align="center"><font class="main">
	<input type="submit" name="new" value="$closedShop::TEXT{'PAY_NFO'} -->"></div></td></tr>
	</table>
	|;
    closedShop::Footer();
}

###############################################################################
# FUNCTION:  Display
# DESCRIPTION: view cart
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DISPLAY {
    print qq|<font class="main">
	  <form action="|;

    if ( $closedShop::ForceSSL eq "Y" && $closedShop::ForceCartURL ) {
        print $closedShop::ForceCartURL;
    }
    else {
        print $program_name;
    }

    print qq|" method="POST">
	  <input type="hidden" name="action" value="Checkout">
	  <input type="hidden" name="selected" value="1">
	  <table width="$closedShop::TableWidth" class="main"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'CHK_PROC'} - Step 1</font></th></tr>

	  <tr class="mainrow">
	  <td colspan="2"><div align="center"><font class="main"><strong>$closedShop::TEXT{'RET_SP'}</strong></div></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'EMAIL'}: </td>
	  <td><font class="main"> <input type="text" name="email" size="20"></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'PASS'}: </td>
	  <td><font class="main"> <input type="password" name="password1" size="20"></td></tr>
	  <tr class="mainrow">
	  <td colspan="2"><div align="center"><font class="main">
	  <input type="submit" name="login" value="$closedShop::TEXT{'LOGIN'}"></div></td></tr>

	  <tr class="mainrow"><td colspan="2">&nbsp;</td></tr>
	  
	  <tr class="mainrow">
	  <td colspan="2"><div align="center"><font class="main"><strong>$closedShop::TEXT{'NEW_SP'}</strong><br>
	  $closedShop::TEXT{'FLL_ALL'}</div></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'FNAME'}: </td>
	  <td><font class="main"> <input type="name" name="fname" size="20"></td></tr>
	   <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'LNAME'}: </td>
	  <td><font class="main"> <input type="name" name="lname" size="20"></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'EMAIL'}: </td>
	  <td><font class="main"> <input type="text" name="email2" size="20"></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'PASS'}: </td>
	  <td><font class="main"> <input type="password" name="password2" size="20"></td></tr>
  	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'CNFNEW_PASS'}: </td>
	  <td><font class="main"> <input type="password" name="confirm" size="20"></td></tr>
	  <tr class="mainrow"><td colspan="2"><div align="center"><font class="main">
	  <input type="submit" name="new" value="$closedShop::TEXT{'CR_ACC'}"></div></td></tr>
	  </table>|;
    closedShop::Footer();
}

###############################################################################
# FUNCTION:  ViewCart
# DESCRIPTION: view the shoppers cart
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ViewCart {
    my $message = shift;
    closedShop::Header($message);
    my $mesg = "";
    if ( $message ne "nohttp" ) {
        $mesg = $message;
    }

    print qq|
	<font color='red' class='main'>$mesg</font>
	<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="6">
	<font class="head"> $closedShop::TEXT{'SHP_CRT'}</font></th></tr>
	<form action="$program_name" method='POST'>
	<input type='hidden' name='action' value='UpdateQuantities'>
	<tr class="mainrow">
	<td><font class="main"><strong>$closedShop::TEXT{'NAME'}</strong></td>
	<td><font class="main"><strong>$closedShop::TEXT{'QTY'}</strong></td>
	<td><font class="main"><strong>$closedShop::TEXT{'ATT'}</strong></td>
	<td><font class="main"><strong>$closedShop::TEXT{'PRC'}</strong></td>
	<td><font class="main"><strong>$closedShop::TEXT{'TOT'}</strong></td>
	<td><font class="main"><strong>$closedShop::TEXT{'REM'}</strong></td>
	</tr>|;

    my @ary  = ();
    my $cart = closedShop::GetCookie("cart");
    if ($cart) {
        @ary = split( /$closedShop::CookieItemDelimiter/, $cart )
          if ( defined($cart) );
    }

    my $k     = 1;
    my $grand = 0;
    foreach (@ary) {
        my $uniq_id = closedShop::URLEncode($_);
        if ( !$_ ) { next; }
        my ( $id, $qty, $attr ) =
          split( /$closedShop::CookieSectionDelimiter/, $_ );
        if ( !$id || $id eq "NULL" ) { next; }
        if ( $id eq "gc" ) {
            my ( $id, $recipient_name, $recipient_email, $gc_amount ) =
              split( /$closedShop::CookieSectionDelimiter/, $_ );

            my $total = sprintf( "%.2f", $gc_amount );
            $grand += $total;

            print qq|
	   	  <tr class="mainrow">
		  <td colspan="3"><font class="main"> $closedShop::TEXT{'GC'} ($closedShop::CurrencyMarker $gc_amount)</td>
		  <td><font class="main"> 1 x $closedShop::CurrencyMarker $gc_amount</td>
		   <td><font class="main"> $closedShop::CurrencyMarker $total</td>
		  <td><font class="main"> 
		  <a href="$program_name?action=DeleteProduct&id=$uniq_id">$closedShop::TEXT{'REM'}</a></td>
		  </tr>  |;
            $k++;
            next;
        }

        my @attr = ();
        if ($attr) {
            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
        }
        $attr = "";

        my $attr_cost = 0.00;
        foreach my $at (@attr) {
            my ( $prod_id, $attr_id, $data_id ) =
              split( /$closedShop::GiftCertDelimiter/, $at );
            my $SQL = qq|select name,data,price from $closedShop::sql_tables{"attributes"},
	   $closedShop::sql_tables{"attribute_values"} where product_id='$prod_id' and
	   $closedShop::sql_tables{"attributes"}.id = '$attr_id' and
	   $closedShop::sql_tables{"attribute_values"}.id = '$data_id'|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $atname, $atdata, $atprice ) = $sth->fetchrow_array();
            $attr .= "<nobr><u>$atname</u>: $atdata";
            if ( $atprice > 0 ) {
                $attr .= " (+$closedShop::CurrencyMarker$atprice)";
                $attr_cost += $atprice;
            }
            $attr .= "</nobr><BR>";
        }

        my $SQL = qq| select name,price,shipping_price,description,
	              category,reduced_shipping_price from
		      $closedShop::sql_tables{"item"} where id='$id' |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ( $name, $price, $ship, $desc, $cat, $rship ) = $sth->fetchrow_array();

        my $total = sprintf( "%.2f", ( $price + $attr_cost ) * $qty );

        $grand += $total;

        print qq|
	   	  <tr class="mainrow">
		  <td><font class="main"> <a 
		  href="$closedShop::FileNames{item}?action=ShowItem&id=$id">$name</a></td>
		  <td><font class="main">
		  |;

        if ( $closedShop::EnableInventory eq "Y" && $qty <= 0 ) {
            print qq|<font color='red'>OUT</font>|;
        }
        else {
            print qq|<input type='text' size='3' name='qty$uniq_id' value='$qty'>|;
        }

        print qq|</td>
		  <td><font class="main"> $attr &nbsp;</td>
		  <td><font class="main"> $qty x $closedShop::CurrencyMarker$price|;

        if ( $attr_cost > 0 ) {
            print qq|<BR><nobr>$qty x (+$closedShop::CurrencyMarker$attr_cost)</nobr>|;
        }

        print qq|</td>|;

        print qq| 
		  <td><font class="main"> $closedShop::CurrencyMarker$total</td>
		  <td><font class="main"> 
		  <a href="$program_name?action=DeleteProduct&id=$uniq_id">$closedShop::TEXT{'REM'}</a></td>
		  </tr>  |;
        $k++;
    }

    if ( $k == 1 ) {
        print qq|<tr class="mainrow"><td colspan="6"><font class="main">
	<center><strong><br>$closedShop::TEXT{'NO_CART'}</strong><br><br></td></tr>
	|;
    }

    my $lastitem = "";

    my $contshop = qq|$closedShop::FileNames{item}|;
    if (   $closedShop::ContinueShoppingURL ne ""
        && $closedShop::ContinueShoppingURL ne "http://" )
    {
        $contshop = $closedShop::ContinueShoppingURL;
    }

    $grand = sprintf( "%.2f", $grand );
    print qq|<tr class="mainrow"><td colspan="3" align="right">&nbsp;</td>
	<td colspan='2'><font class="main">
	<strong>Sub $closedShop::TEXT{'TOT'}:</td><td><font class="main"> $closedShop::CurrencyMarker$grand</strong></td></tr>
	<tr class="mainrow"><td colspan="3"><font class="main"> <centeR>
	<input type='submit' value='$closedShop::TEXT{'UPDATE_QTY'}'></form></td>
  	<td colspan='2'><font class="main">|;

    if ( $grand < $closedShop::MinimumOrder ) {
        print qq|<font class="main">$closedShop::TEXT{'MIN_OR'}
		 $closedShop::CurrencyMarker $closedShop::MinimumOrder|;
    }
    else {
        print qq|<a href="$program_name?action=Checkout"><b>$closedShop::TEXT{'CHKOUT'}</b></a>|;
    }

    print qq|</td><td colspan="2"><font class='main'>
	<a href="$contshop"><b>$closedShop::TEXT{'CONT_SHOP'}</b></a></td>
	</td>
	</tr></table>
	<br>
	|;
    closedShop::Footer();

}

###############################################################################
# FUNCTION:  DeleteProduct
# DESCRIPTION: delete a product
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DeleteProduct {
    my $id = closedShop::ValidateInputCustomer( closedShop::URLDecode( $query->param("id") ) );

    my $cart = closedShop::GetCookie("cart");
    $cart =~ s/$id//g;
    my $cookie = new CGI::Cookie( -name => 'cart', -value => "$cart" );
    closedShop::SaveCookie( "cart", "$cart" );
    print redirect( -Cookie => [$cookie], -URL => "$program_name" );
}

################################################################################
# FUNCTION:  LoginUser
# DESCRIPTION: process screen
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginUser {
    my $uid     = shift;
    my $session = closedShop::CreateSession();
    my $SQL     = qq| update $closedShop::sql_tables{"user"} set session="$session" where id = "$uid" |;
    my $sth     = closedShop::DatabaseQuery($SQL);
    closedShop::SetSessionCookie( $session, $uid );
}

################################################################################
# FUNCTION:  ValidateCard
# DESCRIPTION: validates card - courtesy of Business::CreditCard
# from CPAN - thank you Ivan Kohler
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub ValidateCard {
    my ($number) = @_;
    my $return_number = $number;
    my ( $i, $sum, $weight );

    

    # if ($number eq "4000000000000001") { return "4000000000000001"; }

    return 0 if $number =~ /[^\d\s]/;

    $number =~ s/\D//g;

    return 0 unless length($number) >= 13 && 0 + $number;

    for ( $i = 0 ; $i < length($number) - 1 ; $i++ ) {
        $weight = substr( $number, -1 * ( $i + 2 ), 1 ) * ( 2 - ( $i % 2 ) );
        $sum += ( ( $weight < 10 ) ? $weight : ( $weight - 9 ) );
    }

    return $return_number if substr( $number, -1 ) == ( 10 - $sum % 10 ) % 10;
}
