#!/usr/bin/perl

################################################################################
#
# PROGRAM:  Install.pl - from closedShop - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: Install the program
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-06-12   cbf    Created
# 2001-06-22   cbf    Added support for renaming file names
# 2001-07-03   cbf    Added a list of states to choose from
# 2001-07-24   cbf    Mailprogram can be customized
# 2001-08-12   cbf    All input validated
# 2001-09-08   cbf    Added Secure Image URL
# 2001-09-27   dsb    Added a merchant username
# 2001-10-18   cbf    Added database location
# 2002-05-12   cbf    Added minimum order amount, added continue shopping vars,
#                       final return vars
# 2003-02-27   cbf    Added Show shipping prices option
# 2003-04-01   cbf    Added ShowTax option, added currency marker,
#                     Added new taxing options (state,national,internat)
# 2003-04-02   cbf    Added gift certificates, country listings, pay by check
# 2003-06-10   cbf    Added PayPal support
# 2003-08-23   cbf    Added Ebay support, allow access to admin through ip
# 2003-08-25   cbf    Added different language support
# 2003-08-29   cbf    Added Ebay categories into SQL
# 2004-02-18   cbf    Fixed annoying category problem where the cat doesn't show
# 2004-07-07   cbf    Removed many unneeded things during install
# 2005-03-03   cbf    added in code for Affiliates System
################################################################################

use strict;
use CGI;
use DBI;
use Fcntl ':flock';

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query        = new CGI;
my $program_name = "Install.pl";

my $varsfile  = "closedShop.vars";
my @Languages = ( "English", "Tongan" );

my $font             = qq| <font style="font-family: verdana,sans serif, arial; font-size: 12px"> |;
my $tc               = "#DDDDDD";
my $mc               = "#EEEEEE";
my $cannotcreatemesg = qq|
<table border='0' width='400'><tr><td>$font
Could not create the file
<b>$varsfile</b>. This is a problem. Your web server program needs to be able to create
a configuration file, but is unable to. To solve this problem you may need to
contact your webhoster and ask them for your webserver to write to this folder.
Or, if you have access to your server through SSH, you can try to do the following:<BR>
<BR>Change the group of the cgi-bin folder to apache:<BR>
<code>[user\@website cgi-bin]# chgrp apache .</code><BR><BR>
Change permissions so apache group can write to the folder<BR>
<code>[user\@website cgi-bin]# chmod 775 .</code><BR></td></tr></table>
|;
my $cannotwrite = qq|
<table border='0' width='400'><tr><td>$font
Although the
<b>$varsfile</b> exists, closedShop could not write data to it. This is a problem. 
Your web server program needs to be able to write the data. 
To solve this problem remove the file $varsfile from the closedShop folder and
try to install again</td></tr></table>
|;
################################################################################

Main();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main {
    my $action = $query->param('action');
    if ( IsInstalled() ) {
        print "Location: Admin.pl\n\n";
    }
    elsif ( $action eq "Install" ) {
        Install();
    }
    else {
        InstallScreen("");
    }
}

################################################################################
# FUNCTION:  CreateSQL
# DESCRIPTION: create sql tables
################################################################################
# MODIFICATION HISTORY
# 2001-07-25   cbf    created
################################################################################
sub CreateSQL {
    open( FILE, "sql.sql" ) || closedShop::ErrorMessage(
        "Could not find the sql.sql
	file needed to install closedShop on mySQL"
    );
    my @data = <FILE>;
    my $joined = join( "", @data );
    close(FILE);

    my $u   = "<user>";
    my $o   = "<order>";
    my $a   = "<admin>";
    my $i   = "<item>";
    my $g   = "<giftcertificate>";
    my $c   = "<categories>";
    my $e   = "<ebay>";
    my $af  = "<affiliates>";
    my $as  = "<affiliate_sales>";
    my $at  = "<attributes>";
    my $av  = "<attribute_values>";
    my $max = "<maxmind>";
    my $gr  = "<groups>";

    $joined =~ s/$u/$closedShop::sql_tables{"user"}/g;
    $joined =~ s/$o/$closedShop::sql_tables{"order"}/g;
    $joined =~ s/$a/$closedShop::sql_tables{"admin"}/g;
    $joined =~ s/$i/$closedShop::sql_tables{"item"}/g;
    $joined =~ s/$g/$closedShop::sql_tables{"gc"}/g;
    $joined =~ s/$c/$closedShop::sql_tables{"cat"}/g;
    $joined =~ s/$e/$closedShop::sql_tables{"ebay"}/g;
    $joined =~ s/$af/$closedShop::sql_tables{"affiliates"}/g;
    $joined =~ s/$as/$closedShop::sql_tables{"affiliate_sales"}/g;
    $joined =~ s/$at/$closedShop::sql_tables{"attributes"}/g;
    $joined =~ s/$av/$closedShop::sql_tables{"attribute_values"}/g;
    $joined =~ s/$max/$closedShop::sql_tables{"maxmind"}/g;
    $joined =~ s/$gr/$closedShop::sql_tables{"groups"}/g;

    @data = split( /;/, $joined );

    foreach (@data) {
        chomp($_);
        if ($_) {
            my $SQL = qq|$_|;
            my $sth = closedShop::DatabaseQuery($SQL);
        }
    }
}

################################################################################
# FUNCTION:  IsInstalled
# DESCRIPTION: checks to see if it has been run once or not
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub IsInstalled {
    open( FILE, "$varsfile" );
    my @data = <FILE>;
    my $data = join( "", @data );
    close(FILE);

	# if no cart file found, definetly not installed
    if ( $data !~ /FileName_cart/ ) { return 0; }
    else { return 1; }
}

################################################################################
# FUNCTION:  Install
# DESCRIPTION: install
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub Install {

    use closedShop;
    open( FILE, ">$varsfile" );
    close(FILE);
    system("chmod 600 $varsfile");

    if ( !( -e $varsfile ) ) {
        return closedShop::ErrorMessage($cannotcreatemesg);
    }

    open( FILE, ">$varsfile" ) or closedShop::ErrorMessage($cannotwrite);
    flock( FILE, LOCK_EX );
    foreach ( $query->param() ) {
        if (   $_ eq "AdminUsername"
            || $_ eq "AdminPassword"
            || $_ eq "MerchantUsername"
            || $_ eq "MerchantPassword" )
        {
            next;
        }
        if ( $_ =~ /category/ ) { next; }
        my @var = $query->param("$_");
        my $var = join( ",", @var );
        $var =~ s/\n/<BR>/g;
        $var =~ s/~~~/~*~/g;
        print FILE qq|$_~~~$var\n|;
    }
    flock( FILE, LOCK_UN );
    close(FILE);

    my $AUser = $query->param("AdminUsername");
    my $APass = $query->param("AdminPassword");
    my $MUser = $query->param("MerchantUsername");
    my $MPass = $query->param("MerchantPassword");

    closedShop::Initialize( "", 1 );
    CreateSQL();
    CreateAdmin( $AUser, $APass );
    CreateAdmin( $MUser, $MPass );

    for ( my $k = 1 ; $k < $closedShop::MaxCats ; $k++ ) {
        my $catname = closedShop::ValidateInputAdmin( $query->param("category$k") );
        $catname =~ s/"/'/g;
        my $short = lc($catname);
        $short =~ tr/a-zA-Z0-9//cd;
        if ($catname) {
            my $SQL = qq| insert into $closedShop::sql_tables{"cat"} set
	      name="$catname",code="$short",status="PUBLIC",style="text"|;
            my $sth = closedShop::DatabaseQuery($SQL);
        }
    }

    # install ebay categories
    open( FILE, "ebaylist.list" )
      || return closedShop::ErrorMessage(
        "Cannot
		open ebaylist file in order to insert codes"
      );
    my @ebaydata = <FILE>;
    close(FILE);
    foreach (@ebaydata) {
        chomp($_);
        if ( !$_ ) { next; }
        my @tmp = split( /~~/, $_ );
        my $SQL = qq|insert into $closedShop::sql_tables{"ebay"} set
	    id="$tmp[0]",name="$tmp[1]",parent="$tmp[2]" |;
        my $sth = closedShop::DatabaseQuery($SQL);
    }

    my $immesg = "";
    open( FILE, ">" . $query->param("IMAGEPATH") . "/test516613632465.tst" );
    if ( !( -e $query->param("IMAGEPATH") . "/test516613632465.tst" ) ) {
        $immesg = qq|<BR><BR><font color='red'>But there 
      seems to be a problem with your image directory (|
          . $query->param("IMAGEPATH") . qq|). 
      closedShop was not
      able to create a file in this directory. This means you are not currently
      able to upload images of products. Check the documentation for help<BR><BR>
      In addition, you are not able to use the tinyMCE editor. You may still copy the
      tiny_mce folder from the current folder and put it in your image folder manually
      in order to use this 3rd party add-on.</font>|;
    }
    else {
        $immesg = "<BR><BR>closedShop has attempted to copy two pictures and the 
    tinyMCE to your images folder at " . $query->param("IMAGEPATH") . ". If
    these objects do not appear then you may manually copy them from this
    folder to the image folder";
        use File::Copy;
        copy( "noimage.jpg",     $query->param("IMAGEPATH") . "/noimage.jpg" );
        copy( "photoexists.gif", $query->param("IMAGEPATH") . "/photoexists.gif" );

        # don't want to use this, couldn't find simpler way
        system( 'cp', '-r', 'tiny_mce', $query->param("IMAGEPATH") );

    }
    unlink( $query->param("IMAGEPATH") . "/test516613632465.tst" );

    Header();
    print qq|
    <table style="border: 1px solid black;" width='500'><tr><td style='font-family: verdana, sans serif;
    font-size: 12px'>
    You have successfully installed closedShop.
	If you have trouble, and believe closedShop did not install properly,
	the best bet is to delete the file called closedShop.vars in this directory
	and run Install.pl again.
	$immesg
	<br>
	<br>
	Your next step is to login into the <a href="$closedShop::FileNames{admin}">Admin Panel</a>.<BR>
	You will find more options to configure in "Edit Program Variables"
		</td></tr></table>
	|;
    Footer();
}

################################################################################
# FUNCTION:  CreateAdmin
# DESCRIPTION: create admin
################################################################################
# MODIFICATION HISTORY
# 2001-07-25   cbf    created
################################################################################
sub CreateAdmin {
    my ( $AUser, $APass ) = @_;
    my $SQL = qq| insert into $closedShop::sql_tables{"admin"} set username="$AUser",
	password="$APass"|;
    my $sth = closedShop::DatabaseQuery($SQL);
}

################################################################################
# FUNCTION:  InstallScreen
# DESCRIPTION: install screen
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub InstallScreen {
    use closedShop;
    open( FILE, ">test_cs_file" )
      || return closedShop::ErrorMessage($cannotcreatemesg);
    unlink("test_cs_file");

    my $message = shift;
    Header();

    print qq|<br>
	<b>The variables listed below are the ones that are absolutely required.<BR>
	There are many more variables that can be changed once you have installed.<BR>
	Visit the Admin.pl script and go to "Change Variables" to fully control
	closedShop<BR><BR>
	You must have access to a SQL database before you begin installation
	and you must know the access information for it.</b><br>
	<br>
	
	<table width="600" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Company Info</b></font></td></tr>
	<form action="$program_name" method="POST">
	<input type="hidden" name="action" value="Install">
	
	<tr bgcolor="$mc"><td>$font<b>Company Name</b>: </font></td><td>$font 
	<input type="text" name="CompanyName" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company Email</b>: </font></td><td>$font 
	<input type="text" name="CompanyEmail" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company Country</b>: </font></td><td>$font 
	|;
    closedShop::ListCountries("CompanyCountry");
    print qq|</td></tr>

	 
	 <tr bgcolor="$mc"><td>$font<b>Currency Marker</b>: </font></td><td>$font 
	<select name="CurrencyMarker">
	<option value="\$">\$</option>
	<option value="&pound;">&pound;</option>
	<option value="&#8364;">&#8364;</option>
	<option value="&yen;">&yen;</option>
	<option value="&cent;">&cent;</option>
	<option value="&#8224;">&#8224;</option>
  </select>
	</td></tr>
	 
	 <tr bgcolor="$mc"><td>$font<b>Select Accepted Credit Cards or Checks</b>:</td><td>$font 
	
	<input type="checkbox" name="creditcard" value="paypal">PayPal<br>  
	<input type="checkbox" name="creditcard" value="check">Checks<br>  
	<input type="checkbox" name="creditcard" value="mastercard">Mastercard<br>
	<input type="checkbox" name="creditcard" value="visa">Visa<br>
	<input type="checkbox" name="creditcard" value="discover">Discover<br>
	<input type="checkbox" name="creditcard" value="amex">American Express<br>
	<input type="checkbox" name="creditcard" value="diner">Diner's Club<br>
	<input type="checkbox" name="creditcard" value="cod">Cash on Delivery<br>
	
	 </td></tr>

	 <tr bgcolor="$mc"><td>$font<b>Create Categories</b>:<br>
	 <i>Create as many as you need<BR>You can create more later</i></td><td>$font 
	  |;

    for ( my $k = 1 ; $k < 16 ; $k++ ) {
        print qq|Category $k: <input type="text" name="category$k" value="" size="20"><br>|;
    }

    print qq|	
	 </td></tr>
	 
	
	 </table><br><br>

	  <table width="600" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Administration Details</b></font></td></tr>
	
	<tr><td colspan="2" bgcolor="$mc">
	$font Two administrative users will be created. The Admin User can access and edit all
	program variables being created on this screen. The Merchant User will not be able to
	access or edit the server specific variables.
	</font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Admin Username</b>: </font></td><td>$font 
	<input type="text" name="AdminUsername" value="admin" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Admin Password</b>: </font></td><td>$font 
	<input type="text" name="AdminPassword" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Merchant Username</b>: (Optional) </font></td><td>$font 
	<input type="text" name="MerchantUsername" value="merchant" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Merchant Password</b>: (Optional)</font></td><td>$font 
	<input type="text" name="MerchantPassword" value="" size="20"></td></tr>

	<tr bgcolor="$mc"><td>$font<b>Default Language</b>:</font></td>
	<td>$font <select name="Language">|;
    foreach (@Languages) {
        print qq|<option value="$_">$_</option>\n|;
    }
    print qq|</select></td></tr>

	</table><br>
	<br>
	
	 
	 <table width="600" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - SQL Database Info</b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Database Name</b>: </font></td><td>$font 
	<input type="text" name="DatabaseName" value="" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Database Username</b>: </font></td><td>$font 
	<input type="text" name="DatabaseUsername" value="" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Database Password</b>: </font></td><td>$font 
	<input type="text" name="DatabasePassword" value="" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Database Location</b><br>
	<i>Only change if you know what you are doing</i>: </font></td><td>$font 
	<input type="text" name="DatabaseLocation" value="localhost" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Database Table Names</b>: <BR>
	</font></td><td>$font 
	Admin Table: <input type="text" name="admintable" value="closedshop_admin" size="20"><br>
	Item Table: <input type="text" name="itemtable" value="closedshop_item" size="20"><br>
	Order Table: <input type="text" name="ordertable" value="closedshop_order" size="20"><br>
	User Table: <input type="text" name="usertable" value="closedshop_user" size="20"><br>
	Gift Certificate Table: <input type="text" name="gctable" value="closedshop_gc" size="20"><BR>
	Categories Table: <input type="text" name="cattable" value="closedshop_cat" size="20"><BR>
	Ebay Table: <input type="text" name="ebaytable" value="closedshop_ebay"size="20"><BR>
	Affiliates Table: <input type="text" name="affiliatetable" value="closedshop_affliates" size="20"><BR>
	Affiliate Sales Table: <input type="text" name="affiliate_salestable" value="closedshop_affiliate_sales" size="20"><BR>
	Attributes Table: <input type="text" name="attributetable" value="closedshop_attributes" size="20"><BR>
	Attribute Values Table: <input type="text" name="attribute_valuestable" value="closedshop_attribute_values" size="20"><BR>
	Groups Table: <input type="text" name="groupstable" value="closedshop_groups" size="20"><BR>
	MaxMind Table: <input type="text" name="maxmindtable" value="closedshop_maxmind" size="20"><BR>
	    
        <br>
	</td></tr>
		 </table><br><br>
	
	 <table width="600" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Server Specifics</b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Website URL</b>: <br>
	No Trailing Slash</font></td><td>$font |;
    my $host = $ENV{'HTTP_HOST'};
    if ($host) {
        $host = "http://$host";
    }
    else {
        $host = "http://";
    }

    print qq|
	<input type="text" name="website" value="$host" size="40"></td></tr>
	|;
    my $script_url = $ENV{'SCRIPT_URI'};
    if ($script_url) {
        my @tmp = split( /\//, $script_url );
        $tmp[$#tmp] = "";
        $script_url = join( "/", @tmp );
    }
    else {
        $script_url = $ENV{'REQUEST_URI'};
        my @tmp = split( /\//, $script_url );
        $tmp[$#tmp] = "";
        $script_url = join( "/", @tmp );
        $script_url = substr( $script_url, 0, length($script_url) - 1 );
        $host = $ENV{'HTTP_HOST'};
        if ( $script_url && $host ) {
            $script_url = "http://$host$script_url";
        }
        else {
            $script_url = "http://";
        }
    }
    print qq|
	<tr bgcolor="$mc"><td>$font<b>URL to closedShop Folder</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="urltofolder" value="$script_url" size="40"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Server Path to Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="IMAGEPATH" value="" size="40"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>URL to Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="IMAGEURL" value="" size="40"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Mail Program</b>: </font></td><td>$font 
	<input type="text" name="mailprog" value="/usr/sbin/sendmail" size="30"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Email Flags</b>: </font></td><td>$font 
	<input type="text" name="emailflags" value="-t" size="30"></td></tr>
	
	</table><br><br>
	 <table width="600" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Style</b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Table Top Colors</b>: </font></td><td>$font
	<input type="text" name="headrow_color" value="#99CC99" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Table Main Colors</b>: </font></td><td>$font 
	<input type="text" name="mainrow_color" value="#EEEEEE" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Table Width</b>: </font></td><td>$font
	<input type="text" name="TableWidth" value="550" size="20"></td></tr>
        <tr bgcolor="$mc"><td>$font<b>Table Border</b>: </font></td><td>$font
	<input type="text" name="table_border" value="1px solid black"  size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Table Border Spacing</b>: </font></td><td>$font
	<input type="text" name="table_borderspacing" value="1 px" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Main Font Face</b>: </font></td><td>$font
	<input type="text" name="fontmain_family" value="verdana,arial,sans-serif" size="30"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Main Font Size</b>: </font></td><td>$font
	<input type="text" name="fontmain_size" value="12px" size="30"></td></tr>
	
	<tr  bgcolor="$mc" ><td class='main'>$font<b>Main Row Hover Color</b>: </font></td><td class='main'>$font
        <input type="text" name="mainrow_hover" value="#DDDDDD" size="30"></td></tr>
	<tr  bgcolor="$mc" ><td class='main'>$font<b>Input Button Hover Color</b>: </font></td><td class='main'>$font
        <input type="text" name="input_hover" value="yellow" size="30"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Header Font Face</b>: </font></td><td>$font
	<input type="text" name="fonthead_family" value="verdana,arial,sans-serif" size="30"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Header Font Size</b>: </font></td><td>$font
	<input type="text" name="fonthead_size" value="15px" size="30"></td></tr></table>
	
	<BR><BR>
	<table width="600" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - File names</b></font></td></tr>
	
		<tr><td colspan="2" bgcolor="$tc">$font<b><i>Important - these must be a full URL if they will be called using cgi-wrap or on a separate secure server. See the README file.</i></b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>URL for Admin.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_admin" value="Admin.pl" size="40"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL for Cart.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_cart" value="Cart.pl" size="40"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL for User.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_user" value="User.pl" size="40"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL for Item.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_item" value="Item.pl" size="40"></td></tr>


	
	</table><br><br>
	
	 <table width="600" border="0"><tr><td colspan="2" bgcolor="$tc"><center>
	$font <font color="blue"><b><input type="submit" value="Install"></b></font></td></tr>
	</table></form>
	 
	</center></div>|;

    Footer();
}

################################################################################
# FUNCTION:  Header
# DESCRIPTION: header
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub Header {
    print "Content-type: text/html \n\n";
    print qq|<html><head><title>closedShop Installation</title></head>
	<body bgcolor="white">
	$font
	<h3>closedShop Installation</h3>
	|;
}

################################################################################
# FUNCTION:  Footer
# DESCRIPTION: footer
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub Footer {
    print qq|
	</body>
	</html>
	|;
}

