#!/usr/bin/perl 

################################################################################
#
# PROGRAM:  Item.pl - from closedShop - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: Shows items, allows for searches
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-08   cbf    Created
# 2001-06-07   cbf    Allowed for searching in any category, show attributes
# 2001-06-24   cbf    cosmetic changes
# 2001-08-12   cbf    All input validated
# 2001-09-08   cbf    Added SecureImageURL variable use
# 2001-09-18   cbf    A mark appears if a picture is available
# 2002-01-20   cbf    Allowed for thumbnail displayal of categories
# 2002-02-01   cbf    Only shows intro text for cat if its first page
# 2002-02-11   cbf    Make use of public/private categories
# 2003-02-27   cbf    If ShowShipping is no, doesn't show shipping cost
# 2003-03-31   cbf    Searching will now split words and search
# 2003-04-02   cbf    Added Gift Certificate
# 2003-10-17   cbf    Added Template support for Items and Category
# 2004-02-18   db     SecureImage URL bug fixes B.Darnell@student.umist.ac.uk
# 2004-07-07   cbf    Image pops up to full size when clicked upon
# 2004-07-12   cbf    Added all categories template
# 2005-03-21   cbf    Images can be externally linked
################################################################################

use CGI::Carp(fatalsToBrowser);
use closedShop;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();
###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query        = new CGI;
my $program_name = $closedShop::FileNames{"item"};

my %actions = (
    ShowItem             => "Show an Item",
    Search               => "Item Search",
    ShowCategory         => "Display a Category",
    ShowGiftCertificates => "Shows Gift Certificates",
    ShowAllCategories    => "Prints All Categories",
    ShowImage            => "Shows an Image",
    SetAffiliate         => "Sets the Affiliate"
);

###############################################################################

Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main {
    my $action = $query->param('action');
    $action = closedShop::ValidateInputCustomer($action);
    $action = "cs" if ( !defined($action) );
    if ( !defined( $actions{$action} ) ) {
        Search("");
    }
    else {
        eval($action);
    }
}

###############################################################################
# FUNCTION:  SetAffiliate
# DESCRIPTION: sets the affiliate
################################################################################
# MODIFICATION HISTORY
# 2005-03-04   cbf    Created
##############################################################################
sub SetAffiliate {
    my $id = closedShop::ValidateInputCustomer( $query->param("id") );
    $id =~ tr/0-9//cd;

    use CGI qw/:standard/;
    use CGI::Cookie;
    my $cookie1 = new CGI::Cookie( -name => 'affiliate_id', -value => $id );
    closedShop::SaveCookie( "affiliate_id", "$id" );
    print header(
        -Cookie   => [$cookie1],
        -type     => "text/html",
        -Location => "$program_name"
    );

}

###############################################################################
# FUNCTION:  ShowImage
# DESCRIPTION: shows an image
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ShowImage {
    my $secure = closedShop::ValidateInputCustomer( $query->param("secure") );
    my $id     = closedShop::ValidateInputCustomer( $query->param("id") );
    my $nodata = closedShop::ValidateInputCustomer( $query->param("nodata") );

    print "Content-type: text/html \n\n";
    print qq|<html>
  <head>
  <style type='text/css'>
  <!--
     font.main {
	  font-family: $closedShop::fontmain_family;
	  color: $closedShop::fontmain_color;
          font-size: $closedShop::fontmain_size;
          font-style: $closedShop::fontmain_style;
          font-variant: $closedShop::fontmain_variant;
        }
  -->
  </style>
  </head>
  <body bgcolor='white'><font class='main'>
  <center>|;
    my $extra = "";
    if ($nodata) {
        $extra = qq| width='300' |;
    }
    if ($secure) {
        print qq|<img src='$closedShop::SecureImageURL/$id.jpg' $extra>|;
    }
    else {
        print qq|<img src='$closedShop::IMAGEURL/$id.jpg' $extra>|;
    }
    print qq|<BR><BR><a href="#" onClick="javascript:window.close();">Close Window</a></body></html>|;
}

###############################################################################
# FUNCTION:  ShowGiftCerticates
# DESCRIPTION: shows gifts certficates
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ShowGiftCertificates {
    my @denoms = closedShop::GetGiftCertificateInfo();
    if ( !@denoms ) {
        return Search();
    }
    else {
        my $message = closedShop::ValidateInputCustomer( $query->param("message") );
        closedShop::Header();
        print qq|<center><font color='red'>$message</font></center>
	  <form action="$closedShop::FileNames{cart}" method="POST">
	  <input type="hidden" name="action" value="AddGiftCertificate">
	  <table width="$closedShop::TableWidth" border="1"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'PURCH'} $closedShop::TEXT{'GC'}</font></th></tr>
 	  <tr class="mainrow"><td valign="top"><table border="0" width="100%">
	  <tr class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'RECIP'} $closedShop::TEXT{'NAME'}</strong>:</td>
	  <td><font class="main"> <input type="text" name="recipient_name" size="20"></td></tr>
	  <tr class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'RECIP'} $closedShop::TEXT{'EMAIL'}</strong>:</td>
	  <td><font class="main"> <input type="text" name="recipient_email" size="20"></td></tr>

	  <tr class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'SELECT'} $closedShop::TEXT{'GC'} $closedShop::TEXT{'AMNT'}</strong>:</td>
	  <td><font class="main"> <select name="gc_amount">|;

        foreach (@denoms) {
            print qq|<option value="$_">$closedShop::CurrencyMarker $_</option>\n|;
        }

        print qq|</select>
		</td></tr>
		<tr class="mainrow"><td colspan="2">
	  <font class="main"> <center>
	  <input type="submit" value="$closedShop::TEXT{'ADD'} $closedShop::TEXT{'GC'} $closedShop::TEXT{'TO'} $closedShop::TEXT{'SHP_CRT'}"></td></form>
	  </tr>
	  </form></table>
		</td></tr></table>
	  |;
        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  ShowItem
# DESCRIPTION: displays an item
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ShowItem {
    my $id = $query->param("id");
    $id =~ tr/0-9//cd;
    if ( !$id || $id == 0 ) { $id = 1; }

    my $SQL = qq| select name,price,shipping_price,description,
	              category,quantity,image_url,weight from $closedShop::sql_tables{"item"} where
				  id="$id" |;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $name, $price, $ship, $desc, $cat, $quantity, $image_url, $weight ) = $sth->fetchrow_array();

    $SQL = qq|select name,id,ordering from $closedShop::sql_tables{"attributes"}
		where product_id='$id' order by ordering|;
    $sth = closedShop::DatabaseQuery($SQL);
    my @attributes = ();
    while ( my ( $atname, $atid, $atorder ) = $sth->fetchrow_array() ) {
        my $att  = "$atname: <select name='attribute$atorder'>";
        my $SQL2 = qq|select data,price,id from $closedShop::sql_tables{"attribute_values"}
         where attribute_id='$atid' order by ordering|;
        my $sth2 = closedShop::DatabaseQuery($SQL2);
        while ( my ( $atdata, $atprice, $dataid ) = $sth2->fetchrow_array() ) {
            $att .= qq|<option value='$id$closedShop::GiftCertDelimiter$atid$closedShop::GiftCertDelimiter$dataid'>
	 $atdata|;
            if ( $atprice > 0 ) {
                $att .= qq| ($closedShop::TEXT{'ADD'} $closedShop::CurrencyMarker$atprice) |;
            }
            $att .= qq|</option>\n|;
        }
        $att .= "</select>";
        push @attributes, $att;
    }

    my $imurl = "";
    if (   -e "$closedShop::IMAGEPATH/$id.jpg"
        || -e "$closedShop::SIMAGEPATH/$id.jpg" )
    {
        if ( $closedShop::FileNames{"item"} =~ /https/ ) {
            $imurl = "$closedShop::SecureImageURL/$id.jpg";
        }
        else {
            $imurl = "$closedShop::IMAGEURL/$id.jpg";
        }
    }
    elsif ($image_url) {
        $imurl = $image_url;
    }

    if ( -e $closedShop::item_template ) {
        my $check = DisplayItemTemplate( $id, $name, $price, $ship, $desc, $cat, $quantity, \@attributes, $imurl );
        if ( $check == 1 ) { return; }
    }

    closedShop::Header();
    print qq|
	  <form action="$closedShop::FileNames{cart}" method="POST">
	  <input type="hidden" name="action" value="AddProduct">
	  <input type="hidden" name="id" value="$id">
	  <table width="$closedShop::TableWidth" border="1"><tr class="headrow"><th colspan="2">
	  <font class="head"> $name</font></th></tr>
 	  <tr class="mainrow"><td>|;

    if (   -e "$closedShop::IMAGEPATH/$id.jpg"
        || -e "$closedShop::SIMAGEPATH/$id.jpg" )
    {
        my $nodata = "";
        my ( $width, $height ) = closedShop::ImageDecode("$closedShop::IMAGEPATH/$id.jpg");
        if ( !$width ) {
            ( $width, $height ) = closedShop::ImageDecode("$closedShop::SIMAGEPATH/$id.jpg");
        }
        $height += 40;
        if ( !$width ) { $width = 310; $height = 400; $nodata = "&nodata=1"; }

        my $imgname = $name;
        $imgname =~ s/ //g;
        if ( $closedShop::FileNames{"item"} =~ /https/ ) {

            print qq|<a href="| . $closedShop::FileNames{'item'} . qq|?action=ShowItem&id=$id#" onclick="javascript:window.open('| . $closedShop::FileNames{"item"} . qq|?action=ShowImage&secure=1&id=$id$nodata','$imgname','width=$width,height=$height');"><img border="0"
		  alt="Click to view larger image" width="220"
		  src="$closedShop::SecureImageURL/$id.jpg"></a><br>\n|;
        }
        else {
            print qq|<a href="| . $closedShop::FileNames{'item'} . qq|?action=ShowItem&id=$id#" onclick="javascript:window.open('| . $closedShop::FileNames{"item"} . qq|?action=ShowImage&secure=0&id=$id$nodata','$imgname','width=$width,height=$height,resizable=yes');"><img
		  width="220" border="0" alt="Click to view larger image"
		  src="$closedShop::IMAGEURL/$id.jpg"></a><br>\n|;
        }
    }
    elsif ($image_url) {
        my $imgname = $name;
        $imgname =~ s/ //g;
        print qq|<a href="| . $closedShop::FileNames{'item'} . qq|?action=ShowItem&id=$id#" onClick="javascript:window.open('$image_url','$imgname','width=300,height=300,resizable=yes');"><img
		   border="0" alt="Click to view larger image"
		  src="$image_url"></a>|;
    }
    else {
        if (   -e "$closedShop::IMAGEPATH/noimage.jpg"
            || -e "$closedShop::SIMAGEPATH/noimage.jpg" )
        {
            print qq|<img
		   border="0" alt="There is no picture"
		  src="$closedShop::IMAGEURL/noimage.jpg">|;
        }
        else {
            print qq|<font class="main"><b><div align="center">$closedShop::TEXT{'NOIMAGE'}</div></b><br>\n|;
        }
    }

    print qq|</td><td valign="top"><table border="0" style='border: 1px solid black;' cellpadding=1 cellspacing=1 width="100%">
	  <tr style='background-color: white;' class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'PROD'} $closedShop::TEXT{'NAME'}</strong>:</td>
	  <td><font class="main"> $name</td></tr>
	  <tr class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'CAT'}</strong>:</td>
	  <td><font class="main"><a 
	href="$closedShop::FileNames{'item'}?action=ShowCategory&category=$cat">$closedShop::categories{$cat}</a></td></tr>
	  <tr style='background-color: white;' class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'PRC'}</strong>:</td>
	  <td><font class="main"> $closedShop::CurrencyMarker$price</td></tr>|;

    if ( $closedShop::ShowShipping eq "ITEM" ) {
        print qq|
	  <tr  class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'SHP_PRC'}</strong>:</td>
	  <td><font class="main"> $closedShop::CurrencyMarker$ship</td></tr>|;
    }

    my $bcolor = "style='background-color: white;'";
    if ( @attributes > 0 ) {

        if ( $closedShop::ShowShipping eq "NONE" ) {
            $bcolor = "";
        }
        print qq|
	    <tr class="mainrow" $bcolor><td valign='top'><font class="main"> <strong>$closedShop::TEXT{'ATT'}</strong>:</td>
	    <td valign='top'><font class="main"> |
          . join( "<BR>", @attributes ) . qq|&nbsp;</td></tr>
	    |;
    }
    else {
        $bcolor = "";
    }

    if ( $closedShop::ShowStock eq "Y" ) {
        if ($bcolor) {
            $bcolor = "";
        }
        else {
            $bcolor = "style='background-color: white;'";
        }
        print qq|
	    <tr  $bcolor class="mainrow"><td><font class="main"> <strong>$closedShop::TEXT{'QTY'}</strong>:</td>
	    <td><font class="main"> $quantity</td></tr>
	    |;
    }

    if ( $weight && $weight != 0 ) {
        if ($bcolor) {
            $bcolor = "";
        }
        else {
            $bcolor = "style='background-color: white;'";
        }
        print qq|
	    <tr  $bcolor class="mainrow"><td><font class="main">
	     <strong>$closedShop::TEXT{'WGHT'}</strong>:</td>
	    <td><font class="main"> $weight $closedShop::WeightIdentifier</td></tr>
	    |;
    }

    print qq|
	  </table></td></tr>
	  <tr class="mainrow"><td colspan="2"><font class="main"> $desc</td></tr>
	  <tr class="mainrow"><td colspan="1">
	  <font class="main"> 
	  $closedShop::TEXT{'QTY'}: <input type="text" name="quantity" value="1" size="3" maxlength="3">
	  &nbsp;&nbsp;&nbsp;&nbsp;|;
    if ( $closedShop::EnableInventory eq "Y" && $quantity < 1 ) {
        my $emailmesg = "I would like to order $name, but it is out of stock.";
        my $space     = "%20";
        $emailmesg =~ s/ /$space/ig;
        print qq|<b><a href="mailto:$closedShop::CompanyEmail?subject=$emailmesg">$closedShop::TEXT{'ITEM'} $closedShop::TEXT{'OUTSTOCK'}</a></b>|;
    }
    else {
        print qq| <input type="submit" value="$closedShop::TEXT{'ADD_CRT'}">|;
    }

    print qq|</td></form>
	  <form action="$closedShop::FileNames{cart}" method="POST"><td>  <font class="main">
	  <input type="hidden" name="action" value="AddWishList">
	  <input type="hidden" name="id" value="$id">
	  <input type="submit" value="$closedShop::TEXT{'ADD_WISH'}"></td></tr>
	  </form></table>
	  |;
    closedShop::Footer();
}

###############################################################################
# FUNCTION:  DisplayItemTemplate
# DESCRIPTION: displays an item
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DisplayItemTemplate {
    my ( $id, $name, $price, $ship, $desc, $cat, $qty, $attr, $imgurl ) = @_;

    my @attributes = @{$attr};
    my $attributes = join( "<BR>", @attributes );

    open( FILE, "$closedShop::item_template" ) || return 0;
    my @data = <FILE>;
    my $data = join( "", @data );
    $data =~ s/\n/ /g;
    close(FILE);

    my $category = $closedShop::categories{$cat};
    my $addurl   = $closedShop::FileNames{cart} . "?action=AddProduct&id=$id&quantity=1";
    my $wishurl  = $closedShop::FileNames{cart} . "?action=AddWishList&id=$id";
    my $caturl   = $closedShop::FileNames{item} . "?action=ShowCategory&category=$cat";

    my $addformpost = qq| <form action="$closedShop::FileNames{cart}" method="POST">
	  <input type="hidden" name="action" value="AddProduct">
	  <input type="hidden" name="id" value="$id">|;

    my $str_id          = "<# id #>";
    my $str_name        = "<# name #>";
    my $str_price       = "<# price #>";
    my $str_ship        = "<# shipping #>";
    my $str_desc        = "<# description #>";
    my $str_qty         = "<# quantity #>";
    my $str_cat         = "<# category #>";
    my $str_att         = "<# attributes #>";
    my $str_image       = "<# image_url #>";
    my $str_caturl      = "<# cat_url #>";
    my $str_addurl      = "<# add_url #>";
    my $str_wishurl     = "<# wish_url #>";
    my $str_addformpost = "<# add_formpost #>";

    $data =~ s/$str_id/$id/g;
    $data =~ s/$str_name/$name/g;
    $data =~ s/$str_price/$price/g;
    $data =~ s/$str_ship/$ship/g;
    $data =~ s/$str_desc/$desc/g;
    $data =~ s/$str_qty/$qty/g;
    $data =~ s/$str_cat/$category/g;
    $data =~ s/$str_att/$attributes/g;
    $data =~ s/$str_addurl/$addurl/g;
    $data =~ s/$str_wishurl/$wishurl/g;
    $data =~ s/$str_caturl/$caturl/g;
    $data =~ s/$str_image/$imgurl/g;
    $data =~ s/$str_addformpost/$addformpost/g;

    # courtesy of nashdj from Perlmonks.org
    $data =~ s/<# {(.*?)} #>/EvaluateCode("item",$1,$category,$name,$price,$qty,$desc)/eg;

    $data =~ s/<#(.*?)#>//g;

    print $closedShop::HTML_HEADER;
    print $data;
    return 1;
}

###############################################################################
# FUNCTION:  DisplayAllCategoryTemplate
# DESCRIPTION: displays an item
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DisplayAllCategoriesTemplate {

    my %categories = ();
    my $search     = $closedShop::FileNames{"item"};

    if ( $search !~ /http/ ) {
        $search = $closedShop::urltofolder . "/" . $closedShop::FileNames{"item"};
    }

    foreach ( sort keys %closedShop::categories ) {
        my (@data) = closedShop::GetCategoryInfo($_);
        $data[ eval( $#data + 1 ) ] = "$search?action=ShowCategory&category=$_";
        $categories{$_} = \@data;
    }

    open( FILE, "$closedShop::allcat_template" ) || return 0;
    my @data = <FILE>;
    my $data = join( "", @data );
    $data =~ s/\n/ /g;
    close(FILE);

    # courtesy of nashdj from Perlmonks.org
    $data =~ s/<# {(.*?)} #>/EvaluateCode("all",$1,\%categories)/eg;

    $data =~ s/<#(.*?)#>//g;

    print $closedShop::HTML_HEADER;
    print $data;
    return 1;
}

###############################################################################
# FUNCTION:  EvaluateCodeInHTML
# DESCRIPTION: does evalus
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub EvaluateCode {
    my ( $code, $categories, $category, $name, $price, $qty, $desc, $items, %categories );
    my $sel   = shift;
    my %items = ();
    if ( $sel eq "all" ) {
        ( $code, $categories ) = @_;
        %categories = %{$categories};
    }
    elsif ( $sel eq "item" ) {
        ( $code, $category, $name, $price, $qty, $desc ) = @_;
    }
    elsif ( $sel eq "cat" ) {
        ( $code, $category, $items ) = @_;
        %items = %{$items};
    }

    my $str;

    # which comes from Gisle Aas (gisle@ActiveState.com)
    use IOString;
    my $str_fh = IO::String->new($str);
    my $old_fh = select $str_fh;

    eval $code;

    select $old_fh;
    if ($@) { $str = $@; }
    $str;
}

###############################################################################
# FUNCTION:  DisplayCatTemplate
# DESCRIPTION: displays a category
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DisplayCatTemplate {
    my ( $code, $category, $intro ) = @_;

    open( FILE, "$closedShop::cat_template" ) || return 0;
    my @data = <FILE>;
    my $data = join( "", @data );
    $data =~ s/\n/ /g;
    close(FILE);

    my $caturl = $closedShop::FileNames{item} . "?action=ShowCategory&category=$code";

    my $SQL = qq| select id,name,price,quantity from $closedShop::sql_tables{"item"}
	              where category="$code" order by name |;
    my $sth = closedShop::DatabaseQuery($SQL);

    my %items = ();

    # format - id,name,price,qty left, item url, image url,
    while ( my ( $id, $name, $price, $quantity ) = $sth->fetchrow_array() ) {
        $items{$id} = qq|$id~#~$name~#~$price~#~$quantity~#~$closedShop::FileNames{item}?action=ShowItem&id=$id&quantity=1~#~|;
        if (   -e "$closedShop::IMAGEPATH/$id.jpg"
            || -e "$closedShop::SIMAGEPATH/$id.jpg" )
        {
            if ( $closedShop::FileNames{"item"} =~ /https/ ) {
                $items{$id} .= qq|$closedShop::SecureImageURL/$id.jpg|;
            }
            else {
                $items{$id} .= qq|$closedShop::IMAGEURL/$id.jpg|;
            }
        }
    }

    my $str_cat     = "<# category #>";
    my $str_intro   = "<# intro #>";
    my $str_caturl  = "<# cat_url #>";
    my $str_catcode = "<# cat_code #>";

    $data =~ s/$str_cat/$category/g;
    $data =~ s/$str_intro/$intro/g;
    $data =~ s/$str_caturl/$caturl/g;
    $data =~ s/$str_catcode/$code/g;

    # courtesy of nashdj from Perlmonks.org
    $data =~ s/<# {(.*?)} #>/EvaluateCode("cat",$1,$category,\%items)/eg;

    $data =~ s/<#(.*?)#>//g;

    print $closedShop::HTML_HEADER;
    print $data;
    return 1;
}

###############################################################################
# FUNCTION:  ShowCategory
# DESCRIPTION: displays an item
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ShowCategory {
    my $cat = $query->param("category");
    $cat =~ tr/a-zA-Z0-9//cd;
    my $lastitem = $query->param("lastitem");
    $lastitem =~ tr/0-9//cd;
    my $startitem = $query->param("startitem");
    $startitem =~ tr/0-9//cd;
    my $grouping = $query->param("grouping");
    $grouping =~ tr/0-9//cd;

    my $SQL2 = qq| select COUNT(id) from $closedShop::sql_tables{"item"} 
	              where category="$cat"  |;
    my $sth2 = closedShop::DatabaseQuery($SQL2);
    my ($numitems) = $sth2->fetchrow_array();

    my $mark = qq|<div align="left">(P) = $closedShop::TEXT{'PHOTOA'}</font></div>|;

    my $photoe = "";
    if ( -e "$closedShop::IMAGEPATH/photoexists.gif" ) {
        $photoe = "$closedShop::IMAGEURL/photoexists.gif";
    }
    elsif ( -e "$closedShop::SIMAGEPATH/photoexists.gif" ) {
        $photoe = "$closedShop::SecureImageURL/photoexists.gif";
    }

    if ($photoe) {
        $mark = qq|<div align="left"><img src="$photoe"> 
	  <font color="Red"> = $closedShop::TEXT{'PHOTOA'}</font></div>|;
    }

    my ( $intro, $style, $numimages, $status, $name, $children, $parent ) = closedShop::GetCategoryInfo($cat);

    $intro .= qq|<BR><BR>| if ($intro);
    if ( $grouping > 1 ) { $intro = ""; }

    if ( $status eq "PRIVATE" || $status eq "NOCAT" ) {
        closedShop::Header();
        print qq|<br>
	  <table width="300" border="1"><tr class="headrow"><th colspan="1">
	  <font class="head">$closedShop::TEXT{'CAT'} $closedShop::TEXT{'ERR'}</font></th></tr>
	  <tr class="mainrow">
	  <td><font class="main"> $closedShop::TEXT{'BAD_CATMESG'}</td>
	  </tr></table><br>
	  <br>  |;
        return closedShop::Footer();
    }

    if ( -e $closedShop::cat_template ) {
        my $check = DisplayCatTemplate( $cat, $name, $intro );
        if ( $check == 1 ) { return; }
    }

    closedShop::Header();

    print qq|<font class="main">  $intro |;

    my $search = $closedShop::FileNames{"item"};

    if ($parent) {
        my $SQL = qq|select name,code from $closedShop::sql_tables{"cat"} where id='$parent'|;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ( $nm, $cd ) = $sth->fetchrow_array();
        print qq|</center><B>Categories:</b> |;
        DoParent( $nm, $cd, $parent, $search );
        print qq|$name<BR><center>|;
    }

    if ($children) {
        my @child = split( /~/, $children );
        print qq|</center><b>Sub Categories:</b> |;
        foreach (@child) {
            my $SQL = qq|select name,code from $closedShop::sql_tables{"cat"} where id='$_'|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $nm, $cd ) = $sth->fetchrow_array();
            if ($cd) {
                print qq|<a href="$search?action=ShowCategory&category=$cd">$nm</a> \| |;
            }
        }
    }
    print qq|<center>|;

    if ( $style eq "text" ) {
        my $SQL3 = qq| select count(id) from $closedShop::sql_tables{"item"}
	              where category="$cat" |;
        my $sth3 = closedShop::DatabaseQuery($SQL3);
        my ($numitems) = $sth3->fetchrow_array();

        if ( $numitems > 30 ) {
            print qq|<center>Jump to: |;
            foreach (qw/A B C D E F G H I J K L M N O P Q R S T U V W X Y Z/) {
                print qq|<A href="#$_">$_</a> |;
            }
            print qq|</center>|;
        }

        print qq|
	  <table width="$closedShop::TableWidth" border="1"><tr class="headrow"><th colspan="3">
	  <font class="main"> <font size="3"><font color="blue">$closedShop::TEXT{'CAT'} -
	  $closedShop::categories{$cat}</font></th></tr>
	  <tr class="mainrow">
	  <td><font class="main"><strong>$closedShop::TEXT{'NAME'}</strong></td>
	  <td><font class="main"><strong>$closedShop::TEXT{'PRC'}</strong></td>
	  <td><font class="main"><strong>$closedShop::TEXT{'SHP_CRT'}</strong></td>
	  </tr>|;

        my $k      = 0;
        my $letter = "";
        my $SQL    = qq| select id,name,price,quantity,image_url from $closedShop::sql_tables{"item"}
	              where category="$cat" order by name |;
        my $sth = closedShop::DatabaseQuery($SQL);

        while ( my ( $id, $name, $price, $quantity, $image_url ) = $sth->fetchrow_array() ) {

            my $SQL2 = qq|select name from $closedShop::sql_tables{"attributes"}
		where product_id='$id' limit 1|;
            my $sth2 = closedShop::DatabaseQuery($SQL2);

            my ($atcheck) = $sth2->fetchrow_array();

            my $picturemark = "";
            my $firstletter = substr( $name, 0, 1 );
            if (   -e "$closedShop::IMAGEPATH/$id.jpg"
                || -e "$closedShop::SIMAGEPATH/$id.jpg"
                || $image_url )
            {
                if (   -e "$closedShop::IMAGEPATH/photoexists.gif"
                    || -e "$closedShop::SIMAGEPATH/photoexists.gif" )
                {
                    if ( $closedShop::FileNames{"item"} =~ /https/ ) {
                        $picturemark = qq|<img src="$closedShop::SecureImageURL/photoexists.gif"> |;
                    }
                    else {
                        $picturemark = qq|<img src="$closedShop::IMAGEURL/photoexists.gif"> |;
                    }
                }
                else {
                    $picturemark = qq|<font color="red">(P)</font> |;
                }
            }

            my $printletter = "";
            if ( $firstletter ne $letter ) {
                $letter      = $firstletter;
                $printletter = $letter;
            }
            print qq|
	   	  <tr class="mainrow"><td>
		  <font class="main"> <a name="$printletter">$picturemark</a> <a 
		  href="$program_name?action=ShowItem&id=$id">$name</a></td>
		  <td><font class="main"> $closedShop::CurrencyMarker $price</td>
		  <td><font class="main"> |;

            if ( $closedShop::EnableInventory eq "Y" && $quantity < 1 ) {
                print qq|<b>$closedShop::TEXT{'OUTSTOCK'}</b>|;
            }
            elsif ($atcheck) {
                print qq|<a href="$closedShop::FileNames{item}?action=ShowItem&id=$id">Select Attributes</a> |;
            }
            else {
                print qq|<a href="$closedShop::FileNames{cart}?action=AddProduct&id=$id&quantity=1">$closedShop::TEXT{'ADD_CRT'}</a> |;
            }

            print qq|</td></tr>|;

            $k++;
        }

        if ( $k == 0 ) {
            print qq|
	   	  <tr class="mainrow"><td colspan="3">
		  <font class="main"> <b><div align="center">$closedShop::TEXT{'BAD_CATMESG2'}</div>
		  </b></td></tr>|;
        }

        print qq|</table><BR>$mark|;
    }
    elsif ( $style eq "thumbnail" ) {
        if ( !$grouping || $grouping > $numitems ) { $grouping = 1; }

        my $ng = $grouping + $numimages;
        my $og = $grouping - $numimages;
        if ( $og < 1 ) { $og = 1; }

        my $ngdisplay = $ng - 1;
        if ( $ngdisplay > $numitems ) { $ngdisplay = $numitems; }
        my %imagesizes = (
            "1"  => "400",
            "2"  => "250",
            "4"  => "200",
            "8"  => "200",
            "16" => "100"
        );
        my %newrow = ( "1" => "1", "2" => "2", "4" => "2", "8" => "2", "16" => "4" );
        print qq|
	  <table width="$closedShop::TableWidth" border="1"><tr class="headrow">
	  <th colspan="$newrow{$numimages}">
	  <font class="main"> <font size="3"><font color="blue">$closedShop::TEXT{'CAT'} -
	  $closedShop::categories{$cat}<br></font><Font size="2">
	  Displaying Items $grouping - $ngdisplay out of $numitems</font></th></tr>|;

        my $k = 1;
        my $j = 1;
        my $t = 1;

        my $tmpimages = $numimages;
        if ( ( $numitems - $grouping + 1 ) < $numimages ) {
            $tmpimages = $numitems - $grouping + 1;
        }

        my $SQL = qq| select id,name,price,quantity,image_url from $closedShop::sql_tables{"item"} 
	              where category="$cat" order by name |;
        my $sth = closedShop::DatabaseQuery($SQL);

        while ( my ( $id, $name, $price, $quantity, $image_url ) = $sth->fetchrow_array() ) {
            if ( $t < $grouping ) { $t++; next; }

            if ( $k == 1 ) {
                print qq|<tr class="mainrow">|;
            }

            PrintItem( $name, $price, $id, $imagesizes{$numimages}, $quantity, $image_url );

            if ( $k == $newrow{$numimages} ) {
                print qq| </tr> |;
                $k = 0;
            }
            if ( $tmpimages == $j ) {
                print qq|<tr class="mainrow">
		  <td colspan="$newrow{$numimages}"><center><font class="main"> <A 
		  href="$program_name?action=ShowCategory&category=$cat&grouping=$og">&lt;&lt;
			$closedShop::TEXT{'PREV'} $closedShop::TEXT{'PAGE'}</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A
		  href="$program_name?action=ShowCategory&category=$cat&grouping=$ng">
		  $closedShop::TEXT{'NEXT'} $closedShop::TEXT{'PAGE'}&gt;&gt;</a>
		  </td></tr>
		  |;
                last;
            }
            $k++;
            $j++;
        }
        print qq|</table>|;
        if ( $numitems < 1 ) {
            print qq|<BR>
	    <table width="300" border="1"><tr class="headrow"><th colspan="1">
	  <font class="main"> <font size="3"><font color="blue">$closedShop::TEXT{'CAT'} $closedShop::TEXT{'ERR'}</font></th></tr>
	  <tr class="mainrow">
	  <td><font class="main">$closedShop::TEXT{'BAD_CATMESG2'}</td>
	  </tr></table><br>
	  <br>  |;
        }

    }

    closedShop::Footer();
}

###############################################################################
# FUNCTION:  DoParent
# DESCRIPTION: prints recursive parent
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DoParent {
    my ( $nm, $cd, $id, $url ) = @_;
    my $SQL = qq|select parent,name,code from $closedShop::sql_tables{"cat"}
    where id="$id"|;

    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $parent, $name, $code ) = $sth->fetchrow_array();

    if ($parent) { DoParent( $name, $code, $parent, $url ); }
    print qq|<a href="$url?action=ShowCategory&category=$code"><u>$name</u></a> 
      <font size="3">&gt;</font> |;
}

###############################################################################
# FUNCTION:  PrintAllCategories
# DESCRIPTION: prints all categories
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ShowAllCategories {
    if ( -e $closedShop::allcat_template ) {
        my $check = DisplayAllCategoriesTemplate();
        if ( $check == 1 ) { return; }
    }

    closedShop::Header();
    print qq| <table width="$closedShop::TableWidth" border="1">
    <tr class="headrow"><th>
	  <font class="head"> $closedShop::TEXT{'CATS'}</font></th></tr>
	  <tr class="mainrow"><td><font class="main">
	   |;
    closedShop::PrintCategoryList( "", 0, "", "list" );
    print qq|</td></tr></table>|;
    closedShop::Footer();
}

###############################################################################
# FUNCTION:  PrintItem
# DESCRIPTION: print item stuff
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub PrintItem {
    my ( $name, $price, $id, $imsize, $quantity, $image_url ) = @_;

    my $SQL2 = qq|select name from $closedShop::sql_tables{"attributes"}
		where product_id='$id' limit 1|;
    my $sth2 = closedShop::DatabaseQuery($SQL2);

    my ($atcheck) = $sth2->fetchrow_array();

    print qq|<td><centeR><font class="main"> <A 
		  href="$program_name?action=ShowItem&id=$id">|;

    if ( $closedShop::FileNames{"item"} =~ /https/ ) {
        if (   -e "$closedShop::IMAGEPATH/$id.jpg"
            || -e "$closedShop::SIMAGEPATH/$id.jpg" )
        {
            print qq|<img border="0"
		    alt="Click to view product info" width="$imsize"
		    src="$closedShop::SecureImageURL/$id.jpg">|;
        }
        elsif ($image_url) {
            print qq|<img border="0"
		    alt="Click to view product info" width="$imsize"
		    src="$image_url">|;
        }
        else {
            print qq|<table border="1"><tr><td><font class="main">
			<b><div align="center">$closedShop::TEXT{'NOIMAGE'}</div>
			</b></td></tr></table>|;
        }
    }
    else {
        if (   -e "$closedShop::IMAGEPATH/$id.jpg"
            || -e "$closedShop::SIMAGEPATH/$id.jpg" )
        {
            print qq|<img width="$imsize" border="0" alt="Click to
		    view product info" src="$closedShop::IMAGEURL/$id.jpg">|;
        }
        elsif ($image_url) {
            print qq|<img border="0"
		    alt="Click to view product info" width="$imsize"
		    src="$image_url">|;
        }

        else {
            print qq|<table border="1"><tr><td><font class="main">
			<b><div align="center">$closedShop::TEXT{'NOIMAGE'}</div></b>
			</td></tr></table>|;
        }
    }

    print qq|</a><br><A href="$program_name?action=ShowItem&id=$id">$name</a>
		 <br>Price: $closedShop::CurrencyMarker$price <BR> |;
    if ( $closedShop::EnableInventory && $quantity < 1 ) {
        print qq|<b>$closedShop::TEXT{'OUTSTOCK'}</b>|;
    }
    elsif ( !$atcheck ) {
        print qq|<a 
	href="$closedShop::FileNames{cart}?action=AddProduct&id=$id&quantity=1">$closedShop::TEXT{'ADD_CRT'}</a></td>|;
    }
}

###############################################################################
# FUNCTION:  Search
# DESCRIPTION: allows for searching
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Search {
    closedShop::Header();
    if ( $query->param("selected") == 1 ) {
        my $SQL = qq|select id,name,price,category,quantity,image_url from 
	               $closedShop::sql_tables{"item"} where 1=1 |;
        if ( $query->param("name") ) {
            my $na = $query->param("name");
            $na = closedShop::ValidateInputCustomer($na);
            my @name = split( / /, $na );
            $SQL .= qq| AND ( 1=1 AND |;
            foreach my $n (@name) {
                $SQL .= qq| name LIKE "%$n%" AND |;
            }
            $SQL .= qq| 1=1 )|;
        }
        if ( $query->param("category") ) {
            my $cat = $query->param("category");
            $cat = closedShop::ValidateInputCustomer($cat);
            $SQL .= qq| and category = "$cat" |;
        }
        if ( $query->param("description") ) {
            my $description = $query->param("description");
            $description = closedShop::ValidateInputCustomer($description);
            my @desc = split( / /, $description );
            $SQL .= qq| AND ( 1=0 OR |;
            foreach my $d (@desc) {
                $SQL .= qq| description LIKE "%$d%" OR |;
            }
            $SQL .= qq| 1=0 )|;
        }
        $SQL .= qq| order by category |;
        my $sth = closedShop::DatabaseQuery($SQL);

        my $mark   = qq|<div align="left">(P) = $closedShop::TEXT{'PHOTOA'}</font></div>|;
        my $photoe = "";
        if ( -e "$closedShop::IMAGEPATH/photoexists.gif" ) {
            $photoe = "$closedShop::IMAGEURL/photoexists.gif";
        }
        elsif ( -e "$closedShop::SIMAGEPATH/photoexists.gif" ) {
            $photoe = "$closedShop::SecureImageURL/photoexists.gif";
        }

        if ($photoe) {
            $mark = qq|<div align="left"><img src="$photoe">
	    <font color="Red"> = $closedShop::TEXT{'PHOTOA'}</font></div>|;
        }

        if ( $closedShop::SearchThumbImages ne "Y" ) {

            print qq|<font class="main">$mark
	  <table width="$closedShop::TableWidth" border="1"><tr class="headrow"><th colspan="4">
	  <font class="head"> $closedShop::TEXT{'SRCH_RES'}</font></th></tr>
	  <tr class="mainrow">
	  <td><font class="main"><strong>$closedShop::TEXT{'NAME'}</strong></td>
	  <td><font class="main"><strong>$closedShop::TEXT{'CAT'}</strong></td>
	  <td><font class="main"><strong>$closedShop::TEXT{'PRC'}</strong></td>
	  <td><font class="main"><strong>$closedShop::TEXT{'SHP_CRT'}</strong></td>
	  </tr>|;

            my $k = 0;
            while ( my ( $id, $name, $price, $category, $quantity, $image_url ) = $sth->fetchrow_array() ) {
                my $SQL2 = qq|select name from $closedShop::sql_tables{"attributes"}
		where product_id='$id' limit 1|;
                my $sth2 = closedShop::DatabaseQuery($SQL2);

                my ($atcheck) = $sth2->fetchrow_array();

                my $picturemark = "";
                if (   -e "$closedShop::IMAGEPATH/$id.jpg"
                    || -e "$closedShop::SIMAGEPATH/$id.jpg"
                    || $image_url )
                {
                    if (   -e "$closedShop::IMAGEPATH/photoexists.gif"
                        || -e "$closedShop::SIMAGEPATH/photoexists.gif" )
                    {
                        if ( $closedShop::FileNames{"item"} =~ /https/ ) {
                            $picturemark = qq|<img src="$closedShop::SecureImageURL/photoexists.gif"> |;
                        }
                        else {
                            $picturemark = qq|<img src="$closedShop::IMAGEURL/photoexists.gif"> |;
                        }
                    }
                    else {
                        $picturemark = qq|<font color="red">(P)</font> |;
                    }
                }
                my ( $intro, $style, $numimages, $status, $cname ) = closedShop::GetCategoryInfo($category);
                if ( $status ne "PRIVATE" ) {
                    print qq|
	   	  <tr class="mainrow"><td>
		  <font class="main"> $picturemark <a
		  href="$program_name?action=ShowItem&id=$id&quantity=1">$name</a></td>
		  <td><font class="main"> <a
	 href="$program_name?action=ShowCategory&category=$category">$cname</a></td>
		  <td><font class="main">$closedShop::CurrencyMarker $price</td>
		  <td><font class="main"> |;
                    if ( $closedShop::EnableInventory eq "Y" && $quantity < 1 ) {
                        print qq|<b>$closedShop::TEXT{'OUTSTOCK'}</b>|;
                    }
                    elsif ($atcheck) {
                        print qq|<a href="$closedShop::FileNames{item}?action=ShowItem&id=$id">Select Attributes</a>|;
                    }
                    else {
                        print qq|<a href="$closedShop::FileNames{cart}?action=AddProduct&id=$id&quantity=1">$closedShop::TEXT{'ADD_CRT'}</a>|;
                    }
                    print qq|</td>  </tr>  |;
                    $k++;
                }
            }

            if ( $k == 0 ) {
                print qq|<tr class="mainrow"><td colspan="4">
		  <font class="main"> 
		  <div align="center">$closedShop::TEXT{'NOMATCH'}</div>
		  </td></tr>|;
            }

            print qq|<tr class="headrow"><th colspan="4">
	   	<font class="main"> <font size="3"><font color="blue">
	   	$closedShop::TEXT{'SRCH_RET'} $k $closedShop::TEXT{'ITMS'}</font></th></tr></table>
	   	<font class="main">  <br>
	   	<div align="center"><a href="$program_name">$closedShop::TEXT{'SRCH_AG'}</a></div>|;
        }
        else {

            print qq|
		<table width="$closedShop::TableWidth" border="1"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'SRCH_RES'}</font></th></tr>|;

            my $k = 0;
            while ( my ( $id, $name, $price, $category, $quantity, $image_url ) = $sth->fetchrow_array() ) {

                my $SQL2 = qq|select name from $closedShop::sql_tables{"attributes"}
				where product_id='$id' limit 1|;
                my $sth2 = closedShop::DatabaseQuery($SQL2);
                my ($atcheck) = $sth2->fetchrow_array();

                my $picture = "";
                if ( $closedShop::FileNames{"item"} =~ /https/ ) {
                    $picture = qq|<img width=150 style='border: 1px solid black;' src="$closedShop::SecureImageURL/$id.jpg" border=0> |;
                }
                else {
                    $picture = qq|<img width=150 style='border: 1px solid black;' src="$closedShop::IMAGEURL/$id.jpg" border=0> |;
                }

                if ($image_url) {
                    $picture = qq|<img width=150 style='border: 1px solid black;' src="$image_url" border=0> |;
                }

                if ( !( -e "$closedShop::IMAGEPATH/$id.jpg" || -e "$closedShop::SIMAGEPATH/$id.jpg" || $image_url ) ) {
                    $picture = qq|<table style='width: 200px; 
            			border: 1px solid black; height: 100px;'>
            			<tr><td><center>
            			<font class='main'>$closedShop::TEXT{NOIMAGE}</font>
            			</td></tr></table>|;
                }

                my ( $intro, $style, $numimages, $status, $cname ) = closedShop::GetCategoryInfo($category);
                if ( $status ne "PRIVATE" ) {

                    if ( $k % 2 == 0 ) {
                        print qq| <tr class="mainrow"> |;
                    }

                    print qq|
	   	 <td><center>
		  <font class="main"> <a
		  href="$program_name?action=ShowItem&id=$id&quantity=1">$picture</a>
		  <BR><a
		  href="$program_name?action=ShowItem&id=$id&quantity=1">$name</a><BR>
	$closedShop::TEXT{CAT}: <A href="$program_name?action=ShowCategory&category=$category">$cname</a><BR>
		  $closedShop::CurrencyMarker$price<BR> |;

                    if ( $closedShop::EnableInventory eq "Y" && $quantity < 1 ) {
                        print qq|<b>$closedShop::TEXT{'OUTSTOCK'}</b>|;
                    }
                    elsif ($atcheck) {
                        print qq|<a href="$closedShop::FileNames{item}?action=ShowItem&id=$id">Select Attributes</a>|;
                    }
                    else {
                        print qq|<a href="$closedShop::FileNames{cart}?action=AddProduct&id=$id&quantity=1">$closedShop::TEXT{'ADD_CRT'}</a>|;
                    }
                    print qq|</td>   |;

                    if ( $k % 2 == 1 ) { print qq| </tr> |; }
                    $k++;
                }
            }

            if ( $k % 2 == 1 ) { print qq|<td>&nbsp;</td></tr>|; }

            if ( $k == 0 ) {
                print qq|<tr class="mainrow"><td colspan="2">
		  <font class="main"> 
		  <div align="center">$closedShop::TEXT{'NOMATCH'}</div>
		  </td></tr>|;
            }

            print qq|<tr class="headrow"><th colspan="2">
	   	<font class="main"> <font size="3"><font color="blue">
	   	$closedShop::TEXT{'SRCH_RET'} $k $closedShop::TEXT{'ITMS'}</font></th></tr></table>
	   	<font class="main">  <br>
	   	<div align="center"><a href="$program_name">$closedShop::TEXT{'SRCH_AG'}</a></div>|;
        }
    }
    else {

        print qq|
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="Search">
	  <input type="hidden" name="selected" value="1">
	  <table width="$closedShop::TableWidth" border="1"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'SEARCH'}</font></th></tr>
	    <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'BY'} $closedShop::TEXT{'NAME'}:</td>
	  <td><font class="main"> <input type="text" name="name" size="20"></td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'BY'} $closedShop::TEXT{'CAT'}:</td>
	  <td><font class="main">  |;

        closedShop::PrintCategoryList( "category", 1 );

        print qq|</td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'BY'} $closedShop::TEXT{'DESC'}:</td>
	  <td><font class="main"> <input type="text" name="description" size="20"></td></tr>
	  
	  <tr class="mainrow"><td colspan="2"><center><font class="main">
	  <input type="submit" value="$closedShop::TEXT{'SEARCH'}"></td></tr></form>
	  </table>
	  |;
        my @denoms = closedShop::GetGiftCertificateInfo();
        if ( $denoms[0] ) {
            print qq|<font class="main">
			 <a 
		  href="$closedShop::FileNames{item}?action=ShowGiftCertificates">$closedShop::TEXT{'PURCH'} $closedShop::TEXT{'GC'}</a>
			<BR>
			|;
        }
    }
    closedShop::Footer();
}

