#!/usr/bin/perl 

################################################################################
#
# PROGRAM:  Upgrade.pl - from closedShop - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: Upgrade program to go from 2.1 to 2.2
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2003-04-02  cbf   Created
# 2003-04-02   cbf    Added gift certificates, country listings, pay by check
# 2003-07-10  cbf    Added ip address
# 2004-02-15  cbf    updated for 1.5->1.6
################################################################################

use CGI::Carp(fatalsToBrowser);
use strict;
use CGI;
use DBI;
use closedShop;
use Fcntl ':flock';

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query        = new CGI;
my $program_name = "Upgrade.pl";
my $font         = "<font face='verdana, sans serif' size='2'>";
my @Languages    = ( "English", "Tongan" );

closedShop::Initialize( "", 1 );
Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main {
    my $action = $query->param('action');
    if ( IsInstalled() ) {
        print "Location: Admin.pl\n\n";
    }
    elsif ( $action eq "Upgrade" ) {
        Upgrade();
    }
    else {
        UpgradeScreen("");
    }
}

################################################################################
# FUNCTION:  IsInstalled
# DESCRIPTION: checks to see if it has been run once or not
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub IsInstalled {
    return 0;
}

################################################################################
# FUNCTION:  Upgrade
# DESCRIPTION: install
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub Upgrade {
    closedShop::Initialize( "", 1 );

    UpdateVariables();
    UpdateSQL();

    Header();
    print qq|You have successfully (fingers crossed) upgraded closedShop from 
	2.1 to 2.2
	This file will now change its own permissions so that it can no longer
	be run from a web browser. If you need to try the Upgrade again, you 
	will have to rechange permissions	
	<br>
	<br>
	Your next step might be to login into the <a href="$closedShop::FileNames{admin}">Admin Panel</a>
	|;
    Footer();
    system("chmod 644 $program_name");
}

################################################################################
# FUNCTION:  UpdateVariables
# DESCRIPTION: update vars
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub UpdateVariables {

    open( FILE, ">>$closedShop::varsfile" );
    flock( FILE, LOCK_EX );
    foreach ( $query->param() ) {
        my $data = $query->param("$_");
        $data =~ s/~~~/~*~/g;
        $data =~ s/\n/<BR>/g;
        print FILE qq|$_$closedShop::VarsFileDelimiter| . $data . qq|\n|;
    }

  	my @weights = ();
    my @shipordertotal = ();
    for ( my $t = 0 ; $t < 8 ; $t++ ) {
            my $wend = $query->param("WeightClassEnd$t");
            if ( !$wend ) { $wend = "Max"; }
            
            my $stcend = $query->param("ShipTotalClassEnd$t");
            if ( !$wend ) { $wend = "Max"; }

            my @weight = ( $t, $query->param("WeightClassStart$t"), $wend, $query->param("WeightClassPrice$t") );
            my $wt = join( $closedShop::CookieSectionDelimiter, @weight );
            push @weights, $wt;
            
            my @shiporder = ( $t, $query->param("ShipTotalClassStart$t"), $stcend, $query->param("ShipTotalClassPrice$t") );
            my $stc = join( $closedShop::CookieSectionDelimiter, @shiporder );
            push @shipordertotal, $stc;
      }
      my $wts = join( $closedShop::GiftCertDelimiter, @weights );
      my $stcs = join( $closedShop::GiftCertDelimiter, @shipordertotal );
        
      print FILE qq|WeightClasses$closedShop::VarsFileDelimiter| . $wts . qq|\n|;
      print FILE qq|ShipTotalClasses$closedShop::VarsFileDelimiter| . $stcs . qq|\n|;

    flock( FILE, LOCK_UN );
    close(FILE);
}

################################################################################
# FUNCTION:  UpdateSQL
# DESCRIPTION: update SQL
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub UpdateSQL {
    
    my $SQL = qq|alter table $closedShop::sql_tables{item} 
    add weight decimal(9,4)  |;

    my $sth = closedShop::DatabaseQuery($SQL);

    my $SQL = qq|alter table $closedShop::sql_tables{order} 
    change stax stax decimal(9,5)  |;

    my $sth = closedShop::DatabaseQuery($SQL);

    my $SQL = qq|alter table $closedShop::sql_tables{order} 
     change itax itax decimal(9,5) |;

    my $sth = closedShop::DatabaseQuery($SQL);

    my $SQL = qq|alter table $closedShop::sql_tables{order} 
     change ctax ctax decimal(9,5)  |;

    my $sth = closedShop::DatabaseQuery($SQL);

  
}

################################################################################
# FUNCTION:  InstallScreen
# DESCRIPTION: install screen
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub UpgradeScreen {
    my $message = shift;
    Header();
	closedShop::WriteJavaScript();
	
    print qq|
	<table width="600" border="0"><tr class="headrow"><td colspan="2">
	<font class="head"> Installation - Company Info</font></td></tr>
	<form action="$program_name" method="POST" enctype="multipart/form-data">
	<input type="hidden" name="action" value="Upgrade">
	
  	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='shipinfo' style='visibility: visible; 
	position: relative; z-index: 1; display: block;' name='shipinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
	
	<tr class="mainrow"><td>$font<b>Shipping:</b><BR>
	
	 </font></td><td>$font
	 |;
	   
        print qq|
	 <input type="radio" name="ShowShipping" onClick="show('itemdiv'); hide('nonediv'); hide('totaldiv'); hide('weightdiv');" value="ITEM" checked> Per Item<BR>
	 <input type="radio" name="ShowShipping" onClick="show('weightdiv'); hide('itemdiv'); hide('totaldiv'); hide('nonediv');" value="WEIGHT"  > By Weight<BR>
	 <input type="radio" name="ShowShipping" onClick="show('totaldiv'); hide('itemdiv'); hide('nonediv'); hide('weightdiv');" value="TOTAL"  > By Order Total<BR>
	 <input type="radio" name="ShowShipping" onClick="show('nonediv'); hide('itemdiv'); hide('totaldiv'); hide('weightdiv');" value="N"  > None
	 </td></tr>
	
	 <tr class="mainrow"><td>$font<b>$closedShop::TEXT{'WGHT'} Identifier </b>: </font></td><td><font class="main">
	<select name="WeightIdentifier">|;

        print qq|
	<option value="lbs">lbs</option>
	<option value="kg">kg</option>
  </select>
  
	</td></tr>
	<script language='JavaScript'>
	<!--
	function ChangeWeightStart(j,k,name) {
		var eobject = eval(name + "End" + j);
	    var sobject = eval(name + "Start" + k);
	    var sbox = document.getElementById(sobject.id);
	    var ebox = document.getElementById(eobject.id);
	    
		sbox.value = eval(parseInt(ebox.value)+1);
		
	}
	
	function CheckWeight(j,name) {
		var k = parseInt(j) + 1;
		var eobject = eval(name + "End" + j);
	    var sobject = eval(name + "Start" + j);
	    var sbox = document.getElementById(sobject.id);
	    var ebox = document.getElementById(eobject.id);

	    var nextobject = eval(name + "Start" + k);
	    var nextbox = document.getElementById(nextobject.id);
	    
	    if (parseInt(ebox.value) < parseInt(sbox.value)) {
	    	ebox.value = eval(parseInt(sbox.value) + 1);
	    	nextbox.value = eval(parseInt(sbox.value) + 2);
	    }
	    if (ebox.value == "") {
	    	nextbox.value = "";
	    }
	}
	
	-->
	</script>
	
	<tr class="mainrow"><td colspan='2'>
	<div style="visibility: collapse;">
	<div id='nonediv' name='nonediv' style="position: relative;
		  display: block; visibility: visible; z-index: 1;">
		 $font No shipping prices will be displayed
		 or calculated for the customer</font>
	</div>
	
	<div id='totaldiv' name='totaldiv' style="position: relative;
		  display: none; visibility: hidden; z-index: 1;">
		$font Shipping will be calculated from
		 the total cost of the order by the classes defined below.<BR>
	|;
	
	    for ( my $t = 0 ; $t < 8 ; $t++ ) {
	  
            my ( $s, $e ) = ( "", "" );
            if ( $t == 0 ) { $s = qq| value="0"   |; }
      
            if ( $t == 7 ) { $e = qq| value="Max"  disabled |; }
            
            my $js = "";
            if ( $t != 7 ) { 
              $js = qq| onChange="javascript:CheckWeight($t,'ShipTotalClass');" 
              onKeyUp="javascript:ChangeWeightStart($t,$t+1,'ShipTotalClass');" |; 
              }

            print qq|	  
	  Start: $closedShop::CurrencyMarker<input type='text' size='4' $s 
	  id='ShipTotalClassStart$t' name='ShipTotalClassStart$t' > 
	  &nbsp;&nbsp;
	  End: $closedShop::CurrencyMarker<input $js type='text' $e size='4' 
	  id='ShipTotalClassEnd$t' name='ShipTotalClassEnd$t'>
	  &nbsp;&nbsp;
	  Price: $closedShop::CurrencyMarker<input type='text' size='6' 
	  name='ShipTotalClassPrice$t'>
	  <BR>
	    |;
	  }
	  
	  print qq|
		 </font>
	</div>
	
	<div id='itemdiv' name='itemdiv' style="position: relative;
		  display: none; visibility: hidden;  z-index: 1;">
		$font Shipping will be calculated by the
		 per-item shipping cost</font>
	</div>
	
	<div id='weightdiv' name='weightdiv' style="position: relative;
		   display: none; visibility: hidden; z-index: 100;">
	$font<b>
	Weight Classes: (enter the price for weight ranges)</b><BR>
	If you don't need all the weight classes, the last entry that only 
	has a start range will be considered maximum class. 
	<BR>
	
	|;

        for ( my $t = 0 ; $t < 8 ; $t++ ) {
            
            my ( $s, $e ) = ( "", "" );
            if ( $t == 0 ) { $s = qq| value="0"   |; }
            
            if ( $t == 7 ) { $e = qq| value="Max"  disabled |; }

            my $js = "";
            if ( $t != 7 ) { 
              $js = qq| onChange="javascript:CheckWeight($t,'WeightClass');" 
              onKeyUp="javascript:ChangeWeightStart($t,$t+1,'WeightClass');" |; 
            }

            print qq|	  
	  Start: <input type='text' size='4' $s id='WeightClassStart$t' 
	  name='WeightClassStart$t' > 
	  &nbsp;($closedShop::WeightIdentifier)&nbsp;&nbsp;&nbsp;
	  End: <input $js type='text' $e size='4' 
	  id='WeightClassEnd$t' name='WeightClassEnd$t'>
	  &nbsp;($closedShop::WeightIdentifier) &nbsp;&nbsp;&nbsp;
	  Price: $closedShop::CurrencyMarker<input type='text'
	   size='6' name='WeightClassPrice$t'>
	  <BR>
	  |;

        }

        print qq|
        
        </div>
        </div>
	 </td></tr>
	</table>
	</div>
	</div>
	</td></tr>	

	
	 <tr class="mainrow"><td>$font<b>Show search results as
	 thumbnails: </font></td><td>$font
	 <input type="radio" name="SearchThumbImages" value="Y" > $closedShop::TEXT{'YES'}<BR>
	 <input type="radio" name="SearchThumbImages" value="N" checked> $closedShop::TEXT{'NO'}
	 </td></tr>
	
	<tr class="mainrow"><td colspan="2">
	$font <Center><input type="submit" value="Upgrade System"></font></td></tr></table></form>
	 
	</center></div>|;

    Footer();
}

################################################################################
# FUNCTION:  Header
# DESCRIPTION: header
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub Header {
    print "Content-type: text/html \n\n";
    print qq|<html><head><title>closedShop Installation</title>
<style type="text/css">
    <!--
        font.main {
	  font-family: verdana,arial,sans-serif;
	  color: blue;
          font-size: 13px;
          font-style: oblique;
          font-variant: small-caps;
        }
        font.head {
 	  font-family: verdana,arial,sans-serif;
	  color: blue;
          font-size: 13px;
          font-style: normal;
          font-variant: small-caps;
        }
	tr.mainrow {
          background-color: #EEEEEE;
	  border: 0px dashed black;
        }
	tr.headrow {
          background-color: #99CC99;
	  border: 0px dashed black;
        }
	table.main {
	  border: 0px dashed black;
	  border-spacing: 1 px;
	  
	}
	select.small {
	  font-family: verdana;
	  font-size: 10px;
	}
     -->
     </style>

        </head>
	<body bgcolor="white">
	$closedShop::font
	<h3>closedShop Upgrade</h3>
	|;
}

################################################################################
# FUNCTION:  Footer
# DESCRIPTION: footer
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub Footer {
    print qq|
	</body>
	</html>
	|;
}

