#!/usr/bin/perl 

################################################################################
#
# PROGRAM:  User.pl - from closedShop - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: User accounts
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-03   cbf    Created
# 2001-08-12   cbf    All input validated
# 2003-04-02   cbf    Added gift certificates, country listings, pay by check
# 2003-08-27   cbf    Added order tracking facilities
# 2003-08-28   cbf    Added logout for user screen
# 2003-10-09   cbf    Converted all text to use new multi-lingual facilities
################################################################################

#use CGI::Carp(fatalsToBrowser);
use closedShop;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query        = new CGI;
my $program_name = "$closedShop::FileNames{user}";

my %actions = (
    EditInfo          => "$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'NFO'}",
    Wishlist          => $closedShop::TEXT{"VW_WISH"},
    ViewOrders        => $closedShop::TEXT{"VW_ORD"},
    ChangePassword    => $closedShop::TEXT{"CHG_PASS"},
    Logout            => $closedShop::TEXT{"LOGOUT"},
    ChangeLanguage    => $closedShop::TEXT{'DEF_LANG'},
    NewAffiliate      => "New Affiliate",
    RetrieveAffiliate => "Retrieve Affiliate"
);
################################################################################

Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main {
    my $action = $query->param('action');
    $action = closedShop::ValidateInputCustomer($action);
    if ( $action eq "Login" ) {
        Login();
    }
    elsif ( $action eq "NewAccount" ) {
        NewAccount();
    }
    elsif ( $action eq "RetrieveAffiliate" ) {
        RetrieveAffiliate();
    }
    elsif ( $action eq "NewAffiliate" ) {
        NewAffiliate();
    }
    elsif ( $action eq "ChangeLanguage" ) {
        ChangeLanguage();
    }
    elsif ( $actions{$action} eq "" ) {
        LoginScreen( $closedShop::TEXT{"PLS_LOGIN"} );
    }
    else {
        my ( $session, $uid ) = closedShop::GetSessionCookie();
        if ( !closedShop::Authenticate( $session, $uid, "user" ) ) {
            return LoginScreen( $closedShop::TEXT{"IMPROP_LOGIN"} );
        }
        eval($action);
    }
}

###############################################################################
# FUNCTION:  ChangeLanguage
# DESCRIPTION: Chnage language user uses
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ChangeLanguage {
    my $lang = $query->param("Language");
    use CGI qw/:standard/;
    use CGI::Cookie;
    my $cookie1 = new CGI::Cookie( -name => 'cs_language', -value => "$lang" );
    closedShop::SaveCookie( "cs_language", "$lang" );

    print $query->redirect(
        -Cookie   => [$cookie1],
        -location => $ENV{'HTTP_REFERER'}
    );

}

###############################################################################
# FUNCTION:  RetrieveAffiliate
# DESCRIPTION: sends email with affiliate details
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub RetrieveAffiliate {

    if ( closedShop::ValidateInputCustomer( $query->param("select") eq "email" ) ) {
        my $email = closedShop::ValidateInputCustomer( $query->param("email") );

        my $SQL = qq|select id,fname,lname from $closedShop::sql_tables{"affiliates"} 
				 where email='$email' |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ( $id, $fname, $lname ) = $sth->fetchrow_array();

        $SQL = qq|select commission,sale,owed,status,date_sold from
      $closedShop::sql_tables{"affiliate_sales"} where aid='$id' |;

        $sth = closedShop::DatabaseQuery($SQL);

        my $body = qq|
Affiliate account summary for $fname $lname at $closedShop::CompanyName

------------------------------------------------------------------------
Date of Sale		\|Amt. Owed		\|Sale		\|Status
------------------------------------------------------------------------
|;

        while ( my ( $comm, $sale, $owed, $status, $date_sold ) = $sth->fetchrow_array() ) {
            $body .= qq|$date_sold		\$$owed		\$$sale		$status\n|;
        }

        my $link = $closedShop::FileNames{"item"};
        if ( $link !~ /^http/ ) {
            $link = $closedShop::urltofolder . "/" . $closedShop::FileNames{"item"};
        }

        $body .= qq|------------------------------------------------------------------------
      
Your HTML link to connect as an affiliate is:

$link?action=SetAffiliate&id=$id

Thank You,
$closedShop::CompanyName
$closedShop::website
|;

        if ($id) {
            closedShop::SendEmail( $email, "Affiliate Account Summary", $body );
        }

        closedShop::Header();

        print qq|
    <font size='3' class='main'>
      Your account summary has been successfully mailed to $email. <BR>Please 
      check your email in the next few minutes to view the details.
    </font><BR><BR><BR>
    
    |;

        closedShop::Footer();
    }
    else {
        closedShop::Header();
        print qq|
       <table width='$closedShop::TableWidth' class="main"><tr class='headrow'>
			 <td colspan='2' class='head'>Enter your Email address to retrieve
			 your affiliate sales</td></tr>
			 <form action='$program_name' method='POST'>
			 <input type='hidden' name='action' value='RetrieveAffiliate'>
			 <input type='hidden' name='select' value='email'>
 			 
			 
			 <tr class='mainrow'><td>Email</td><td><input type='text' name='email' size='20'></td></tr>
			 <tr class='headrow'><td colspan='2'><Center><input type='submit' name='submit' value='Retrieve Account Summary'></td></tr>
			 </table>
			 </form>
			 <BR><BR>
			 
       |;

        closedShop::Footer();

    }

}

###############################################################################
# FUNCTION:  NewAffiliate
# DESCRIPTION: shows form to add yourself as new affiliate
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub NewAffiliate {
    if ( closedShop::ValidateInputCustomer( $query->param("select") eq "add" ) ) {
        my $fname   = closedShop::ValidateInputCustomer( $query->param("fname") );
        my $lname   = closedShop::ValidateInputCustomer( $query->param("lname") );
        my $address = closedShop::ValidateInputCustomer( $query->param("address") );
        my $city    = closedShop::ValidateInputCustomer( $query->param("city") );
        my $state   = closedShop::ValidateInputCustomer( $query->param("state") );
        my $country = closedShop::ValidateInputCustomer( $query->param("country") );
        my $zip     = closedShop::ValidateInputCustomer( $query->param("zip") );
        my $phone   = closedShop::ValidateInputCustomer( $query->param("phone") );
        my $email   = closedShop::ValidateInputCustomer( $query->param("email") );

        my $SQL = qq|insert into $closedShop::sql_tables{"affiliates"} 
		set fname='$fname',lname='$lname',
		address='$address',city='$city',status='UNAPPROVED',
		state='$state',country="$country",zip='$zip',phone='$phone',email='$email' |;

        my $sth = closedShop::DatabaseQuery($SQL);

        closedShop::Header();

        print qq|<BR><BR>
    <font class='main'>
      Your information has been successfully submitted. <BR>You will be contacted
      regarding your status as an affiliate.
    </font><BR><BR>
    
    |;

        closedShop::Footer();
    }
    else {
        closedShop::Header();
        print qq|
       <table width='$closedShop::TableWidth' class="main"><tr class='headrow'>
			 <td colspan='2' >Become an Affiliate</td></tr>
			 <form action='$program_name' method='POST'>
			 <input type='hidden' name='action' value='NewAffiliate'>
 			 <input type='hidden' name='select' value='add'>
	<tr class='mainrow'><td>$closedShop::TEXT{'FNAME'}</td><td><input type='text' name='fname' size='20'></td></tr>
	<tr class='mainrow'><td>$closedShop::TEXT{'LNAME'}</td><td><input type='text' name='lname' size='20'></td></tr>
	<tr class='mainrow'><td>$closedShop::TEXT{'EMAIL'}</td><td><input type='text' name='email' size='25'></td></tr>
	<tr class='mainrow'><td>$closedShop::TEXT{'ADDRESS'}</td><td><textarea rows='3' cols='35' name='address'></textarea></td></tr>
	 <tr class='mainrow'><td>$closedShop::TEXT{'CITY'}</td><td><input type='text' name='city' size='20'></td></tr><br />
			 |;

        if ( $closedShop::RestrictState ne "Y" ) {
            print qq|
			 <tr class='mainrow'><td>$closedShop::TEXT{'STATE'}</td><td>|;
            closedShop::ListStates( "state", '' );
            print qq|</td></tr>|;
        }
        print qq|
      			 <tr class='mainrow'><td>$closedShop::TEXT{'COUNTRY'}</td><td>|;
        closedShop::ListCountries( "country", "",, "restrict" );
        print qq|</td></tr>
			 <tr class='mainrow'><td>$closedShop::TEXT{'ZIP'}</td><td><input type='text' name='zip' size='15'></td></tr>
			 <tr class='mainrow'><td>$closedShop::TEXT{'PHONE'}</td><td><input type='text' name='phone' size='15'></td></tr>
			 <tr class='headrow'><td colspan='2'><center><input type='submit' value='Join our Network of Affiliates'></td></tr>
			 </table>
			 </form><font class='main'>
			 <a href='$program_name?action=RetrieveAffiliate'>Retrieve your account summary through email</a><BR><BR>
       |;

        closedShop::Footer();

    }

}

###############################################################################
# FUNCTION:  Wishlist
# DESCRIPTION: The user will log in, sending the username and password
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Wishlist {
    my ( $session, $uid ) = closedShop::GetSessionCookie();
    my $itemid = $query->param("id");
    $itemid =~ tr/0-9//cd;
    if ( $query->param("Remove") && $itemid ) {
        my $SQL = qq| select wishlist_ids from $closedShop::sql_tables{"user"}
               where id = "$uid" |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ($wish) = $sth->fetchrow_array();

        $itemid =~ tr/0-9//cd;
        $itemid = $itemid . ":";
        $wish =~ s/$itemid//g;
        $SQL = qq| update $closedShop::sql_tables{"user"} set wishlist_ids="$wish"
               where id = "$uid" |;
        $sth = closedShop::DatabaseQuery($SQL);
    }
    return UserScreen( $closedShop::TEXT{"REM_WISH"}, $uid );
}

###############################################################################
# FUNCTION:  Login
# DESCRIPTION: The user will log in, sending the username and password
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Login {
    my $username = closedShop::ValidateInputCustomer( $query->param('username') );
    my $password = closedShop::ValidateInputCustomer( $query->param('password') );

    my $SQL = qq| select id from $closedShop::sql_tables{"user"} where email = "$username" and
	              password = "$password" |;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ($uid) = $sth->fetchrow_array();

    # if we pass this condition, the user has logged in, so retrieve information
    if ( !$uid ) {
        return LoginScreen( $closedShop::TEXT{"BAD_EM"} );
    }
    else {
        LoginTheUser($uid);
    }
}

###############################################################################
# FUNCTION:  Logout
# DESCRIPTION: The user will log out
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Logout {
    my ( $session, $uid ) = closedShop::GetSessionCookie();
    if ( closedShop::Authenticate( $session, $uid, "user" ) ) {
        my $SQL = qq| update $closedShop::sql_tables{"user"} set session=""
		where id="$uid"	and session="$session" |;
        my $sth = closedShop::DatabaseQuery($SQL);
    }

    return LoginScreen( $closedShop::TEXT{"LG_OUT"} );
}

################################################################################
# FUNCTION:  ChangePassword
# DESCRIPTION: ChangePassword
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub ChangePassword {
    my $old     = closedShop::ValidateInputCustomer( $query->param("old") );
    my $new     = closedShop::ValidateInputCustomer( $query->param("new") );
    my $confirm = closedShop::ValidateInputCustomer( $query->param("confirm") );

    my ( $session, $uid ) = closedShop::GetSessionCookie();

    my $SQL = qq| select email from $closedShop::sql_tables{"user"}
               where password = "$old" and id = "$uid" |;
    my $sth = closedShop::DatabaseQuery($SQL);
    if ( !$sth->fetchrow_array() ) {
        return UserScreen( $closedShop::TEXT{"IN_PASS"}, $uid );
    }
    elsif ( $new ne $confirm ) {
        return UserScreen( $closedShop::TEXT{"NO_MATCH"}, $uid );
    }
    else {
        $SQL = qq| update $closedShop::sql_tables{"user"}
              set password="$new" where id = "$uid" |;
        $sth = closedShop::DatabaseQuery($SQL);
        return UserScreen( $closedShop::TEXT{"UP_PASS"}, $uid );
    }
}

################################################################################
# FUNCTION:  EditInfo
# DESCRIPTION: edits info
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub EditInfo {
    my $fname   = closedShop::ValidateInputCustomer( $query->param("fname") );
    my $lname   = closedShop::ValidateInputCustomer( $query->param("lname") );
    my $email   = closedShop::ValidateInputCustomer( $query->param("email") );
    my $address = $query->param("address");
    $address =~ s/"/'/g;
    my $city    = closedShop::ValidateInputCustomer( $query->param("city") );
    my $state   = closedShop::ValidateInputCustomer( $query->param("state") );
    my $country = closedShop::ValidateInputCustomer( $query->param("country") );
    my $zip     = closedShop::ValidateInputCustomer( $query->param("zip") );
    my $phone   = closedShop::ValidateInputCustomer( $query->param("phone") );

    my ( $session, $uid ) = closedShop::GetSessionCookie();

    my $SQL = qq| update $closedShop::sql_tables{"user"}
              set fname="$fname",lname='$lname',address="$address",city="$city",state="$state",
			  zip="$zip",phone="$phone",country="$country" where id = "$uid" |;
    my $sth = closedShop::DatabaseQuery($SQL);

    $SQL = qq| select email from $closedShop::sql_tables{"user"}
               where email = "$email" and id != "$uid" |;
    $sth = closedShop::DatabaseQuery($SQL);
    if ( $sth->fetchrow_array() ) {
        return UserScreen( $closedShop::TEXT{"EM_USED"}, $uid );
    }
    else {
        $SQL = qq| update $closedShop::sql_tables{"user"}
              set email="$email" where id = "$uid" |;
        $sth = closedShop::DatabaseQuery($SQL);
        return UserScreen( $closedShop::TEXT{"SUCC_UPDATE"}, $uid );
    }
}

################################################################################
# FUNCTION:  UserScreen
# DESCRIPTION: Displays the admin screen
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub UserScreen {
    my $message = shift;
    closedShop::Header($message);
    $message = "" if ( $message eq "nohttp" );
    my $uid = shift;

    print qq|<br>
	<font class="main">	<h4>$closedShop::TEXT{"WELCOME"} $closedShop::CompanyName.</h4>
	<font color="red">$message</font><br><font class="main">
	|;

    my $SQL = qq|select $closedShop::sql_tables{"groups"}.name,
		discount from $closedShop::sql_tables{"user"},$closedShop::sql_tables{"groups"}
		where $closedShop::sql_tables{"user"}.id = '$uid' and 
		$closedShop::sql_tables{"user"}.groupid = $closedShop::sql_tables{"groups"}.id |;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $gname, $gdiscount ) = $sth->fetchrow_array();

    if ($gname) {
        print qq|
	 <table style='border: 1px solid black;' width='95%'><tr>
      <td style='font-size: 13px;'>You are in the $gname group 
      which receives a $gdiscount\% discount</td></tr></table><BR>
	|;
    }

    print qq|
      
	
	<table border="0" width="$closedShop::TableWidth"><tr><td valign="top">
	
	<td width="60%" rowspan="2" valign="top"><font class="main">	|;

    $SQL = qq| select fname,lname,address,city,state,country,zip,phone,email,wishlist_ids 
                from $closedShop::sql_tables{"user"} where id = "$uid" |;
    $sth = closedShop::DatabaseQuery($SQL);
    my ( $fname, $lname, $address, $city, $state, $country, $zip, $phone, $email, $wishlist_ids ) = $sth->fetchrow_array();

    print qq|
	<table  class="main" width="275" border="0"><tr class="headrow"><td colspan="1"  >
	<font class="head">
	<a name="EditInfo">$closedShop::TEXT{"US_INFO"}</a></font></td></tr>
	<form action="$program_name" method="POST">
	<input type="hidden" name="action" value="EditInfo">
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"FNAME"}</b>: </font><font size="1">
	<input type="text" name="fname" value="$fname" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"LNAME"}</b>: </font><font size="1">
	<input type="text" name="lname" value="$lname" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"EMAIL"}</b></font>: <font size="1"><input type="text" name="email" value="$email" size="20"> </td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"ADDRESS"}</b></font>: <font size="1"><textarea rows="2" name="address" cols="20">$address</textarea> </td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"CITY"}</b></font>: <font size="1"><input type="text" name="city" value="$city" size="20"> </td></tr>
	|;

    if ( $closedShop::RestrictState ne "Y" ) {
        print qq|<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"STATE"}</b></font>:
	|;
        closedShop::ListStates( "state", $state );
        print qq|
	</td></tr>
	|;
    }
    print qq|
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"ZIP"}</b></font>: <font size="1"><input type="text" name="zip" value="$zip" size="12"> </td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"COUNTRY"}</b></font>:
	|;
    closedShop::ListCountries( "country", $country,, "restrict" );
    print qq|
	</td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{"PHONE"}</b></font>:<font size="1"> <input type="text" name="phone" value="$phone" size="20"> </td></tr>
	<tr class="mainrow"><td><font class="main"><center><b><input type="submit" value="$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'NFO'}" name="EditInfo"></b></center></td></tr>
    <tr class="headrow"><td colspan="2" class="headrow">
	<font class="main"> &nbsp;</td></tr>  
	 </form></table>
	 </td>
	 <td valign='top'>
	 <table class="main" border="0" width="100%"><tr class="headrow"><td colspan="2" >
	 <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="ChangePassword">
	<font class="head"> <a name="ChangePassword">$closedShop::TEXT{"CHG_PASS"}</a></b></font></td></tr>
	 <tr class="mainrow">
	 <td><font class="main"><b>$closedShop::TEXT{"OLD_PASS"}</b></td><td><font class="main">
	<font size="1"><input type="password" name="old" value="" size="12"> </td>
	</tr>
	<tr  class="mainrow">
	 <td><font class="main"><b>$closedShop::TEXT{"NEW_PASS"}</b></td><td><font class="main">
	<font size="1"> <input type="password" name="new" value="" size="12"> </td>
	</tr>
	<tr  class="mainrow">
	
	 <td><font class="main"><b>$closedShop::TEXT{"CNFNEW_PASS"}</b></td><td><font class="main">
	<font size="1"> <input type="password" name="confirm" value="" size="12"> </td>
	</tr> 

	<tr class="mainrow"><td colspan="3"><font class="main"><center>
	<b><input type="submit" value="$closedShop::TEXT{'CHG_PASS'}" name="ChangePassword"></b></center></td></tr></form>
 <tr class="headrow"><td colspan="2" class="headrow">
	<font class="main"> &nbsp;</td></tr>
	 </table>
	 	 
	 <table class="main" border="0" width="100%"><tr class="headrow" ><td colspan="2" >
	 <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="Logout">
	<font class="head"> <a
name="Logout">$closedShop::TEXT{"LOGOUT"}</a></font></td></tr>
	<tr class="mainrow"><td colspan="3"><font class="main"><center>
	<b><input type="submit" value="$closedShop::TEXT{'LOGOUT'}"
name="Logout"></b></center></td></tr></form>
 <tr class="headrow"><td colspan="2" class="headrow">
	<font class="main"> &nbsp;</td></tr>
	 </table>
	 
	
	</td>
	 
	 </tr>
	</table>
	 
	 
	 <table border="0" width="$closedShop::TableWidth"><tr><td valign="top">
	 <tr><td valign="top"  width="100%" colspan="2"><font class="main">
	 <table class="main" border="0" width="100%"><tr  class="headrow"><td colspan="6">
	<font class="head"> <a name="ViewOrders">$closedShop::TEXT{"OR_LIST"}</a></font></td></tr>
	 <tr class="mainrow">
	 <td><font class="main"><b>ID</b></td>
	 <td><font class="main"><b>$closedShop::TEXT{"ITMS"}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{"DBOUGHT"}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{"DPROC"}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{"STAT"}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{"TOT"}</b></td></tr>|;

    $SQL = qq| select id,item_ids,DATE_FORMAT(date_bought,"%M %d %Y"),
			DATE_FORMAT(date_processed,"%M %d %Y"),complete,
	                total,tracking,trackno from $closedShop::sql_tables{"order"} 
	                where complete="Y" and user_id="$uid" order by date_bought DESC|;

    $sth = closedShop::DatabaseQuery($SQL);
    my $k = 0;
    while ( my ( $oid, $iids, $date_bought, $date_processed, $complete, $total, $tracking, $trackno ) = $sth->fetchrow_array() ) {
        $k++;
        my @ary = ();
        if ( $iids =~ /$closedShop::CookieItemDelimiter/ ) {
            @ary = split( /$closedShop::CookieItemDelimiter/, $iids );
        }
        else {
            @ary = split( /~/, $iids );
        }

        my $item_text = "";
        foreach (@ary) {
            my ( $iid, $qty, $attr );

            if ( $_ =~ /$closedShop::CookieSectionDelimiter/ ) {
                ( $iid, $qty, $attr ) =
                  split( /$closedShop::CookieSectionDelimiter/, $_ );
            }
            else {
                ( $iid, $qty, $attr ) = split( /\|/, $_ );
            }
            if ( !$iid ) { next; }
            my $SQL2 = qq| select name,category from $closedShop::sql_tables{"item"} where
				      id="$iid" |;
            my $sth2 = closedShop::DatabaseQuery($SQL2);
            my ( $name, $cat ) = $sth2->fetchrow_array();

            my @attr = ();
            if ($attr) {
                @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
            }

            $item_text .= qq|$qty x <a href="$closedShop::FileNames{item}?action=ShowItem&id=$iid">$name</a>  
	(<a href="$closedShop::FileNames{item}?action=ShowCategory&category=$cat">$closedShop::categories{$cat}</a>)<BR>
		  |;
        }

        if ($date_processed) { $complete = $closedShop::TEXT{"ORD_COMP"}; }
        else { $complete = $closedShop::TEXT{"PEND"}; }
        if ( $date_processed eq "" ) { $date_processed = "--"; }
        my $shipping = "";
        if ($tracking) {
            my @tmp =
              split( /~~~/, $closedShop::ShippingCompanies{"$tracking"} );
            my $trackname = $tmp[0];
            if ( !$trackname ) { $trackname = $tracking; }
            my $trackurl = $tmp[1];
            $trackurl =~ s/<NUMBER>/$trackno/g;
            if ($trackurl) {
                $trackurl = qq|<a href="$trackurl"
                        target="_blank">$closedShop::TEXT{"PCK_TRK"}</a> (No. $trackno)|;
            }
            else {
                $trackurl = qq|$closedShop::TEXT{"TRK"} #: $trackno|;
            }
            $shipping .= qq|$closedShop::TEXT{"SHIP_CO"}: <u>$trackname</u> &nbsp;&nbsp;&nbsp;&nbsp;$trackurl|;
        }

        print qq|
		    <tr class="mainrow">
			<td><font class="main"> $oid</td>
			<td><font class="main"> $item_text</td>
			<td><font class="main"> $date_bought</td>
			<td><font class="main"> $date_processed</td>
			<td><font class="main"> $complete</td>
			<td><font class="main"> $closedShop::CurrencyMarker  $total</td></tr>
			|;
        if ($shipping) {
            print qq|<tr><td colspan="1">&nbsp;</td><td colspan='5'><font class="main"> $shipping</td>
		    </tr>|;
        }
    }

    if ( $k == 0 ) {
        print qq|<tr  class="mainrow"><td colspan="6">
	<font class="main"> <div align="center"><b>$closedShop::TEXT{"NO_ORD"}</b></div></font></td></tr>|;
    }

    print qq| <tr class="headrow"><td colspan="6" >
	<font class="main"> &nbsp;</td></tr>
	 </table>
	 </td></tr>
	
	  <tr><td valign="top"  width="100%" colspan="2"><font class="main">
	 <table class="main" border="0" width="100%"><tr class="headrow"><td colspan="6" >
	<font class="head"> <a name="Wishlist">$closedShop::TEXT{"WSH"}</a></font></td></tr>
	 <tr class="mainrow">
	 <td><font class="main"><b>$closedShop::TEXT{"NAME"}</b></td>
	 <td><font class="main"><b>$closedShop::TEXT{"CAT"}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'ITEM'} $closedShop::TEXT{"PRC"}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{"RM_WISH"}</b></td>
	  </tr>|;

    my @wish = split( /:/, $wishlist_ids );

    $k = 0;
    foreach (@wish) {
        $k++;
        my $SQL2 = qq| select name,category,price from 
        	$closedShop::sql_tables{"item"} where id="$_" |;
        my $sth2 = closedShop::DatabaseQuery($SQL2);
        my ( $n, $c, $p ) = $sth2->fetchrow_array();
        print qq|
		<tr class="mainrow">
	 	<td><font class="main"><a href="$closedShop::FileNames{item}?action=ShowItem&id=$_">$n</a></td>
	 	<td><font class="main"> $closedShop::categories{$c}</td>
	  	<td><font class="main"> $closedShop::CurrencyMarker $p</td>
		<td><font class="main"> 
		<a href="$program_name?action=Wishlist&Remove=1&id=$_">$closedShop::TEXT{"RM_WISH"}</a></td></tr>
		  |;
    }
    if ( $k == 0 ) {
        print qq|<tr class="mainrow"><td colspan="5" >
	<font class="main"> <div align="center"><b>$closedShop::TEXT{"NO_WISH"}</b></div></font></td></tr>|;
    }

    print qq|
	 <tr class="headrow"><td colspan="5" >
	<font class="main"> &nbsp;</td></tr>
	 </table>
	 </td></tr>
	 </table>
	</center></div>|;

    closedShop::Footer();
}

################################################################################
# FUNCTION:  LoginScreen
# DESCRIPTION: login screen for admin
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginScreen {
    my ( $session, $uid ) = closedShop::GetSessionCookie();
    if ( closedShop::Authenticate( $session, $uid, "user" ) ) {
        return UserScreen( "", $uid );
    }

    my $mesg = $query->param("message") || $_[0];
    closedShop::Header();
    print qq| <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="Login">
	<font class="main">
	<font color="red">$mesg</font>
	<table class="main" border="0"><tr class="headrow"><td colspan="2">
	<font class="head">
	<div align="center"><b>$closedShop::TEXT{"LOGIN"}</div></td></tr><tr class="mainrow"><td><font class="main">
	$closedShop::TEXT{"EMAIL"}: </td><td><input type="text" name="username" size="25"></td></tr>
	<tr class="mainrow"><td><font class="main">
	$closedShop::TEXT{"PASS"}: </td><td><input type="password" name="password" size="25"></td></tr>
	<tr class="mainrow"><td  colspan="2">
	<div align="center"><input type="submit" name="admin_login" value="$closedShop::TEXT{'LOGIN'}"></div>
	</form>
	
	<tr class="mainrow"><td colspan="2">&nbsp;</td></tr>
	  
	  <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="NewAccount">
	  <tr class="mainrow">
	  <td colspan="2"><div align="center"><font class="main"><strong>$closedShop::TEXT{"CR_ACC"}</strong><br>
	  Please fill required fields</div></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{"FNAME"}: </td>
	  <td><font class="main"> <input type="name" name="fname" size="20"></td></tr>
	   <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{"LNAME"}: </td>
	  <td><font class="main"> <input type="name" name="lname" size="20"></td></tr>
	  <tr class="mainrow"><td><font class="main">$closedShop::TEXT{"EMAIL"}: </td>
	  <td><font class="main"> <input type="text" name="email2" size="20"></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{"PASS"}: </td>
	  <td><font class="main"> <input type="password" name="password2" size="20"></td></tr>
  	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{"CNFNEW_PASS"}: </td>
	  <td><font class="main"> <input type="password" name="confirm" size="20"></td></tr>
	  <tr class="mainrow"><td colspan="2"><div align="center"><font class="main">
	  <input type="submit" name="new" value="$closedShop::TEXT{'CR_ACC'}"></div></td></tr>
	  </table>
	<br></form>
	|;

    closedShop::Footer();
}

################################################################################
# FUNCTION:  NewAccount
# DESCRIPTION: NewAccount
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub NewAccount {
    my $fname = $query->param("fname");
    $fname = closedShop::ValidateInputCustomer($fname);
    my $lname = $query->param("lname");
    $lname = closedShop::ValidateInputCustomer($lname);
    my $email = $query->param("email2");
    $email = closedShop::ValidateInputCustomer($email);
    my $password = $query->param("password2");
    $password = closedShop::ValidateInputCustomer($password);
    my $confirm = $query->param("confirm");
    $confirm = closedShop::ValidateInputCustomer($confirm);

    if ( $password ne $confirm ) {
        return LoginScreen( $closedShop::TEXT{"NO_MATCH"} );
    }
    if ( !$fname || !$lname || !$email || !$password ) {
        return LoginScreen( $closedShop::TEXT{"FLL_ALL"} );
    }
    if ( !( closedShop::CheckEmail($email) ) ) {
        return LoginScreen( $closedShop::TEXT{"NOGOOD_EM"} );
    }
    my $SQL = qq| select email from $closedShop::sql_tables{"user"} 
	             where email="$email" |;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ($temail) = $sth->fetchrow_array();
    if ($temail) {
        return LoginScreen( $closedShop::TEXT{"EM_USED"} );
    }

    $SQL = qq| insert into $closedShop::sql_tables{"user"} 
	           set email="$email",fname="$fname",lname='$lname',password="$password" |;
    $sth = closedShop::DatabaseQuery($SQL);
    my $new_id = $sth->{'mysql_insertid'};

    my %cookies    = fetch CGI::Cookie;
    my $lastaction = closedShop::GetCookie('cs_lastaction');
    my $itemid     = closedShop::GetCookie('cs_lastaction_item');

    if ( $lastaction eq "AddWishList" ) {
        closedShop::AddItemToWishList( $new_id, $itemid );
    }

    LoginTheUser($new_id);
}

################################################################################
# FUNCTION:  LoginTheUser
# DESCRIPTION: process screen
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginTheUser {
    my $uid     = shift;
    my $session = closedShop::CreateSession();
    my $SQL     = qq| update $closedShop::sql_tables{"user"} set session="$session" where id = "$uid" |;
    my $sth     = closedShop::DatabaseQuery($SQL);
    closedShop::SetSessionCookie( $session, $uid );
    UserScreen( "nohttp", $uid );
}

