################################################################################
#
# PROGRAM:  closedShop.pm - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: Common functions needed for the closedShop
#
# Author: Chris Fleizach  -  <chris@fightliteracy.com>
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-03   cbf    Created
# 2001-06-17   cbf    All variables stored in a file now, and are loaded from
#                       that file.
# 2001-06-22   cbf    Loads file names from variable file
# 2001-07-03   cbf    Fixed grammatical bugs
# 2001-07-24   cbf    Mailprogram can be customized
# 2001-08-05   cbf    SetSessionCookie was not setting cookie correctly always
# 2001-09-08   cbf    Added SecureImageURL variable
# 2001-09-18   cbf    Added a check for validity of email address
# 2001-10-18   cbf    Added database location
# 2002-01-20   cbf    Added category file variable, and info retrieval
# 2002-05-12   cbf    Added minimum order amount, added continue shopping vars,
#                       final return vars
# 2002-05-14   cbf    Continue Shopping and Final Order link variables added
# 2003-04-01   cbf    Added ShowTax option
# 2003-04-02   cbf    Added gift certificates, country listings, pay by check
# 2003-06-10   cbf    Added PayPal capabilities
# 2003-07-10   cbf    Added sub categories, final message variable
# 2003-08-23   cbf    Added different language support
# 2003-08-26   cbf    changed validateInput to accept latin characters
# 2003-09-25   cbf    Added secure paths for different secure and non secure sys
# 2003-11-10   cbf    Added language changing facilities to the footer
# 2004-02-18   bd     Bug fixes in sub cat links from B.Darnell@student.umist.ac.uk
# 2005-01-11   gyp    changed validate input to use dbi->quote instead of
#                        tr, courtesy of gypsy@freeq.com
# 2005-03-09   cbf    Validate input for cookies now
# 2005-04-04   cbf    Header/Footer redone
################################################################################
package closedShop;

$VERSION = "2.2";

=head1 NAME

closedShop - Module for holding common functions

=head1 SYNOPSIS
	use closedShop;
=cut

################################################################################
# FUNCTION:  Initialize
# DESCRIPTION: Initialize all constant needed throughout the program
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Added the Initialize subroutine to make program more
#                     understandable
################################################################################
sub closedShop::Initialize {
    use vars qw/  $dbh $HTML_HEADER $font $IMAGEURL
      $dsn $DatabaseUsername $DatabasePassword $tc $mc
      %sql_tables $IMAGEPATH %categories $website
      %AcceptedCards $urltofolder $TaxState
      $TaxRate $CompanyName $CompanyEmail $mailprog
      $CompanyPhone %FileNames $CompanyAddress $SecureImageURL
      $CompanyState $CompanyZip $CompanyCity $varsfile
      $MinimumOrder $ContinueShoppingURL $CompletedOrderURL
      $ShowShipping $ShowTax $CurrencyMarker $gcfile $CompanyCountry
      $DatabaseName $DatabaseLocation $TableWidth $PayPalEmail $MaxCats
      $CompanyFax $FinalMessage %PrimalHash $IPAccess $EbayAuction
      $EbayUsername $EbayPassword @Languages $sheader $sfooter $SIMAGEPATH
      %ShippingCompanies $EnableInventory $Language %TEXT
      $fontmain_family $fontmain_color $fontmain_size $fontmain_style
      $fontmain_variant $fonthead_family $fonthead_color $fonthead_size
      $fonthead_style $fonthead_variant $mainrow_color $headrow_color
      $table_border $table_borderspacing $table_backgroundimage
      $cat_template $item_template %Languages $allcat_template
      $EnablePreCharge $PreChargeMerchantID $PreChargeSecurityKey1
      $PreChargeSecurityKey2 $emailflags $CookieAttributeDelimiter
      $CookieItemDelimiter $CookieSectionDelimiter $ShowStock $mainrow_hover
      $input_hover $RestrictState $RestrictCountry %KnownCards
      $MaxAttributeOptions $MaxAttributes $UseTinyMCE $TinyMCEFormat
      $MaxMindLicenseKey $EnableMaxMind $EnableTaxExempt $EnableAuthorizeNET
      $AuthorizeNETLoginID $AuthorizeNETTransactionKey $ForceSSL $ForceCartURL
      $WeightIdentifier $ShipByWeight %WeightClasses %ShipTotalClasses $SearchThumbImages
      $EmailFooter
      /;

    $varsfile                 = "closedShop.vars";
    $CookieAttributeDelimiter = ":_:";
    $VarsFileDelimiter        = "~~~";
    $CookieItemDelimiter      = "~_~";
    $CookieSectionDelimiter   = "#_#";
    $GiftCertDelimiter        = "~:~";
    $MaxCats                  = 20;

    LoadVariables( "$varsfile", $_[1] );

}



################################################################################
# FUNCTION:  LoadVariables
# DESCRIPTION: load variables from a file
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Added the Initialize subroutine to make program more
#                     understandable
################################################################################
sub closedShop::LoadVariables {
    my %hash = ();
    open( FILE, "$_[0]" ) || ErrorMessage("Cannot open variable file $_[0]");
    while (<FILE>) {
        chomp($_);
        my @ary = split( /~/, $_ );
        $hash{ $ary[0] } = $ary[1];
        if ( !$ary[1] ) {
            @ary = split( /$closedShop::VarsFileDelimiter/, $_ );
            $hash{ $ary[0] } = $ary[1];
        }
    }
    close(FILE);
    %PrimalHash = %hash;

    $DatabaseName     = $hash{"DatabaseName"};
    $DatabaseLocation = $hash{"DatabaseLocation"};
    $DatabaseUsername = $hash{"DatabaseUsername"};
    $DatabasePassword = $hash{"DatabasePassword"};
    $dsn              = "DBI:mysql:" . $DatabaseName . ":" . $DatabaseLocation;

    $website     = $hash{"website"};
    $urltofolder = $hash{"urltofolder"};

    $Language  = $hash{"Language"};
    %Languages = (
        English => "language.en",
        Chinese => "language.zh",
        Tongan  => "language.to"
    );

    %ShippingCompanies = (
        UPS   => "United Parcel Service~~~http://wwwapps.ups.com/WebTracking/processInputRequest?HTMLVersion=5.0&sort_by=status&tracknums_displayed=1&TypeOfInquiryNumber=T&loc=en_US&InquiryNumber1=<NUMBER>&AgreeToTermsAndConditions=yes&track.x=19&track.y=5",
        FedEx => "Federal Express~~~http://www.fedex.com/cgi-bin/tracking?action=track&tracknumbers=<NUMBER>&language=english&cntry_code=us&mps=y&ascend_header=1",
        USPS  => "United States Postal Service~~~http://trkcnfrm1.smi.usps.com/netdata-cgi/db2www/cbd_243.d2w/output?CAMEFROM=OK&strOrigTrackNum=<NUMBER>"
    );

    $CompanyName     = $hash{"CompanyName"};
    $CompanyEmail    = $hash{"CompanyEmail"};
    $CompanyPhone    = $hash{"CompanyPhone"};
    $CompanyFax      = $hash{"CompanyFax"};
    $CompanyAddress  = $hash{"CompanyAddress"};
    $CompanyZip      = $hash{"CompanyZip"};
    $CompanyCity     = $hash{"CompanyCity"};
    $CompanyState    = $hash{"CompanyState"};
    $CompanyCountry  = $hash{"CompanyCountry"};
    $IPAccess        = $hash{"IPAccess"};
    $EnableInventory = $hash{"EnableInventory"};

    $TaxState       = $hash{"CompanyState"};
    $TaxRate        = $hash{"TaxRate"};
    $TaxRateCountry = $hash{"TaxRateCountry"};
    $TaxRateInt     = $hash{"TaxRateInt"};
    $CurrencyMarker = $hash{"CurrencyMarker"};
    $MinimumOrder   = $hash{"MinimumOrder"};
    $ForceSSL       = $hash{"ForceSSL"};
    $ForceCartURL   = $hash{"ForceCartURL"};

    $MaxAttributes       = $hash{"MaxAttributes"};
    $MaxAttributeOptions = $hash{"MaxAttributeOptions"};
    $SearchThumbImages   = $hash{"SearchThumbImages"};

    if ( !$MaxAttributes       || $MaxAttributes <= 0 )       { $MaxAttributes       = 3; }
    if ( !$MaxAttributeOptions || $MaxAttributeOptions <= 0 ) { $MaxAttributeOptions = 5; }

    $PayPalEmail                = $hash{"PayPalEmail"};
    $EbayAuction                = $hash{"EbayAuction"};
    $EbayUsername               = $hash{"EbayUsername"};
    $EbayPassword               = $hash{"EbayPassword"};
    $EnableMaxMind              = $hash{"EnableMaxMind"};
    $MaxMindLicenseKey          = $hash{"MaxMindLicenseKey"};
    $EnableAuthorizeNET         = $hash{"EnableAuthorizeNET"};
    $AuthorizeNETLoginID        = $hash{"AuthorizeNETLoginID"};
    $AuthorizeNETTransactionKey = $hash{"AuthorizeNETTransactionKey"};

    $EnablePreCharge       = $hash{"EnablePreCharge"};
    $PreChargeMerchantID   = $hash{"PreChargeMerchantID"};
    $PreChargeSecurityKey1 = $hash{"PreChargeSecurityKey1"};
    $PreChargeSecurityKey2 = $hash{"PreChargeSecurityKey2"};
    $UseTinyMCE            = $hash{"UseTinyMCE"};
    $TinyMCEFormat         = $hash{"TinyMCEFormat"};
    $EnableAuthorizeNET    = $hash{"EnableAuthorizeNET"};
    if ( !$TinyMCEFormat ) { $TinyMCEFormat = "default"; }

    $HTML_HEADER = "Content-Type: text/html; charset=utf-8 \n\n";
    $font        = $hash{"font"};

    $tc = $hash{"tc"};
    $mc = $hash{"mc"};

    $fontmain_family  = $hash{"fontmain_family"};
    $fontmain_color   = $hash{"fontmain_color"};
    $fontmain_size    = $hash{"fontmain_size"};
    $fontmain_style   = $hash{"fontmain_style"};
    $fontmain_variant = $hash{"fontmain_variant"};

    $fonthead_family  = $hash{"fonthead_family"};
    $fonthead_color   = $hash{"fonthead_color"};
    $fonthead_size    = $hash{"fonthead_size"};
    $fonthead_style   = $hash{"fonthead_style"};
    $fonthead_variant = $hash{"fonthead_variant"};
    $mainrow_color    = $hash{"mainrow_color"};
    $headrow_color    = $hash{"headrow_color"};
    $mainrow_hover    = $hash{"mainrow_hover"};
    $input_hover      = $hash{"input_hover"};

    $header  = $hash{"header"};
    $footer  = $hash{"footer"};
    $sheader = $hash{"sheader"};
    $sfooter = $hash{"sfooter"};

    $item_template   = $hash{"item_template"};
    $cat_template    = $hash{"cat_template"};
    $allcat_template = $hash{"allcat_template"};

    $ContinueShoppingURL = $hash{"ContinueShoppingURL"};
    $CompletedOrderURL   = $hash{"CompletedOrderURL"};

    $RestrictCountry  = $hash{"RestrictCountry"};
    $RestrictState    = $hash{"RestrictState"};
    $ShowShipping     = $hash{"ShowShipping"};
    $ShowTax          = $hash{"ShowTax"};
    $WeightIdentifier = $hash{"WeightIdentifier"};
    $ShipByWeight     = $hash{"ShipByWeight"};
    $EnableTaxExempt  = $hash{"EnableTaxExempt"};
    $ShowStock        = $hash{"ShowStock"};
    my @weights = split( /$GiftCertDelimiter/, $hash{"WeightClasses"} );
    my @shiptotals = split( /$GiftCertDelimiter/, $hash{"ShipTotalClasses"} );

    foreach (@weights) {
        my @tmp = split( /$CookieSectionDelimiter/, $_ );
        my $level = shift(@tmp);
        $WeightClasses{$level} = \@tmp;
    }
    
    foreach (@shiptotals) {
        my @tmp = split( /$CookieSectionDelimiter/, $_ );
        my $level = shift(@tmp);
        $ShipTotalClasses{$level} = \@tmp;
    }

    $IMAGEPATH  = $hash{"IMAGEPATH"};
    $SIMAGEPATH = $hash{"SIMAGEPATH"};
    $IMAGEURL   = $hash{"IMAGEURL"};

    $FinalMessage = $hash{"FinalMessage"};
    $EmailFooter  = $hash{"EmailFooter"};

    # top color and main color for tables
    $TableWidth            = $hash{"TableWidth"};
    $table_border          = $hash{"table_border"};
    $table_borderspacing   = $hash{"table_borderspacing"};
    $table_backgroundimage = $hash{"table_backgroundimage"};

    %sql_tables = (
        admin            => $hash{"admintable"},
        item             => $hash{"itemtable"},
        order            => $hash{"ordertable"},
        user             => $hash{"usertable"},
        gc               => $hash{"gctable"},
        cat              => $hash{"cattable"},
        ebay             => $hash{"ebaytable"},
        affiliates       => $hash{"affiliatetable"},
        affiliate_sales  => $hash{"affiliate_salestable"},
        attributes       => $hash{"attributetable"},
        attribute_values => $hash{"attribute_valuestable"},
        groups           => $hash{"groupstable"},
        maxmind          => $hash{"maxmindtable"},
    );

    %KnownCards = (
        paypal     => "PayPal",
        mastercard => "Master Card",
        amex       => "American Express",
        discover   => "Discover",
        visa       => "Visa",
        diner      => "Diner's Club",
        check      => "Check",
        cod        => "Cash on Delivery"
    );

    my $comma = ",";
    my @cards = ();
    @cards = split( /$comma/, $hash{"creditcard"} ) if ( $hash{"creditcard"} );
    foreach ( sort @cards ) {
        if ( $KnownCards{"$_"} ) {
            $AcceptedCards{"$_"} = $KnownCards{"$_"};
        }
    }

    %FileNames = (
        item  => "$hash{FileName_item}",
        admin => "$hash{FileName_admin}",
        cart  => "$hash{FileName_cart}",
        user  => "$hash{FileName_user}"
    );

    $SecureImageURL = $hash{"SecureImageURL"};

    $mailprog   = $hash{"mailprog"};
    $emailflags = $hash{"emailflags"};

    $dbh = Connect();

    if ( $sql_tables{"cat"} && !$_[1] ) {
        my $SQL = qq|select code,name from $sql_tables{"cat"} order by code|;
        my $sth = DatabaseQuery($SQL);
        while ( my ( $c, $n ) = $sth->fetchrow_array() ) {
            $categories{"$c"} = $n;
        }
    }

    LoadLanguage( $_[1] );

}

################################################################################
# FUNCTION:  Cleanup
# DESCRIPTION: To be called when exiting
################################################################################
# MODIFICATION HISTORY
# 2000-11-02   cbf    added comments,
################################################################################
sub closedShop::Cleanup {
    if ($dbh) {
        $dbh->disconnect();
    }
    exit(0);
}

################################################################################
# FUNCTION:  LoadLanguage
# DESCRIPTION: loads languages
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::LoadLanguage {
    my $lang = "English";
    if ( $_[0] != 1 ) {
        $lang = closedShop::GetLangCookie();
    }

    if ( !$lang ) {
        $lang = $closedShop::Language;
        if ( !$lang ) {
            $lang = "English";
        }
    }

    my $file = $closedShop::Languages{$lang};
    open( FILE, "$file" );
    my @data = <FILE>;
    close(FILE);
    foreach (@data) {
        chomp($_);
        my @tmp = split( /\|/, $_ );
        $closedShop::TEXT{ $tmp[0] } = $tmp[1];
    }
}

###############################################################################
# FUNCTION:  ImageDecode
# DESCRIPTION: find info about images
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub closedShop::ImageDecode {
    my ($filename) = @_;

    # first try it as a gif
    # courtesy of
    # http://www.glennf.com/writing/gifdecode.html
    open( GIF, $filename ) || return ( undef, undef );
    my $buf = '';
    my $n = read GIF, $buf, 10;
    close GIF;

    if ( $n < 10 ) {
        my ( $head, $width, $height ) = unpack( "A6vv", $buf );
        if ( $head =~ /^GIF8[79]a/ ) {
            return ( $width, $height );
        }
    }

    # now try as jpeg courtesy of
    # http://forums.invisionpower.com/lofiversion/index.php/t12693.html
    my ( $buffer, $id, $length );
    open( JPG_FILE, '<' . $filename ) || return ( undef, undef );
    binmode JPG_FILE;
    sysread( JPG_FILE, $buffer, 2 );
    unless ( $buffer eq "\xFF\xD8" ) {
        return ( undef, undef );
    }
    while ( sysread( JPG_FILE, $buffer, 4 ) == 4 ) {
        ( $id, $length ) = unpack( "nn", $buffer );
        if ( ( $id == 0xFFC2 ) || ( $id == 0xFFC0 ) ) {
            sysread( JPG_FILE, $buffer, 5 );
            close(JPG_FILE);
            return ( unpack( "cnn", $buffer ) )[ 2, 1 ];
        }
        else {
            sysread( JPG_FILE, $buffer, $length - 2 );
        }
    }
    close(JPG_FILE);
    return ( undef, undef );
}

################################################################################
# FUNCTION:  PrintCategoryLast
# DESCRIPTION: prints category list
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::PrintCategoryList {
    my ( $sname, $any, $stat, $what, $chosen ) = @_;
    if ( $what ne "list" ) { print qq|<select name="$sname">|; }
    else { print qq|<table  cellspacing='1' cellpadding='1' width='100%'>|; }
    if ($any)    { print qq|<option value="">$closedShop::TEXT{'ANYCAT'}</option>\n  |; }
    if ($chosen) { print qq|<option value="$chosen">$closedShop::categories{$chosen}</option>\n|; }

    foreach ( sort keys %closedShop::categories ) {
        my ( $intro, $style, $numimages, $status, $name, $children, $parent ) = closedShop::GetCategoryInfo($_);
        if ($parent) { next; }
        if ( $status ne "PRIVATE" || $stat eq "ALL" ) {
            if ( $what eq "list" ) {
                my $search = $closedShop::FileNames{"item"};
                if ( $search !~ /http/ ) {
                    $search = $closedShop::urltofolder . "/" . $closedShop::FileNames{"item"};
                }
                print qq|<tr class='mainrow'><td><font class='main'><a href="$search?action=ShowCategory&category=$_">$name</a></td></tr>\n|;
            }
            else {
                print qq|<option value="$_">$closedShop::categories{"$_"}</option>\n|;
            }
            DoChildren( $children, 2, $what );
        }
    }
    if ( $what eq "list" ) { print qq|</table>|; }
    else { print qq| </select> |; }
}

################################################################################
# FUNCTION:  DoChildren
# DESCRIPTION: recursive function to print child categories
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::DoChildren {
    my ( $children, $level, $what ) = @_;
    my $space  = "&nbsp;" x $level;
    my $space2 = "&nbsp;&nbsp;" x $level;

    if ($children) {
        my @child = split( /~/, $children );
        foreach (@child) {
            if ( !$_ ) { next; }
            my $SQL = qq|select name,code,children from $closedShop::sql_tables{"cat"} where id="$_"|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $nm, $cd, $ch2 ) = $sth->fetchrow_array();
            if ($cd) {
                if ( $what eq "list" ) {
                    my $search = $closedShop::FileNames{"item"};
                    if ( $search !~ /http/ ) {
                        $search = $closedShop::urltofolder . "/" . $closedShop::FileNames{"item"};
                    }
                    print qq|<tr class='mainrow'><td><font class='main'>$space2\|-<a href="$search?action=ShowCategory&category=$cd">$nm</a></td></tr>|;
                }
                else {
                    print qq|<option value="$cd">$space\|-$nm</option>\n |;
                }
            }
            if ($ch2) { DoChildren( $ch2, $level + 2, $what ); }
        }
    }
}

################################################################################
# FUNCTION:  GetNewCategoryCode
# DESCRIPTION: gets anew category code
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::GetNewCategoryCode {
    my $catname = shift;
    my $short   = lc($catname);
    $short =~ tr/a-zA-Z0-9//cd;
    my $testname = 1;
    while ($testname) {
        my $r = rand(999) % 100;
        $short .= $r;
        my $SQL = qq|select name from $sql_tables{"cat"} where code="$temp"|;
        my $sth = DatabaseQuery($SQL);
        ($testname) = $sth->fetchrow_array();
    }
    return $short;
}

################################################################################
# FUNCTION:  PrintEbaySubCat
# DESCRIPTION: create admin
################################################################################
# MODIFICATION HISTORY
# 2001-07-25   cbf    created
################################################################################
sub closedShop::PrintEbaySubCat {
    my ( $name, $id, $parent ) = @_;

    my $SQL = qq|select name,id from $closedShop::sql_tables{"ebay"}
	where parent="$parent"|;
    my $sth = closedShop::DatabaseQuery($SQL);

    if ( $sth->rows > 0 ) {
        my $onChange = qq|onChange="MakeNewList()"|;
        if ( $name eq "sub5" ) { $onChange = ""; }
        print qq|<select name="$name" $onChange>|;
        print qq|<option value="">Select Category</option>|;
        while ( my ( $n, $i ) = $sth->fetchrow_array() ) {
            my $s = "";
            if ( $i == $id ) { $s = "selected"; }
            print qq|<option value="$i" $s>$n</option>\n|;
        }
        print qq|</select>|;
    }
    else {
        print qq|No Sub Category
	<input type="hidden" name="$name" value="">|;
    }

}

################################################################################
# FUNCTION:  PrintEbayCategoryMain
# DESCRIPTION: create admin
################################################################################
# MODIFICATION HISTORY
# 2001-07-25   cbf    created
################################################################################
sub closedShop::PrintEbayCategoryMain {
    print qq|
	<script language="JavaScript">
	<!--
	var newwin;
	function changesub() {
	 newwin = window.open('$closedShop::FileNames{admin}?action=PrintEbaySubCategory&main='+document.stockForm.Category1Option.value,'SubCategory','width=350,height=300,scrollbars=yes,resizable=yes');
	}
	-->
	</script>

	<select name="Category1Option" onChange="changesub()">
	<option value="">$closedShop::TEXT{'SELECT'} Main $closedShop::TEXT{'CAT'}</option>
	|;

    my $SQL = qq|select id,name from $closedShop::sql_tables{"ebay"}
		where parent=0|;
    my $sth = closedShop::DatabaseQuery($SQL);

    while ( my ( $id, $name ) = $sth->fetchrow_array() ) {
        print qq|<option value="$id">$name</option>\n|;
    }

    print qq|
        </select>
	|;
}

################################################################################
# FUNCTION:  LoadEbayCountries
# DESCRIPTION: return ebay country code
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::LoadEbayCountries {

    my %ebayhash = (
        "United States of America"  => 1,
        "Canada"                    => 2,
        "United Kingdom"            => 3,
        "Afghanistan"               => 4,
        "Albania"                   => 5,
        "Algeria"                   => 6,
        "American Samoa"            => 7,
        "Andorra"                   => 8,
        "Angola"                    => 9,
        "Anguilla"                  => 10,
        "Antigua and Barbuda"       => 11,
        "Argentina"                 => 12,
        "Armenia"                   => 13,
        "Aruba"                     => 14,
        "Australia"                 => 15,
        "Austria"                   => 16,
        "Azerbaijan Republic"       => 17,
        "Bahamas"                   => 18,
        "Bahrain"                   => 19,
        "Bangladesh"                => 20,
        "Barbados"                  => 21,
        "Belarus"                   => 22,
        "Belgium"                   => 23,
        "Belize"                    => 24,
        "Benin"                     => 25,
        "Bermuda"                   => 26,
        "Bhutan"                    => 27,
        "Bolivia"                   => 28,
        "Bosnia and Herzegovina"    => 29,
        "Botswana"                  => 30,
        "Brazil"                    => 31,
        "British Virgin Islands"    => 32,
        "Brunei Darussalam"         => 33,
        "Bulgaria"                  => 34,
        "Burkina Faso"              => 35,
        "Burma"                     => 36,
        "Burundi"                   => 37,
        "Cambodia"                  => 38,
        "Cameroon"                  => 39,
        "Cape Verde Islands"        => 40,
        "Cayman Islands"            => 41,
        "Central African Republic"  => 42,
        "Chad"                      => 43,
        "Chile"                     => 44,
        "China"                     => 45,
        "Colombia"                  => 46,
        "Comoros"                   => 47,
        "Congo, Democratic "        => 48,
        "Congo, Republic "          => 49,
        "Cook Islands"              => 50,
        "Costa Rica"                => 51,
        "Cote d Ivoire "            => 52,
        "Croatia, Republic of"      => 53,
        "Cyprus"                    => 55,
        "Czech Republic"            => 56,
        "Denmark"                   => 57,
        "Djibouti"                  => 58,
        "Dominica"                  => 59,
        "Dominican Republic"        => 60,
        "Ecuador"                   => 61,
        "Egypt"                     => 62,
        "El Salvador"               => 63,
        "Equatorial Guinea"         => 64,
        "Eritrea"                   => 65,
        "Estonia"                   => 66,
        "Ethiopia"                  => 67,
        "Falkland Islands"          => 68,
        "Fiji"                      => 69,
        "Finland"                   => 70,
        "France"                    => 71,
        "French Guiana"             => 72,
        "French Polynesia"          => 73,
        "Gabon Republic"            => 74,
        "Gambia"                    => 75,
        "Georgia"                   => 76,
        "Germany"                   => 77,
        "Ghana"                     => 78,
        "Gibraltar"                 => 79,
        "Greece"                    => 80,
        "Greenland"                 => 81,
        "Grenada"                   => 82,
        "Guadeloupe"                => 83,
        "Guam"                      => 84,
        "Guatemala"                 => 85,
        "Guernsey"                  => 86,
        "Guinea"                    => 87,
        "Guinea-Bissau"             => 88,
        "Guyana"                    => 89,
        "Haiti"                     => 90,
        "Honduras"                  => 91,
        "Hong Kong"                 => 92,
        "Hungary"                   => 93,
        "Iceland"                   => 94,
        "India"                     => 95,
        "Indonesia"                 => 96,
        "Iran"                      => 97,
        "Ireland"                   => 99,
        "Israel"                    => 100,
        "Italy"                     => 101,
        "Jamaica"                   => 102,
        "Jan Mayen"                 => 103,
        "Japan"                     => 104,
        "Jersey"                    => 105,
        "Jordan"                    => 106,
        "Kazakhstan"                => 107,
        "Kenya Coast Republic"      => 108,
        "Kiribati"                  => 109,
        "Korea South"               => 111,
        "Kuwait"                    => 112,
        "Kyrgyzstan"                => 113,
        "Laos"                      => 114,
        "Latvia"                    => 115,
        "Lebanon"                   => 116,
        "Liechtenstein"             => 120,
        "Lithuania"                 => 121,
        "Luxembourg"                => 122,
        "Macau"                     => 123,
        "Macedonia"                 => 124,
        "Madagascar"                => 125,
        "Malawi"                    => 126,
        "Malaysia"                  => 127,
        "Maldives"                  => 128,
        "Mali"                      => 129,
        "Malta"                     => 130,
        "Marshall Islands"          => 131,
        "Martinique"                => 132,
        "Mauritania"                => 133,
        "Mauritius"                 => 134,
        "Mayotte"                   => 135,
        "Mexico"                    => 136,
        "Micronesia"                => 226,
        "Moldova"                   => 137,
        "Monaco"                    => 138,
        "Mongolia"                  => 139,
        "Montserrat"                => 140,
        "Morocco"                   => 141,
        "Mozambique"                => 142,
        "Namibia"                   => 143,
        "Nauru"                     => 144,
        "Nepal"                     => 145,
        "Netherlands"               => 146,
        "Netherlands Antilles"      => 147,
        "New Caledonia"             => 148,
        "New Zealand"               => 149,
        "Nicaragua"                 => 150,
        "Niger"                     => 151,
        "Nigeria"                   => 152,
        "Niue"                      => 153,
        "Norway"                    => 154,
        "Oman"                      => 155,
        "Pakistan"                  => 156,
        "Palau"                     => 157,
        "Panama"                    => 158,
        "Papua New Guinea"          => 159,
        "Paraguay"                  => 160,
        "Peru"                      => 161,
        "Philippines"               => 162,
        "Poland"                    => 163,
        "Portugal"                  => 164,
        "Puerto Rico"               => 165,
        "Qatar"                     => 166,
        "Romania"                   => 167,
        "Russian Federation"        => 168,
        "Rwanda"                    => 169,
        "Saint Helena"              => 170,
        "Saint Kitts-Nevis"         => 171,
        "Saint Lucia"               => 172,
        "Saint Pierre and Miquelon" => 173,
        "Saint Vincent "            => 174,
        "San Marino"                => 175,
        "Saudi Arabia"              => 176,
        "Senegal"                   => 177,
        "Seychelles"                => 178,
        "Sierra Leone"              => 179,
        "Singapore"                 => 180,
        "Slovakia"                  => 181,
        "Slovenia"                  => 182,
        "Solomon Islands"           => 183,
        "Somalia"                   => 184,
        "South Africa"              => 185,
        "Spain"                     => 186,
        "Sri Lanka"                 => 187,
        "Suriname"                  => 189,
        "Svalbard"                  => 190,
        "Swaziland"                 => 191,
        "Sweden"                    => 192,
        "Switzerland"               => 193,
        "Syria"                     => 194,
        "Tahiti"                    => 195,
        "Taiwan"                    => 196,
        "Tajikistan"                => 197,
        "Tanzania"                  => 198,
        "Thailand"                  => 199,
        "Togo"                      => 200,
        "Tonga"                     => 201,
        "Trinidad and Tobago"       => 202,
        "Tunisia"                   => 203,
        "Turkey"                    => 204,
        "Turkmenistan"              => 205,
        "Turks and Caicos Islands"  => 206,
        "Tuvalu"                    => 207,
        "Uganda"                    => 208,
        "Ukraine"                   => 209,
        "United Arab Emirates"      => 210,
        "Uruguay"                   => 211,
        "Uzbekistan"                => 212,
        "Vanuatu"                   => 213,
        "Vatican City State"        => 214,
        "Venezuela"                 => 215,
        "Vietnam"                   => 216,
        "Virgin Islands (U.S.)"     => 217,
        "Wallis and Futuna"         => 218,
        "Western Sahara"            => 219,
        "Western Samoa"             => 220,
        "Yemen"                     => 221,
        "Yugoslavia"                => 222,
        "Zambia"                    => 223,
        "Zimbabwe"                  => 224,
        "Reunion"                   => 227
    );
    return %ebayhash;
}

################################################################################
# FUNCTION:  ReturnEbayCountryCode
# DESCRIPTION: return ebay country code
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::ReturnEbayCountryCode {
    my ($country) = @_;
    my %ebayhash = LoadEbayCountries();
    return $ebayhash{$country};
}

################################################################################
# FUNCTION:  GetCategoryInfo
# DESCRIPTION: creates an image
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::GetCategoryInfo {
    my $category = shift;

    my $SQL = qq|select intro,style,numimages,status,name,children,parent 
	  from $sql_tables{"cat"} where code="$category"|;
    my $sth = DatabaseQuery($SQL);
    my ( $intro, $style, $numimages, $status, $name, $children, $parent ) = $sth->fetchrow_array();

    if ($name) {
        return ( $intro, $style, $numimages, $status, $name, $children, $parent );
    }
    else {
        return ( "", "", "", "NOCAT", "", "", "" );
    }
}

################################################################################
# FUNCTION:  GetGiftCertificateCode
# DESCRIPTION: gets new code for a gift cert
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::GetGiftCertificateCode {
    my $size  = 20;
    my $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    my @array = split( //, $chars );
    my @dots  = ();
    while ( $size-- ) {
        push @dots, $array[ rand(@array) ];
    }
    return join( "", @dots );
}

################################################################################
# FUNCTION:  GetGiftCertificateInfo
# DESCRIPTION: creates an image
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::GetGiftCertificateInfo {
    open( FILE, "$closedShop::varsfile" )
      || closedShop::ErrorMessage("Can't open file $closedShop::varsfile");
    my @data = <FILE>;
    close(FILE);

    my @denoms = ();
    foreach (@data) {
        my @ar = split( /$closedShop::VarsFileDelimiter/, $_ );
        if ( $ar[0] eq "GiftCertificates" ) {
            chomp( $ar[1] );
            @denoms = split( /$closedShop::GiftCertDelimiter/, $ar[1] );
        }
    }

    return @denoms;
}

################################################################################
# FUNCTION:  MonthName
# DESCRIPTION: returns month name
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::MonthName {
    my $m = shift;
    if    ( $m == 1 )  { return "Jan"; }
    elsif ( $m == 2 )  { return "Feb"; }
    elsif ( $m == 3 )  { return "Mar"; }
    elsif ( $m == 4 )  { return "Apr"; }
    elsif ( $m == 5 )  { return "May"; }
    elsif ( $m == 6 )  { return "Jun"; }
    elsif ( $m == 7 )  { return "Jul"; }
    elsif ( $m == 8 )  { return "Aug"; }
    elsif ( $m == 9 )  { return "Sep"; }
    elsif ( $m == 10 ) { return "Oct"; }
    elsif ( $m == 11 ) { return "Nov"; }
    elsif ( $m == 12 ) { return "Dec"; }

}

################################################################################
# FUNCTION:  GetLangCookie
# DESCRIPTION: gets info in cookies
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::GetLangCookie {
    return closedShop::GetCookie("cs_language");
}

################################################################################
# FUNCTION:  GetCookieData
# DESCRIPTION: gets info in cookies
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::GetSessionCookie {

    my $session = closedShop::GetCookie("session");
    my $id      = closedShop::GetCookie("id");

    return ( $session, $id );
}

################################################################################
# FUNCTION:  CreateSession
# DESCRIPTION: gets info in cookies
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::CreateSession {
    my $size  = int( rand(256) + 255 );
    my $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    my @array = split( //, $chars );
    my @dots  = ();
    while ( $size-- ) {
        push @dots, $array[ rand(@array) ];
    }

    my $line = "";
    foreach (@dots) {
        if ( int( rand( time ^ $$ ) ) % 2 == 0 ) {
            $line .= $_;
        }
    }

    my $add = "";
    for ( my $k = 0 ; $k < 40 ; $k++ ) {
        my $n1 = int( rand( length($line) ) );
        my $sub = substr( $line, $n1, 1 );
        $add .= "$sub$n1";
    }

    my $session = int( rand(99999999) ) . crypt( time * rand(999), "YT" ) . $add . crypt( rand(730300334) * rand(31443), "EP" );
    $session =~ tr/a-zA-Z0-9//cd;
    return $session;
}

################################################################################
# FUNCTION:  Authenticate
# DESCRIPTION: authenticates the user
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::Authenticate {
    my ( $session, $id, $table ) = @_;
    my $SQL = qq| select id from $sql_tables{"$table"} where session = "$session" 
    				and id = '$id'|;
    my $sth = DatabaseQuery($SQL);
    my ($t_session) = $sth->fetchrow_array();

    if ( !$t_session ) {
        return 0;
    }
    else {
        return 1;
    }
}

################################################################################
# FUNCTION:  LogoffAdmin
# DESCRIPTION: logouts user
################################################################################
# MODIFICATION HISTORY
# 2005-04-04   cbf    added comments
################################################################################
sub closedShop::LogoffAdmin {
    my $id          = shift;
    my $SQL         = qq| update $sql_tables{"admin"} set session = "" |;
    my $sth         = DatabaseQuery($SQL);
    my ($t_session) = $sth->fetchrow_array();

    use CGI qw/:standard/;
    use CGI::Cookie;
    my $cookie1 = new CGI::Cookie( -name => 'session', -value => '' );
    my $cookie2 = new CGI::Cookie( -name => 'id',      -value => '' );

    print header( -Cookie => [ $cookie1, $cookie2 ], -type => "text/html" );

}

################################################################################
# FUNCTION:  GetAffiliateID
# DESCRIPTION: gets the affiliate id
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::GetAffiliateID {
    return closedShop::GetCookie("affiliate_id");
}

################################################################################
# FUNCTION:  SetSessionCookie
# DESCRIPTION: sets session of admin with a cookie
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments
################################################################################
sub closedShop::SetSessionCookie {
    my ( $session, $id, $location ) = @_;
    use CGI qw/:standard/;
    use CGI::Cookie;
    my $cookie1 = new CGI::Cookie( -name => 'session', -value => $session );
    my $cookie2 = new CGI::Cookie( -name => 'id',      -value => $id );
    closedShop::SaveCookie( "id",      "$id" );
    closedShop::SaveCookie( "session", "$session" );

    if ($location) {
        print header(
            -Cookie  => [ $cookie1, $cookie2 ],
            -type    => "text/html",
            location => "$location"
        );
    }
    else {
        print header( -Cookie => [ $cookie1, $cookie2 ], -type => "text/html" );
    }
}

################################################################################
# FUNCTION:  ValidateInputAdmin
# DESCRIPTION:  Anything that we read in from a $query->param should be checked
#               to make sure they only contain word characters. This here will
#               only accept things that are safe for databases using
#               the dbi->quote function
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments,
################################################################################
sub closedShop::ValidateInputAdmin {
    my $input = shift;

    # remove unsafe database characters
    $input = ConvertChars($input);

    # $input = $dbh->quote($input);

    if ( !defined($input) || $input eq "NULL" ) { return ""; }

    if ( length($input) > 3000 ) { $input = substr( $input, 0, 3000 ); }
    return $input;
}

################################################################################
# FUNCTION:  ConvertChars
# DESCRIPTION:  changes chars to html codes
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments,
################################################################################
sub ConvertChars {
    my $input = shift;

    my $apos = "&#39;";
    $input =~ s/'/$apos/g;

    my $quot = "&#34;";
    $input =~ s/"/$quot/g;

    my $caret = "&#95;";
    $input =~ s/\^/$caret/g;

    my $backslash = "&#92;";
    $input =~ s/\\/$backslash/g;

    return $input;
}

################################################################################
# FUNCTION:  ValidateInputCustomer
# DESCRIPTION:  Anything that we read in from a $query->param should be checked
#               to make sure they only contain word characters. This here will
#               only accept things that are safe for databases using
#               the dbi->quote function, customer removes tags so
#               they can't insert HTML or javascript into things
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments,
################################################################################
sub closedShop::ValidateInputCustomer {
    my $input = shift;

    # remove unsafe database characters
    $input = ConvertChars($input);

    # $input = $dbh->quote($input);
    $input =~ s/<[^>.]*>//ig;

    if ( !defined($input) || $input eq "NULL" ) { return ""; }

    if ( length($input) > 2000 ) { $input = substr( $input, 0, 2000 ); }
    return $input;
}

################################################################################
# FUNCTION:  URLEncode
# DESCRIPTION: Footer at bottom of each page
################################################################################
# MODIFICATION HISTORY
# 2000-07-29   cbf    Created
################################################################################
sub closedShop::URLEncode {
    my ($str) = @_;
    $str =~ s/([^A-Za-z0-9])/sprintf("%%%02X", ord($1))/seg;
    return $str;

}
################################################################################
# FUNCTION:  URLDecode
# DESCRIPTION: Footer at bottom of each page
################################################################################
# MODIFICATION HISTORY
# 2000-07-29   cbf    Created
################################################################################
sub closedShop::URLDecode {
    my ($str) = @_;
    $str =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
    return $str;

}

################################################################################
# FUNCTION:  Header
# DESCRIPTION: header at bottom of each page
################################################################################
# MODIFICATION HISTORY
# 2000-07-29   cbf    Created
################################################################################
sub closedShop::Header {

    print $HTML_HEADER unless ( $_[0] eq "nohttp" );
    my $h = $header;

    if ( !( -e $h ) ) { $h = $sheader; }
    if ( ( -e $h ) ) {
        open( FILE, "$h" );
        my @data = <FILE>;
        close(FILE);
        print join( "", @data );
    }

    WriteStyleSheets();
    WriteJavaScript();

    print "<table border='0'  cellpadding='0' cellspacing='0'><tr><td>";
}

################################################################################
# FUNCTION:  CurrencyFormat
# DESCRIPTION: formats currency nicely
# courtesy of: http://www.web-source.net/web_development/currency_formatting.htm
################################################################################
# MODIFICATION HISTORY
# 2000-07-29   cbf    Created
################################################################################
sub closedShop::CurrencyFormat {
    my $delimiter = ',';    # replace comma if desired
    my ( $n, $d ) = split /\./, shift, 2;
    my @a = ();
    while ( $n =~ /\d\d\d\d/ ) {
        $n =~ s/(\d\d\d)$//;
        unshift @a, $1;
    }
    unshift @a, $n;
    $n = join $delimiter, @a;
    $n = "$n\.$d" if $d =~ /\d/;
    return $n;
}

################################################################################
# FUNCTION:  Footer
# DESCRIPTION: Footer at bottom of each page
################################################################################
# MODIFICATION HISTORY
# 2000-07-29   cbf    Created
################################################################################
sub closedShop::Footer {

    my $cart   = closedShop::GetCookie("cart");
    my $userid = closedShop::GetCookie("id");

    my ( $session, $uid ) = closedShop::GetSessionCookie();

    if ( defined($cart) ) {
        my @ary = split( /;/, $cart );
        foreach (@ary) {
            my ( $key, $value ) = split( /=/, $_ );
            if ( $key eq "cart" && !$value ) { $cart = ""; }
        }
    }

    if ( !defined($userid) ) {
        $userid = 0;
    }

    if ( closedShop::Authenticate( $session, $uid, "user" ) ) {
        my $SQL = "select discount from $sql_tables{'groups'},$sql_tables{'user'} where
	  $sql_tables{'user'}.id = '$uid' and $sql_tables{'user'}.groupid = $sql_tables{'groups'}.id";
        my $sth = DatabaseQuery($SQL);
        my ($discount) = $sth->fetchrow_array();

        if ( $discount && $discount > 0 ) {
            print qq|<BR><center>
	  <table style='border: 1px solid gray;' width='50%'><tr><td><center>
	  <font class="main">You are eligible 
	  for a discount of $discount\%</font></td></tr></table></center><br>|;
        }

    }

    if ( $cart && $userid != 1 ) {
        print qq|<BR><font class="main">
	 <center><b><a href="| . $closedShop::FileName{'cart'} . qq|">$closedShop::TEXT{'VIEW_CRT'}</a></b></center>|;
    }

    my $year = ( (localtime)[5] % 100 ) + 2000;

    print qq|
	<form action="$closedShop::FileNames{'user'}" name="Lang" method="POST">
	<input type="hidden" name="action" value="ChangeLanguage"><div align="right">
	<font size="1" style="font-family: $closedShop::fontmain_family;">
	$closedShop::TEXT{'DEF_LANG'}: 
	<select style="border: 1px solid black; font-family: $closedShop::fontmain_family; font-size: 10px;" 
	onChange="javascript:document.Lang.submit();" 
	name="Language">|;
    my $lang = GetLangCookie();
    if ( !$lang ) { $lang = $closedShop::Language; }

    print qq|<option value="$lang">$lang</option>
	<option value="$lang">-----</option>|;
    foreach ( sort keys %closedShop::Languages ) {
        print qq|<option value="$_">$_</option>\n|;
    }
    print qq|</select><BR>
	&copy; 2001-$year
	<a target="_blank" href="http://closedshop.sourceforge.net/">closedShop ($VERSION)</a>
	<br>Open Source Shopping</font></form>
	</div></td></tr></table>|;

    my $f = $footer;
    if ( !( -e $f ) ) { $f = $sfooter; }

    if ( -e $f ) {
        open( FILE, "$f" );
        my @data = <FILE>;
        close(FILE);
        print join( "", @data );
    }
}

################################################################################
# FUNCTION:  WriteStyleSheets
# DESCRIPTION: now there are style sheets
################################################################################
# MODIFICATION HISTORY
# 2000-07-29   cbf    Created
################################################################################
sub closedShop::WriteJavaScript {

    if ( $closedShop::UseTinyMCE eq "Y" ) {

        print qq~
	 <!-- tinyMCE -->
<script language="javascript" type="text/javascript" src="$closedShop::$IMAGEURL/tiny_mce/tiny_mce.js"></script>

<script language="javascript" type="text/javascript">
     	tinyMCE.init({
		mode : "specific_textareas",
		theme : "$closedShop::TinyMCEFormat"
	});
   </script>
       <!-- /tinyMCE -->
       ~;
    }

    print qq~
<script language="JavaScript" type="text/javascript">
<!--

function showDiv(id) {
    dom = document.getElementById ? 1 : 0;
    ie4 = (document.all && !dom) ? 1 : 0;
    opera = (navigator.userAgent.indexOf('Opera')!=-1) ? 1 : 0;
    supported = ((dom||ie4) && !(opera)) ? 1 : 0;

    if(!supported) return;

    if (dom && !document.all)
     document.all = document.getElementsByTagName("*")

        obj = (dom) ? document.getElementById(id) : document.all(id)

        if(obj.style.visibility=='hidden')
        obj.style.visibility='visible';

        else
        obj.style.visibility='hidden';
}


var browserType;

if (document.layers) {browserType = "nn4"}
if (document.all) {browserType = "ie"}
if (window.navigator.userAgent.toLowerCase().match("gecko")) {browserType= "gecko"}

function hide(id) {
  if (browserType == "gecko" )
     document.poppedLayer = eval('document.getElementById(id)');
  else if (browserType == "ie")
     document.poppedLayer = eval('document.all[id]');
  else
     document.poppedLayer = eval('document.layers[id]');
  document.poppedLayer.style.visibility = "hidden";
    document.poppedLayer.style.display = "none";
}

function toggle(id) {
	if (browserType == "gecko" )
    document.poppedLayer = eval('document.getElementById(id)');
  else if (browserType == "ie")
     document.poppedLayer = eval('document.all[id]');
  else
     document.poppedLayer = eval('document.layers[id]');
     
     if (document.poppedLayer.style.visibility == "visible") {
      document.poppedLayer.style.visibility = "hidden";
	  document.poppedLayer.style.display = "none";
     }
     else {
	  document.poppedLayer.style.visibility = "visible";
	  document.poppedLayer.style.display = "block";
	}
}

function show(id) {
  if (browserType == "gecko" )
    document.poppedLayer = eval('document.getElementById(id)');
  else if (browserType == "ie")
     document.poppedLayer = eval('document.all[id]');
  else
     document.poppedLayer = eval('document.layers[id]');
  document.poppedLayer.style.visibility = "visible";
  document.poppedLayer.style.display = "block";
}

function Confirm(url) {

  var answer = confirm("Are you sure you wanted to delete?");
  if (answer != 0) {
    return true;
  }
  else {
   return false;
  }
}	
	 
function Sure(url) {
  var answer = confirm("Are you sure you wanted to delete?");
  if (answer != 0) {
    location.href = url;
    return true;
  }
  else {
   return false;
  }
}
  
function open_window(url,title,x,y) {
    newwin = window.open(url,title,'width=' + x + ',height=' + y + ',scrollbars=yes,resizable=yes');
}	
      
-->
</script>

	 
	 ~;
}

################################################################################
# FUNCTION:  WriteStyleSheets
# DESCRIPTION: now there are style sheets
################################################################################
# MODIFICATION HISTORY
# 2000-07-29   cbf    Created
################################################################################
sub closedShop::WriteStyleSheets {
    my $bg = "";
    if ($table_backgroundimage) {
        $bg = qq|background-image: url($table_backgroundimage);|;
    }
    print qq|
    <style type="text/css">
    <!--
  
	font.main {
	  font-family: $closedShop::fontmain_family;
	  color: $closedShop::fontmain_color;
    	  font-size: $closedShop::fontmain_size;
    	  font-style: $closedShop::fontmain_style;
   	  font-variant: $closedShop::fontmain_variant;
        }
	
	font.small {
	  font-family: $closedShop::fonthead_family;
	  color: $closedShop::fonthead_color;
          font-size: 10px;
	}
	
  	font.head {
 	  font-family: $closedShop::fonthead_family;
	  color: $closedShop::fonthead_color;
          font-size: $closedShop::fonthead_size;
          font-style: $closedShop::fonthead_style;
          font-variant: $closedShop::fonthead_variant;
        }
        
    font.receipt {
    	font-family: verdana, sans serif;
    	font-size: 11px;
    }
        
	tr.mainrow {
          background-color: $closedShop::mainrow_color;
	  border: $closedShop::table_border;
	  font-family: $closedShop::fontmain_family;
	  color: $closedShop::fontmain_color;
    	  font-size: $closedShop::fontmain_size;
   	  font-style: $closedShop::fontmain_style;
    	  font-variant: $closedShop::fontmain_variant;
        }
	tr.headrow {
          background-color: $closedShop::headrow_color;
	  border: $closedShop::table_border;
	  font-family: $closedShop::fonthead_family;
	  color: $closedShop::fonthead_color;
          font-size: $closedShop::fonthead_size;
          font-style: $closedShop::fonthead_style;
          font-variant: $closedShop::fonthead_variant;
        }
	table.main {
	  border: $closedShop::table_border;
	  border-spacing: $closedShop::table_borderspacing;
	  $bg
	}
	select.small {
	  font-family: verdana;
	  font-size: 10px;
	}
	
	input, textarea, select {
	  border: 1px solid #000000;
	  font-size: $closedShop::fontmain_size;
	  font-family: $closedShop::fonthead_family;
	}
	
	tr.mainrow:hover {
	   background-color: $closedShop::mainrow_hover;
	}
	
	input:hover, textarea:hover {
	   background-color: $closedShop::input_hover;
	}
	
     -->
     </style>

   |;

}

################################################################################
# FUNCTION:  ListStates
# DESCRIPTION: list states/provinces to choose
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments, uses variables set at the top
################################################################################
sub closedShop::ListStates {
    my ( $varname, $initial ) = @_;
    print qq|
  <select name="$varname">
  |;

    if ($initial) {
        print qq|<option value="$initial">$initial</option>|;
    }

    print qq|
    <OPTION  VALUE="NONE">Not Applicable</option>
   		<option>Alabama</option>
			<option>Alaska</option>
			<option>Arizona</option>
			<option>Arkansas</option>
			<option>California</option>
			<option>Colorado</option>
			<option>Connecticut</option>
			<option>DC</option>
			<option>Delaware</option>
			<option>Florida</option>
			<option>Georgia</option>
			<option>Hawaii</option>
			<option>Idaho</option>
			<option>Illinois</option>
			<option>Indiana</option>
			<option>Iowa</option>
			<option>Kansas</option>
			<option>Kentucky</option>
			<option>Louisiana</option>
			<option>Maine</option> 
			<option>Maryland</option> 
			<option>Massachusetts</option> 
			<option>Michigan</option>
			<option>Minnesota</option> 
			<option>Missouri</option> 
			<option>Mississippi</option> 
			<option>Montana</option> 
			<option>Nebraska</option> 
			<option>Nevada</option> 
			<option>New Hampshire</option>
			<option>New Jersey</option> 
			<option>New Mexico</option> 
			<option>New York</option> 
			<option>North Carolina</option>
			<option>North Dakota</option> 
			<option>Ohio</option> 
			<option>Oklahoma</option> 
			<option>Oregon</option>
			<option>Pennsylvania</option> 
			<option>Rhode Island</option> 
			<option>South Carolina</option>
			<option>South Dakota</option> 
			<option>Tennessee</option> 
			<option>Texas</option> 
			<option>Utah</option>
			<option>Vermont</option> 
			<option>Virginia</option> 
			<option>Washington</option>
			<option>West Virginia</option> 
			<option>Wisconsin</option> 
			<option>Wyoming</option>
	   <OPTION>Alberta</option>
      <OPTION>British Columbia</option>
      <OPTION>Manitoba</option>
      <OPTION>New Brunswick</option>
      <OPTION>Newfoundland</option>
      <OPTION>NW Terr. / Nunavut</option>
      <OPTION>Nova Scotia</option>
     <OPTION>Ontario</option>
    <OPTION>Prince Edward Island</option>
      <OPTION>Quebec</option>
      <OPTION>Saskatchewan</option>
     <OPTION>Yukon Territory</option>
  </select>
  |;

}

################################################################################
# FUNCTION:  ListCountries
# DESCRIPTION: list countries to choose
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments, uses variables set at the top
################################################################################
sub closedShop::ListCountries {
    my ( $varname, $initial, $restrict ) = @_;
    print qq|
  <select name="$varname">
  |;

    if ($initial) {
        print qq|<option value="$initial">$initial</option>|;
    }

    print qq|<option value=''>Please Select</option>|;

    my %countries = closedShop::LoadEbayCountries();

    if ( $restrict eq "restrict" && $closedShop::RestrictCountry eq "Y" ) {
        foreach ( sort keys %countries ) {
            if ( $closedShop::CompanyCountry eq $_ ) {
                print qq|<option value='$_'>$_</option>\n|;
                break;
            }
        }
    }
    else {
        foreach ( sort keys %countries ) {
            print qq|<option value='$_'>$_</option>\n|;
        }
    }

    print qq|</select>|;
}

################################################################################
# FUNCTION:  AddItemToWishList
# DESCRIPTION: adds items ot a wishlist
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments, uses variables set at the top
################################################################################
sub closedShop::AddItemToWishList {
    my ( $uid, $id ) = @_;
    my $SQL    = qq|select wishlist_ids from $closedShop::sql_tables{"user"} where id="$uid"|;
    my $sth    = closedShop::DatabaseQuery($SQL);
    my ($wish) = $sth->fetchrow_array();

    if ($id) {
        if ( $wish !~ /$id/ || !$wish ) {
            $wish .= $id . qq|:|;
        }
        else {
            $wish = $wish;
        }
    }

    $SQL = qq|update $closedShop::sql_tables{"user"}
	set wishlist_ids = "$wish" where id="$uid"|;

    $sth = closedShop::DatabaseQuery($SQL);
}

################################################################################
# FUNCTION:  Connect
# DESCRIPTION: Connect to the mySQL database
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments, uses variables set at the top
################################################################################
sub closedShop::Connect {
    $dbh = DBI->connect( $dsn, $DatabaseUsername, $DatabasePassword )
      || ErrorMessage(
        "$closedShop::TEXT{'MYSQL_ERR'}
		   $dsn $DatabaseUsername"
      );
    return $dbh;
}

################################################################################
# FUNCTION:  ErrorMessage
# DESCRIPTION: When something fails should print the error message that is
#              passed and we also output to a log. Then we call Cleanup and
#              exit the program
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    added comments, outputs to a log
################################################################################
sub closedShop::ErrorMessage {
    Header();
    print qq|$closedShop::font
	<B><BR><font color="red">$closedShop::TEXT{'ERR'}</b></font>
	         <BR><Br>$_[0] <BR>$DBI::err $DBI::errstr<BR><BR>
		<a href="javascript:history.go(-1);">&lt;--- $closedShop::TEXT{'GBACK'}</a>|;
    Cleanup();
    exit(0);
}

################################################################################
# FUNCTION:  DatabaseQuery
# DESCRIPTION:  Allows for a query to the database, for a general query
#               If they didn't send a where clause, we have to ignore it
#               Returns a sth handle so that the user can define what the sth
#               fetchrow_array should return
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    added comments,
################################################################################
sub closedShop::DatabaseQuery {
    my $SQL = shift;
    if ( !$dbh ) { return; }
    my $sth = $dbh->prepare($SQL) || ErrorMessage($SQL);
    $sth->execute() || ErrorMessage($SQL);
    return $sth;
}

################################################################################
# FUNCTION:  SendEmail
# DESCRIPTION: send email to everyone
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub closedShop::SendEmail {
    my ( $toemail, $subject, $body ) = @_;
    if ( !( CheckEmail($toemail) ) ) { return; }
    open( MAIL, "|$mailprog $emailflags" ) || ErrorMessage("Can't open $mailprog: $!");

    print MAIL "To: $toemail\n";
    print MAIL "Reply-to: $CompanyEmail\n";
    print MAIL "From: $CompanyEmail\n";

    if (   $body =~ /</ && $body =~ />/
        || $closedShop::EmailFooter =~ /</ && $closedShop::EmailFooter =~ />/ )
    {
        print MAIL "Mime-Version: 1.0\n";
        print MAIL "Content-type: text/html; charset=\"iso-8859-1\"\n";

    }

    print MAIL "Subject: $subject\n\n";
    print MAIL "\n$body\n\n";
    if ($closedShop::EmailFooter) {
        print MAIL "$closedShop::EmailFooter\n\n";
    }
    close(MAIL);
}

################################################################################
# FUNCTION:  CheckEmail
# DESCRIPTION: makes sure email is pretty much all correct
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub CheckEmail {
    my $email = shift;
    if (   $email =~ /(@.*@)|(\.\.)|(@\.)|(\.@)|(^\.)/
        || $email !~ /^.+\@(\[?)[a-zA-Z0-9\-\.]+\.([a-zA-Z]{2,3}|[0-9]{1,3})(\]?)$/ )
    {
        return 0;
    }
    else {
        return 1;
    }
}

################################################################################
# FUNCTION:  GetCookie
# DESCRIPTION: generic get cookie
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub GetCookie {
    my $key = shift;

    use CGI qw/:standard/;
    use CGI::Cookie;
    my %cookies = fetch CGI::Cookie;
    my $value   = "";

    if ( defined($cookies{"$key"}) ) {
        $value = closedShop::ValidateInputCustomer( $cookies{"$key"}->value );
    }
    else {
      # insert URL rewriting
      # 
    }

    return $value;
}

################################################################################
# FUNCTION:  SaveCookie
# DESCRIPTION: maps to db
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub SaveCookie {
    my ( $key, $data ) = @_;

	
}

###############################################################################
# FUNCTION:  GetShippingCosts
# DESCRIPTION: gets shipping costs
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub GetShippingCostsItem {
    my ( $ship, $qty, $rship, $weight ) = @_;
    my $shiptotal  = 0;
    my $shipstring = "";
    if ( $closedShop::ShowShipping eq "ITEM" ) {
        if ( $qty > 1 && $rship && $rship >= 0 ) {
            $shiptotal = sprintf( "%.2f", $ship + ( ( $qty - 1 ) * $rship ) );
            $shipstring = "1 x $closedShop::CurrencyMarker $ship<BR>
           	  &nbsp;&nbsp" . eval( $qty - 1 ) . " x $closedShop::CurrencyMarker" . eval( ( $qty - 1 ) * $rship ) . 
           	  "<BR><b>$closedShop::CurrencyMarker" . $shiptotal;
        }
        else {
            $shiptotal = sprintf( "%.2f", $qty * $ship );
            $shipstring = "$qty x $closedShop::CurrencyMarker$ship";
        }
    }
    elsif ( $closedShop::ShowShipping eq "WEIGHT" ) {
        my $tweight = $qty * $weight;
        $shipstring = $closedShop::TEXT{'WGT'} . " $tweight " . 
           $closedShop::WeightIdentifier;
    }
    elsif ($closedShop::ShowShipping eq "NONE" ) {
        $shipstring = "$closedShop::CurrencyMarker0.00 ";
    }
    return ( $shiptotal, $shipstring );
}

###############################################################################
# FUNCTION:  GetShippingCosts
# DESCRIPTION: gets shipping costs
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub GetShippingCostsClass {
    my ( $weight, $subtotal ) = @_;
    my $shiptotal  = 0;
    my $shipstring = "";
    if ( $closedShop::ShowShipping eq "WEIGHT" ) {
        foreach ( keys %closedShop::WeightClasses ) {
            my ( $sw, $ew, $wp ) = @{ $closedShop::WeightClasses{$_} };
            if ( !defined($sw) || !defined($wp) ) { next; }
            if ( $ew eq "Max"  || $ew eq "" ) { $ew = 3999999999; }
            if ( $weight >= $sw && $weight <= $ew ) {
                $shiptotal  = $wp;
                $shipstring = " $weight $closedShop::WeightIdentifier";
            }
        }
    }
    elsif ( $closedShop::ShowShipping eq "TOTAL" ) {
       
        foreach ( keys %closedShop::ShipTotalClasses ) {
            my ( $sw, $ew, $wp ) = @{ $closedShop::ShipTotalClasses{$_} };
            if ( !defined($sw) || !defined($wp) ) { next; }
            if ( $ew eq "Max" || $ew eq "" ) { $ew = 3999999999; }
            if ( $subtotal >= $sw && $subtotal <= $ew ) {
                $shiptotal  = $wp;
                $shipstring = "";
            }
        }
    }
    elsif ($closedShop::ShowShipping eq "NONE" ) {
        $shipstring = "";
    }
    return ( $shiptotal, $shipstring );
}

###############################################################################
# FUNCTION:  GetTaxCosts
# DESCRIPTION: gets tax costs
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub GetTaxCosts {
    my ( $grand, $taxExemptID, $b_state, $b_country, $staxrate, $ctaxrate, $itaxrate ) = @_;
    my ( $stax, $ctax, $itax ) = ( "0.00", "0.00", "0.00" );

    if ( !defined($staxrate) ) { $staxrate = $closedShop::TaxRate; }
    if ( !defined($itaxrate) ) { $itaxrate = $closedShop::TaxRateInt; }
    if ( !defined($ctaxrate) ) { $ctaxrate = $closedShop::TaxRateCountry; }

    if ( $closedShop::ShowTax eq "N"
        || ( $closedShop::EnableTaxExempt eq "Y" && $taxExemptID != "" ) )
    {
        return ( $stax, $ctax, $itax );
    }

    if ( $closedShop::CompanyState eq $b_state ) {
        $stax = sprintf( "%.2f", $staxrate * $grand );
    }

    if ( $closedShop::CompanyCountry eq $b_country ) {
        $ctax = sprintf( "%.2f", $ctaxrate * $grand );
    }
    else {
        $itax = sprintf( "%.2f", $itaxrate * $grand );
    }

    return ( $stax, $ctax, $itax );

}

1;
