#!/usr/bin/perl

################################################################################
#
# PROGRAM:  Admin.pl - from closedShop - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: Admininstration for shopping cart
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-03   cbf    Created
# 2001-06-07   cbf    Added attributes to products
# 2001-06-09   cbf    Added a field for product id specified by merchant
# 2001-06-11   cbf    Removed image file if file was not uploaded
# 2001-06-20   cbf    Added ability to change program variables
# 2001-06-22   cbf    Added ability to change filenames
# 2001-06-24   cbf    When displaying order, filename wasn't correct on a link
# 2001-07-24   cbf    Mailprogram can be customized
# 2001-08-05   cbf    Attributes weren't showing in admin screen
#                     All input is validated
# 2001-08-12   cbf    Added instructions about "processing orders"
# 2001-09-08   cbf    Added SecureImageURL variable use
# 2001-09-13   cbf    Made Edit/Delete list be sorted by category, put edit
#                      button next to each item, to reduce scroll time
# 2001-09-17   cbf    Let admin remove customers from cust. list
# 2001-09-18   cbf    Let admin email all customers a la mail program
# 2001-09-27   dsb    Change variables takes account of Merchant/Admin uname
# 2001-10-18   cbf    Added database location
# 2002-01-20   cbf    Each category now has variables to be edited
# 2002-01-29   cbf    Rudimentary Download Order History Log
# 2002-02-11   cbf    Let categories be public or private
# 2002-05-12   cbf    Added minimum order amount, added continue shopping vars,
#                       final return vars
# 2002-05-12   cbf    Removed extra space from customer section
# 2003-01-06   cbf    Added Item list to Download order history log
# 2003-02-27   cbf    Added Show shipping prices option
# 2003-03-01   cbf    Added Important links back, it went away somehow
# 2003-04-01   cbf    Added currency marker, different taxing
#                     Added new taxing options (state,national,internat)
# 2003-04-02   cbf    Added gift certificates, country listings, pay by check
# 2003-07-10   cbf    Added ip address for orders
# 2003-07-11   cbf    Added mySQL data backup
# 2003-08-23   cbf    Added Ebay Possibilities
# 2003-08-23   cbf    Added access to Admin page restrict to IP
# 2003-08-25   cbf    Added importing capabilities
# 2003-08-25   cbf    Added different language support
# 2003-10-17   cbf    Added Template support for Items and Category
# 2004-02-18   cbf    Fixed annoying category problem where the cat doesn't show
# 2004-02-19   cbf    Fixed bug in edit product where it didn't change the
#                      attributes on loading. Unfortunately it didn't make v1.6
# 2004-05-07   cbf    Editing bug fixes, goaded on by john.jones@mac.com
#                      and Trevor van Bremen
# 2004-07-07   cbf    Fixed Reduced Shipping bugs
# 2004-07-08   cbf    Added Data Exporting
# 2004-07-13   cbf    Added Sales Analysis Reports
# 2004-07-27   cbf    Added PreCharge Support
# 2004-08-24   peale  fixed error on line 1898
# 2005-01-23   gyp    added vv2 code for cc's gypsy@freeq.com
# 2005-03-03   cbf    added in code for Affiliates System
# 2005-04-04   cbf    added stats to admin, cod for payment, new admin options
# 2005-04-27   cbf    attribute section in add/edit looks better, has prices now
# 2005-05-02   cbf    attributes can be set to be any number
# 2005-05-03   cbf    uses TinyMCE for textareas
# 2005-05-18   cbf    displays MaxMind information for orders if enabled
################################################################################

use CGI::Carp(fatalsToBrowser);
use closedShop;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query        = new CGI;
my $program_name = "$closedShop::FileNames{admin}";

my %actions = (
    AddProduct           => $closedShop::TEXT{'ADD_PROD'},
    EditProduct          => "$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'PROD'}",
    DeleteProduct        => $closedShop::TEXT{'DEL_PROD'},
    ViewOrders           => $closedShop::TEXT{'VIEW_PROC'},
    ViewCustomers        => $closedShop::TEXT{'VIEW_CUST'},
    AdminScreen          => $closedShop::TEXT{'ADMIN_PAN'},
    ChangeVariables      => $closedShop::TEXT{'ED_PROG'},
    EditCategories       => "$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'CAT'}",
    ViewLinks            => $closedShop::TEXT{'VIEW_LNK'},
    GiftCertificate      => $closedShop::TEXT{'GC_MNG'},
    BackupSQL            => $closedShop::TEXT{'BK_SQL'},
    ImportProducts       => $closedShop::TEXT{'IMPORT'},
    ExportData           => $closedShop::TEXT{'EXPORT'},
    PrintEbaySubCategory => "Print Ebay Sub Categories",
    SalesReports         => $closedShop::TEXT{'SALE_RP'},
    Affiliate            => $closedShop::TEXT{'AFFILIATE'},
    Groups               => "Group Discount Management",
    Logoff               => "Logout of System",
    PrintReceipt         => "Print Reciept"
);
################################################################################

Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main {
    my $action = $query->param('action');
    $action = "cs" if ( !defined($action) );

    my $IP = $closedShop::IPAccess;
    if ($IP) {
        if (   $ENV{'REMOTE_ADDR'} ne $IP
            && $ENV{'REMOTE_HOST'} !~ /$IP/ )
        {
            return closedShop::ErrorMessage("$closedShop::TEXT{'NO_IP'} $ENV{'REMOTE_ADDR'}");
        }
    }

    if ( $action eq "Login" ) {
        Login();
    }
    elsif ( $actions{$action} eq "" ) {
        my ( $session, $uid ) = closedShop::GetSessionCookie();
        if ( closedShop::Authenticate( $session, $uid, "admin" ) ) {
            AdminScreen();
        }
        else {
            LoginScreen( $closedShop::TEXT{'PLS_LOGIN'} );
        }
    }
    else {
        my ( $session, $uid ) = closedShop::GetSessionCookie();
        if ( !closedShop::Authenticate( $session, $uid, "admin" ) ) {
            return LoginScreen( $closedShop::TEXT{'IMPROP_LOGIN'} );
        }
        else {
            eval($action);
        }
    }
}

###############################################################################
# FUNCTION:  Logoff
# DESCRIPTION: display sales reports
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Logoff {
    closedShop::LogoffAdmin();
    return LoginScreen( $closedShop::TEXT{'PLS_LOGIN'}, "nohttp" );
}

###############################################################################
# FUNCTION:  SalesReports
# DESCRIPTION: display sales reports
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub SalesReports {
    closedShop::Header();
    print qq|
                <font class="main">
          <table width="$closedShop::TableWidth"  class="main">
          <tr class="headrow"><th colspan="7">
          <font class="head">$closedShop::TEXT{'SALE_RP'}</th></tr>
           <tr class="mainrow"><td colspan='7'>
           <b>$closedShop::TEXT{'SALE_BY_DATE'}</b></td></tr>
	   |;

    my %years = ();
    my $SQL   = qq|select total,YEAR(date_processed),MONTH(date_processed)
   from    $closedShop::sql_tables{"order"} where date_processed IS NOT NULL|;
    my $sth = closedShop::DatabaseQuery($SQL);
    while ( my ( $total, $year, $month, $monthname ) = $sth->fetchrow_array() ) {
        if ( !( $year && $month ) ) { next; }
        my %months = ();
        if ( $years{$year} ) {
            %months = %{ $years{$year} };
        }

        my @data = ();
        if ( $months{$month} ) {
            @data = @{ $months{$month} };
        }

        push @data, $total;
        $months{$month} = \@data;
        $years{$year}   = \%months;
    }

    my $allsales = 0;
    my $alltotal = 0;

    foreach my $y ( sort keys %years ) {
        my %months = %{ $years{$y} };
        print qq|<tr class='mainrow'><td colspan='7'><font class='main'><b>$closedShop::TEXT{'YEAR'}: $y (# $closedShop::TEXT{'SALES'} / $closedShop::TEXT{'TOT'})</td></tr>
	    <tr class='mainrow'>|;
        my $yearsales = 0;
        my $yeartotal = 0;

        for ( my $m = 1 ; $m < 13 ; $m++ ) {
            if ( $m == 7 ) {
                print qq|<td><font class='small'><b>$closedShop::TEXT{'TOT'}</b></td></tr><tr class='mainrow'>|;
            }
            print qq|<td valign='top'><font class='small'>
         <B>| . closedShop::MonthName($m) . qq|</B><BR>|;

            if ( $months{$m} ) {
                my @sales = @{ $months{$m} };
                print eval( $#sales + 1 ) . qq| / |;
                my $sum = 0;
                foreach my $sale (@sales) { $sum += $sale; }
                print qq|$closedShop::CurrencyMarker $sum|;
                $yeartotal += $sum;
                $yearsales += eval( $#sales + 1 );
            }
            else { print qq|&nbsp;|; }

            print qq|</td>|;

        }

        $allsales += $yearsales;
        $alltotal += $yeartotal;
        print qq|<td><font class='small'>$yearsales / $closedShop::CurrencyMarker $yeartotal<br>Avg. Sale: $closedShop::CurrencyMarker| . sprintf( "%.2f", $yeartotal / $yearsales ) . qq|</td></tr>|;
    }

    if ( $allsales > 0 ) {
        print qq|<tr class='mainrow'><td colspan='2'><font class='main'><B>$closedShop::TEXT{'ALL'} $closedShop::TEXT{'TOT'}</b></td>
    <td colspan='2'><font  class='main'><B># $closedShop::TEXT{'SALES'}</b>: $allsales</td><td colspan='2'><font class='main'><b>$closedShop::TEXT{'TOT'} $closedShop::TEXT{'SALES'}:</b> $closedShop::CurrencyMarker $alltotal</td>
    <td colspan='2'><font class='main'><B>Avg. $closedShop::TEXT{'SALES'}</b>: $closedShop::CurrencyMarker|
          . sprintf( "%.2f", $alltotal / $allsales ) . qq|</td></tr>|;
    }
    else {
        print qq|<tr class='mainrow'><Td colspan='7'>$closedShop::TEXT{'NOSALESPROC'}</td></tr>|;
        print qq|</table>|;
         BottomAdmin();
        return closedShop::Footer();
    }

    print qq|<tr class="headrow"><th colspan="7"><font class='small'>&nbsp;</th></tr></table><BR><BR>

          <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="7">
          <font class="head">$closedShop::TEXT{'SALE_RP'}</th></tr>
           <tr class="mainrow"><td colspan='3'>
           <b>$closedShop::TEXT{'SALE_BY_ITEM'} - $closedShop::TEXT{'BYQTY'}</b></td></tr>
	   <tr class="mainrow"><td><b><font class='main'>$closedShop::TEXT{'NAME'}</b></td>
	   <td><b><font class='main'>$closedShop::TEXT{'CODE'}</td>
	   <td><b><font class='main'>$closedShop::TEXT{'QTY_SOLD'}</td></tr>
	   |;

    $SQL = qq|select item_ids
       from    $closedShop::sql_tables{"order"} where date_processed IS NOT NULL|;
    $sth = closedShop::DatabaseQuery($SQL);
    my %topqty  = ();
    my %topsale = ();
    my %names   = ();
    $names{"gc"} = "Gift Certificate";
    my $totalsold = 0;
    my $totalsale = 0;
    while ( my ($itemids) = $sth->fetchrow_array() ) {
        my @ary = split( /$closedShop::CookieItemDelimiter/, $itemids );
        foreach (@ary) {
            if ( !$_ ) { next; }
            my ( $id, $qty, $attr ) = split( /$closedShop::CookieSectionDelimiter/, $_ );
            my $SQL2 = qq|select price,name,product_code from $closedShop::sql_tables{"item"} where
		id='$id'|;
            my $sth2 = closedShop::DatabaseQuery($SQL2);
            my ( $price, $name, $code ) = $sth2->fetchrow_array();

            if ( $id eq "gc" ) {
                my ( $id, $recipient_name, $recipient_email, $gc_amount ) =
                  split( /$closedShop::CookieSectionDelimiter/, $_ );
                $topqty{$id}++;
                $totalsold++;
                $topsale{$id} += $gc_amount;
                $totalsale    += $gc_amount;
            }
            else {
                $names{$id} = $name . "~~~" . $code;
                $topqty{$id}  += $qty;
                $totalsold    += $qty;
                $topsale{$id} += $price * $qty;
                $totalsale    += $price * $qty;
            }
        }
    }

    my @keys = sort { $topqty{$b} <=> $topqty{$a} } keys %topqty;
    foreach my $d (@keys) {
        my ( $na, $co ) = split( /~~~/, $names{$d} );
        if ( !$na ) { next; }
        print qq|<tr class='mainrow'><td><font class='main'>$na</td><td><font class='main'>$co</td>
      <td><font class='main'>$topqty{$d}</td></tr>|;
    }

    print qq|<tr class='mainrow'><td colspan='2' ><font class='main'><b>$closedShop::TEXT{'TOT'}</b></td>
    <td><font class='main'><b>$totalsold</b></td></tr>
    <tr class='mainrow'><td colspan='3'>&nbsp;</td></tr>
     <tr class="mainrow"><td colspan='3'>
           <b>$closedShop::TEXT{'SALE_BY_ITEM'} - $closedShop::TEXT{'BYTOTAL'}</b></td></tr>
	   <tr class="mainrow"><td><b><font class='main'>$closedShop::TEXT{'NAME'}</b></td>
	   <td><b><font class='main'>$closedShop::TEXT{'CODE'}</td>
	   <td><b><font class='main'>$closedShop::TEXT{'TOT'} $closedShop::TEXT{'SALES'}</td></tr>|;

    @keys = sort { $topsale{$b} <=> $topsale{$a} } keys %topsale;
    foreach my $d (@keys) {
        my ( $na, $co ) = split( /~~~/, $names{$d} );
        if ( !$na ) { next; }
        print qq|<tr class='mainrow'><td><font class='main'>$na</td><td><font class='main'>$co</td>
      <td><font class='main'>$closedShop::CurrencyMarker $topsale{$d}</td></tr>|;
    }

    print qq|<tr class='mainrow'><td colspan='2' ><font class='main'><b>$closedShop::TEXT{'TOT'}</b></td>
    <td><font class='main'><b>$closedShop::CurrencyMarker $totalsale</b></td></tr>
    <tr class="headrow"><th colspan="3"><font class='small'>&nbsp;</th></tr>
    </table><BR><BR>
       <table width="$closedShop::TableWidth"  class="main">
       <tr class="headrow"><th colspan="4">
          <font class="head">$closedShop::TEXT{'SALE_RP'}</th></tr>
           <tr class="mainrow"><td colspan='4'>
           <b>$closedShop::TEXT{'SALE_BY_ITEM'} - $closedShop::TEXT{'BYLOCATION'}</b></td></tr>
	   <tr class="mainrow"><td ><b><font class='main'>$closedShop::TEXT{'LOCATION'}</b></td>
	   <td><b><font class='main'># $closedShop::TEXT{'SALES'}</td>
	   <td><b><font class='main'>$closedShop::TEXT{'TOT'} $closedShop::TEXT{'SALES'}</td>
	   <td><b><font class='main'>Average $closedShop::TEXT{'SALES'}</td></tr>
	   |;
    $SQL = qq|select b_state,b_country,total
       from    $closedShop::sql_tables{"order"} where date_processed IS NOT NULL|;
    $sth = closedShop::DatabaseQuery($SQL);

    my %countries    = ();
    my %countrysales = ();
    while ( my ( $state, $country, $total ) = $sth->fetchrow_array() ) {
        my %states = ();
        if ( $countries{$country} ) {
            %states = %{ $countries{$country} };
        }
        my @data = ();
        if ( $states{$state} ) {
            @data = @{ $states{$state} };
        }

        $data[0]++;
        $data[1] += $total;
        $states{$state}      = \@data;
        $countries{$country} = \%states;
        my @cdata = ();
        if ( $countrysales{$country} ) {
            @cdata = @{ $countrysales{$country} };
        }
        $cdata[0]++;
        $cdata[1] += $total;
        $countrysales{$country} = \@cdata;
    }

    foreach my $c ( keys %countries ) {
        my %states = %{ $countries{$c} };
        my @cdata  = @{ $countrysales{$c} };
        print qq|<tr class='mainrow'><td><font class='main'><b>$c</b></td>
      <td><font class='main'><b>| . $cdata[0] . qq|</td>
      <td><font class='main'><b>$closedShop::CurrencyMarker| . $cdata[1] . qq|</td>
      <td><font class='main'><b>$closedShop::CurrencyMarker|
          . sprintf( "%.2f", $cdata[1] / $cdata[0] ) . qq|</td>
      </tr>|;
        foreach my $s ( sort keys %states ) {
            my @data = @{ $states{$s} };
            print qq|<tr class='mainrow'><td><font class='main'>
	&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
	$s</td>
      <td><font class='main'>| . $data[0] . qq|</td>
      <td><font class='main'>$closedShop::CurrencyMarker| . $data[1] . qq|</td>
      <td><font class='main'>$closedShop::CurrencyMarker|
              . sprintf( "%.2f", $data[1] / $data[0] ) . qq|</td>
      </tr>|;
        }
    }

    print qq|<tr class="headrow"><th colspan="4"><font class='small'>&nbsp;</th></tr>
	  </table>Note: Totals include processed sales only|;

    BottomAdmin();

    closedShop::Footer();
}

###############################################################################
# FUNCTION:  BottomAdmin
# DESCRIPTION: bottom of admin page
###############################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub BottomAdmin {
    print qq|<BR><BR>
  <div align="center">
  <Table width="$closedShop::TableWidth" style='border: 1px solid black'></form>
  <Tr ><form action="$program_name" method="POST"><td align='left'>
            <input type="hidden" name="action" value="AdminScreen">
            <input type="submit" value="$closedShop::TEXT{'ADMIN_PAN'}"></form></div></center>
    </td><td width='35'>&nbsp;</td><form action="$program_name" method="POST">
    <td align='right'>
            <input type="hidden" name="action" value="Logoff">
            <input type="submit" value="Logout of System"></form></div>
     </td></tr></table><BR><BR>
  |;

}

###############################################################################
# FUNCTION:  affiliates
# DESCRIPTION: affiliates and Commissions
###############################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub Groups {
    if ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Assign" ) {
        AssignGroupUsers();
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Add" ) {
        ActionGroup("add");
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Delete" ) {
        ActionGroup( "delete", closedShop::ValidateInputAdmin( $query->param("id") ) );
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Edit" ) {
        if ( closedShop::ValidateInputAdmin( $query->param("submit") ) == 1 ) {
            ActionGroup( "edit", closedShop::ValidateInputAdmin( $query->param("id") ) );
        }
        else {
            ShowEditGroup( closedShop::ValidateInputAdmin( $query->param("id") ) );
        }
    }
    else {
        DisplayGroups();
    }
}

###############################################################################
# FUNCTION:  Displayaffiliates
# DESCRIPTION: default screen for affiliates
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub AssignGroupUsers {

    if ( closedShop::ValidateInputAdmin( $query->param("submit") ) == 1 ) {
        my @mems    = $query->param("members");
        my $groupid = closedShop::ValidateInputAdmin( $query->param("groupid") );

        my $SQL = qq|update $closedShop::sql_tables{"user"} set groupid='' where groupid='$groupid'|;
        my $sth = closedShop::DatabaseQuery($SQL);

        foreach (@mems) {
            $SQL = qq|update $closedShop::sql_tables{"user"} set groupid='$groupid' where id='$_'|;

            $sth = closedShop::DatabaseQuery($SQL);
        }
        return DisplayGroups("You have updated the group members");
    }
    else {

        my $id = closedShop::ValidateInputAdmin( $query->param("id") );

        my $SQL = qq|select id,name,discount from $closedShop::sql_tables{"groups"} where id='$id'|;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ( $gid, $gname, $gdiscount ) = $sth->fetchrow_array();

        closedShop::Header();

        print qq|</center><font class='head'>Select Group Members for $gname ($gdiscount \%)</font><BR>|;

        print qq|
    
    <table  width="$closedShop::TableWidth" valign='top' class="main">
    <form action='$program_name' method='POST'><tr class='headrow'>
	<td valign='top' colspan='3' class='head'>Assign Users to $gname</td>	 
	</tr><input type='hidden' name='action' value='Groups'>
 	<input type='hidden' name='select' value='Assign'>
	<input type='hidden' name='groupid' value='$gid'>
	<input type='hidden' name='submit' value='1'>
	<tr class='mainrow'><td><u>Name</u></td>
	<td><u>Email</u></td>
	<td><u>Assign</u></td>
	</tr>
	|;

        $SQL = qq|select $closedShop::sql_tables{"user"}.id from 
	        $closedShop::sql_tables{"user"},$closedShop::sql_tables{"groups"} 
		where groupid = $closedShop::sql_tables{"groups"}.id and
		$closedShop::sql_tables{"groups"}.id = '$id'|;
        $sth = closedShop::DatabaseQuery($SQL);
        my %members = ();
        while ( my ($i) = $sth->fetchrow_array() ) {
            $members{$i} = 1;
        }

        $SQL = qq|select $closedShop::sql_tables{"user"}.id,$closedShop::sql_tables{"groups"}.name from 
	        $closedShop::sql_tables{"user"},$closedShop::sql_tables{"groups"} 
		where groupid = $closedShop::sql_tables{"groups"}.id and
		$closedShop::sql_tables{"groups"}.id != '$id'|;
        $sth = closedShop::DatabaseQuery($SQL);
        my %nonmembers = ();
        while ( my ( $i, $gn ) = $sth->fetchrow_array() ) {
            $nonmembers{$i} = $gn;
        }

        $SQL = qq|select fname,lname,email,id from $closedShop::sql_tables{"user"}|;
        $sth = closedShop::DatabaseQuery($SQL);

        while ( my ( $fn, $ln, $e, $i ) = $sth->fetchrow_array() ) {
            my $checked  = "";
            my $disabled = "";
            my $mesg     = "";
            if ( $members{$i} == 1 ) {
                $checked = "checked";
            }

            if ( $nonmembers{$i} ) {
                $disabled = "disabled";
                $mesg     = " (in $nonmembers{$i})";
            }

            print qq|<tr class='mainrow'><td>$fn $ln</td><td>$e</td><td>
	  <input type='checkbox' name='members' $disabled $checked value='$i'>$mesg</td>
	  </tr> |;
        }

        print qq|
	<tr class='mainrow'><td colspan='3'><center><input type='submit' value='Add Members'></td></tr>
	</table></form><BR><BR>|;

        BottomAdmin();

        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  EditAffiliate
# DESCRIPTION: edits the affiliate user data
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub ActionGroup {
    my ( $action, $id ) = @_;

    my $name     = closedShop::ValidateInputAdmin( $query->param("name") );
    my $discount = closedShop::ValidateInputAdmin( $query->param("discount") );

    my $SQL     = "";
    my $message = "";
    if ( $action eq "edit" ) {
        $SQL = qq|update $closedShop::sql_tables{"groups"} 
		  set name='$name',discount='$discount'
		  where id='$id' |;
        $message = "You have edited the group";
    }
    elsif ( $action eq "add" ) {
        $SQL = qq|insert into $closedShop::sql_tables{"groups"} 
		 set name='$name',discount='$discount' |;

        $message = "You have added the group";
    }
    elsif ( $action eq "delete" ) {
        $SQL = qq|delete from $closedShop::sql_tables{"groups"} 
		  where id='$id' |;
        my $sth = closedShop::DatabaseQuery($SQL);
        $SQL = qq|update $closedShop::sql_tables{"user"} 
		  set groupid=''  where groupid='$id'|;
        $message = "You have removed the group";
    }

    my $sth = closedShop::DatabaseQuery($SQL);

    return DisplayGroups($message);
}

###############################################################################
# FUNCTION:  ShowEditGroup
# DESCRIPTION: default screen for affiliates
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub ShowEditGroup {
    my ($id) = @_;
    my $SQL = qq|select id,name,discount from $closedShop::sql_tables{"groups"} 
    		where id='$id'|;
    my $sth = closedShop::DatabaseQuery($SQL);

    my $uncount = 0;
    my ( $id, $name, $discount ) = $sth->fetchrow_array();

    closedShop::Header();

    print qq|
     <table  width="300" valign='top' class="main">
    <form action='$program_name' method='POST'><tr class='headrow'>
	<td valign='top' colspan='2' class='head'>Add a New Group</td>	 
	</tr><input type='hidden' name='action' value='Groups'>
 	<input type='hidden' name='select' value='Edit'>
	<input type='hidden' name='submit' value='1'>
	<input type='hidden' name='id' value='$id'>
	<tr class='mainrow'><td>Name</td><td><input value='$name' type='text' name='name' size='25'></td></tr>
	<tr class='mainrow'><td>Percent Discount</td><td><input value='$discount' type='text' name='discount' size='5'> \%</td></tr>
	<tr class='mainrow'><td colspan='2'><center><input type='submit' value='Edit Group'></td></tr>
	</table></form><BR><BR>
 	|;
    BottomAdmin();
    closedShop::Footer();

}

###############################################################################
# FUNCTION:  Displayaffiliates
# DESCRIPTION: default screen for affiliates
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub DisplayGroups {
    closedShop::Header();
    my ($message) = @_;

    print qq|</center><font class='head'>Group Discount Information</font><BR>|;

    if ($message) {
        $message = qq|<table style='border: 1px solid black;' width='80%'><tr>
      <td><font color="red" class='main'>$message</td></tr></table><BR>|;
    }

    print qq|
    $message
    <table  width="300" valign='top' class="main">
    <form action='$program_name' method='POST'><tr class='headrow'>
	<td valign='top' colspan='2' class='head'>Add a New Group</td>	 
	</tr><input type='hidden' name='action' value='Groups'>
 	<input type='hidden' name='select' value='Add'>
	<tr class='mainrow'><td>Name</td><td><input type='text' name='name' size='25'></td></tr>
	<tr class='mainrow'><td>Percent Discount</td><td><input type='text' name='discount' size='5'> \%</td></tr>
	<tr class='mainrow'><td colspan='2'><center><input type='submit' value='Add New Group'></td></tr>
	</table></form><BR><BR>
  	<table  class="main" summary=""width="$closedShop::TableWidth">
	<tr class='headrow'>
	<td colspan='4' >Groups</td>
	</tr><tr class='mainrow'>
	<td><b>Name</b></td>
	<td><B>Discount</b></td>
	<td><b># Members</b></td>
	<Td><B>Action</b></td>
	</tr>|;

    my $SQL = qq|select id,name,discount from $closedShop::sql_tables{"groups"} order by name|;
    my $sth = closedShop::DatabaseQuery($SQL);

    my $uncount = 0;
    while ( my ( $id, $name, $discount ) = $sth->fetchrow_array() ) {
        my $SQL2          = qq|select count(id) from $closedShop::sql_tables{"user"} where groupid='$id'|;
        my $sth2          = closedShop::DatabaseQuery($SQL2);
        my ($num_members) = $sth2->fetchrow_array();

        my $SQL2 = qq|select $closedShop::sql_tables{"user"}.fname,
	$closedShop::sql_tables{"user"}.lname,
	  $closedShop::sql_tables{"user"}.email from $closedShop::sql_tables{"user"},
	  $closedShop::sql_tables{"groups"} where $closedShop::sql_tables{"user"}.groupid =
	  $closedShop::sql_tables{"groups"}.id  and $closedShop::sql_tables{"groups"}.id='$id'|;
        my $sth2 = closedShop::DatabaseQuery($SQL2);
        my @mems = ();
        while ( my ( $fn, $ln, $e ) = $sth2->fetchrow_array() ) {
            push @mems, "$fn $ln$closedShop::VarsFileDelimiter$e";
        }

        print qq|<tr class='mainrow'><td><a href="#a" title='Show Group Members' 
	onClick="javascript:showDiv('mem$id');">$name</a>
		<div name='code' ID='mem$id' style="visibility: hidden; position: absolute;">
		<table style='border: 1px solid black; background-color: tan;' width='300'>
		<tr><td class='mainrow'>
		<font class='main'>
		|;
        foreach (@mems) {
            my @ar = split( /$closedShop::VarsFileDelimiter/, $_ );
            print qq|<li style='list-style-type: circle;'><a href='mailto:$ar[2]'>$ar[0] $ar[1]</a>|;
        }
        if ( $#mems < 0 ) {
            print qq|No customers have been assigned to this group|;
        }

        print qq|
		</td></tr></table></div></td>
		<td>$discount \%</td><td><center>$num_members</td>
		<td>
		  <table width='100%' style='border: 1px solid gray;'><Tr><td 
		  style='font-size: 11px;'>
		    <a href='$program_name?action=Groups&select=Assign&id=$id'>Assign Customers</a></td></tr>
		    </table> <table width='100%' style='border: 1px solid gray;'>
		  <tr><td  style='font-size: 11px;'>
		    <a href='$program_name?action=Groups&select=Edit&id=$id'>Edit Group</a>
		  </td></tr></table> <table width='100%' style='border: 1px solid gray;'>
		  <tr><td  style='font-size: 11px;'>
		  <A onClick=\"javascript:Sure('$program_name?action=Groups&select=Delete&id=$id');\"
		href='#'>Remove Group</a>
		  </td></tr>
		  </table>
		</td>
		</tr>|;
        $uncount++;
    }
    if ( $uncount == 0 ) {
        print qq|<tr class='mainrow'><td colspan='4'>No Groups</td></tr>|;
    }

    print qq|</table>|;

    BottomAdmin();

    closedShop::Footer();
}

###############################################################################
# FUNCTION:  affiliates
# DESCRIPTION: affiliates and Commissions
###############################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub Affiliate {
    if ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "ViewMore" ) {
        ViewMoreAffiliateInfo( "", closedShop::ValidateInputAdmin( $query->param("id") ) );
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "AddAffiliate" ) {
        ActionAffiliate("add");
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Delete" ) {
        DeleteAffiliate( closedShop::ValidateInputAdmin( $query->param("id") ) );
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Edit" ) {
        ActionAffiliate( "edit", closedShop::ValidateInputAdmin( $query->param("id") ) );
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Status" ) {
        AffiliateStatus( closedShop::ValidateInputAdmin( $query->param("id") ), closedShop::ValidateInputAdmin( $query->param("status") ) );
    }
    elsif ( closedShop::ValidateInputAdmin( $query->param("select") ) eq "Mark" ) {
        MarkAffiliate( closedShop::ValidateInputAdmin( $query->param("id") ), closedShop::ValidateInputAdmin( $query->param("sid") ) );
    }
    else {
        DisplayAffiliates();
    }
}

###############################################################################
# FUNCTION:  EditAffiliate
# DESCRIPTION: edits the affiliate user data
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub ActionAffiliate {
    my ( $action, $id ) = @_;

    my $fname      = closedShop::ValidateInputAdmin( $query->param("fname") );
    my $lname      = closedShop::ValidateInputAdmin( $query->param("lname") );
    my $address    = closedShop::ValidateInputAdmin( $query->param("address") );
    my $city       = closedShop::ValidateInputAdmin( $query->param("city") );
    my $state      = closedShop::ValidateInputAdmin( $query->param("state") );
    my $country    = closedShop::ValidateInputAdmin( $query->param("country") );
    my $zip        = closedShop::ValidateInputAdmin( $query->param("zip") );
    my $phone      = closedShop::ValidateInputAdmin( $query->param("phone") );
    my $email      = closedShop::ValidateInputAdmin( $query->param("email") );
    my $commission = closedShop::ValidateInputAdmin( $query->param("commission") );

    my $SQL     = "";
    my $message = "";
    if ( $action eq "edit" ) {
        $SQL = qq|update $closedShop::sql_tables{"affiliates"} 
		set fname='$fname',lname='$lname',address='$address',city='$city',
				 state='$state',country='$country',zip='$zip',phone='$phone',email='$email',
				 commission='$commission'
				 where id='$id' |;
        $message = "You have edited the affiliate";
    }
    elsif ( $action eq "add" ) {
        $SQL = qq|insert into $closedShop::sql_tables{"affiliates"} 
			set fname='$fname',lname='$lname',address='$address',city='$city',status='APPROVED',
				 state='$state',country='$country',zip='$zip',phone='$phone',email='$email',
				 commission='$commission' |;

        $message = "You have added the affiliate";
    }

    my $sth = closedShop::DatabaseQuery($SQL);
    if ( $action eq "add" ) {
        $id = $sth->{'mysql_insertid'};
    }

    return ViewMoreAffiliateInfo( $message, $id );
}

###############################################################################
# FUNCTION:  MarkAffiliate
# DESCRIPTION: marks affiliate data
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub MarkAffiliate {
    my ( $id, $sid ) = @_;

    $id  =~ tr/0-9//cd;
    $sid =~ tr/0-9//cd;

    if ($id) {
        my $extra = "";
        if ($sid) {
            $extra = " and id = '$sid' ";
        }

        my $SQL = qq|update $closedShop::sql_tables{"affiliate_sales"} 
				 set status='PAID' where aid='$id' $extra |;
        my $sth = closedShop::DatabaseQuery($SQL);
    }

    return ViewMoreAffiliateInfo( "You have updated the status of the commissions", $id );
}

###############################################################################
# FUNCTION:  AffiliateStatus
# DESCRIPTION: approve or deny affiliate
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub AffiliateStatus {
    my ( $id, $status ) = @_;

    $id =~ tr/0-9//cd;

    if ($id) {
        my $SQL = "";
        if ( $status eq 'APPROVED' ) {
            $SQL = qq|update $closedShop::sql_tables{"affiliates"} 
				set status='APPROVED' where id='$id' |;

            my $link = $closedShop::FileNames{"item"};
            if ( $link !~ /^http/ ) {
                $link = $closedShop::urltofolder . "/" . $closedShop::FileNames{"item"};
            }
            my $body = qq|Congratulations,

You have been approved as an Affiliate for $closedShop::CompanyName.
      
Your HTML link to connect as an affiliate is:

$link?action=SetAffiliate&id=$id

Thank You,
$closedShop::CompanyName
$closedShop::website
|;
            my $SQL2 = qq|select email from $closedShop::sql_tables{"affiliates"} 
				 where id='$id' |;
            my $sth2 = closedShop::DatabaseQuery($SQL2);
            my ($email) = $sth2->fetchrow_array();
            closedShop::SendEmail( $email, "Affiliate Account", $body );

        }
        else {
            $SQL = qq|delete from $closedShop::sql_tables{"affiliates"} 
				          where id='$id' |;
        }
        my $sth = closedShop::DatabaseQuery($SQL);
    }

    return DisplayAffiliates("You have updated the status of the affiliate");
}

###############################################################################
# FUNCTION:  DeleteAffiliate
# DESCRIPTION: shows more detailed info about affiliates
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub DeleteAffiliate {
    my ($id) = @_;

    if ( closedShop::ValidateInputAdmin( $query->param("sure") ) eq "yes" ) {
        my $SQL = qq|delete from $closedShop::sql_tables{"affiliates"} where id='$id'|;
        my $sth = closedShop::DatabaseQuery($SQL);

        $SQL = qq|delete from $closedShop::sql_tables{"affiliate_sales"} where aid='$id'|;
        $sth = closedShop::DatabaseQuery($SQL);

        return DisplayAffiliates("You have removed the affiliate");
    }

    my $SQL = qq|select fname,lname from $closedShop::sql_tables{"affiliates"} where id='$id'|;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $fname, $lname ) = $sth->fetchrow_array();

    closedShop::Header();

    print qq|
    </center>
    <font class='main'>
		<a href='$program_name?action=Affiliate'>Return to Affiliate Home</a>
		<table summary="" width='300'><tr class='headrow'>
			  <form action='$program_name' method='POST'>
			 <input type='hidden' name='action' value='Affiliate'>
 			 <input type='hidden' name='select' value='Delete'>
			 <input type='hidden' name='sure' value='yes'>
 			 <input type='hidden' name='id' value='$id'>
			 <td class='head'>Confirm Deletion</td></tr>
			 <tr class='mainrow'>
			 <td>Are you sure that you want to remove the affiliate $fname $lname and all of their
			 corresponding records?</td></tr>
			 <tr class='mainrow'><td><center><input type='submit' value='Delete Affiliate'></td></tr></form>
			 </table><BR><BR>
			 <a href='javascript:history.go(-1);'>Go back</a>
		|;
    BottomAdmin();
    closedShop::Footer();

}

###############################################################################
# FUNCTION:  ViewMoreAffiliateInfo
# DESCRIPTION: shows more detailed info about affiliates
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub ViewMoreAffiliateInfo {
    my ( $message, $id ) = @_;

    my $SQL = qq|select commission,fname,email,address,city,state,country,zip,phone,lname
		  from $closedShop::sql_tables{"affiliates"} where id='$id'|;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ( $commission, $fname, $email, $address, $city, $state, $country, $zip, $phone, $lname ) = $sth->fetchrow_array();

    closedShop::Header();

    my $link = $closedShop::FileNames{"item"};

    if ( $link !~ /^http/ ) {
        $link = $closedShop::urltofolder . "/" . $closedShop::FileNames{"item"};
    }

    if ($message) {
        print qq|<table style='border: 1px solid black;' width='80%'><tr>
      <td><font color="red">$message</td></tr></table><BR>|;
    }

    print qq|</centeR>
    <font class='main'>
		<a href='$program_name?action=Affiliate'>Return to Affiliate Home</a><BR><BR>
    	<table  border='0'><tr class='headrow'>
			 <td colspan='2' class='head'>Affiliate: $fname $lname (<a href="mailto:$email">$email</a>)</td></tr>
		<tr class='mainrow'><td>
		Current Commission Level: <b style='background-color: yellow'>&nbsp;&nbsp;$commission \%</b></td></tr>
		<tr class='mainrow'><td>
		Affiliate Link: <b style='background-color: yellow'>&nbsp;&nbsp;$link?action=SetAffiliate&id=$id&nbsp;&nbsp;</b> </td></tr>
		<tr class='mainrow'><td>
		Click to <a href="#" onClick="javascript:showDiv('code');">Show Affiliate HTML Code</a>
		<div name='code' ID='code' style="visibility: hidden; position: absolute;">
		<table style='border: 1px solid black; background-color: tan;' width='300'>
		<tr><td class='mainrow'>
		<font face='courier' size='2'>
		&lt;B&gt;&lt;a href="$link?action=SetAffiliate&id=$id"&gt;Shop At 
		$closedShop::CompanyName!&lt;/a&gt;&lt;B&gt;</font>
		
		</td>
		</tr>
		</table>
		</div></td></tr></table>
		
		|;

    print qq|</center>
		<table width='300' border='0'><tr class='headrow'>
			 <td colspan='2' class='head'>Edit Affiliate</td></tr>
			 <form action='$program_name' method='POST'>
			 <input type='hidden' name='action' value='Affiliate'>
 			 <input type='hidden' name='select' value='Edit'>
 			 <input type='hidden' name='id' value='$id'>
			 <tr class='mainrow'><td>First Name</td><td><input type='text' value='$fname' name='fname' size='15'></td></tr>
			 <tr class='mainrow'><td>Last Name</td><td><input type='text' value='$lname' name='lname' size='15'></td></tr>
			 <tr class='mainrow'><td>Email</td><td><input type='text' value='$email' name='email' size='20'></td></tr>
			 <tr class='mainrow'><td>Address</td><td><textarea rows='3' cols='35' name='address'>$address</textarea></td></tr>
			 <tr class='mainrow'><td>City</td><td><input type='text' value='$city' name='city' size='15'></td></tr><br />
			 <tr class='mainrow'><td>State</td><td>|;
    closedShop::ListStates( "state", "$state" );
    print qq|</td></tr>
			 <tr class='mainrow'><td>Country</td><td>|;
    closedShop::ListCountries( "country", "$country", "restrict" );
    print qq|</td></tr>
			 <tr class='mainrow'><td>Zip</td><td><input type='text' value='$zip' name='zip' size='15'></td></tr>
			 <tr class='mainrow'><td>Phone</td><td><input type='text' value='$phone' name='phone' size='15'></td></tr>
			 <tr class='mainrow'><td>Commission: </td><td><input type='text' value='$commission' name='commission' size='5'>\%</td></tr>
			 <tr class='mainrow'><td colspan='2'><center><input type='submit' value='Edit Affiliate'></td></tr>
			  </form>
			 </table><BR><BR>
		
  			<table summary="" width='$closedShop::TableWidth'><tr class='headrow'>
				<td colspan='4' class='headrow'>Sales Information</td>
				</tr><tr class='mainrow'>
				<td><b>Date</b></td>
				<td><B>Amount Owed (Comm.)</b></td>
				<td><b>Sale</b></td>
				<td><b>Status</b></td>
				</tr>|;

    $SQL = qq|select id,sale,owed,status,commission,DATE_FORMAT(date_sold,"%M %e, %Y") from 
	 $closedShop::sql_tables{"affiliate_sales"} where aid='$id' order by date_sold DESC|;
    $sth = closedShop::DatabaseQuery($SQL);

    my ( $numsales, $totalsales, $totalpaid, $unpaid ) = ( 0, 0, 0 );
    while ( my ( $sid, $sale, $owed, $status, $comm, $date ) = $sth->fetchrow_array() ) {
        print qq|<tr class='mainrow'><td>$date</td>
				 <td>$closedShop::CurrencyMarker $owed (at \%$comm)</td><td>$closedShop::CurrencyMarker $sale</td>
				 <td><u>$status</u>&nbsp;&nbsp;&nbsp;
				 |;
        if ( $status ne "PAID" ) {
            print qq|
	 <a href="$program_name?action=Affiliate&select=Mark&id=$id&sid=$sid">Mark as Paid</a><br />
	 |;
            $unpaid += $owed;
        }
        else {
            $totalpaid += $owed;
        }

        print qq|</td></tr>|;
        $numsales++;
        $totalsales += $sale;
    }

    print qq|
		<tr class='headrow'><td colspan='1'># Sales: $numsales </td><td>
		Total Owed (Unpaid): $closedShop::CurrencyMarker $unpaid</td><td>
		Total Paid: $closedShop::CurrencyMarker $totalpaid </td><tD colspan='1'>
		Total Sales Generated: $closedShop::CurrencyMarker $totalsales
	 	</td></tr>
		</table>
		|;
    BottomAdmin();

    closedShop::Footer();

}

###############################################################################
# FUNCTION:  Displayaffiliates
# DESCRIPTION: default screen for affiliates
################################################################################
# MODIFICATION HISTORY
# 2005-03-1   cbf    Created
##############################################################################
sub DisplayAffiliates {
    closedShop::Header();
    my ($message) = @_;

    print qq|</center><font class='head'>Affiliate Information</font><BR>|;

    if ($message) {
        print qq|<table style='border: 1px solid black;' width='80%'><tr>
      <td><font color="red">$message</td></tr></table>|;
    }

    print qq|
    
    <table  width="300" valign='top' class="main"><form action='$program_name' method='POST'><tr class='headrow'>
			 <td valign='top' colspan='2' class='head'>Add New Affiliate</td>
			 
			 </tr>
			 <input type='hidden' name='action' value='Affiliate'>
 			 <input type='hidden' name='select' value='AddAffiliate'>
			 <tr class='mainrow'><td>First Name</td><td><input type='text' name='fname' size='15'></td></tr>
			 <tr class='mainrow'><td>Last Name</td><td><input type='text' name='lname' size='15'></td></tr>
			 <tr class='mainrow'><td>Email</td><td><input type='text' name='email' size='25'></td></tr>
			 <tr class='mainrow'><td>Address</td><td><textarea rows='3' cols='25' name='address'></textarea></td></tr>
			 <tr class='mainrow'><td>City</td><td><input type='text' name='city' size='20'></td></tr><br />
			 <tr class='mainrow'><td>State</td><td>|;
    closedShop::ListStates( "state", '' );
    print qq|</td></tr>
			  <tr class='mainrow'><td>Country</td><td>|;
    closedShop::ListCountries( "country", "$closedShop::CompanyCountry", "restrict" );
    print qq|</td></tr>
			 <tr class='mainrow'><td>Zip</td><td><input type='text' name='zip' size='15'></td></tr>
			 <tr class='mainrow'><td>Phone</td><td><input type='text' name='phone' size='15'></td></tr>
			 <tr class='mainrow'><td>Commission: </td><td><input type='text'  name='commission' size='5'>\%</td></tr>
			 <tr class='mainrow'><td colspan='2'><center><input type='submit' value='Add New Affiliate'></td></tr>
			 </table>
			 </form>
<BR><BR>
  			<table  class="main" summary=""width="$closedShop::TableWidth">
			<tr class='headrow'>
				<td colspan='4' >Unapproved Affiliates</td>
				</tr><tr class='mainrow'>
				<td><b>Name</b></td>
				<td><B>Email</b></td>
				<td><b>State</b></td>
				<td><b>Action</b></td>
				</tr>|;

    my $SQL = qq|select id,fname,lname,email,state from $closedShop::sql_tables{"affiliates"} 
		where status='UNAPPROVED' order by lname|;
    my $sth = closedShop::DatabaseQuery($SQL);

    my $uncount = 0;
    while ( my ( $id, $fname, $lname, $email, $state ) = $sth->fetchrow_array() ) {
        print qq|<tr class='mainrow'><td>$fname $lname</td>
		<td><a href='mailto:$email'>$email</a></td><td>$state</td>
		<td>
		<a href="$program_name?action=Affiliate&select=Status&status=APPROVED&id=$id">Approve</a><br />
		<a href="$program_name?action=Affiliate&select=Status&status=DENIED&id=$id">Deny</a><br />
		</td></tr>|;
        $uncount++;
    }
    if ( $uncount == 0 ) {
        print qq|<tr class='mainrow'><td colspan='4'>No Affiliates</td></tr>|;
    }

    print qq|
	<tr class='headrow' ><td colspan='4' class='head'>Approved Affiliates</td>
	</tr><tr class='mainrow'>
	<td><b>Name</b></td>
	<td><B>Amount Owed</b></td>
	<td><b>Sales</b></td>
	<td><b>Action</b></td>
	</tr>|;

    $SQL = qq|select id,commission,fname,lname,email from $closedShop::sql_tables{"affiliates"} 
		 where status='APPROVED' order by lname|;
    $sth = closedShop::DatabaseQuery($SQL);

    my ( $affcount, $totalsales, $totalowed ) = ( 0, 0, 0 );
    while ( my ( $id, $comm, $fname, $lname, $email ) = $sth->fetchrow_array() ) {
        my $SQL2    = qq|select SUM(sale) from $closedShop::sql_tables{"affiliate_sales"} where aid='$id'|;
        my $sth2    = closedShop::DatabaseQuery($SQL2);
        my ($sales) = $sth2->fetchrow_array();

        $SQL2 = qq|select SUM(owed) from $closedShop::sql_tables{"affiliate_sales"} where aid='$id'
				 						   and status='NOTPAID'|;
        $sth2 = closedShop::DatabaseQuery($SQL2);
        my ($owed) = $sth2->fetchrow_array();

        if ( !$owed )  { $owed  = "0.00"; }
        if ( !$sales ) { $sales = "0.00"; }

        print qq|<tr class='mainrow'><td><a href='mailto:$email'>$fname $lname</a></td>
		<td>$closedShop::CurrencyMarker $owed (at $comm \%)</td><td>$closedShop::CurrencyMarker $sales</td>
		<td>
		<a href="$program_name?action=Affiliate&select=ViewMore&id=$id">View More Info</a><br />
		<a href="$program_name?action=Affiliate&select=Delete&id=$id">Delete</td>
		</tr>|;
        $affcount++;
        $totalowed  += $owed;
        $totalsales += $sales;
    }
    if ( $affcount == 0 ) {
        print qq|<tr class='mainrow'><td colspan='4'>No Affiliates</td></tr>|;
    }

    print qq|
			 <tr class='headrow'><td colspan='1'># Affiliates: $affcount</td>
			 <td>
			 Total Owed: $closedShop::CurrencyMarker $totalowed</td>
			 <td colspan='2'>Total Sales: $closedShop::CurrencyMarker $totalsales</td></tr>
			 </table>
		|;
    BottomAdmin();

    closedShop::Footer();
}

###############################################################################
# FUNCTION:  ExportData
# DESCRIPTION: exports data
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ExportData {
    if ( closedShop::ValidateInputAdmin( $query->param("selected") ) == 1 ) {
        my $data = closedShop::ValidateInputAdmin( $query->param("data") );
        if ( $data eq "orders" ) {
            ExportOrders( closedShop::ValidateInputAdmin( $query->param("format") ) );
        }
        elsif ( $data eq "products" ) {
            ExportProducts( closedShop::ValidateInputAdmin( $query->param("format") ) );
        }
        elsif ( $data eq "customers" ) {
            ExportCustomers( closedShop::ValidateInputAdmin( $query->param("format") ) );
        }
        else {
            $query = new CGI( {} );
            ExportData();
        }
    }
    else {
        closedShop::Header();
        print qq|
                <font class="main">
          <form action="$program_name" method="POST">
          <input type="hidden" name="action" value="ExportData">
          <input type="hidden" name="selected" value="1">
          <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
          <font class="head">$closedShop::TEXT{'EXPORT'}</th></tr>
           <tr class="mainrow">
                 <td width='50%'><font class="main"> 
		 $closedShop::TEXT{'SELECT_DATA'}:<BR>
		 $closedShop::TEXT{'EXP_INSTR'}</td>
	<td width='50%'> <font class="main">
	<input type='radio' name='data' value='products'>$closedShop::TEXT{'PRODS'}<BR>
	<input type='radio' name='data' value='orders'>$closedShop::TEXT{'DL_HIS'}<BR>
	<input type='radio' name='data' value='customers'>$closedShop::TEXT{'CUST'}<BR>
            </td></tr>
	    
	    <tr class="mainrow">
                 <td width='50%'><font class="main"> 
		 Data Format</td>
	<td width='50%'> <font class="main">
	<input type='radio' name='format' value='textcomma'>Text (comma separated)<BR>
	<input type='radio' name='format' value='texttab'>Text (tab separated)<BR>
	<input type='radio' name='format' value='excel'>Excel <BR>
	<i>(need Spreadsheet::WriteExcel module)</i>
            </td></tr>
	    
            <tr class="mainrow"><td colspan="2"><centeR>
               <input type="submit" value="$closedShop::TEXT{'EXPORT'}"></td></tr>
		</form>           

	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
	</table>
        
                 |;
        BottomAdmin();

        closedShop::Footer();

    }
}

###############################################################################
# FUNCTION:  ImportProducts
# DESCRIPTION: imports a list of products
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ImportProducts {
    my $excelmesg = $closedShop::TEXT{'EXCEL_MESG'};

    if ( $query->param("selected") == 1 ) {
        my $category = $query->param("category");
        my $file     = $query->param("file");

        my @data = ();
        if ($file) {
            while (<$file>) { push @data, $_; }
        }
        my @file     = split( /\./, $file );
        my $numcount = 0;

        if ( $file[$#file] =~ /xls/ ) {
            eval("use Spreadsheet::ParseExcel");
            if ($@) {
                return AdminScreen($excelmesg);
            }
            else {
                open( FILE, ">tmplist.xls" );
                print FILE @data;
                close(FILE);

                require Spreadsheet::ParseExcel;
                my $oBook = Spreadsheet::ParseExcel::Workbook->Parse('tmplist.xls');
                my ( $iR, $iC, $oWkS, $oWkC );
                foreach my $oWkS ( @{ $oBook->{Worksheet} } ) {
                    for ( my $iR = $oWkS->{MinRow} + 1 ; defined $oWkS->{MaxRow} && $iR <= $oWkS->{MaxRow} ; $iR++ ) {
                        my $SQL = qq| insert into $closedShop::sql_tables{"item"} set
            			  category="$category",|;
                        my $j = 0;
                        for ( my $iC = $oWkS->{MinCol} ; defined $oWkS->{MaxCol} && $iC <= $oWkS->{MaxCol} ; $iC++ ) {
                            $oWkC = $oWkS->{Cells}[$iR][$iC];
                            my $field = $oWkS->{Cells}[ $oWkS->{MinRow} ][$iC];
                            $SQL .= lc( $field->Value ) . qq|="| . $oWkC->Value . qq|"|;
                            if ( $j != $oWkS->{MaxCol} ) { $SQL .= qq|,|; }
                            $j++;
                        }
                        my $sth = closedShop::DatabaseQuery($SQL);

                        $numcount++;
                    }
                }
            }
        }
        else {
            my @fieldlist = split( /\",\"/, $data[0] );
            for ( my $k = 1 ; $k < @data ; $k++ ) {
                chomp( $data[$k] );
                if ( !$data[$k] ) { next; }
                my @list = split( /\",\"/, $data[$k] );
                my $SQL = qq| insert into $closedShop::sql_tables{"item"} set
			category="$category",|;
                for ( my $j = 0 ; $j < @fieldlist ; $j++ ) {
                    $fieldlist[$j] =~ s/"//g;
                    $fieldlist[$j] = lc( $fieldlist[$j] );
                    $list[$j] =~ s/"//g;
                    $SQL .= qq|$fieldlist[$j]="$list[$j]"|;
                    if ( $j != $#fieldlist ) { $SQL .= qq|,|; }
                }
                $numcount++;
                my $sth = closedShop::DatabaseQuery($SQL);
            }
        }

        return AdminScreen( $closedShop::TEXT{'DONE_IM'} . $numcount );
    }
    else {
        closedShop::Header();
        print qq|
                <font class="main">
          <form action="$program_name"
enctype="multipart/form-data" method="POST">
          <input type="hidden" name="action" value="ImportProducts">
          <input type="hidden" name="selected" value="1">
          <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
          <font class="head">$closedShop::TEXT{'IMPORT'}</th></tr>
           <tr class="mainrow">
                 <td><font class="main"> $closedShop::TEXT{'SELECT'} $closedShop::TEXT{'CAT'}:</td>
	<td> <font class="main">|;
        closedShop::PrintCategoryList( "category", 0, "ALL" );
        print qq|
            </td></tr>
    <tr class="mainrow">
	<td><font class="main"> $closedShop::TEXT{'CH_FILE'}:</td>
	<td><font class="main"> <input type="file" name="file" size="20"><BR>
	$closedShop::TEXT{'SEE_IN'}
    </tr>
            <tr class="mainrow"><td colspan="2"><centeR>
               <input type="submit" value="$closedShop::TEXT{'IMPORT'}"></td></tr>
            <tr class="mainrow"><td colspan="2">
<font class="main">
$closedShop::TEXT{'IMP_IN1'}
<BR>
<code>
<dd>"name","description","product_code","price","shipping_price"</dd><BR>
<dd>"Green Widgets","This is a green 
widget","GW123","32.50","2.00"</dd><BR>
<dd>"Blue Widgets","This is a blue widget","BW123","10.50",""</dd><BR>
</code><BR><font class="main">
$closedShop::TEXT{'IMP_IN2'}<BR><BR>

	</td></tr>

	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
	</table>
       
                 |;
        BottomAdmin();

        closedShop::Footer();

    }

}

###############################################################################
# FUNCTION:  PrintEbaySubCategory
# DESCRIPTION: prints teh ebay sub cat
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub PrintEbaySubCategory {
    my $main = $query->param("main");
    my $sub1 = $query->param("sub1");
    if ( !$sub1 ) { $sub1 = -1; }
    my $sub2 = $query->param("sub2");
    if ( !$sub2 ) { $sub2 = -1; }
    my $sub3 = $query->param("sub3");
    if ( !$sub3 ) { $sub3 = -1; }
    my $sub4 = $query->param("sub4");
    if ( !$sub4 ) { $sub4 = -1; }
    my $sub5 = $query->param("sub5");
    if ( !$sub5 ) { $sub5 = -1; }

    my $SQL = qq|select name from $closedShop::sql_tables{"ebay"}
	where id="$main"|;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ($mname) = $sth->fetchrow_array();

    print $closedShop::HTML_HEADER;
    print qq|<html>
	<head>|;
    closedShop::WriteStyleSheets();
    print qq|<script language="JavaScript">
	<!--
	function MakeNewList() {
	 var sub1 = document.subform1.sub1.value;
	 var sub2 = document.subform1.sub2.value;
	 var sub3 = document.subform1.sub3.value;
	 var sub4 = document.subform1.sub4.value;
	 var sub5 = document.subform1.sub5.value;
         var url = "$closedShop::FileNames{"admin"}?action=PrintEbaySubCategory&main=$main&sub1="+sub1+"&sub2="+sub2+"&sub3="+sub3+"&sub4="+sub4;
	 window.location.href = url;
	}
	function Submitted() {
	   if (document.subform1.sub1.value != "") {
             var w = document.subform1.sub1.selectedIndex;
	     var text = document.subform1.sub1.options[w].text;
             window.opener.document.stockForm.CategoryMenu1.value = document.subform1.sub1.value;
             window.opener.document.stockForm.subcat1.value = text;
           }
	   else {
	     window.opener.document.stockForm.subcat1.value = "";
	     window.opener.document.stockForm.CategoryMenu1.value = "";
	   }
	   if (document.subform1.sub2.value != "") {
             var w = document.subform1.sub2.selectedIndex;
	     var text = document.subform1.sub2.options[w].text;
             window.opener.document.stockForm.CategoryMenu2.value = document.subform1.sub2.value;
             window.opener.document.stockForm.subcat2.value = text;
           }
	   else {
	     window.opener.document.stockForm.subcat2.value = "";	
	     window.opener.document.stockForm.CategoryMenu2.value = "";
	   }

	   if (document.subform1.sub3.value != "") {
             var w = document.subform1.sub3.selectedIndex;
	     var text = document.subform1.sub3.options[w].text;
             window.opener.document.stockForm.CategoryMenu3.value = document.subform1.sub3.value;
             window.opener.document.stockForm.subcat3.value = text;
           }
	   else {
	     window.opener.document.stockForm.subcat3.value = "";	
	     window.opener.document.stockForm.CategoryMenu3.value = "";
	   }

	   if (document.subform1.sub4.value != "") {
             var w = document.subform1.sub4.selectedIndex;
	     var text = document.subform1.sub4.options[w].text;
             window.opener.document.stockForm.CategoryMenu4.value = document.subform1.sub4.value;
             window.opener.document.stockForm.subcat4.value = text;
           }
	   else {
	     window.opener.document.stockForm.subcat4.value = "";	
	     window.opener.document.stockForm.CategoryMenu4.value = "";
	   }

	   if (document.subform1.sub5.value != "") {
             var w = document.subform1.sub5.selectedIndex;
	     var text = document.subform1.sub5.options[w].text;
             window.opener.document.stockForm.CategoryMenu5.value = document.subform1.sub5.value;
             window.opener.document.stockForm.subcat5.value = text;
           }
	   else {
	     window.opener.document.stockForm.subcat5.value = "";	
	     window.opener.document.stockForm.CategoryMenu5.value = "";
	   }

	 self.close();
	}
	-->
	</script>
	</head>
	<body bgcolor='white'><center>
  <table border="0" width="90%"><tr class="headrow"><td
colspan="2"><font class="head">
<center>$closedShop::TEXT{'SELECT'} $closedShop::TEXT{'SUBCAT'}
   for<BR>$mname</center></td></tr>
   <form name="subform1" action="javascript:Submitted()">
	<input type="hidden" name="main" value="$main">
  <tr class="mainrow"><td><font class="main">
  Sub-Category 1: </td><td><font class="main">|;
    closedShop::PrintEbaySubCat( "sub1", $sub1, $main );
    print qq|</td></tr><tr  class="mainrow"><td>
  <font class="main"> Sub-Category 2:</td><td><font class="main">|;
    closedShop::PrintEbaySubCat( "sub2", $sub2, $sub1 );
    print qq|</td></tr><tr class="mainrow"><td>
  <font class="main"> Sub-Category 3:</td><td><font class="main">|;
    closedShop::PrintEbaySubCat( "sub3", $sub3, $sub2 );
    print qq|</td></tr><tr  class="mainrow"><td>
  <font class="main"> Sub-Category 4:</td><td><font class="main">|;
    closedShop::PrintEbaySubCat( "sub4", $sub4, $sub3 );
    print qq|</td></tr><tr  class="mainrow"><td>
  <font class="main"> Sub-Category 5:</td><td><font class="main">|;
    closedShop::PrintEbaySubCat( "sub5", $sub5, $sub4 );
    print qq| </td></tr></table><BR><input type="submit" value="$closedShop::TEXT{'ENT_SUB'}"></form>
   </body></html> |;
}

###############################################################################
# FUNCTION:  BackupSQL
# DESCRIPTION: backup SQL database
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub BackupSQL {
    if ( $query->param("selected") == 1 ) {
        my $file = $query->param("file");
        open( FILE, ">$file" ) || closedShop::ErrorMessage("can't open $file");
        my $run = "mysqldump -u$closedShop::DatabaseUsername -p$closedShop::DatabasePassword --databases $closedShop::DatabaseName > $file";
        $run = `$run`;
        return AdminScreen( "$closedShop::TEXT{'BCK_MESG'}<BR>" . `pwd` . "/$file" );
    }
    else {
        closedShop::Header();
        print qq|
    		<font class="main">
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="BackupSQL">
	  <input type="hidden" name="selected" value="1">
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'BK_SQL'}</th></tr>
    <tr class="mainrow">
		 <td><font class="main"> $closedShop::TEXT{'BKSQL_MESG'}</td>
       <td><font class="main">
	 <input type="text" name="file" size="25" value="csBackupSQL.sql"></td>
	   </tr>   
	     <tr class="headrow"><td colspan="2"><centeR>
	       <input type="submit" value="$closedShop::TEXT{'BK_SQL'}"></td></tr></table>
	
		 |;
        BottomAdmin();

        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  GiftCertificate
# DESCRIPTION: gift certificate management
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub GiftCertificate {
    if ( $query->param("selected") eq "Denomination" ) {
        my @denom = $query->param("denom");
        my $d     = join( $closedShop::GiftCertDelimiter, @denom );

        open( FILE, "$closedShop::varsfile" );
        my @data = <FILE>;
        close(FILE);

        my @newdata = ();
        foreach (@data) {
            my @ar = split( /$closedShop::VarsFileDelimiter/, $_ );
            if ( $ar[0] ne "GiftCertificates" ) {
                push @newdata, $_;
            }
        }

        push @newdata, "GiftCertificates" . $closedShop::VarsFileDelimiter . $d . "\n";

        open( FILE, ">$closedShop::varsfile" );
        flock( FILE, LOCK_EX );
        print FILE join( "", @newdata );
        flock( FILE, LOCK_UN );
        close(FILE);
        return AdminScreen( $closedShop::TEXT{'GC_UP'} );
    }
    elsif ( $query->param("selected") eq "Enable" ) {
        my $id  = closedShop::ValidateInputAdmin( $query->param("id") );
        my $SQL = qq|update $closedShop::sql_tables{"gc"} set active="Y" 
	             where id="$id"|;
        my $sth = closedShop::DatabaseQuery($SQL);
        $query = new CGI( {} );
        return GiftCertificate( $closedShop::TEXT{'GC_EN'} );

    }
    else {
        my $mesg = $_[0];
        closedShop::Header();
        my @denoms = closedShop::GetGiftCertificateInfo();
        my %marks  = ();
        foreach (@denoms) {
            $marks{"$_"} = "checked";
        }

        print qq|
		<font class="main">
		<font color="red">$mesg</font>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="GiftCertificate">
	  <input type="hidden" name="selected" value="Denomination">
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'GC_MNG'}</th></tr>
    <tr class="mainrow">
		 <td colspan="2"><font class="main"> $closedShop::TEXT{'GC_MESG2'}<BR>
		 |;
        my $url = "$closedShop::FileNames{item}?action=ShowGiftCertificates";
        if ( $url !~ /^http/ ) {
            $url = $closedShop::urltofolder . "/" . $url;
        }
        print qq|<a href="$url">$url</a><BR>	</td>	</tr>
		<tr class="mainrow">
		 <td width="180"><font class="main"> $closedShop::TEXT{'GC_MESG3'}</td>
	    <td><font class="main"> 
			<input type="checkbox" name="denom" |
          . $marks{"5.00"} . qq| value="5.00">$closedShop::CurrencyMarker 5.00<BR>
			<input type="checkbox" name="denom" |
          . $marks{"10.00"} . qq| value="10.00">$closedShop::CurrencyMarker 10.00<BR>
			<input type="checkbox" name="denom" |
          . $marks{"25.00"} . qq| value="25.00">$closedShop::CurrencyMarker 25.00<BR>
			<input type="checkbox" name="denom" |
          . $marks{"50.00"} . qq| value="50.00">$closedShop::CurrencyMarker 50.00<BR>
			<input type="checkbox" name="denom" |
          . $marks{"100.00"} . qq| value="100.00">$closedShop::CurrencyMarker 100.00<BR>
			<input type="checkbox" name="denom" |
          . $marks{"200.00"} . qq| value="200.00">$closedShop::CurrencyMarker 200.00<BR>
			<input type="checkbox" name="denom" |
          . $marks{"500.00"} . qq| value="500.00">$closedShop::CurrencyMarker 500.00<BR>
			<BR>
			<input type="submit" value="$closedShop::TEXT{'NEW_DNM'}">
			</td></form>
		</tr>	
			<tr class="mainrow">
		 <td><font class="main"> $closedShop::TEXT{'O_GC'} </td>
	    <td><font class="main"> 
			|;
        my $SQL = qq|select id,amount_left,initial_amount,customer_name,customer_id,
		DATE_FORMAT(date_bought,"%M %e %Y"),gc_code,active from
		$closedShop::sql_tables{"gc"} where closed != 'Y' order by date_bought ASC|;
        my $sth = closedShop::DatabaseQuery($SQL);
        my $k;

        for ( $k = 1 ; my (@ary) = $sth->fetchrow_array() ; $k++ ) {
            my $style  = "#ffe4c4";
            my $active = qq|<a href="$closedShop::FileNames{admin}?action=GiftCertificate&selected=Enable&id=$ary[0]">$closedShop::TEXT{'ENABLE'} $closedShop::TEXT{'GC'}</a>|;
            if ( $ary[7] eq "Y" ) {
                $active = $closedShop::TEXT{'ACTI'};
            }
            if ( $k % 2 ) { $style = "#faebd7"; }
            print qq| <Center><table  class="main" cellpadding="0" 
				   cellspacing="2" width="100%"><tr bgcolor="$style"><td>
					 <font class="main">
					 $k) $ary[3] ($closedShop::TEXT{'BO_ON'} $ary[5])<BR> $closedShop::TEXT{'RM_BC'}: $closedShop::CurrencyMarker
					 $ary[1] / $closedShop::CurrencyMarker $ary[2]<BR>
					 GC Code: $ary[6] - $active</td></tr></table>|;
        }

        if ( $k == 1 ) {
            print qq|<font class="main"> $closedShop::TEXT{'NO_OGC'}|;
        }

        print qq|</td></tr>
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr></table>
			|;
        BottomAdmin();

        closedShop::Footer();

    }
}

###############################################################################
# FUNCTION:  ExportCustomers
# DESCRIPTION: download customer log
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ExportCustomers {
    my ($format) = @_;

    my $SQL = qq| select fname,lname,email,address,city,state,country,zip,phone,
    $closedShop::sql_tables{"groups"}.name,$closedShop::sql_tables{"user"}.id
	  from ($closedShop::sql_tables{"user"} LEFT JOIN $closedShop::sql_tables{"groups"}
	  ON $closedShop::sql_tables{"groups"}.id = $closedShop::sql_tables{"user"}.groupid)
	  where email != "" order by lname  |;
    my $sth = closedShop::DatabaseQuery($SQL);

    my $sep = "";
    my ( $workbook, $worksheet, $style );
    if ( $format eq "textcomma" ) {
        print "Content-type: text/html \n\n<pre>";
        $sep = ",";
    }
    elsif ( $format eq "texttab" ) {
        print "Content-type: text/html \n\n<pre>";
        $sep = "\t";
    }
    elsif ( $format eq "excel" ) {

        eval("use Spreadsheet::WriteExcel");
        if ($@) {
            return AdminScreen(
                "You need to install SpreadSheet::WriteExcel to 
	 export in Excel format"
            );
        }
        require Spreadsheet::WriteExcel;
        print "Content-type: application/vnd.ms-excel\n\n";
        $workbook  = Spreadsheet::WriteExcel->new("-");
        $worksheet = $workbook->add_worksheet( $closedShop::TEXT{'CUST'} );
    }

    my $y = 0;
    foreach (
        qw/ Count FirstName LastName Email Address City State Country
        Zip Phone GroupName ID/
      )
    {
        if ( $format eq "excel" ) {
            $worksheet->write( 0, $y, $_ );
        }
        else {
            print qq|"$_"$sep|;
        }
        $y++;
    }

    if ( $format eq "excel" ) {

        # $style->set_bold(0);
        # $style->set_color('white');
    }
    else {
        print qq|<BR>|;
    }

    my $k = 1;
    while ( my (@ary) = $sth->fetchrow_array() ) {
        $_ =~ s/<BR>//g;
        $_ =~ s/\n//g;
        if ( $format eq "excel" ) {
            $worksheet->write( $k, 0, $k, $style );
        }
        else {
            print "$k$sep";
        }

        my $e = 1;
        foreach (@ary) {
            if ( $format eq "excel" ) {
                $worksheet->write( $k, $e, $_ );
            }
            else {
                print qq|"$_"$sep|;
            }
            $e++;
        }

        if ( $format ne "excel" ) {
            print qq|<BR>|;
        }
        $k++;
    }

}

###############################################################################
# FUNCTION:  ExportProducts
# DESCRIPTION: download product log
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ExportProducts {
    my ($format) = @_;

    my $SQL = qq| select name,product_code,category,price,shipping_price,
	              reduced_shipping_price,description,
		      quantity,id  from $closedShop::sql_tables{"item"} order by name  |;
    my $sth = closedShop::DatabaseQuery($SQL);

    my $sep = "";
    my ( $workbook, $worksheet, $style );
    if ( $format eq "textcomma" ) {
        print "Content-type: text/html \n\n<pre>";
        $sep = ",";
    }
    elsif ( $format eq "texttab" ) {
        print "Content-type: text/html \n\n<pre>";
        $sep = "\t";
    }
    elsif ( $format eq "excel" ) {
        eval("use Spreadsheet::WriteExcel");
        if ($@) {
            return AdminScreen(
                "You need to install SpreadSheet::WriteExcel to 
	 export in Excel format"
            );
        }
        require Spreadsheet::WriteExcel;
        print "Content-type: application/vnd.ms-excel\n\n";
        $workbook  = Spreadsheet::WriteExcel->new("-");
        $worksheet = $workbook->add_worksheet( $closedShop::TEXT{'PRODS'} );
    }

    my $y = 0;
    foreach (
        qw/Count Name ProductCode Category Price ShippingPrice ReducedShippingPrice
        Description Attributes QuantityRemaining ID/
      )
    {
        if ( $format eq "excel" ) {
            $worksheet->write( 0, $y, $_ );
        }
        else {
            print qq|"$_"$sep|;
        }
        $y++;
    }

    if ( $format eq "excel" ) {

        # $style->set_bold(0);
        # $style->set_color('white');
    }
    else {
        print qq|\n|;
    }

    my $k = 1;
    while ( my (@ary) = $sth->fetchrow_array() ) {

        my $SQL2 = qq|select $closedShop::sql_tables{"attributes"}.name,
		$closedShop::sql_tables{"attribute_values"}.data,price from
		$closedShop::sql_tables{"attributes"},$closedShop::sql_tables{"attribute_values"}
		where product_id='$ary[8]' and $closedShop::sql_tables{"attribute_values"}.attribute_id =
		$closedShop::sql_tables{"attributes"}.id order by $closedShop::sql_tables{"attributes"}.name |;
        my $sth2       = closedShop::DatabaseQuery($SQL2);
        my $attributes = '';
        while ( my (@ats) = $sth2->fetchrow_array() ) {
            $attributes .= "$ats[0]: $ats[1],";
        }
        $attributes = substr( $attributes, 0, length($attributes) - 1 );

        $ary[9] = $ary[8];
        $ary[8] = $ary[7];
        $ary[7] = $attributes;

        if ( $format eq "excel" ) {
            $worksheet->write( $k, 0, $k );
        }
        else {
            print "$k$sep";
        }

        my $e = 1;
        foreach (@ary) {
            $_ =~ s/\n//g;
            $_ =~ s/<br[^.]*>//ig;
            $_ =~ s/<p>/ /ig;
            $_ =~ s/<\/p>//ig;
            if ( $format eq "excel" ) {
                $worksheet->write( $k, $e, $_ );
            }
            else {
                print qq|"$_"$sep|;
            }
            $e++;
        }

        if ( $format ne "excel" ) {
            print qq|\n|;
        }
        $k++;
    }

}

###############################################################################
# FUNCTION:  ExportOrders
# DESCRIPTION: download order log
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ExportOrders {
    my ($format) = @_;

    my $SQL = qq| select DATE_FORMAT(date_bought,"%W, %M %d %Y"),
	              total,b_fname,b_lname,b_address,b_city,b_state,
                  b_zip,b_country,b_phone,s_fname,s_lname,s_address,s_city,s_state,
                  s_zip,s_country,s_phone,cc_name,cc,cc_num,cc_month,cc_year,cc_vcode,item_ids,
		    ip_address,id  from $closedShop::sql_tables{"order"} 
		          where complete="Y" order by date_bought  |;
    my $sth = closedShop::DatabaseQuery($SQL);

    my $sep = "";
    my ( $workbook, $worksheet, $style );
    if ( $format eq "textcomma" ) {
        print "Content-type: text/html \n\n<pre>";
        $sep = ",";
    }
    elsif ( $format eq "texttab" ) {
        print "Content-type: text/html \n\n<pre>";
        $sep = "\t";
    }
    elsif ( $format eq "excel" ) {
        eval("use Spreadsheet::WriteExcel");
        if ($@) {
            return AdminScreen(
                "You need to install SpreadSheet::WriteExcel to 
	 export in Excel format"
            );
        }
        require Spreadsheet::WriteExcel;
        print "Content-type: application/vnd.ms-excel\n\n";
        $workbook  = Spreadsheet::WriteExcel->new("-");
        $worksheet = $workbook->add_worksheet( $closedShop::TEXT{'PRODS'} );
    }

    my $y = 0;
    foreach (
        qw/Count Date_Bought Total Billing_FirstName Billing_LastName Billing_Address Billing_City
        Billing_State Billing_Zip Billing_Country Billing_Phone Shipping_FirstName
        Shipping_LastName Shipping_Address Shipping_City Shipping_State
        Shipping_Zip Shipping_Country Shipping_Phone CC_Name Credit_Card
        CC_Number CC_Month CC_Year CCV_Code Items IP_Address ID/
      )
    {

        if ( $format eq "excel" ) {
            $worksheet->write( 0, $y, $_ );
        }
        else {
            print qq|"$_"$sep|;
        }
        $y++;
    }

    if ( $format ne "excel" ) {
        print qq|\n|;
    }

    if ( $format eq "excel" ) {

        # $style->set_bold(0);
        #  $style->set_color('white');
    }

    my $k = 1;
    while ( my (@ary) = $sth->fetchrow_array() ) {
        my $j = 0;
        if ( $format eq "excel" ) {
            $worksheet->write( $k, 0, $k );
        }
        else {
            print "$k$sep";
        }

        foreach (@ary) {
            $_ =~ s/\n//g;
            $_ =~ s/<br[^.]*>//ig;
            $_ =~ s/<p[^.]*>//ig;
            if ( $j == 24 ) {
                my @items = split( /$closedShop::CookieItemDelimiter/, $_ );
                my $t     = 1;

                my $itemdisplay = "";
                foreach my $its (@items) {
                    if ( !$its ) { next; }
                    my ( $iid, $qty, $attr ) = split( /$closedShop::CookieSectionDelimiter/, $its );
                    if ( $iid eq "gc" ) {
                        my ( $id, $recipient_name, $recipient_email, $gc_amount ) =
                          split( /$closedShop::CookieSectionDelimiter/, $its );
                        $itemdisplay .= qq|Gift Certificate for $recipient_name for $closedShop::CurrencyMarker $gc_amount; |;
                    }
                    else {
                        my $SQL2 = qq| select name,price,shipping_price,description,
	                    category,product_code from $closedShop::sql_tables{"item"} where
				  id="$iid" |;
                        my $sth2 = closedShop::DatabaseQuery($SQL2);
                        my ( $name, $price, $ship, $desc, $cat, $code ) = $sth2->fetchrow_array();

                        my @attr = ();
                        if ($attr) {
                            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
                        }

                        my $attr      = "";
                        my $attr_cost = 0.00;
                        foreach my $at (@attr) {
                            if ( !$at ) { next; }
                            my ( $prod_id, $attr_id, $data_id ) = split( /$closedShop::GiftCertDelimiter/, $at );
                            my $SQL3 = qq|select name,data,price from $closedShop::sql_tables{"attributes"},
	   		      $closedShop::sql_tables{"attribute_values"} where product_id='$prod_id' and
	   		      $closedShop::sql_tables{"attributes"}.id = '$attr_id' and
	   		      $closedShop::sql_tables{"attribute_values"}.id = '$data_id'|;
                            my $sth3 = closedShop::DatabaseQuery($SQL3);
                            my ( $atname, $atdata, $atprice ) = $sth3->fetchrow_array();
                            $attr .= "$atname=$atdata";
                            if ( $atprice > 0 ) {
                                $attr .= " (+$closedShop::CurrencyMarker$atprice)";
                                $attr_cost += $atprice;
                            }
                            $attr .= ",";
                        }
                        $attr = substr( $attr, 0, length($attr) - 1 );

                        my $total = sprintf( "%.2f", $qty * ( $price + $ship + $attr_cost ) );
                        $itemdisplay .= qq|$name ($code) x $qty ($attr) ($closedShop::categories{$cat}) Total Cost: $closedShop::CurrencyMarker $total; |;
                    }
                }
                if ( $format eq "excel" ) {
                    $worksheet->write( $k, $j + 1, $itemdisplay );
                }
                else {
                    print qq|"$itemdisplay"$sep|;
                }

            }
            else {
                if ( $format eq "excel" ) {
                    $worksheet->write( $k, $j + 1, $_ );
                }
                else {
                    print qq|"$_"$sep|;
                }
            }

            $j++;
        }
        if ( $format ne "excel" ) {
            print qq|\n|;
        }

        $k++;
    }
}

###############################################################################
# FUNCTION:  EmailCustomers
# DESCRIPTION: emails all customers
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub EmailCustomers {
    my ( $subject, $body ) = @_;
    my $k = 0;

    my $SQL = qq| select email,fname,lname from $closedShop::sql_tables{"user"}
	  where email != ""|;
    my $sth = closedShop::DatabaseQuery($SQL);
    while ( my ( $em, $fname, $lname ) = $sth->fetchrow_array() ) {
        my $newsub  = $subject;
        my $newbody = $body;

        my ( $fn, $ln ) = ( ":fname:", ":lname:" );

        $newsub  =~ s/$fn/$fname/g;
        $newbody =~ s/$fn/$fname/g;

        $newsub  =~ s/$ln/$lname/g;
        $newbody =~ s/$ln/$lname/g;

        closedShop::SendEmail( $em, $newsub, $newbody );
        $k++;
    }

}

###############################################################################
# FUNCTION:  EditCategories
# DESCRIPTION: add a product to database
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub EditCategories {
    if ( $query->param("selected") eq "RemoveCategory" ) {
        if ( $query->param("delete") eq "Y" ) {
            my $cat = closedShop::ValidateInputAdmin( $query->param("category") );
            my $SQL = qq|select parent,children,id from $closedShop::sql_tables{"cat"}
	where code="$cat"|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $pa, $ch, $id ) = $sth->fetchrow_array();

            my @child = split( /~/, $ch );
            foreach (@child) {
                if ( !$_ ) { next; }
                $SQL = qq|update $closedShop::sql_tables{"cat"} set parent="$pa"
	  where id="$_"|;
                $sth = closedShop::DatabaseQuery($SQL);
            }

            $SQL = qq|select children from $closedShop::sql_tables{"cat"} 
	  where id="$pa"|;
            $sth = closedShop::DatabaseQuery($SQL);
            my ($oldch) = $sth->fetchrow_array();

            $oldch =~ s/~$id~//g;
            $oldch .= $ch;

            $SQL = qq|update $closedShop::sql_tables{"cat"} set children="$oldch"
	where id="$pa"|;
            $sth = closedShop::DatabaseQuery($SQL);

            $SQL = qq|delete from $closedShop::sql_tables{"cat"} where
	code = "$cat" |;
            $sth = closedShop::DatabaseQuery($SQL);
            $SQL = qq|delete from $closedShop::sql_tables{"item"} where
	category = "$cat" |;
            $sth = closedShop::DatabaseQuery($SQL);
            return AdminScreen( $closedShop::TEXT{'SUCC_RMC'} );
        }
        return AdminScreen( $closedShop::TEXT{'FAIL_RMC'} );
    }
    elsif ( $query->param("selected") eq "AddSubCategory" ) {
        my $cat = $query->param("category");
        my $sub = $query->param("subcat");
        my $SQL = qq|select id,children from $closedShop::sql_tables{"cat"} where
	code = "$cat" |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ( $parent, $children ) = $sth->fetchrow_array();
        my $code = closedShop::GetNewCategoryCode($sub);

        $SQL = qq|insert into $closedShop::sql_tables{"cat"} set name="$sub",
      parent="$parent",code="$code",status="PUBLIC",style="text"|;
        $sth = closedShop::DatabaseQuery($SQL);
        my $newid = $sth->{'mysql_insertid'};

        $children .= "~$newid~";
        $SQL = qq|update $closedShop::sql_tables{"cat"} set children="$children"
      where id ="$parent"|;
        $sth = closedShop::DatabaseQuery($SQL);

        return AdminScreen( $closedShop::TEXT{'SUCC_ADDSUB'} );
    }
    elsif ( $query->param("selected") == 1 ) {
        foreach ( sort keys %closedShop::categories ) {
            my $intro = closedShop::ValidateInputAdmin( $query->param("intro_$_") );
            $intro =~ s/"/'/g;
            my $name = closedShop::ValidateInputAdmin( $query->param("name_$_") );
            $name =~ s/"/'/g;
            my $style = closedShop::ValidateInputAdmin( $query->param("style_$_") );
            $style =~ s/"/'/g;
            my $numimages = closedShop::ValidateInputAdmin( $query->param("numimages_$_") );
            my $status    = closedShop::ValidateInputAdmin( $query->param("status_$_") );

            my $SQL = qq|update $closedShop::sql_tables{"cat"} set name="$name",
               intro="$intro",style="$style",numimages="$numimages",status="$status"
               where code="$_"|;
            my $sth = closedShop::DatabaseQuery($SQL);
        }

        return AdminScreen( $closedShop::TEXT{'SUCC_EDC'} );
    }
    else {
        closedShop::Header();
        print qq|<font class="main"><b>$closedShop::TEXT{'JM_C'}</b>: |;
        my $t = 0;
        foreach ( sort keys %closedShop::categories ) {
            print qq|<A href="#$_">$closedShop::categories{$_}</a>|;
            if ( $t % 3 == 2 ) { print "<BR>"; }
            else { print " - "; }
            $t++;
        }
        print qq|
	      <Table  class="main" width="$closedShop::TableWidth"><tr><td>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="EditCategories">
	  <input type="hidden" name="selected" value="1">
	  <table width="60%"  class="main"><tr class="headrow"><th>
	  <font class="head">$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'CAT'}</th></tr>
      |;

        foreach ( sort keys %closedShop::categories ) {
            my ( $intro, $style, $numimages, $status, $name, $child, $parent ) = closedShop::GetCategoryInfo($_);
            my $SQL1 = qq|select name,code from $closedShop::sql_tables{"cat"} 
	      where id="$parent"|;
            my $sth1 = closedShop::DatabaseQuery($SQL1);
            my ( $parent_name, $pcode ) = $sth1->fetchrow_array();

            my $checktext    = "";
            my $checkthumb   = "";
            my $checkprivate = "";
            my $checkpublic  = "";
            if ( $style eq "thumbnail" ) { $checkthumb = "checked"; }
            else { $checktext = "checked"; }

            if ( $status eq "PRIVATE" ) { $checkprivate = "checked"; }
            else { $checkpublic = "checked"; }

            print qq|
          <tr class="mainrow"><td>
		  <font class="main"> <A name="$_">$closedShop::TEXT{'CAT'} $closedShop::TEXT{'NAME'}</a>:
		    <BR><input type="text" name="name_$_" 
                  value="$closedShop::categories{$_}" size="25"><BR>|;
            if ($parent_name) {
                print qq|$closedShop::TEXT{'PARENT'} $closedShop::TEXT{'CAT'}:
			     <a href="#$pcode">$parent_name</a><BR>|;
            }
            print qq|
	      <b>$closedShop::TEXT{'INT_TXT'}</b><br>
		  <textarea  mce_editable='true' style='height: 170px; width: 230px;' name="intro_$_">$intro</textarea><br>
		  <br>
		  <b>$closedShop::TEXT{'CAT'} $closedShop::TEXT{'DSP_T'}</b><br>
		  <input type="radio" name="style_$_" $checktext value="text"> $closedShop::TEXT{'TXT_D'}
		  &nbsp;&nbsp;&nbsp;
		  <input type="radio" name="style_$_" $checkthumb value="thumbnail"> $closedShop::TEXT{'THM_D'}<br>
		  $closedShop::TEXT{'NUM_THM'}: <select name="numimages_$_">
		  |;
            if ($numimages) {
                print qq|<option value="$numimages">$numimages $closedShop::TEXT{'ITMS'}</option>|;
            }
            print qq|
		  <option value="16">16 $closedShop::TEXT{'ITMS'}</option>
		  <option value="8">8 $closedShop::TEXT{'ITMS'}</option>
		  <option value="4">4 $closedShop::TEXT{'ITMS'}</option>
		  <option value="2">2 $closedShop::TEXT{'ITMS'}</option>
		  <option value="1">1 $closedShop::TEXT{'ITMS'}</option>
		  </select><br>
		  
		  <br><strong>$closedShop::TEXT{'CAT'} $closedShop::TEXT{'STAT'}</strong><br>
		  <input type="radio" name="status_$_" $checkpublic value="PUBLIC"> $closedShop::TEXT{'PUBV'}
		  &nbsp;&nbsp;&nbsp;
		  <input type="radio" name="status_$_" $checkprivate value="PRIVATE">
		  $closedShop::TEXT{'PRIVH'}<br>
		  </td></tr>
		  <tr class='headrow'><td colspan='2'>&nbsp;</td></tr>
		|;
        }

        print qq|
	  <tr class="headrow">
	  <td colspan="2"><div align="center"><input type="submit" value="$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'CAT'}"></div></td>
	  </tr></form></table>
	    </td><td width="5">&nbsp;</td>
	      <form action="$program_name" method="POST">
		<input type="hidden" name="action" value="EditCategories">
		<input type="hidden" name="selected" value="AddSubCategory">
	      <td valign="top">
		 <table width="35%"  class="main"><tr class="headrow"><th colspan="2">
	  <font class="main"> <font size="3"><font color="blue">$closedShop::TEXT{'ENT_SUB'}</font></th></tr>
	    <tr class="mainrow"><td> <font class="main">$closedShop::TEXT{'SELECT'} $closedShop::TEXT{'CAT'}:</td>
	      <td> <font class="main">|;

        closedShop::PrintCategoryList( "category", 0, "ALL" );

        print qq|
	    </td></tr>
	    <tr class="mainrow"><td> <font class="main"> $closedShop::TEXT{'SUBCAT'} $closedShop::TEXT{'NAME'}:</td>
	    <td><input type="text" name="subcat" size="15"></td></tr>
	    <tr class="headrow">
	    <td colspan="2"><center> <font class="main"><input type="submit" value="$closedShop::TEXT{'ENT_SUB'}"></td>
	    </tr></form></table>
	    <BR><form action="$program_name" method="POST">
	    <input type="hidden" name="selected" value="RemoveCategory">
           <input type="hidden" name="action" value="EditCategories">
            <table width="35%"  class="main"><tr class="headrow"><th colspan="2">
	    <font class="head"> $closedShop::TEXT{'REM'} $closedShop::TEXT{'CAT'}</th></tr>
	    <tr class="mainrow"><td> <font class="main"> $closedShop::TEXT{'SELECT'} $closedShop::TEXT{'CAT'}:</td>
	    <td> <font class="main">|;

        closedShop::PrintCategoryList( "category", 0, "ALL" );

        print qq|</td></tr>
	  <tr class="mainrow"><td> <font class="main"> $closedShop::TEXT{'CNF_DELMESG'}</td>
	    <td> <font class="main"><input type="radio" name="delete" value="Y">$closedShop::TEXT{'YES'} <BR>
	    <input type="radio" name="delete" value="N" checked> $closedShop::TEXT{'NO'}</td></tr>
	    <tr class="headrow">
	    <td colspan="2"> <font class="main"><center><input type="submit" value="$closedShop::TEXT{'REM'} $closedShop::TEXT{'CAT'}"></td>
	    </tr></form>

	    </table>
	    </td></tr></table>
	  
	      |;
        BottomAdmin();

        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  ViewCustomers
# DESCRIPTION: add a product to database
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ViewCustomers {
    my $mesg = "";
    if ( $query->param("selected") eq "RemoveCustomer" ) {
        my $id   = $query->param("id");
        my $SQL2 = qq| delete from $closedShop::sql_tables{"user"}
	  where id = "$id" |;
        my $sth2 = closedShop::DatabaseQuery($SQL2);
        $mesg = qq|<div align="center"><font class="main">
	  <font color="red">$closedShop::TEXT{'RM_CUST'}</font></div><br>|;
    }
    elsif ( $query->param("selected") eq "EmailCustomer" ) {
        my $subject = $query->param("subject");
        my $body    = $query->param("body");
        EmailCustomers( $subject, $body );
        $mesg = qq|<div align="center"><font class="main">
	  <font color="red">$closedShop::TEXT{'SUCCEM_CUST'}</font></div><br>|;
    }
    elsif ( $query->param("selected") eq "EmailCustomerDisplay" ) {
        closedShop::Header();
        print qq|
	  <form action="$program_name" method="POST">
	   <input type="hidden" name="action" value="ViewCustomers">
	   <input type="hidden" name="selected" value="EmailCustomer">
	   <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	  <font class="head">$closedShop::TEXT{'EM_CUST'} </th></tr>
 	  <tr class="mainrow">
		<td><font class="main"> $closedShop::TEXT{'SUBJECT'}</td>
		<td><font class="main"> <input type="text" name="subject" size="30"></td></tr>
	  <tr class="mainrow"><td><font class="main"> $closedShop::TEXT{'BODY'}</td>
	  <td><font class="main"> 
	  <table border='0'><tr><td>
	  <textarea mce_editable='true' name="body" cols="40" rows="5" wrap="physical"></textarea>
	  </td><td valign='top'><font class="main"> 
	  <b>Mail Merging</b>:<BR>
	  &nbsp;&nbsp;&nbsp;&nbsp;:fname: - insert customer's first name<BR>
	  &nbsp;&nbsp;&nbsp;&nbsp;:lname: - insert customer's last name<BR>
	  </td></tr></table>
	  </td>
	  <tr class="headrow">
	  <td colspan="2"><div align="center"><input type="submit" value="$closedShop::TEXT{'EM_CUST'}"></div></td>
	  </tr></form></table>
	|;
        BottomAdmin();
        return closedShop::Footer();
    }

    closedShop::Header();

    print qq|$mesg
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="6">
	  <font class="main"> <font size="3"><font color="blue">$closedShop::TEXT{'VIEW_CUST'}</font></th></tr>
 	  <tr class="mainrow">
	<td><font class="main"><b>First Name</b></td>
	<td><font class="main"><b>Last Name</b></td>
	<td><font class="main"><b>$closedShop::TEXT{'EMAIL'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'ADDRESS'}/<br>$closedShop::TEXT{'PHONE'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'TOT'} $closedShop::TEXT{'PURCHASE'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'REM'}</b></td>
	  </tr>|;

    my $SQL3 = qq| select fname,lname,email,address,city,state,country,zip,phone,id
	  from $closedShop::sql_tables{"user"} where email != "" order by lname|;
    my $sth3 = closedShop::DatabaseQuery($SQL3);

    my $k = 0;
    while ( my ( $fname, $lname, $email, $address, $city, $state, $country, $zip, $phone, $id ) = $sth3->fetchrow_array() ) {
        print qq|
	   <tr class="mainrow">

		<td><form action="$program_name" method="POST">
	   <input type="hidden" name="action" value="ViewCustomers">
	   <input type="hidden" name="selected" value="RemoveCustomer">
	   <input type="hidden" name="id" value="$id"><font class="main"> $fname</td>
		<td>$lname</td>
		<td><font class="main"><a href="mailto:$email">$email</a></td>
	    <td><font class="main"> $address<br>$city, $state $zip<br>
		<BR>$country<BR>$phone</td>
	    |;

        my $SQL2 = qq| select sum(total) from $closedShop::sql_tables{"order"} where
	  date_processed is NOT NULL and user_id = "$id"|;
        my $sth2 = closedShop::DatabaseQuery($SQL2);
        my ($sum) = $sth2->fetchrow_array();
        if ( !$sum ) { $sum = "0.00"; }

        print qq|<td><font class="main"> $closedShop::CurrencyMarker $sum</td>
	  <td><font class="main"> <input type="submit" value="$closedShop::TEXT{'REM'}"></td></form>
	  </tr>|;
        $k++;
    }

    print qq|<tr class="headrow">
		<td colspan="6"><font class="main"> <div align="center"><b>$closedShop::TEXT{'TOT'} $closedShop::TEXT{'CUST'}: $k</b></div></td></tr>
	    </table></form>
		<br>
		
		<div align="center"><form action="$program_name" method="POST">
	   <input type="hidden" name="action" value="ViewCustomers">
	   <input type="hidden" name="selected" value="EmailCustomerDisplay">
	   <input type="submit" value="$closedShop::TEXT{'EM_CUST'}"></form></div></center>
	   
	   </div>
				
		|;
    BottomAdmin();
    closedShop::Footer();

}

###############################################################################
# FUNCTION:  AddProduct
# DESCRIPTION: add a product to database
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AddProduct {
    my $mesg = shift;
    if ( $query->param("selected") == 1 ) {
        my $name = $query->param("name");
        $name = closedShop::ValidateInputAdmin($name);
        my $category = $query->param("category");
        $category = closedShop::ValidateInputAdmin($category);
        my $price = $query->param("price");
        $price = closedShop::ValidateInputAdmin($price);
        my $code = $query->param("code");
        $code = closedShop::ValidateInputAdmin($code);
        my $shipping = $query->param("shipping");
        $shipping = closedShop::ValidateInputAdmin($shipping);
        my $rshipping = $query->param("ReducedShipping");
        $rshipping = closedShop::ValidateInputAdmin($rshipping);
        if ( !$rshipping ) { $rshipping = -1; }
        my $weight = $query->param("weight");
        $weight = closedShop::ValidateInputAdmin($weight);
        $weight =~ tr/0-9\.//cd;
        my $quantity = $query->param("quantity");
        $quantity =~ tr/0-9//cd;
        $quantity = 1 if ( !$quantity );
        my $desc = $query->param("desc");
        $desc = closedShop::ValidateInputAdmin($desc);
        my $image     = $query->param("image");
        my $image_url = closedShop::ValidateInputAdmin( $query->param("image_url") );

        if ( !$name || !$price || !$category ) {
            $query = new CGI( {} );
            return AddProduct( $closedShop::TEXT{'FLL_ALL'} );
        }

        my $EbayItemTitle = closedShop::ValidateInputAdmin( $query->param("EbayItemTitle") );
        if ( !$EbayItemTitle ) { $EbayItemTitle = $name; }
        my $EbayItemDesc = closedShop::ValidateInputAdmin( $query->param("EbayItemDesc") );
        if ( !$EbayItemDesc ) { $EbayItemDesc = $desc; }
        my $EbayQuantity = closedShop::ValidateInputAdmin( $query->param("EbayQuantity") );
        if ( !$EbayQuantity ) { $EbayQuantity = $quantity; }
        my $EbayDuration      = closedShop::ValidateInputAdmin( $query->param("EbayDuration") );
        my $EbayStartingPrice = closedShop::ValidateInputAdmin( $query->param("EbayStartingPrice") );
        if ( !$EbayStartingPrice ) { $EbayStartingPrice = $price; }

        my $Category1Option = closedShop::ValidateInputAdmin( $query->param("Category1Option") );
        my $CategoryMenu1   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu1") );
        my $CategoryMenu2   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu2") );
        my $CategoryMenu3   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu3") );
        my $CategoryMenu4   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu4") );
        my $CategoryMenu5   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu5") );

        my $PayInstructions = closedShop::ValidateInputAdmin( $query->param("PayInstructions") );

        my $SQL = qq| insert into $closedShop::sql_tables{"item"} set 
             name="$name",weight="$weight",
	     price="$price",shipping_price="$shipping",description="$desc",
	     category="$category",product_code="$code",
	     quantity="$quantity",reduced_shipping_price="$rshipping",image_url="$image_url" |;
        my $sth    = closedShop::DatabaseQuery($SQL);
        my $new_id = $sth->{'mysql_insertid'};

        my $k = 1;
        while ( $query->param("attribute$k-name") ) {
            my $atname = closedShop::ValidateInputAdmin( $query->param("attribute$k-name") );

            $SQL = qq|insert into  $closedShop::sql_tables{"attributes"} set
		product_id="$new_id",name='$atname',ordering='$k'|;
            $sth = closedShop::DatabaseQuery($SQL);
            my $attr_id = $sth->{'mysql_insertid'};

            my $t = 1;
            while ( $query->param("attribute$k-$t-data") ) {
                my $atdata  = closedShop::ValidateInputAdmin( $query->param("attribute$k-$t-data") );
                my $atprice = closedShop::ValidateInputAdmin( $query->param("attribute$k-$t-price") );

                $SQL = qq|insert into  $closedShop::sql_tables{"attribute_values"} set
		attribute_id="$attr_id",data='$atdata',price='$atprice',ordering='$t'|;

                $sth = closedShop::DatabaseQuery($SQL);
                $t++;
            }
            $k++;
        }

        if ($image) {
            use Cwd;
            my $currdir = getcwd;

            open( OUTFILE, ">$closedShop::IMAGEPATH/$new_id.jpg" )
              || closedShop::ErrorMessage(
                "$closedShop::TEXT{'CANT_UP'} $closedShop::IMAGEPATH<BR>
            $closedShop::TEXT{'CANT_UPMESG'} $closedShop::IMAGEPATH'<BR>
            $closedShop::TEXT{'PATH_LOC_ERROR'} $currdir"
              );
            while (<$image>) { print OUTFILE; }
            close(OUTFILE);

            if ( !( stat("$closedShop::IMAGEPATH/$new_id.jpg") )[7] ) {
                unlink("$closedShop::IMAGEPATH/$new_id.jpg");
            }

        }

        my $appendmesg = "";
        if ( $query->param("DoEbay") eq "Y" ) {
            my $pid = $$;
            if ( defined( my $cid = fork() ) ) {
                if ( $cid == 0 ) {
                    PostEbayItem( $EbayItemTitle, $EbayItemDesc, $EbayQuantity, $EbayDuration, $EbayStartingPrice, $Category1Option, $CategoryMenu1, $CategoryMenu2, $CategoryMenu3, $CategoryMenu4, $CategoryMenu5, $PayInstructions, $new_id );
                }
                else {
                    $appendmesg = $closedShop::TEXT{'EBAY_THREAD'};
                }
            }
            else {
                $appendmesg = $closedShop::TEXT{'EBAY_BADTHREAD'};
            }
        }

        return AdminScreen("$closedShop::TEXT{'SUCC_ADDI'} $appendmesg");

    }
    else {
        closedShop::Header();
        print qq|<font class='main' color='red'>$mesg</font>
	  <form action="$program_name" method="POST" enctype="multipart/form-data"
		name="stockForm">
	  <input type="hidden" name="action" value="AddProduct">
	  <input type="hidden" name="selected" value="1">
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'ADD_PROD'}</th></tr>
 	 
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'PROD'} $closedShop::TEXT{'NAME'}:</td>
	  <td><font class="main"> <input type="text" name="name" size="20"></td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'PROD'} Code/Id:</td>
	  <td><font class="main"> <input type="text" name="code" size="20"></td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'CAT'}:</td>
	  <td><font class="main"> |;
        closedShop::PrintCategoryList( "category", 0, "ALL" );
        print qq|
	  </td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'PROD'} $closedShop::TEXT{'PRC'}:</td>
	  <td><font class="main"> $closedShop::CurrencyMarker <input type="text" name="price" size="6"></td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'SHP_PRC'}:</td>
	  <td><font class="main"> $closedShop::CurrencyMarker <input type="text" name="shipping" size="6"></td></tr>
	  <tr class="mainrow"><td><center><font class="main"> 
		$closedShop::TEXT{'REDUCE'} $closedShop::TEXT{'SHP_PRC'}:</td>
	  <td><font class="main"> $closedShop::CurrencyMarker <input
type="text" name="ReducedShipping" size="6" value="">
<BR>$closedShop::TEXT{'REDU_MESG'}</td>
</td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'WGHT'}:</td>
	  <td><font class="main"> <input type="text" value="" name="weight" size="6"> $closedShop::WeightIdentifier</td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'QTY'} $closedShop::TEXT{'INSTOCK'}:</td>
	  <td><font class="main"> <input type="text" value="1" name="quantity" size="4"></td></tr>
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'PROD'} $closedShop::TEXT{'DESC'}:</td>
	  <td><font class="main"> <textarea mce_editable='true' name="desc" cols="60" rows="8"></textarea></td></tr>
  	  
	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'IMAGE'}:<br>
	  (JPEG Format)</td>
	  <td><font class="main"> <input type="file" name="image" size="20"></td></tr>
	  
		<tr class="mainrow"><td><center><font class="main"> Or $closedShop::TEXT{'IMAGE'} URL:
	  </td>
	  <td><font class="main"> <input type="text" name="image_url" size="20"> (<i>External Link to Image</i>)</td></tr>
	  
		
	    <tr class="headrow"><td colspan='2'>
	   <font class="main"><b> $closedShop::TEXT{'ADD'} $closedShop::TEXT{'ATT'}:</b></td>
	   </tr>
	   <td colspan='2'>
	  <font class="main">
	  <table  class="main" width="100%">
	  |;

        for ( my $j = 1 ; $j < $closedShop::MaxAttributes + 1 ; $j++ ) {

            if ( ( $j - 1 ) % 3 == 0 ) {
                print qq|<tr class='mainrow'>|;
            }

            print qq|<td style='border: 1px solid black;' valign='top'><font class="main">
	  <u style='background-color: $closedShop::input_hover'>$closedShop::TEXT{'ATTR'} $j:</u><br>
	  $closedShop::TEXT{'ATTR'} $closedShop::TEXT{'NAME'}: <input 
	  type="text" name="attribute$j-name"  size="10">
	   <table style='border: 1px solid gray;' width='100%'><tr style='font-size: 10px;'>
	  <td valign='top'><font ><U>$closedShop::TEXT{'ATT'}</td><td><u>$closedShop::TEXT{'PRC'} ($closedShop::TEXT{'EXTRA'})
	   </td></tr>|;

            for ( my $t = 1 ; $t < $closedShop::MaxAttributeOptions + 1 ; $t++ ) {
                print qq|<tr><td style='font-size: 10px;'>$t) 
	       <input style='font-size: 10px;' type="text" name="attribute$j-$t-data"  size="16">
	       </td><td style='font-size: 10px;'>
	        $closedShop::CurrencyMarker <input style='font-size: 10px;' type="text" name="attribute$j-$t-price" value="0.00" size="5">
	       </td></tr>
	       |;
            }

            print qq|</table></td>|;

            if ( $j % 3 == 0 ) {
                print qq|</tr>|;
            }

        }
        print qq| </tr></table>
	</td></tr>
	|;

        if ( $closedShop::EbayAuction eq "Y" ) {
            eval("use LWP::UserAgent");
            if ($@) {
                print qq|<tr class="mainrow"><td colspan="2">
        	<center><font class="main"> <b>Ebay $closedShop::TEXT{'ERR'}</b></td></tr>
        	<tr class="mainrow">
          	<td colspan="2"><font class="main"> $closedShop::TEXT{'POST_EBAY_ERR'}:<BR>
		<li>LWP::UserAgent
		<li>HTTP::Request::Common
		which can be found at 
		<a href="http://search.cpan.org">http://search.cpan.org</a> </td></tr>
		|;
            }
            else {
                DisplayEbayData();
            }
        }

        print qq|
	  <tr class="headrow"><th colspan="2">
	  <font class="main"> <input type="submit" value="$closedShop::TEXT{'ADD'} $closedShop::TEXT{'PROD'}"></th></tr>
	  </form></table>
	  
	  |;
        BottomAdmin();
        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  PostEbayItem
# DESCRIPTION: sends the data to Ebay
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub PostEbayItem {
    my ( $EbayItemTitle, $EbayItemDesc, $EbayQuantity, $EbayDuration, $EbayStartingPrice, $Category1Option, $CategoryMenu1, $CategoryMenu2, $CategoryMenu3, $CategoryMenu4, $CategoryMenu5, $PayInstructions, $id ) = @_;

    use HTTP::Request;
    use HTTP::Request::Common qw(POST GET);
    use LWP::UserAgent;
    use HTTP::Cookies;

    my $cat1 = $Category1Option;
    my $CAT  = $cat1;
    my $sub1 = $CategoryMenu1;
    if ($sub1) { $CAT = $sub1; }
    my $sub2 = $CategoryMenu2;
    if ($sub2) { $CAT = $sub2; }
    my $sub3 = $CategoryMenu3;
    if ($sub3) { $CAT = $sub3; }
    my $sub4 = $CategoryMenu4;
    if ($sub4) { $CAT = $sub4; }
    my $sub5 = $CategoryMenu5;
    if ($sub5) { $CAT = $sub4; }

    my $itemtitle = $EbayItemTitle;
    my $itemdesc  = $EbayItemDesc;

    my $photourl = "";
    if ( -e "$closedShop::IMAGEPATH/$id.jpg" ) {
        $photourl = $closedShop::IMAGEURL . "/$id.jpg";
    }
    elsif ( -e "$closedShop::SIMAGEPATH/$id.jpg" ) {
        $photourl = $closedShop::SecureImageURL . "/$id.jpg";
    }
    my $duration   = $EbayDuration;
    my $quantity   = $EbayQuantity;
    my $startprice = $EbayStartingPrice;
    my $citystate  = "$closedShop::CompanyCity, $closedShop::CompanyState";
    my $region     = "0";
    my $country    = closedShop::ReturnEbayCountryCode($closedShop::CompanyCountry);

    my ( $paypal, $visa, $discover, $amex ) = ();

    if ($closedShop::PayPalEmail) {
        $paypal = "&PaymentMethodPaypal=Paypal&PaymentMethodPaypalEmail=$closedShop::CompanyEmail";
    }

    if ( $closedShop::AcceptedCards{"visa"} ) {
        $visa = "&PaymentMethodCreditCardVisaMaster=VisaMaster";
    }
    if ( $closedShop::AcceptedCards{"discover"} ) {
        $discover = "&PaymentMethodCreditCardDiscover=Discover";
    }

    if ( $closedShop::AcceptedCards{"amex"} ) {
        $amex = "&PaymentMethodCreditCardAmEx=AmEx";
    }

    my $file       = "ebaycookies.txt";
    my $cookie_jar = HTTP::Cookies->new(
        file            => $file,
        ignore_discards => 1,
        hide_cookie2    => 1
    );

    my @sites = (
        "http://signin.ebay.com/aw-cgi/eBayISAPI.dll?SignIn",
        "http://signin.ebay.com/aw-cgi/eBayISAPI.dll?SignInWelcome&userid=fleizach&Pass=8der754b&keepMeSignInOption=1&rememberMeForSellBid=y",
        "http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&sspagename=h%3Ah%3Asyi%3AUS",
        "http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&SaleFormat=1&CurrentPage=SyiFormat&AuctionFormat=1",
        "http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&SaleFormat=1&CurrentPage=SyiMetaCategory&ShowCategory=1&CategorySelectorStyle=Javascript&Category1Option=$cat1&ButtonContinue=Continue+%3E",
"http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&SaleFormat=1&CurrentPage=SyiLeafCategory&ShowCategory=1&CategorySelectorStyle=Javascript&Category1Option=$cat1&CategoryMenu1=$sub1&CategoryMenu2=$sub2&CategoryMenu3=$sub3&CategoryMenu4=$sub4&CategoryMenu5=$sub5&Category1=$CAT&ButtonContinue=Continue+%3E",
        "http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&SaleFormat=1&CurrentPage=SyiTitle&UseRichTextEditor=false&ItemTitle=$itemtitle&ItemDescription=$itemdesc&ShowCategory=1&ButtonContinue=Continue+%3E",
"http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&SaleFormat=1&ListingCurrencyId=1&CurrentPage=SyiDetails&PhotoBrowseStyle=SELLER_HOSTED&PhotoPersist=$photourl&PhotoUrlChanged=true&EpsBetaOption=In&RelistPhoto=1&ListingDuration=$duration&QuantityListed=$quantity&AuctionStartPrice=$startprice&ItemCityState=$citystate&ItemRegion=$region&ItemCountry=$country&photoDisplayType=0&PictureUrl=$photourl&ThemeId=10&LayoutId=10000&GalleryURL=http%3A%2F%2F&Gallery=0&ShowAsGift=0&CounterStyle=1&ButtonContinue=via_Pics_SELLER_HOSTED",
"http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&SaleFormat=1&CurrentPage=SyiPayment&AutoPayRequest=false&ButtonToggle=0$paypal$visa$discover$amex&PaymentMethodGenericCheck=Check&PaymentMethodGenericMOCashiers=MOCashiers&PaymentMethodGenericSeeItemDescription=SeeItemDescription&ShippingWhoPays=1&ShippingCostsSpecified=false&SingleItemShippingAndHandling=2.75&ShippingInsuranceAvailability=NONE&ShippingInsurancePerItem=0.00&TaxPercent=0.0&PaymentInstructions=$PayInstructions&ShipToType=2&EscrowType=0&ButtonContinue=Continue+%3E",
        "http://cgi5.ebay.com/ws2/eBayISAPI.dll?SellItem&SaleFormat=2&ListingCurrencyId=1&CurrentPage=SyiVerify&ButtonContinue=Submit%20Listing",
    );

    my $ua = LWP::UserAgent->new;
    $ua->cookie_jar($cookie_jar);
    $ua->agent("Mozilla/5.0");
    open( FILE, ">$file" );
    close(FILE);

    for ( my $k = 0 ; $k < @sites ; $k++ ) {
        my $req = GET "$sites[$k]";
        $cookie_jar->add_cookie_header($req);
        $req->header('Connection: Keep-Alive');
        $req->referer( $sites[ $k - 1 ] );
        $req->content_type('application/x-www-form-urlencoded');

        my $content = $ua->request($req);
        open( FILE, ">$file" );
        print FILE $cookie_jar->as_string();
        close(FILE);

        # GET SESSION ID
        if ( $k > 1 ) {
            my $data = $content->as_string();
            my $sp   = qq|id="SessionId" type="hidden" value="|;
            my @data = split( /$sp/, $data );
            $data = substr( $data[1], 0, index( $data, "\"" ) );
            $data = substr( $data, 0, index( $data, "\">" ) );
            $sites[ $k + 1 ] .= "&SessionId=$data";
        }
        if ( $k == $#sites - 1 ) {

            # time before submit get values
            my $alldata = $content->as_string();
            my $data    = $alldata;
            my $sp      = qq|id="tokenstring" type="hidden" value="|;
            my @data    = split( /$sp/, $data );
            $data = substr( $data[1], 0, index( $data, "\"" ) );
            $data = substr( $data, 0, index( $data, "\">" ) );
            $sites[$#sites] .= "&tokenstring=$data";

            my $data1 = $alldata;
            $sp = qq|id="tokentext" type="hidden" value="|;
            my @data1 = split( /$sp/, $data1 );
            $data1 = substr( $data1[1], 0, index( $data1, "\"" ) );
            $data1 = substr( $data1, 0, index( $data1, "\">" ) );
            $sites[$#sites] .= "&tokentext=$data1";
        }

        if ( $k == $#sites ) {
            open( FILE, ">ebay.status" );
            print FILE "<b>Ebay Submission Status</b>: The item has been successfully 
submitted to eBay and you will receive an email from eBay confirming this status.";
            close(FILE);
        }
        else {
            open( FILE, ">ebay.status" );
            print FILE "<b>Ebay Submission Status</b>: The item is still in the process 
of being submitted to eBay. Currently, it is on step $k of $#sites";
            close(FILE);
        }

    }
}

###############################################################################
# FUNCTION:  DisplayEbayData
# DESCRIPTION: diplays Ebay data
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DisplayEbayData {
    my ( $itemtitle, $startingprice, $itemdesc, $quantity ) = @_;
    print qq|
	   <tr class="mainrow"><td colspan="2">
	<center><font class="main"> <b>Ebay Auction Data</b></td></tr>
	<tr class="mainrow">
	  <td><font class="main"> Post Ebay Auction: </td>
	  <td><font class="main"> <input type="radio" name="DoEbay" 
           value="N" checked>Do not post to Ebay<BR>
	   <input type="radio" name="DoEbay" value="Y">Post to Ebay
	</td></tr>

	<tr class="mainrow">
	  <td><font class="main"> $closedShop::TEXT{'ITM_TITLE'}: </td>
	  <td><font class="main"> <input type="text" name="EbayItemTitle" 
           value="$itemtitle" size="25"><BR>
           $closedShop::TEXT{'EBAY_TITLEMESG'}</td></tr>
	<tr class="mainrow">
	  <td><font class="main"> $closedShop::TEXT{'ITEM'} $closedShop::TEXT{'DESC'}: </td>
	  <td><font class="main"> <textarea rows="6" mce_editable='true'
cols="50" name="EbayItemDesc">$itemdesc</textarea><BR>
           $closedShop::TEXT{'EBAY_DESCMESG'}</td></tr>
	<tr class="mainrow">
	  <td><font class="main"> $closedShop::TEXT{'AUC_DUR'}: </td>
	  <td><font class="main"> <select name="EbayDuration">
	<option value="3">3 Days</option>
	<option value="5">5 Days</option>
	<option value="7">7 Days</option>
	<option value="10">10 Days</option>
	</select>
           </td></tr>
	<tr class="mainrow">
	  <td><font class="main"> $closedShop::TEXT{'QTY'}: </td>
	  <td><font class="main"> <input type="text" name="EbayQuantity"
	value="$quantity" size="3"></td></tr>
	<tr class="mainrow">
	  <td><font class="main"> $closedShop::TEXT{'STRT'} $closedShop::TEXT{'PRC'}: </td>
	  <td><font class="main"> $closedShop::CurrencyMarker <input type="text" name="EbayStartingPrice"
	value="$startingprice"><BR>
	$closedShop::TEXT{'EBAY_PRCMESG'}
           </td></tr>
	<tr class="mainrow">
	  <td><font class="main"> Select Main $closedShop::TEXT{'CAT'}: </td>
	  <td><font class="main"> |;
    closedShop::PrintEbayCategoryMain();
    print qq|<BR>
	Sub-Category 1: <input type="text" name="subcat1" size="25" disabled><BR>
	  <input type="hidden" name="CategoryMenu1" value="">
	Sub-Category 2: <input type="text" name="subcat2" size="25" disabled><BR>
	  <input type="hidden" name="CategoryMenu2" value="">
	Sub-Category 3: <input type="text" name="subcat3" size="25" disabled><BR>
	  <input type="hidden" name="CategoryMenu3" value="">
	Sub-Category 4: <input type="text" name="subcat4" size="25" disabled><BR>
	  <input type="hidden" name="CategoryMenu4" value="">
	Sub-Category 5: <input type="text" name="subcat5" size="25" disabled><BR>
	  <input type="hidden" name="CategoryMenu5" value="">
	 </td></tr>
	<tr class="mainrow">
	  <td><font class="main"> $closedShop::TEXT{'PAY_IN'}: </td>
	  <td><font class="main"> <textarea mce_editable='true' name="PayInstructions" cols="40"
	rows="6">Preferred payment is through PayPal. See listing for further details.</textarea></td></tr>|;

}

###############################################################################
# FUNCTION:  ViewLinks
# DESCRIPTION: Views important links
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ViewLinks {
    closedShop::Header();
    my $search = $closedShop::FileNames{"item"};
    my $user   = $closedShop::FileNames{"user"};
    my $cart   = $closedShop::FileNames{"cart"};
    if ( $search !~ /http/ ) {
        $search = $closedShop::urltofolder . "/" . $closedShop::FileNames{"item"};
    }
    if ( $user !~ /http/ ) {
        $user = $closedShop::urltofolder . "/" . $closedShop::FileNames{"user"};
    }
    if ( $cart !~ /http/ ) {
        $cart = $closedShop::urltofolder . "/" . $closedShop::FileNames{"cart"};
    }

    my $gc          = $search . "?action=ShowGiftCertificates";
    my $showallcats = $search . "?action=ShowAllCategories";

    my $newaffiliate = $user . "?action=NewAffiliate";
    my $retaffiliate = $user . "?action=RetrieveAffiliate";

    print qq|
  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
  <font class="head"> $closedShop::TEXT{'IM_LINKS'}</th></tr>

  <tr class="mainrow"><td><font class="main">$closedShop::TEXT{'SRCH'} $closedShop::TEXT{'LINK'}:</td>
  <td><font class="main" size='1'> <A href="$search">$search</a></td></tr>
	<tr class="mainrow"><td><font class="main">$closedShop::TEXT{'USR_LOG'} $closedShop::TEXT{'LINK'}:</td>
  <td><font class="main" size='1'> <A href="$user">$user</a></td></tr>
	<tr class="mainrow"><td><font class="main">$closedShop::TEXT{'SHP_CRT'} $closedShop::TEXT{'LINK'}:</td>
  <td><font class="main" size='1'> <A href="$cart">$cart</a></td></tr>
	<tr class="mainrow"><td><font class="main">$closedShop::TEXT{'GC'} $closedShop::TEXT{'LINK'}:</td>
  <td><font class="main" size='1'> <a href="$gc">$gc</a></td></tr>
<tr class="mainrow"><td><font class="main">  $closedShop::TEXT{'DSP'} $closedShop::TEXT{'ALL'}  $closedShop::TEXT{'CATS'} $closedShop::TEXT{'LINK'}:</td>
	  <td><font class="main" size='1'> <A href="$showallcats">$showallcats</a></td></tr>
	  <tr class="mainrow"><td><font class="main"> Add New Affiliate Link:</td>
	  <td><font class="main" size='1'> <A href="$newaffiliate">$newaffiliate</a></td></tr>
	  <tr class="mainrow"><td><font class="main"> Retrieve Affiliate Info Link:</td>
	  <td><font class="main" size='1'> <A href="$retaffiliate">$retaffiliate</a></td></tr>
	  
	<tr class="mainrow"><td colspan="2"><center><font class="main"> 
	<b> $closedShop::TEXT{'CAT'}  $closedShop::TEXT{'LNKS'}</b></td></tr>
|;

    foreach ( sort keys %closedShop::categories ) {
        my ( $intro, $style, $numimages, $status ) = closedShop::GetCategoryInfo($_);
        if ( $status ne "PRIVATE" ) {
            print qq|<tr class="mainrow"><td><font class="main"> 
	 $closedShop::TEXT{'CAT'} $closedShop::categories{$_}:</td>
	<td><font class="main" size='1'> <a href="$search?action=ShowCategory&category=$_">$search?action=ShowCategory&category=$_</a></td></tr>|;
        }
    }

    print qq|
	<Tr class="headrow"><td colspan="2">&nbsp;</td></tr>	
	</table>
	
	  |;
    BottomAdmin();
    closedShop::Footer();
}

###############################################################################
# FUNCTION:  EditProduct
# DESCRIPTION: edit a product
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub EditProduct {

    if ( $query->param("selected") == 4 ) {
        my $id = $query->param("id");
        $id =~ tr/0-9//cd;

        unlink("$closedShop::IMAGEPATH/$id.jpg");
        unlink("$closedShop::SIMAGEPATH/$id.jpg");
        $query = new CGI( { action => "EditProduct", selected => "2", id => "$id" } );
        return EditProduct();
    }
    elsif ( $query->param("selected") == 3 ) {
        my $name = $query->param("name");
        $name = closedShop::ValidateInputAdmin($name);
        my $category = $query->param("category");
        $category = closedShop::ValidateInputAdmin($category);
        my $price = $query->param("price");
        $price = closedShop::ValidateInputAdmin($price);
        my $code = $query->param("code");
        $code = closedShop::ValidateInputAdmin($code);
        my $shipping = $query->param("shipping");
        $shipping = closedShop::ValidateInputAdmin($shipping);
        my $rshipping = $query->param("ReducedShipping");
        $rshipping = closedShop::ValidateInputAdmin($rshipping);
        if ( !$rshipping ) { $rshipping = -1; }
        my $quantity = $query->param("quantity");
        $quantity =~ tr/0-9//cd;
        $quantity = 1 if ( !$quantity );
        my $weight = $query->param("weight");
        $weight =~ tr/0-9\.//cd;
        my $desc = $query->param("desc");
        $desc = closedShop::ValidateInputAdmin($desc);
        my $image_url = $query->param("image_url");
        my $image     = $query->param("image");
        my $id        = $query->param("id");
        $id =~ tr/0-9//cd;

        my $EbayItemTitle = closedShop::ValidateInputAdmin( $query->param("EbayItemTitle") );
        if ( !$EbayItemTitle ) { $EbayItemTitle = $name; }
        my $EbayItemDesc = closedShop::ValidateInputAdmin( $query->param("EbayItemDesc") );
        if ( !$EbayItemDesc ) { $EbayItemDesc = $desc; }
        my $EbayQuantity = closedShop::ValidateInputAdmin( $query->param("EbayQuantity") );
        if ( !$EbayQuantity ) { $EbayQuantity = $quantity; }
        my $EbayDuration      = closedShop::ValidateInputAdmin( $query->param("EbayDuration") );
        my $EbayStartingPrice = closedShop::ValidateInputAdmin( $query->param("EbayStartingPrice") );
        if ( !$EbayStartingPrice ) { $EbayStartingPrice = $price; }
        my $Category1Option = closedShop::ValidateInputAdmin( $query->param("Category1Option") );
        my $CategoryMenu1   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu1") );
        my $CategoryMenu2   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu2") );
        my $CategoryMenu3   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu3") );
        my $CategoryMenu4   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu4") );
        my $CategoryMenu5   = closedShop::ValidateInputAdmin( $query->param("CategoryMenu5") );

        my $PayInstructions = closedShop::ValidateInputAdmin( $query->param("PayInstructions") );

        my $SQL = qq|select id from $closedShop::sql_tables{"attributes"} where
		product_id="$id"|;
        my $sth = closedShop::DatabaseQuery($SQL);

        while ( my ($aid) = $sth->fetchrow_array() ) {
            my $SQL2 = qq|delete from $closedShop::sql_tables{"attribute_values"} where
	  	attribute_id='$aid'|;
            my $sth2 = closedShop::DatabaseQuery($SQL2);
        }

        $SQL = qq|delete from $closedShop::sql_tables{"attributes"} where
		product_id="$id"|;
        $sth = closedShop::DatabaseQuery($SQL);

        my $k = 1;
        while ( $query->param("attribute$k-name") ) {
            my $atname = closedShop::ValidateInputAdmin( $query->param("attribute$k-name") );

            $SQL = qq|insert into  $closedShop::sql_tables{"attributes"} set
		product_id="$id",name='$atname',ordering='$k'|;
            $sth = closedShop::DatabaseQuery($SQL);
            my $attr_id = $sth->{'mysql_insertid'};

            my $t = 1;
            while ( $query->param("attribute$k-$t-data") ) {
                my $atdata  = closedShop::ValidateInputAdmin( $query->param("attribute$k-$t-data") );
                my $atprice = closedShop::ValidateInputAdmin( $query->param("attribute$k-$t-price") );

                $SQL = qq|insert into  $closedShop::sql_tables{"attribute_values"} set
		attribute_id="$attr_id",data='$atdata',price='$atprice',ordering='$t'|;

                $sth = closedShop::DatabaseQuery($SQL);
                $t++;
            }
            $k++;
        }

        $SQL = qq| update $closedShop::sql_tables{"item"} set name="$name",
	                price="$price",shipping_price="$shipping",description="$desc",
			category="$category",product_code="$code",weight="$weight",
			quantity="$quantity",reduced_shipping_price="$rshipping",image_url="$image_url"
			where id = "$id"|;
        $sth = closedShop::DatabaseQuery($SQL);

        if ($image) {
            open( OUTFILE, ">$closedShop::IMAGEPATH/$id.jpg" )
              || closedShop::ErrorMessage("$closedShop::TEXT{'ERR'} $!");
            while (<$image>) { print OUTFILE; }
            close(OUTFILE);

            if ( !( stat("$closedShop::IMAGEPATH/$id.jpg") )[7] ) {
                unlink("$closedShop::IMAGEPATH/$id.jpg");
                closedShop::ErrorMessage( "unlinking $closedShop::IMAGEPATH/$id.jpg" . ( ( stat("$closedShop::IMAGEPATH/$id.jpg") )[7] ) );
            }
        }

        my $appendmesg = "";
        if ( $query->param("DoEbay") eq "Y" ) {
            my $pid = $$;
            if ( defined( my $cid = fork() ) ) {
                if ( $cid == 0 ) {
                    PostEbayItem( $EbayItemTitle, $EbayItemDesc, $EbayQuantity, $EbayDuration, $EbayStartingPrice, $Category1Option, $CategoryMenu1, $CategoryMenu2, $CategoryMenu3, $CategoryMenu4, $CategoryMenu5, $PayInstructions, $id );
                }
                else {
                    $appendmesg = $closedShop::TEXT{'EBAY_THREAD'};
                }
            }
            else {
                $appendmesg = $closedShop::TEXT{'EBAY_BADTHREAD'};
            }
        }

        return AdminScreen("$closedShop::TEXT{'SUCC_EDITI'} $name $appendmesg");
    }
    elsif ( $query->param("selected") == 2 ) {
        my $id = $query->param("id");
        $id =~ tr/0-9//cd;
        if ( $id == 0 || !$id ) {
            goto START;
        }
        my $SQL = qq| select name,price,shipping_price,description,
			category,product_code,quantity,weight,
			reduced_shipping_price,image_url from $closedShop::sql_tables{"item"} where id = "$id" |;
        my $sth = closedShop::DatabaseQuery($SQL);
        my ( $name, $price, $shipping_price, $description, $category, $code, $quantity, $weight, $ReducedShipping, $image_url ) = $sth->fetchrow_array();

        if ( $ReducedShipping == -1 ) { $ReducedShipping = ""; }

        closedShop::Header();
        print qq|
	  <form action="$program_name" method="POST" enctype="multipart/form-data"
	name="stockForm">
	  <input type="hidden" name="action" value="EditProduct">
	  <input type="hidden" name="selected" value="3">
	  <input type="hidden" name="id" value="$id">
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	  <font class="head">$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'PROD'}</th></tr>
 	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'PROD'}  $closedShop::TEXT{'NAME'}:</td>
	  <td><font class="main"> <input type="text" name="name" value="$name" size="20"></td></tr>
	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'PROD'} Code/Id:</td>
	  <td><font class="main"> <input type="text" name="code" value="$code" size="20"></td></tr>
	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'CAT'}:</td>
	  <td><font class="main"> |;

        closedShop::PrintCategoryList( "category", 0, "ALL", "", "$category" );

        print qq|
	  </td></tr>
	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'PROD'}  $closedShop::TEXT{'PRC'}:</td>
	  <td><font class="main"> <input type="text" value="$price" name="price" size="6"></td></tr>
	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'SHP_PRC'}:</td>
	  <td><font class="main"> <input type="text" value="$shipping_price" name="shipping" size="6"></td></tr>
         <tr class="mainrow"><td><center><font class="main">$closedShop::TEXT{'REDUCE'} $closedShop::TEXT{'SHP_PRC'}:</td>
          <td><font class="main"> $closedShop::CurrencyMarker <input
type="text" name="ReducedShipping" size="6" 
value="$ReducedShipping"><BR>$closedShop::TEXT{'REDU_MESG'}</td>
</td></tr>
 	  <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'WGHT'}:</td>
	  <td><font class="main"> <input type="text" value="$weight" name="weight" size="6"> $closedShop::WeightIdentifier</td></tr>
	
	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'QTY'}  $closedShop::TEXT{'INSTOCK'}:</td>
	  <td><font class="main"> <input type="text" value="$quantity" name="quantity" size="4"></td></tr>
	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'PROD'}  $closedShop::TEXT{'DESC'}:</td>
	  <td><font class="main"> <textarea mce_editable='true' name="desc" cols="60" rows="8">$description</textarea></td></tr>
  	  <tr class="mainrow"><td><center><font class="main">  $closedShop::TEXT{'IMAGE'}:</td>
	  <td><font class="main"> |;

        if (   -e "$closedShop::IMAGEPATH/$id.jpg"
            || -e "$closedShop::SIMAGEPATH/$id.jpg" )
        {

            my ( $width, $height ) = closedShop::ImageDecode("$closedShop::IMAGEPATH/$id.jpg");
            if ( !$width ) {
                ( $width, $height ) = closedShop::ImageDecode("$closedShop::SIMAGEPATH/$id.jpg");
                if ( !$width ) {
                    $width  = 400;
                    $height = 300;
                }
            }

            my $usewidth;
            if ( $width > $closedShop::TableWidth - 100 ) {
                $usewidth = $closedShop::TableWidth - 100;
            }
            else {
                $usewidth = $width;
            }

            my $ratio     = ( $closedShop::TableWidth - 100 ) / $width;
            my $useheight = $ratio * $height;

            print qq|
	<font style='color: red; font-size: 16px; font-variant: small-caps;'>X</font> <a href="$program_name?action=EditProduct&id=$id&selected=4">Remove Picture</a><BR>
	|;
            if ( $closedShop::FileNames{"item"} =~ /https/ ) {
                print qq|<img width='$usewidth' height='$useheight' src="$closedShop::SecureImageURL/$id.jpg"><br>\n|;
            }
            else {
                print qq|<img width='$usewidth' height='$useheight' src="$closedShop::IMAGEURL/$id.jpg"><br>\n|;
            }
        }
        else {
            print qq|<font class="main"><b>$closedShop::TEXT{'NOIMAGE'}</b><br>\n|;
        }

        print qq|
	  Upload new picture<BR>
	  <input type="file" name="image" size="35"></td></tr>
	 <tr class="mainrow"><td><center><font class="main"> Or $closedShop::TEXT{'IMAGE'} URL:</td>
	  <td>
	  |;
        if ($image_url) {
            print qq|<img src="$image_url" border='1'><BR>|;
        }
        print qq|
	  <font class="main"> <input type="text" name="image_url" value="$image_url" size="45"> <BR>
	  (<i>External Link to Image</i>)</td></tr>
	  
	 
	   <tr class="headrow"><td colspan='2'>
	   <font class="main"><b> $closedShop::TEXT{'ADD'} $closedShop::TEXT{'ATT'}:</b></td>
	   </tr>
	   <td colspan='2'>
	  <font class="main">
	  <table  class="main" width="100%"><tr class='mainrow'>
	  |;

        for ( my $j = 1 ; $j < $closedShop::MaxAttributes + 1 ; $j++ ) {

            $SQL = qq| select name,id from $closedShop::sql_tables{"attributes"} where
	           product_id='$id' and ordering='$j'|;
            $sth = closedShop::DatabaseQuery($SQL);
            my @attr = $sth->fetchrow_array();

            if ( ( $j - 1 ) % 3 == 0 ) {
                print qq|<tr class='mainrow'>|;
            }

            print qq|<td valign='top' style='border: 1px solid black;'><font class="main">
	  <u style='background-color: $closedShop::input_hover'>$closedShop::TEXT{'ATTR'} $j:</u><br>
	  $closedShop::TEXT{'ATTR'} $closedShop::TEXT{'NAME'}: <input 
	  type="text" name="attribute$j-name" value="$attr[0]" size="10">
	   <table style='border: 1px solid gray;' width='100%'><tr style='font-size: 10px;'>
	  <td><font >$closedShop::TEXT{'ATTR'}</td><td>$closedShop::TEXT{'PRC'} ($closedShop::TEXT{'EXTRA'})
	   </td></tr>|;

            for ( my $t = 1 ; $t < $closedShop::MaxAttributeOptions + 1 ; $t++ ) {

                my $SQL2 = qq| select data,price,id from $closedShop::sql_tables{"attribute_values"}
	              where attribute_id='$attr[1]' and ordering='$t'|;
                my $sth2 = closedShop::DatabaseQuery($SQL2);
                my @vals = $sth2->fetchrow_array();
                print qq|<tr><td style='font-size: 10px;'>$t)
	       <input style='font-size: 10px;' type="text" name="attribute$j-$t-data" value="$vals[0]" size="16">
	       </td><td style='font-size: 10px;'>
	        $closedShop::CurrencyMarker <input style='font-size: 10px;' type="text" name="attribute$j-$t-price" value="$vals[1]" size="5">
	       </td>
	       |;
            }

            print qq|</table></td> |;

            if ( $j % 3 == 0 ) {
                print qq|</tr>|;
            }

        }
        print qq|</tr></table>
	  
	 </td></tr>|;
        if ( $closedShop::EbayAuction eq "Y" ) {
            eval("use LWP::UserAgent");
            if ($@) {
                print qq|<tr class="mainrow"><td colspan="2">
                <center><font class="main"> <b>Ebay $closedShop::TEXT{'ERR'}</b></td></tr>
                <tr class="mainrow">
                <td colspan="2"><font class="main">$closedShop::TEXT{'POST_EBAY_ERR'}<BR>
                <li>LWP::UserAgent
                <li>HTTP::Request::Common
                which can be found at
                <a href="http://search.cpan.org">http://search.cpan.org</a> </td></tr>
                |;
            }
            else {
                DisplayEbayData( $name, $price, $description, $quantity );
            }
        }

        print qq|
	  <tr class="headrow"><th colspan="2">
	  <font class="main"> <font size="3"><input type="submit" value="$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'PROD'}"></th></tr>
	  </form></table>
	  |;
        BottomAdmin();
        closedShop::Footer();
    }
    elsif ( $query->param("selected") == 1 ) {
        my $cate = $query->param("category");
        closedShop::Header();
        print qq|
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="6">
	  <font class="head">$closedShop::TEXT{'EDIT'} $closedShop::TEXT{'PROD'}
		in $closedShop::TEXT{'CAT'}: $closedShop::Categories{$cate}</font></th></tr>
	  <tr class="mainrow">
	  <td><font class="main"><b>$closedShop::TEXT{'NAME'}</b></td>
	  <td><font class="main"><b>Code/Id</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'PRC'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'QTY'} $closedShop::TEXT{'INSTOCK'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'EDIT'}</b></td>
	  </tr>|;

        my $SQL = qq| select name,price,id,product_code,quantity from
	                $closedShop::sql_tables{"item"} where category="$cate"
	                order by name |;

        my $sth    = closedShop::DatabaseQuery($SQL);
        my $icount = 0;
        while ( my ( $name, $price, $id, $code, $quantity ) = $sth->fetchrow_array() ) {
            print qq|
		    <tr class="mainrow">
			<td><font class="main"> $name</td>
			<td><font class="main"> $code</td>
			<td><font class="main"> $closedShop::CurrencyMarker $price</td>
			<td><font class="main"> $quantity</td>
			  <form action="$program_name" method="POST" 
           			enctype="multipart/form-data">
	 		 <input type="hidden" name="action" value="EditProduct">
	  		<input type="hidden" name="selected" value="2">
			<input type="hidden" name="id" value="$id">
			<td><font class="main"> <input type="submit" value="Edit"></td>
			</tr></form>\n |;
            $icount++;
        }

        print qq|
        <tr class="headrow"><td colspan="5">
	<font class="main"><center><b>$closedShop::TEXT{'TOT'} 
$closedShop::TEXT{'PRODS'}: $icount</td></tr>
		</table></form>|;
        BottomAdmin();
        closedShop::Footer();
    }
    else {
      START:
        closedShop::Header();
        print qq|
	<form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="EditProduct">
	  <input type="hidden" name="selected" value="1">
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	  <font class="head"> $closedShop::TEXT{'SELECT'} $closedShop::TEXT{'CAT'}</font></th></tr>
	 <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'CAT'}:</td>
	  <td><font class="main"> |;

        closedShop::PrintCategoryList( "category", 0, "ALL" );

        print qq|
	 </td></tr>
	    <tr class="headrow"><td colspan="2"><center><input 
		type="submit" value="$closedShop::TEXT{'EDIT'}"></td></tr>
		</table></form>|;
        BottomAdmin();
        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  ViewOrders
# DESCRIPTION: allows merchant to view / process orders
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ViewOrders {
    my $mesg = shift;
    if ( $query->param("selected") == 2 ) {
        if ( $query->param("process") ) {
            my $id = $query->param("id");
            $id =~ tr/0-9//cd;

            my $tracking  = closedShop::ValidateInputAdmin( $query->param("tracking") );
            my $trackno   = closedShop::ValidateInputAdmin( $query->param( $tracking . "No" ) );
            my $trackname = ( split( /~~~/, $closedShop::ShippingCompanies{$tracking} ) )[0];
            my $trackurl  = ( split( /~~~/, $closedShop::ShippingCompanies{$tracking} ) )[1];
            $trackurl =~ s/<NUMBER>/$trackno/g;
            if ( !$trackname ) {
                $trackname = closedShop::ValidateInputAdmin( $query->param("OtherName") );
                $tracking  = $trackname;
            }
            my $trackmesg = "";
            if ($tracking) {
                $trackmesg = qq|\n\n$closedShop::TEXT{'ORD_SHP'} $trackname, $closedShop::TEXT{'W_TRK'} $trackno\n\n|;
            }
            if ( $tracking ne "Other" ) {
                $trackmesg .= qq|$closedShop::TEXT{'FOL_PROG'} $trackurl|;
            }

            my $SQL = qq|update $closedShop::sql_tables{"order"} set
	      date_processed=CURDATE(),tracking="$tracking",
	      trackno="$trackno"where id = "$id" |;
            my $sth = closedShop::DatabaseQuery($SQL);

            $SQL = qq|select user_id,item_ids from $closedShop::sql_tables{"order"}
	              where id = "$id" |;
            $sth = closedShop::DatabaseQuery($SQL);
            my ( $user_id, $item_ids ) = $sth->fetchrow_array();

            my @ary = ();
            if ( $item_ids =~ /$closedShop::CookieItemDelimiter/ ) {
                @ary = split( /$closedShop::CookieItemDelimiter/, $item_ids );
            }
            else {
                @ary = split( /~/, $item_ids );
            }

            foreach (@ary) {
                if ( !$_ ) { next; }
                my ( $id, $qty, $attr );
                if ( $_ =~ /$closedShop::CookieSectionDelimiter/ ) {
                    ( $id, $qty, $attr ) = split( /$closedShop::CookieSectionDelimiter/, $_ );
                }
                else {
                    ( $id, $qty, $attr ) = split( /\|/, $_ );
                }

                if ( $id ne "gc" ) {
                    $SQL = qq|update $closedShop::sql_tables{"item"}
                      set quantity=quantity-'$qty' where id = "$id" |;
                    $sth = closedShop::DatabaseQuery($SQL);
                }
            }

            $SQL = qq|select email,fname,lname,DATE_FORMAT(CURDATE(),"%W, %M %d %Y")
		          from $closedShop::sql_tables{"user"} where id = "$user_id" |;
            $sth = closedShop::DatabaseQuery($SQL);
            my ( $email, $fname, $lname, $now ) = $sth->fetchrow_array();
            my $body = qq|
$fname,

$closedShop::TEXT{'YR_ORD'} (# $id) from $closedShop::CompanyName
$closedShop::TEXT{'PROCNSHP'} $now.

$closedShop::TEXT{'FUR_NFO'}
$closedShop::website
$closedShop::TEXT{'LG_CALL'} $closedShop::CompanyPhone
$trackmesg
$closedShop::TEXT{'THANK'},
$closedShop::CompanyName
$closedShop::CompanyEmail
|;

            # send followup email to customer
            closedShop::SendEmail( $email, $closedShop::TEXT{'ORD_PROC'}, $body );
            $query = new CGI( {} );
            return ViewOrders( $closedShop::TEXT{'SUCC_PROC'} );
        }
        elsif ( $query->param("viewold") ) {
            my $id = $query->param("id");
            $id =~ tr/0-9//cd;
            if ( !$id ) { goto ListAllOldOrders; }
            closedShop::Header();

            DisplayIndividualOrder($id);

            print qq|</table></form>|;
            BottomAdmin();
            closedShop::Footer();
        }
        elsif ( $query->param("deleteold") ) {
            my $id = $query->param("id");
            $id =~ tr/0-9//cd;
            my $SQL = qq|delete from $closedShop::sql_tables{"order"} where id="$id"|;
            my $sth = closedShop::DatabaseQuery($SQL);
            goto ListAllOldOrders;
        }
        else {
            $query = new CGI( {} );
            return ViewOrders();
        }
    }
    elsif ( $query->param("selected") == 1 ) {
        if ( $query->param("process") ) {
            my $id = $query->param("id");
            $id =~ tr/0-9//cd;
            if ( !$id ) { goto ListAllOrders; }
            closedShop::Header();

            DisplayIndividualOrder($id);

            print qq|
		<tr class="mainrow">
	        <td><font class="main"> $closedShop::TEXT{'PCK_TRK'}</div></td>
		<td><font class="main">
		<input type="radio" name="tracking" value="" checked> $closedShop::TEXT{'NONE'}<br>|;
            foreach ( keys %closedShop::ShippingCompanies ) {
                my @tmp = split( /~~~/, $closedShop::ShippingCompanies{$_} );
                my $tmpname = $tmp[0];
                print qq| <nobr><input type="radio" name="tracking"
		    value="$_"> $tmpname No: <input type="text"
		    name="| . $_ . qq|No" size="15"></nobr><BR>|;
            }

            print qq|
		<nobr><input type="radio" name="tracking"
		value="Other"> $closedShop::TEXT{'OTH_SHP'}: $closedShop::TEXT{'NAME'}: <input
		type="text" name="OtherName" size="10"><BR>
		$closedShop::TEXT{'OTH_SHP'} $closedShop::TEXT{'TRK'} #:<input type="text"
		name="OtherNo" size="15"></nobr></td>
		</tr>

		<tr class="headrow">
	        <td align="" colspan="2"><font class="main"><div align="center">
		<input type="submit" name="process" value="$closedShop::TEXT{'PROC'} $closedShop::TEXT{'ORD'}"></div>
		</td></tr></table></form>
		|;
            BottomAdmin();
            closedShop::Footer();
        }
        elsif ( $query->param("deleteorder") ) {
            my $id = $query->param("id");
            $id =~ tr/0-9//cd;
            if ( !$id ) { goto ListAllOrders; }
            my $SQL = qq|delete from $closedShop::sql_tables{"order"}
			             where id = "$id" |;
            my $sth = closedShop::DatabaseQuery($SQL);
            goto ListAllOrders;
        }
        else {
          ListAllOldOrders:
            closedShop::Header();
            print qq|
	 	 <font class="main"><font color="red">$mesg</font>
	 	 <form action="$program_name" method="POST" enctype="multipart/form-data">
	  	<input type="hidden" name="action" value="ViewOrders">
	  	<input type="hidden" name="selected" value="2">
	  	<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="5">
	 	 <font class="head">$closedShop::TEXT{'VW_OORD'}</font></th></tr>
	 	 <tr class="mainrow">
	  	<td><font class="main"><b>$closedShop::TEXT{'VW_DET'}</b></td>
	  	<td><font class="main"><b>$closedShop::TEXT{'DBOUGHT'}</b></td>
	 	 <td><font class="main"><b>$closedShop::TEXT{'NAME'}</b></td>
	 	 <td><font class="main"><b>$closedShop::TEXT{'EMAIL'}</b></td>
	 	 <td><font class="main"><b>$closedShop::TEXT{'TOT'}</b></td></tr>|;

            my $SQL = qq| select id,DATE_FORMAT(date_bought,"%W, %M %d %Y"),user_id,
	                total from $closedShop::sql_tables{"order"}
	                where complete="Y" and date_processed is NOT NULL order by date_bought DESC |;

            my $sth      = closedShop::DatabaseQuery($SQL);
            my $k        = 0;
            my $alltotal = 0;
            while ( my ( $oid, $date_bought, $uid, $total ) = $sth->fetchrow_array() ) {
                $alltotal += $total;
                my $SQL2 = qq| select fname,lname,email from $closedShop::sql_tables{"user"}
	                where id="$uid" |;

                my $sth2 = closedShop::DatabaseQuery($SQL2);
                my ( $fname, $lname, $email ) = $sth2->fetchrow_array();
                my $check = "checked";
                if ( $k != 0 ) { $check = ""; }
                print qq|
		     <tr class="mainrow">
			 <td><font class="main"><input type="radio" name="id" value="$oid" $check></td>
			 <td><font class="main"> $date_bought</td>
			 <td><font class="main"> $fname $lname</td>
			 <td><font class="main"> <a href="mailto:$email">$email</a></td>
			 <td><font class="main"> $closedShop::CurrencyMarker  $total</td></tr>\n |;
                $k++;
            }
            if ( $k == 0 ) {
                print qq|<tr class="mainrow">
		<td colspan='5'><font class='main'><center>No orders have been processed</td></tr>\n |;

            }

            print qq|
        <tr class="headrow"><td colspan="2"><font class="main"><center><input
		type="submit" name="viewold" value="$closedShop::TEXT{'VW_DET'}"><BR><BR>
		<input type="submit" onClick="return Confirm();" name="deleteold" value="$closedShop::TEXT{'DEL_OORD'}"></td>
		<td>&nbsp;</td><td><b><font class="main"> $closedShop::TEXT{'TOT'}: </td>
		<td><font class="main"> $closedShop::CurrencyMarker $alltotal</b></td>
		</tr>
		</table></form>|;
            BottomAdmin();
            closedShop::Footer();
        }
    }
    else {
      ListAllOrders:
        closedShop::Header();
        print qq|
	  <font class="main"><font color="red">$mesg</font>
	  <form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="ViewOrders">
	  <input type="hidden" name="selected" value="1">
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="5">
	  <font class="head">$closedShop::TEXT{'PROC'} $closedShop::TEXT{'ORD'}</font></th></tr>
	  <tr class="mainrow">
	  <td><font class="main"><b>$closedShop::TEXT{'PROC'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'DBOUGHT'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'NAME'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'EMAIL'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'TOT'}</b></td></tr>|;

        my $SQL = qq| select id,DATE_FORMAT(date_bought,"%W, %M %d %Y"),user_id,
	                total from $closedShop::sql_tables{"order"} 
	                where complete="Y" and date_processed is NULL order by date_bought DESC |;

        my $sth4 = closedShop::DatabaseQuery($SQL);
        my $k    = 0;
        while ( my ( $oid, $date_bought, $uid, $total ) = $sth4->fetchrow_array() ) {
            my $SQL2 = qq| select fname,lname,email from $closedShop::sql_tables{"user"} 
	                where id="$uid" |;

            my $sth2 = closedShop::DatabaseQuery($SQL2);
            my ( $fname, $lname, $email ) = $sth2->fetchrow_array();
            my $check = "checked";
            if ( $k != 0 ) { $check = ""; }
            print qq|
		    <tr class="mainrow">
			<td><font class="main"><input type="radio" name="id" value="$oid" $check></td>
			<td><font class="main"> $date_bought</td>
			<td><font class="main"> $fname $lname</td>
			<td><font class="main"> <a href="mailto:$email">$email</a></td>
			<td><font class="main"><nobr> $closedShop::CurrencyMarker $total</nobr></td></tr>\n |;
            $k++;
        }

        print qq|
		 <tr class="mainrow"><td colspan="5"><font class="main">
		 $closedShop::TEXT{'PROCMESG'}
		 </td>
		</tr>
        <tr class="headrow"><td colspan="2"><font 
		class="main"><center><input
		type="submit" name="process" value="$closedShop::TEXT{'VIEW_PROC'}"><br><br>
    		<input
		type="submit" onClick="return Confirm();" name="deleteorder" value="$closedShop::TEXT{'REM'} $closedShop::TEXT{'ORD'}"></td>
		<td>&nbsp;</td><td colspan="2"><font class="main"><center><input
		type="submit" name="viewold" value="$closedShop::TEXT{'VW_OORD'}"></td>
		</tr>
		</table></form>|;
        BottomAdmin();
        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  PrintReceipt
# DESCRIPTION: prints a receipt
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub PrintReceipt {
    my $id = $query->param("id");
    $id =~ tr/0-9//cd;
    if ( !$id ) { $id = 0; }

    print $closedShop::HTML_HEADER;

    print "<html><head>";
    closedShop::WriteStyleSheets();
    print "
    </head>
    <body onLoad='javascript:print();'>
    <font class='receipt' style='font-size: 18px;'>
    <center>
    <b>$closedShop::CompanyName</b></font><BR>

    $closedShop::CompanyAddress<Br>
    $closedShop::CompanyCity, $closedShop::CompanyState $closedShop::CompanyZip<BR>
    $closedShop::CompanyPhone
    </font><BR><BR>    </center>
    ";

    my $SQL = qq|  select b_fname,b_lname,b_address,b_city,b_state,b_country,
                  b_zip,b_phone,s_fname,s_lname,s_address,s_city,s_state,s_country,
                  s_zip,s_phone,item_ids,cc_name,cc,cc_num,cc_month,cc_year,cc_vcode,
		  amount_gc_used,total,sub_total,ship_total,tax,ip_address,
		 tracking,trackno,precharge_score,
        	 precharge_response,DATE_FORMAT(date_bought,"%W, %M %d %Y"),user_id,
		 stax,itax,ctax,score as maxmindscore,binMatch,
		 $closedShop::sql_tables{"maxmind"}.binName,binCountry,
                 binNameMatch,binPhoneMatch,
		 $closedShop::sql_tables{"maxmind"}.binPhone,freeMail,carderEmail,
                 anonymousProxy,ip_region,ip_city,ip_isp,countryMatch,
  		highRiskCountry,distance,shipForward,custPhoneInBillingLoc,
		taxExemptID,AuthorizeNETResult
		  from ($closedShop::sql_tables{"order"} LEFT JOIN 
		  $closedShop::sql_tables{"maxmind"} ON 
		  $closedShop::sql_tables{"maxmind"}.oid = 
		  $closedShop::sql_tables{"order"}.id)
		  where  $closedShop::sql_tables{"order"}.id="$id" 		   
		    |;

    my $sth = closedShop::DatabaseQuery($SQL);
    my (
        $b_fname,    $b_lname,   $b_address,    $b_city,       $b_state,         $b_country,  $b_zip,        $b_phone,               $s_fname,            $s_lname,     $s_address,   $s_city,
        $s_state,    $s_country, $s_zip,        $s_phone,      $item_ids,        $cc_name,    $card,         $cc_num,                $cc_month,           $cc_year,     $cc_vcode,    $amount_gc_used,
        $total_cost, $sub_total, $ship_total,   $tax_total,    $ip_address,      $tracking,   $trackno,      $precharge_score,       $precharge_response, $date_bought, $uid,         $staxrate,
        $itaxrate,   $ctaxrate,  $maxmindscore, $binMatch,     $binName,         $binCountry, $binNameMatch, $binPhoneMatch,         $binPhone,           $freeMail,    $carderEmail, $anonymousProxy,
        $ip_region,  $ip_city,   $ip_isp,       $countryMatch, $highRiskCountry, $distance,   $shipForward,  $custPhoneInBillingLoc, $taxExemptID,        $AuthorizeNETResult
      )
      = $sth->fetchrow_array();

    if ( !$staxrate ) { $staxrate = $closedShop::TaxRate; }
    if ( !$itaxrate ) { $staxrate = $closedShop::TaxRateInt; }
    if ( !$ctaxrate ) { $staxrate = $closedShop::TaxRateCountry; }

    print qq|
      <table width=680 style='border: 1px solid black;' ><tr ><th colspan="2">
	   <b><font class="receipt">$closedShop::TEXT{'ORD'} # $id ($date_bought)</b></th></tr>

		<tr>
        <td valign=top style='border: 1px solid black;'><font class="receipt">
		<strong>$closedShop::TEXT{'BILL_NFO'}</strong><br>
		
		|;
    $SQL = qq|select $closedShop::sql_tables{"groups"}.name,
		discount from $closedShop::sql_tables{"user"},$closedShop::sql_tables{"groups"}
		where $closedShop::sql_tables{"user"}.id = '$uid' and 
		$closedShop::sql_tables{"user"}.groupid = $closedShop::sql_tables{"groups"}.id |;
    $sth = closedShop::DatabaseQuery($SQL);
    my ( $gname, $gdiscount ) = $sth->fetchrow_array();

    my @ipaddr = split( / /, $ip_address );
    $ip_address = $ipaddr[1] . " (" . $ipaddr[0] . ")";
    print qq|
		$b_fname $b_lname<br>
		$b_address<br>
		$b_city, $b_state $b_zip<br>
		$b_country<BR>
		$b_phone<BR>
		</td>
		<td style='border: 1px solid black;' valign='top'><font class="receipt">
		<strong>$closedShop::TEXT{'SHIP'} $closedShop::TEXT{'NFO'}</strong><br>
		$s_fname $s_lname<br>
		$s_address<br>
		$s_city, $s_state $s_zip<br>
		$s_country<BR>
		$s_phone
		</td></tr>
		<tr>
        <td colspan="2"><font class="receipt">
		<strong>$closedShop::TEXT{'PAY_NFO'}</strong><br>
		|;

    print qq|$closedShop::TEXT{'PAY_METH'}: | . $closedShop::KnownCards{$card} . qq|<BR>|;

    if ( $closedShop::EnableTaxExempt eq "Y" && $taxExemptID != "" ) {
        print qq|<BR><BR><u>Tax Exempt ID</u>: $taxExemptID|;

    }

    print qq|
		</td></tr>|;

    print qq|
		<tr ><td colspan="2">
		<b><font class="receipt">$closedShop::TEXT{'PROD'} $closedShop::TEXT{'NFO'}</b>
		<center>
		<table width="100%" style='border: 1px solid black;' cellpadding="1" cellspacing="1">
		<tr >
		<td><font class="receipt"><strong>$closedShop::TEXT{'NAME'}</strong></td>
		<td><font class="receipt"><strong>Code</strong></td>
		<td><font class="receipt"><strong>$closedShop::TEXT{'QTY'}</strong></td>
		<td><font class="receipt"><strong>$closedShop::TEXT{'ATT'}</strong></td>
		<td><font class="receipt"><strong>$closedShop::TEXT{'PRC'}</strong></td>
		<td><font class="receipt"><strong>$closedShop::TEXT{'SHP_PRC'}</strong></td>
		<td><font class="receipt"><strong>$closedShop::TEXT{'TOT'}</strong></td>
		</tr>
		|;

    my @ary = ();
    if ( $item_ids =~ /$closedShop::CookieItemDelimiter/ ) {
        @ary = split( /$closedShop::CookieItemDelimiter/, $item_ids );
    }
    else {
        @ary = split( /~/, $item_ids );
    }

    my $k = 1;
    my ( $itemsub, $shipsub, $totalweight ) = ( 0, 0, 0 );
    foreach (@ary) {
        if ( !$_ ) { next; }

        my ( $iid, $qty, $attr );

        if ( $_ =~ /$closedShop::CookieSectionDelimiter/ ) {
            ( $iid, $qty, $attr ) = split( /$closedShop::CookieSectionDelimiter/, $_ );
        }
        else {
            ( $iid, $qty, $attr ) = split( /\|/, $_ );
        }

        if ( $iid eq "gc" ) {
            my ( $id, $recipient_name, $recipient_email, $gc_amount );
            if ( $_ =~ /$closedShop::CookieSectionDelimiter/ ) {
                ( $id, $recipient_name, $recipient_email, $gc_amount ) =
                  split( /$closedShop::CookieSectionDelimiter/, $_ );
            }
            else {
                ( $id, $recipient_name, $recipient_email, $gc_amount ) = split( /\|/, $_ );
            }

            print qq|
	   	  <tr style='border: 1px solid black;'>
		  <td><font class="receipt"> $closedShop::TEXT{'GC'} ($closedShop::CurrencyMarker $gc_amount)</td>
		  <td><font class="receipt"> GC</td>
		  <td><font class="receipt"> 1</td>
		  <td><font class="receipt"> &nbsp;</td>
		  <td><font class="receipt"> $qty x $closedShop::CurrencyMarker $gc_amount</td>
		  <td><font class="receipt"> N/A</td>
		  <td><font class="receipt"> <nobr>$closedShop::CurrencyMarker $gc_amount</nobr></td>
			|;
            $k++;
            next;
        }

        $SQL = qq| select name,price,shipping_price,description,
	              category,product_code,reduced_shipping_price,weight from
			$closedShop::sql_tables{"item"} where id="$iid" |;
        $sth = closedShop::DatabaseQuery($SQL);
        my ( $name, $price, $ship, $desc, $cat, $code, $rship, $weight ) = $sth->fetchrow_array();
		$totalweight += $qty*$weight;

        my @attr = ();
        if ( $attr =~ /$closedShop::CookieAttributeDelimiter/ ) {
            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
        }
        else {
            @attr = split( /\?/, $attr );
        }

        my @attr = ();
        if ($attr) {
            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
        }
        $attr = "";

        my $attr_cost = 0.00;
        foreach my $at (@attr) {
            my ( $prod_id, $attr_id, $data_id ) = split( /$closedShop::GiftCertDelimiter/, $at );
            my $SQL = qq|select name,data,price from $closedShop::sql_tables{"attributes"},
	   $closedShop::sql_tables{"attribute_values"} where product_id='$prod_id' and
	   $closedShop::sql_tables{"attributes"}.id = '$attr_id' and
	   $closedShop::sql_tables{"attribute_values"}.id = '$data_id'|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $atname, $atdata, $atprice ) = $sth->fetchrow_array();
            $attr .= "<nobr><u>$atname</u>: $atdata";
            if ( $atprice > 0 ) {
                $attr .= " (+$closedShop::CurrencyMarker$atprice)";
                $attr_cost += $atprice;
            }
            $attr .= "</nobr><BR>";
        }
        if ( !$attr ) { $attr = "&nbsp;"; }

        my $total = sprintf( "%.2f", $qty * ( $price + $attr_cost ) );
        $itemsub += $total;

        my ( $shiptotal, $shipstring ) = closedShop::GetShippingCostsItem( $ship, $qty, $rship, $weight);

        $shipsub += $shiptotal;

        print qq|
	   	  <tr style='border: 1px solid black;'>
		  <td><font class="receipt"> $name</a></td>
		  <td><font class="receipt"> $code</td>
		  <td><font class="receipt"> <Center>$qty</center></td>
		  <td><font class="receipt"> $attr&nbsp;</td>
		  <td><font class="receipt"> $qty x $closedShop::CurrencyMarker$price|;
        if ( $attr_cost > 0 ) {
            print qq|<BR>(+$qty x $closedShop::CurrencyMarker$attr_cost)|;
        }

        print qq|</td><td><font class="main">$shipstring</td>
		  <td><font class="main"> <nobr>$closedShop::CurrencyMarker|
          . sprintf( "%.2f", $total + $shiptotal ) . qq|</nobr></td>
		  </tr>  |;
        $k++;
    }

	my ( $shipclass, $shipstring1 ) = closedShop::GetShippingCostsClass( $totalweight, $itemsub);


    if ( $closedShop::EnableTaxExempt eq "Y" && $taxExemptID != "" ) {
        ( $staxrate, $ctaxrate, $itaxrate ) = ( 0, 0, 0 );
        $tax_total = " EXEMPT ";

    }

    print qq|<tr>
		<td colspan="4" align="right">&nbsp;</td>
		<td colspan="2"><font class="receipt">
     	<strong>Item $closedShop::TEXT{'TOT'}: </td>
     	<td><font class="receipt"> $closedShop::CurrencyMarker| . sprintf( "%.2f", $itemsub ) . qq|</strong></td></tr>

		 <tr ><td colspan="4" align="right">&nbsp;</td>
		  <td colspan="2"><font class="receipt"> 
		  <strong>$closedShop::TEXT{'SHIP'} $closedShop::TEXT{'TOT'}:  $shipstring1</td>
      <td><font class="receipt"> $closedShop::CurrencyMarker| . sprintf( "%.2f", $ship_total ) . qq|</strong> 
      </td></tr>

		 <tr ><td colspan="4" align="right">&nbsp;</td>
		  <td colspan="2"><font class="receipt"> 
		  <strong>$closedShop::TEXT{'TAX'} $closedShop::TEXT{'TOT'}:</td>
      <td><font class="receipt"> $closedShop::CurrencyMarker$tax_total</strong> </td></tr>
		|;

    my ( $stax, $ctax, $itax ) = closedShop::GetTaxCosts( $sub_total, $taxExemptID, $b_state, $b_country );
    $tax_total = $stax + $ctax + $itax;

    if ( $closedShop::ShowTax eq "Y" ) {
        my $vrate = ( $staxrate * 100 );
        if ( $staxrate > 0 ) {
            print qq|
		  <tr><td colspan="4" align="right">&nbsp;</td>
		  <td colspan='2'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
		  <font class="receipt"> <strong>$closedShop::TEXT{'TAX'} (for $closedShop::CompanyState at $vrate%):</td>
      <td><font class="receipt"> $closedShop::CurrencyMarker$stax</strong> </td></tr>
			|;
        }
        if ( $ctaxrate > 0 ) {
            my $crate = ( $ctaxrate * 100 );
            print qq|
		     <tr><td colspan="4" align="right">&nbsp;</td>
		     <td colspan='2'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
		     <font class="receipt"> <strong>$closedShop::TEXT{'TAX'}  (for $closedShop::CompanyCountry at $crate%):</td>
         <td><font class="receipt"> $closedShop::CurrencyMarker$ctax</strong> </td></tr>
			   |;
        }
        if ( $itaxrate > 0 && $closedShop::CompanyCountry ne $b_country ) {
            my $irate = ( $itaxrate * 100 );
            print qq|
		     <tr ><td colspan="4" align="right">&nbsp;</td>
		     <td colspan='2'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
		     <font class="receipt"> <strong>$closedShop::TEXT{'TAX'} (for International at $irate%):</td>
         <td><font class="receipt"> $closedShop::CurrencyMarker$itax</strong> </td></tr>
			   |;
        }
    }

    if ( $amount_gc_used > 0 ) {
        print qq|
		  <tr ><td colspan="4" align="right">&nbsp;</td>
			<td  colspan="2"><font class="receipt">
     	<strong>$closedShop::TEXT{'GC'} $closedShop::TEXT{'DED'}:</td><td><font class="main">
		  <strong>$closedShop::CurrencyMarker <font color="red">-($amount_gc_used)</font></strong></strong></td></tr>
		 |;
    }

    if ( $gname || $gdiscount ) {
        my $before = ( $total_cost * ( $gdiscount / 100 ) ) + $total_cost;
        my $disc = sprintf( "%.2f", $before * ( $gdiscount / 100 ) );

        print qq|		
	<tr ><td colspan="4" align="right">&nbsp;</td><td colspan='2'>
	<font class="receipt">
     	 <font color='red'>$gname Discount ($gdiscount\%)</font>:</td>
	 <td><font class="receipt"><font color='red'>-$closedShop::CurrencyMarker$disc</strong></td></tr>
			 |;

    }

    print qq|
		<tr ><td colspan="4" align="right">&nbsp;</td>
		<td  colspan="2"><font class="receipt">
		    <strong>Grand $closedShop::TEXT{'TOT'}:</td>
		    <td><font class="receipt">
		<strong>$closedShop::CurrencyMarker$total_cost</strong></strong></td></tr>
		</table></td></tr>|;

    if ($tracking) {
        my @tmp       = split( /~~~/, $closedShop::ShippingCompanies{"$tracking"} );
        my $trackname = $tmp[0];
        my $trackurl  = $tmp[1];
        if ( !$trackname ) { $trackname = $tracking; }
        $trackurl =~ s/<NUMBER>/$trackno/g;
        if ($trackurl) {
            $trackurl = qq|$closedShop::TEXT{'PCK_TRK'} (No. $trackno)|;
        }
        else {
            $trackurl = qq|$closedShop::TEXT{'TRK'} #: $trackno|;
        }

        print qq|
  		    <tr ><td colspan="2">
                    <font class="receipt"><b>$closedShop::TEXT{'TRK'} $closedShop::TEXT{'NFO'}</b>
                <center></td></tr>
		<tr><td><font class="receipt">
		$closedShop::TEXT{'SHIP_CO'}</td>
		<td><font class="receipt"> $trackname
		</td></tr>
		<tr ><td><font class="receipt">
		$closedShop::TEXT{'PCK_TRK'}:</td>
		<td><font class="receipt"> $trackurl</td></tr>
		
		|;
    }
    print qq|</table></td></tr></table></body></html>|;
}

###############################################################################
# FUNCTION:  DisplayIndividualOrder
# DESCRIPTION: info
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DisplayIndividualOrder {
    my $id = shift;
    $id =~ tr/0-9//cd;
    if ( !$id ) { $id = 0; }

    my $SQL = qq|  select b_fname,b_lname,b_address,b_city,b_state,b_country,
                  b_zip,b_phone,s_fname,s_lname,s_address,s_city,s_state,s_country,
                  s_zip,s_phone,item_ids,cc_name,cc,cc_num,cc_month,cc_year,cc_vcode,
		  amount_gc_used,total,sub_total,ship_total,tax,ip_address,
		 tracking,trackno,precharge_score,
        	 precharge_response,DATE_FORMAT(date_bought,"%W, %M %d %Y"),user_id,
		 stax,itax,ctax,score as maxmindscore,binMatch,
		 $closedShop::sql_tables{"maxmind"}.binName,binCountry,
                 binNameMatch,binPhoneMatch,
		 $closedShop::sql_tables{"maxmind"}.binPhone,freeMail,carderEmail,
                 anonymousProxy,ip_region,ip_city,ip_isp,countryMatch,
  		highRiskCountry,distance,shipForward,custPhoneInBillingLoc,
		taxExemptID,AuthorizeNETResult
		  from ($closedShop::sql_tables{"order"} LEFT JOIN 
		  $closedShop::sql_tables{"maxmind"} ON 
		  $closedShop::sql_tables{"maxmind"}.oid = 
		  $closedShop::sql_tables{"order"}.id)
		  where  $closedShop::sql_tables{"order"}.id="$id" 		   
		    |;

    my $sth = closedShop::DatabaseQuery($SQL);
    my (
        $b_fname,    $b_lname,   $b_address,    $b_city,       $b_state,         $b_country,  $b_zip,        $b_phone,               $s_fname,            $s_lname,     $s_address,   $s_city,
        $s_state,    $s_country, $s_zip,        $s_phone,      $item_ids,        $cc_name,    $card,         $cc_num,                $cc_month,           $cc_year,     $cc_vcode,    $amount_gc_used,
        $total_cost, $sub_total, $ship_total,   $tax_total,    $ip_address,      $tracking,   $trackno,      $precharge_score,       $precharge_response, $date_bought, $uid,         $staxrate,
        $itaxrate,   $ctaxrate,  $maxmindscore, $binMatch,     $binName,         $binCountry, $binNameMatch, $binPhoneMatch,         $binPhone,           $freeMail,    $carderEmail, $anonymousProxy,
        $ip_region,  $ip_city,   $ip_isp,       $countryMatch, $highRiskCountry, $distance,   $shipForward,  $custPhoneInBillingLoc, $taxExemptID,        $AuthorizeNETResult
      )
      = $sth->fetchrow_array();

    if ( !$staxrate ) { $staxrate = $closedShop::TaxRate; }
    if ( !$itaxrate ) { $staxrate = $closedShop::TaxRateInt; }
    if ( !$ctaxrate ) { $staxrate = $closedShop::TaxRateCountry; }

    print qq|<form action="$program_name" method="POST"><font class="main">
	    <input type="hidden" name="action" value="ViewOrders">
	    <input type="hidden" name="selected" value="2">
		<input type="hidden" name="id" value="$id">

		<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="2">
	    <font class="head">$closedShop::TEXT{'ORD'} # $id ($date_bought)</font> (<A href='#a' onClick="javascript:open_window('$closedShop::FileNames{admin}?action=PrintReceipt&id=$id','Receipt',710,400);">Print Receipt</a>)</th></tr>

		<tr class="mainrow">
        <td ><font class="main" valign='top'>
		<strong>$closedShop::TEXT{'BILL_NFO'}</strong><br>
		|;

    $SQL = qq|select $closedShop::sql_tables{"groups"}.name,
		discount from $closedShop::sql_tables{"user"},$closedShop::sql_tables{"groups"}
		where $closedShop::sql_tables{"user"}.id = '$uid' and 
		$closedShop::sql_tables{"user"}.groupid = $closedShop::sql_tables{"groups"}.id |;
    $sth = closedShop::DatabaseQuery($SQL);
    my ( $gname, $gdiscount ) = $sth->fetchrow_array();

    my @ipaddr = split( / /, $ip_address );
    $ip_address = $ipaddr[1] . " (" . $ipaddr[0] . ")";
    print qq|
		$b_fname $b_lname<br>
		$b_address<br>
		$b_city, $b_state $b_zip<br>
		$b_country<BR>
		$b_phone<BR><b>Originating Computer (IP)</b>: $ip_address
		</td>
		<td valign='top'><font class="main">
		<strong>$closedShop::TEXT{'SHIP'} $closedShop::TEXT{'NFO'}</strong><br>
		$s_fname $s_lname<br>
		$s_address<br>
		$s_city, $s_state $s_zip<br>
		$s_country<BR>
		$s_phone
		</td></tr>
		<tr class="mainrow">
        <td colspan="2"><font class="main">
		<strong>$closedShop::TEXT{'PAY_NFO'}</strong><br>
		|;

    print qq|$closedShop::TEXT{'PAY_METH'}: | . $closedShop::KnownCards{$card} . qq|<BR>|;
    if ( !( $card eq "check" || $card eq "paypal" || $card eq "cod" ) ) {
        print qq|
		&nbsp;&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'CC'}</u>: | . $closedShop::KnownCards{$card} . qq|<br>
		&nbsp;&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'CC'} $closedShop::TEXT{'NAME'}</u>: $cc_name<br>
		&nbsp;&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'CC_NUM'}</u>: $cc_num<br>
		&nbsp;&nbsp;&nbsp;&nbsp;<u>$closedShop::TEXT{'EXP_DATE'}</u>: $cc_month / $cc_year<BR>
		  |;

        if ($AuthorizeNETResult) {
            print qq|<BR><BR><u>Authorize.NET Results</u>: $AuthorizeNETResult|;
        }
    }

    if ( $card eq "paypal" ) {
        print qq|&nbsp;&nbsp;&nbsp;&nbsp;Before processing this order, please confirm
      that you have received the payment through PayPal|;
    }

    if ( $closedShop::EnableTaxExempt eq "Y" && $taxExemptID != "" ) {
        print qq|<BR><BR><u>Tax Exempt ID</u>: $taxExemptID|;

    }

    print qq|
		</td></tr>|;

    if (   $closedShop::EnablePreCharge eq "Y"
        && $card ne "check"
        && $card ne "paypal" )
    {
        my $smesg = "";
        if ( $precharge_score <= 250 ) {
            $smesg = "Relatively Low Risk Transaction. No history of chargebacks or unusual activity";
        }
        elsif ( $precharge_score <= 500 ) {
            $smesg = "Standard Risk Transaction. May need to contact client to verify.";
        }
        elsif ( $precharge_score <= 750 ) {
            $smesg = "High Risk Transaction. Has history of free email providers and open proxies.";
        }
        elsif ( $precharge_score <= 1000 ) {
            $smesg = "Very High Risk Transaction. History of chargebacks.";
        }

        my $response = "";
        if ( $precharge_response == 1 ) {
            $response = "<font color='blue'>Transaction Approved</font>";
        }
        elsif ( $precharge_response == 2 ) {
            $response = "<font color='red'>Fraudulent Transaction Detection.</font><BR>
		    <b>It is recommended you do not approve</b>";
        }
        else {
            $response        = "<font color='blue'>Transaction Error ($precharge_response)</font>";
            $precharge_score = "N/A";
            $smesg           = "";
        }

        print qq|<tr class="mainrow">
        <td colspan="2"><font class="main">
		<strong>PreCharge $closedShop::TEXT{'NFO'}</strong><br>
		<u>PreCharge Response</u>: $response<BR>
		<u>PreCharge Score</u>: <font color='blue'>$precharge_score / 1000</font><BR>
		$smesg
		|;

    }

    if ( $closedShop::EnableMaxMind eq "Y" ) {

        print qq|<tr class="mainrow">
        <td colspan="2"><font class="main">
		<strong>MaxMind $closedShop::TEXT{'NFO'}</strong><BR>
		<u><A href='#a' onMouseOver="show('score');"
		       onMouseOut="hide('score');">MaxMind Score</a>
		        <div id='score' name='score' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Overall Fraud Risk Factor based on all outputs <BR>
		      0 (low risk) to 10 (high risk) 
    		     </td></tr></table>
    		    </div>
		       </u>: $maxmindscore<BR>
		  
		  <table style='border: 0px;' width=100%>
		    <Tr>
		
		      <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('binmatch');"
		       onMouseOut="hide('binmatch');"><font style='font-size: 11px;'>
		      Bin Match:</a> $binMatch
		      <div id='binmatch' name='binmatch' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Whether country of issuing bank based on BIN number matches billing address country
    		     </td></tr></table>
    		    </div>
		    </td>
		
		   <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('binname');"
		       onMouseOut="hide('binname');"><font style='font-size: 11px;'>
		      Bin Name:</a> $binName
		      <div id='binname' name='binname' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Name of the bank which issued the credit card based on BIN number. 
		      Available for approximately 96% of BIN numbers.
    		     </td></tr></table>
    		    </div>
		    </td>
		
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('bincountry');"
		       onMouseOut="hide('bincountry');"><font style='font-size: 11px;'>
		      Bin Country:</a> $binCountry
		      <div id='bincountry' name='bincountry' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Whether country of issuing bank based on BIN number matches billing address country
    		     </td></tr></table>
    		    </div>
		    </td>
		    
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('binnamematch');"
		       onMouseOut="hide('binnamematch');"><font style='font-size: 11px;'>
		      Bin Name Match:</a> $binNameMatch
		      <div id='binnamematch' name='binnamematch' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Whether name of issuing bank matches inputted binName. A return value of Yes
		       provides a positive indication that cardholder is in possession of credit card.
    		     </td></tr></table>
    		    </div>
		    </td>
		
		</tr>
		
		
		
   
		<Tr>
		
		      <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('binphonematch');"
		       onMouseOut="hide('binphonematch');"><font style='font-size: 11px;'>
		      Bin Phone Match:</a> $binPhoneMatch
		      <div id='binphonematch' name='binphonematch' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Whether customer service phone number matches inputed binPhone. A return value of Yes provides a 
		      positive indication that cardholder is in possession of credit card.
    		     </td></tr></table>
    		    </div>
		    </td>
		
		   <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('binphone');"
		       onMouseOut="hide('binphone');"><font style='font-size: 11px;'>
		      Bin Phone:</a> $binPhone
		      <div id='binphone' name='binphone' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Customer service phone number listed on back of credit card. 
		      Available for approximately 75% of BIN numbers. 
		      In some cases phone number returned may be outdated.
    		     </td></tr></table>
    		    </div>
		    </td>
		
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('freeemail');"
		       onMouseOut="hide('freeemail');"><font style='font-size: 11px;'>
		      Free Email:</a> $freeMail
		      <div id='freeemail' name='freeemail' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		     Whether e-mail is from free e-mail provider (free e-mail = higher risk)
    		     </td></tr></table>
    		    </div>
		    </td>
		    
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('carder');"
		       onMouseOut="hide('carder');"><font style='font-size: 11px;'>
		      Carder Email:</a> $carderEmail
		      <div id='carder' name='carder' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Whether e-mail is in database of known carders (credit card fraudster)
    		     </td></tr></table>
    		    </div>
		    </td>
		
		</tr>
		
		
	
		<Tr>
		
		      <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('proxy');"
		       onMouseOut="hide('proxy');"><font style='font-size: 11px;'>
		     Anonymous Proxy:</a> $anonymousProxy
		      <div id='proxy' name='proxy' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		     Whether IP address is Anonymous Proxy (anonymous proxy = very high risk)
    		     </td></tr></table>
    		    </div>
		    </td>
		
		   <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('ipregion');"
		       onMouseOut="hide('ipregion');"><font style='font-size: 11px;'>
		      IP Region:</a> $ip_region
		      <div id='ipregion' name='ipregion' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Estimated State/Region of the IP address,
    		     </td></tr></table>
    		    </div>
		    </td>
		
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('ipcity');"
		       onMouseOut="hide('ipcity');"><font style='font-size: 11px;'>
		      IP City:</a> $ip_city
		      <div id='ipcity' name='ipcity' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Estimated City of the IP address
    		     </td></tr></table>
    		    </div>
		    </td>
		    
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('countrymatch');"
		       onMouseOut="hide('countrymatch');"><font style='font-size: 11px;'>
		      Country Match:</a> $countryMatch
		      <div id='countrymatch' name='countrymatch' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Whether country of IP address matches billing address country (mismatch = higher risk)
    		     </td></tr></table>
    		    </div>
		    </td>
		
		</tr>
		
		

		<Tr>
		
		      <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('highrisk');"
		       onMouseOut="hide('highrisk');"><font style='font-size: 11px;'>
		      High Risk Country:</a> $highRiskCountry
		      <div id='highrisk' name='highrisk' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		     Whether IP address or billing address country is in Belarus, Colombia, Egypt, Indonesia, 
		     Lebanon, Macedonia, Malaysia, Nigeria, Pakistan, Ukraine, Vietnam or Yugoslavia
    		     </td></tr></table>
    		    </div>
		    </td>
		
		   <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('distance');"
		       onMouseOut="hide('distance');"><font style='font-size: 11px;'>
		      Distance:</a> $distance
		      <div id='distance' name='distance' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		     Distance from IP address to Billing Location in kilometers (large distance = higher risk)
    		     </td></tr></table>
    		    </div>
		    </td>
		
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('shipforward');"
		       onMouseOut="hide('shipforward');"><font style='font-size: 11px;'>
		      Ship Forward:</a> $shipForward
		      <div id='shipforward' name='shipforward' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		      Whether shipping address is in database of known mail drops
    		     </td></tr></table>
    		    </div>
		    </td>
		    
		    <td style='border: 1px solid black;'><A href='#a' onMouseOver="show('phonebill');"
		       onMouseOut="hide('phonebill');"><font style='font-size: 11px;'>
		      Customer Phone:</a> $custPhoneInBillingLoc
		      <div id='phonebill' name='phonebill' style=" POSITION: 
		      absolute; Z-INDEX: 1; VISIBILITY:hidden;">
    		     <table width='300' style='border: 1px solid black; background-color: tan;'><tr>
 		      <Td ><font style='font-size: 11px;'>
		    Whether the customer phone number is in the billing location. A return 
		    value of Yes provides a positive indication that the phone number listed belongs to the cardholder. 
		    Currently we only support US Phone numbers, in the future we may support other countries.
    		     </td></tr></table>
    		    </div>
		    </td>
		
		</tr>
		
		
		
		</table>
		
		
		</td>
		
		
		</tr>
		
	|;

    }

    print qq|
		<tr class="mainrow"><td colspan="2">
		<font class="main">
		<b>$closedShop::TEXT{'PROD'} $closedShop::TEXT{'NFO'}</b>
		<center>
		<table width="100%"  class="main" cellpadding="1" cellspacing="1">
		<tr class="mainrow" style='background-color: white'>
		<td><font class="main"><strong>$closedShop::TEXT{'NAME'}</strong></td>
		<td><font class="main"><strong>Code</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'QTY'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'ATT'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'PRC'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'SHP_PRC'}</strong></td>
		<td><font class="main"><strong>$closedShop::TEXT{'TOT'}</strong></td>
		</tr>
		|;

    my @ary = ();
    if ( $item_ids =~ /$closedShop::CookieItemDelimiter/ ) {
        @ary = split( /$closedShop::CookieItemDelimiter/, $item_ids );
    }
    else {
        @ary = split( /~/, $item_ids );
    }

    my $k           = 1;
    my $colorchange = '';
    my ( $itemsub, $shipsub, $totalweight ) = ( 0, 0, 0 );
    foreach (@ary) {
        if ( $k % 2 == 0 ) { $colorchange = '#EEEEEE'; }
        else { $colorchange = '#DDDDDD'; }
        if ( !$_ ) { next; }

        my ( $iid, $qty, $attr );

        if ( $_ =~ /$closedShop::CookieSectionDelimiter/ ) {
            ( $iid, $qty, $attr ) = split( /$closedShop::CookieSectionDelimiter/, $_ );
        }
        else {
            ( $iid, $qty, $attr ) = split( /\|/, $_ );
        }

        if ( $iid eq "gc" ) {
            my ( $id, $recipient_name, $recipient_email, $gc_amount );
            if ( $_ =~ /$closedShop::CookieSectionDelimiter/ ) {
                ( $id, $recipient_name, $recipient_email, $gc_amount ) =
                  split( /$closedShop::CookieSectionDelimiter/, $_ );
            }
            else {
                ( $id, $recipient_name, $recipient_email, $gc_amount ) = split( /\|/, $_ );
            }

            print qq|
	   	  <tr class="mainrow" style='background-color: $colorchange'>
		  <td><font class="main"> $closedShop::TEXT{'GC'} ($closedShop::CurrencyMarker $gc_amount)</td>
		  <td><font class="main"> GC</td>
		  <td><font class="main"> 1</td>
		  <td><font class="main"> &nbsp;</td>
		  <td><font class="main"> $qty x $closedShop::CurrencyMarker $gc_amount</td>
		  <td><font class="main"> N/A</td>
		  <td><font class="main"> <nobr>$closedShop::CurrencyMarker $gc_amount</nobr></td>
			|;
            $k++;
            next;
        }

        $SQL = qq| select name,price,shipping_price,description,
	              category,product_code,reduced_shipping_price,weight from
			$closedShop::sql_tables{"item"} where id="$iid" |;
        $sth = closedShop::DatabaseQuery($SQL);
        my ( $name, $price, $ship, $desc, $cat, $code, $rship, $weight ) = $sth->fetchrow_array();
		$totalweight += $qty*$weight;

        my @attr = ();
        if ( $attr =~ /$closedShop::CookieAttributeDelimiter/ ) {
            @attr = split( /$closedShop::CookieAttributeDelimiter/, $attr );
        }
        else {
            @attr = split( /\?/, $attr );
        }

        $attr = "";

        my $attr_cost = 0.00;
        foreach my $at (@attr) {
            my ( $prod_id, $attr_id, $data_id ) = split( /$closedShop::GiftCertDelimiter/, $at );
            my $SQL = qq|select name,data,price from $closedShop::sql_tables{"attributes"},
	   $closedShop::sql_tables{"attribute_values"} where product_id='$prod_id' and
	   $closedShop::sql_tables{"attributes"}.id = '$attr_id' and
	   $closedShop::sql_tables{"attribute_values"}.id = '$data_id'|;
            my $sth = closedShop::DatabaseQuery($SQL);
            my ( $atname, $atdata, $atprice ) = $sth->fetchrow_array();
            $attr .= "<nobr><u>$atname</u>: $atdata";
            if ( $atprice > 0 ) {
                $attr .= " (+$closedShop::CurrencyMarker$atprice)";
                $attr_cost += $atprice;
            }
            $attr .= "</nobr><BR>";
        }
        if ( !$attr ) { $attr = "&nbsp;"; }

        my $total = sprintf( "%.2f", $qty * ( $price + $attr_cost ) );
        $itemsub += $total;

		my ( $shiptotal, $shipstring ) = closedShop::GetShippingCostsItem( $ship, $qty, $rship, $weight);	
        $shipsub += $shiptotal;

        print qq|
	   	  <tr class="mainrow" style='background-color: $colorchange'>
		  <td><font class="main"> <a
		  href="$closedShop::FileNames{item}?action=ShowItem&id=$iid">$name</a></td>
		  <td><font class="main"> $code</td>
		  <td><font class="main"> <Center>$qty</center></td>
		  <td><font class="main"> $attr&nbsp;</td>
		  <td><font class="main"> $qty x $closedShop::CurrencyMarker$price|;
        if ( $attr_cost > 0 ) {
            print qq|<BR>(+$qty x $closedShop::CurrencyMarker$attr_cost)|;
        }

        print qq|</td><td><font class="main">$shipstring</td>
		  <td><font class="main"> <nobr>$closedShop::CurrencyMarker|
          . sprintf( "%.2f", $total + $shiptotal ) . qq|</nobr></td>
		  </tr>  |;
        $k++;
    }

	my ( $shipclass, $shipstring1 ) = closedShop::GetShippingCostsClass( $totalweight, $itemsub);
	if ($shipsub == 0) {
		$shipsub = $shipclass;
	}

    my ( $stax, $ctax, $itax ) = closedShop::GetTaxCosts( $sub_total, $taxExemptID, $b_state, $b_country, $staxrate, $ctaxrate, $itaxrate );

    if ( $closedShop::EnableTaxExempt eq "Y" && $taxExemptID != "" ) {
        ( $staxrate, $ctaxrate, $itaxrate ) = ( 0, 0, 0 );
        $tax_total = " EXEMPT ";
    }

    print qq|<tr class="mainrow" style='background-color: white'>
		<td colspan="4" align="right">&nbsp;</td><td colspan="2">
		<font class="main">
     	<strong>Item $closedShop::TEXT{'TOT'}: </td>
     	<td><font class="main"> $closedShop::CurrencyMarker| . sprintf( "%.2f", $itemsub ) . qq|</strong></td></tr>

		 <tr class="mainrow" style='background-color: white'><td colspan="4" align="right">&nbsp;</td>
		  <td colspan="2"><font class="main"> 
		  <strong>$closedShop::TEXT{'SHIP'} $closedShop::TEXT{'TOT'}: $shipstring1</td>
      <td><font class="main"> $closedShop::CurrencyMarker| . sprintf( "%.2f", $ship_total ) . qq|</strong> 
      </td></tr>

		 <tr class="mainrow" style='background-color: white'><td colspan="4" align="right">&nbsp;</td>
		  <td colspan="2"><font class="main"> 
		  <strong>$closedShop::TEXT{'TAX'} $closedShop::TEXT{'TOT'}:</td>
      <td><font class="main"> $closedShop::CurrencyMarker| . sprintf( "%.2f", $tax_total ) . qq|</strong> 
      </td></tr>
		|;

    if ( $closedShop::ShowTax eq "Y" ) {
        my $vrate = sprintf( "%.2f", ( $staxrate * 100 ) );
        if ( $staxrate > 0 ) {
            print qq|
		  <tr class="mainrow"><td colspan="4" align="right">&nbsp;</td>
		  <td colspan='2'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font class="main"> <strong>$closedShop::TEXT{'TAX'} (for $closedShop::CompanyState at $vrate%):</td>
      <td><font class="main"> $closedShop::CurrencyMarker$stax</strong> </td></tr>
			|;
        }
        if ( $ctaxrate > 0 ) {
            my $crate = sprintf( "%.2f", ( $ctaxrate * 100 ) );
            print qq|
		     <tr class="mainrow"><td colspan="4" align="right">&nbsp;</td>
		     <td colspan='2'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font class="main"> <strong>$closedShop::TEXT{'TAX'}  (for $closedShop::CompanyCountry at $crate%):</td>
         <td><font class="main"> $closedShop::CurrencyMarker$ctax</strong> </td></tr>
			   |;
        }
        if ( $itaxrate > 0 && $closedShop::CompanyCountry ne $b_country ) {
            my $irate = sprintf( "%.2f", ( $itaxrate * 100 ) );
            print qq|
		     <tr class="mainrow"><td colspan="4" align="right">&nbsp;</td>
		     <td colspan='2'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font class="main"> <strong>$closedShop::TEXT{'TAX'} (for International at $irate%):</td>
         <td><font class="main"> $closedShop::CurrencyMarker$itax</strong> </td></tr>
			   |;
        }
    }

    if ( $amount_gc_used > 0 ) {
        print qq|
		  <tr class="mainrow" style='background-color: white'><td colspan="4" align="right">&nbsp;</td>
			<td  colspan="2"><font class="main">
     	<strong>$closedShop::TEXT{'GC'} $closedShop::TEXT{'DED'}:</td><td><font class="main">
		  <strong>$closedShop::CurrencyMarker <font color="red">-($amount_gc_used)</font></strong></strong></td></tr>
		 |;
    }

    if ( $gname || $gdiscount ) {
        my $before = ( $total_cost * ( $gdiscount / 100 ) ) + $total_cost;
        my $disc = sprintf( "%.2f", $before * ( $gdiscount / 100 ) );

        print qq|		
	<tr class="mainrow" style='background-color: white'><td colspan="4" align="right">&nbsp;</td><td colspan='2'><font class="main">
     	 <font color='red'>$gname Discount ($gdiscount\%)</font>:</td>
	 <td><font class="main"><font color='red'>-$closedShop::CurrencyMarker$disc</strong></td></tr>
			 |;

    }

    print qq|
		<tr class="mainrow" style='background-color: white'><td colspan="4" align="right">&nbsp;</td>
		<td  colspan="2"><font class="main">
		    <strong>Grand $closedShop::TEXT{'TOT'}<BR> (amount to charge):</td>
		    <td><font class="main">
		<strong><nobr>$closedShop::CurrencyMarker$total_cost</nobr></strong></td></tr>
		</table></td></tr>|;

    if ($tracking) {
        my @tmp       = split( /~~~/, $closedShop::ShippingCompanies{"$tracking"} );
        my $trackname = $tmp[0];
        my $trackurl  = $tmp[1];
        if ( !$trackname ) { $trackname = $tracking; }
        $trackurl =~ s/<NUMBER>/$trackno/g;
        if ($trackurl) {
            $trackurl = qq|<a href="$trackurl" 
			target="_blank">$closedShop::TEXT{'PCK_TRK'}</a> (No. $trackno)|;
        }
        else {
            $trackurl = qq|$closedShop::TEXT{'TRK'} #: $trackno|;
        }

        print qq|
  		    <tr class="mainrow"><td colspan="2">
                    <font class="main"><b>$closedShop::TEXT{'TRK'} $closedShop::TEXT{'NFO'}</b>
                <center></td></tr>
		<tr class="mainrow"><td><font class="main">
		$closedShop::TEXT{'SHIP_CO'}</td><td><font class="main"> $trackname
		</td></tr><tr class="mainrow"><td><font class="main">
		$closedShop::TEXT{'PCK_TRK'}:</td><td><font class="main"> $trackurl</td></tr>
		<tr class="headrow"><Td colspan="2">&nbsp;</td></tr>
		|;
    }
}

###############################################################################
# FUNCTION:  DeleteProduct
# DESCRIPTION: edit a product
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DeleteProduct {
    if ( $query->param("selected") == 2 ) {
        my $id = $query->param("id");
        $id =~ tr/0-9//cd;
        if ( $id == 0 || !$id ) {
            goto START;
        }
        my $SQL = qq| delete from $closedShop::sql_tables{"item"} where id = "$id" |;
        my $sth = closedShop::DatabaseQuery($SQL);

        return AdminScreen( $closedShop::TEXT{'SUCC_DELP'} );
    }
    elsif ( $query->param("selected") == 1 ) {
        my $cate = $query->param("category");
        closedShop::Header();
        print qq|
	  <form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="DeleteProduct">
	  <input type="hidden" name="selected" value="2">
	  <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><th colspan="6">
	  <font class="head">$closedShop::TEXT{'DEL_PROD'}</font></th></tr>
	  <tr class="mainrow">
	  <td><font class="main"><b>$closedShop::TEXT{'REM'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'CAT'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'NAME'}</b></td>
	  <td><font class="main"><b>Code/Id</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'PRC'}</b></td>
	  <td><font class="main"><b>$closedShop::TEXT{'REM'}</b></td>
	  </tr>|;

        my $SQL = qq| select name,price,id,category,product_code from $closedShop::sql_tables{"item"}
	                where category = "$cate" order by name |;

        my $sth = closedShop::DatabaseQuery($SQL);
        while ( my ( $name, $price, $id, $category, $code ) = $sth->fetchrow_array() ) {
            print qq|
		    <tr class="mainrow">
			<td><font class="main"><input type="radio" name="id" value="$id"></td>
			<td><font class="main"> $closedShop::categories{$category}</td>
			<td><font class="main"> $name</td>
			<td><font class="main"> $code</td>
			<td><font class="main"> $closedShop::CurrencyMarker $price</td>
			<td><font class="main"><input type="submit" value="$closedShop::TEXT{'REM'}"></td>
			</tr>\n |;
        }

        print qq|
        <tr class="headrow"><td colspan="6"><center><input
		type="submit" onClick="return Confirm();" value="$closedShop::TEXT{'DEL_PROD'}"></td></tr>
		</table></form>|;
        BottomAdmin();
        closedShop::Footer();
    }
    else {
      START:
        closedShop::Header();
        print qq|
	<form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="DeleteProduct">
	  <input type="hidden" name="selected" value="1">
	  <table width="$closedShop::TableWidth" class="main"><tr class="headrow"><th colspan="2">
	  <font class="main"> <font size="3"><font color="blue">$closedShop::TEXT{'SELECT'} $closedShop::TEXT{'CAT'}</font></th></tr>
	 <tr class="mainrow"><td><center><font class="main"> $closedShop::TEXT{'CAT'}:</td>
	  <td><font class="main"> |;

        closedShop::PrintCategoryList( "category", 0, "ALL" );

        print qq|
	  </td></tr>
	    <tr class="headrow"><td colspan="2"><center><input
		type="submit" value="$closedShop::TEXT{'SELECT'} $closedShop::TEXT{'ITMS'}"></td></tr>
		</table></form>|;
        BottomAdmin();
        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  ChangeVariables
# DESCRIPTION: change variables of the program
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ChangeVariables {
    my ( $session, $uid ) = closedShop::GetSessionCookie();
    if ( $query->param("selected") == 1 ) {
        open( FILE, "$closedShop::varsfile" );
        my @data = <FILE>;
        close(FILE);
        my %checkhash = ();

        foreach (@data) {
            my @ar = split( /$closedShop::VarsFileDelimiter/, $_ );
            $checkhash{ $ar[0] } = $ar[1];
        }

        my @newdata = ();
        foreach ( $query->param() ) {
            my @var = $query->param("$_");
            $checkhash{$_} = 1;
            my $var = join( ",", @var );
            $var =~ s/~~~/*/g;
            $var =~ s/\n/<BR>/g;
            if ( $_ =~ /category/ ) { next; }
            push @newdata, qq|$_| . $closedShop::VarsFileDelimiter . qq|$var\n|;
        }

        my @weights = ();
        my @shipordertotal = ();
        for ( my $t = 0 ; $t < 8 ; $t++ ) {
            my $wend = $query->param("WeightClassEnd$t");
            if ( !$wend ) { $wend = "Max"; }
            
            my $stcend = $query->param("ShipTotalClassEnd$t");
            if ( !$wend ) { $wend = "Max"; }

            my @weight = ( $t, $query->param("WeightClassStart$t"), $wend, $query->param("WeightClassPrice$t") );
            my $wt = join( $closedShop::CookieSectionDelimiter, @weight );
            push @weights, $wt;
            
            my @shiporder = ( $t, $query->param("ShipTotalClassStart$t"), $stcend, $query->param("ShipTotalClassPrice$t") );
            my $stc = join( $closedShop::CookieSectionDelimiter, @shiporder );
            push @shipordertotal, $stc;
        }
        my $wts = join( $closedShop::GiftCertDelimiter, @weights );
        my $stcs = join( $closedShop::GiftCertDelimiter, @shipordertotal );
        
        $checkhash{"WeightClasses"} = 1;
        $checkhash{"ShipTotalClasses"} = 1;
        push @newdata, qq|WeightClasses$closedShop::VarsFileDelimiter| . $wts . qq|\n|;
        push @newdata, qq|ShipTotalClasses$closedShop::VarsFileDelimiter| . $stcs . qq|\n|;

        foreach ( keys %checkhash ) {
            if ( $checkhash{$_} != 1 ) {
                push @newdata, $_ . $closedShop::VarsFileDelimiter . $checkhash{$_};
            }
        }

        open( FILE, ">$closedShop::varsfile" );
        flock( FILE, LOCK_EX );
        print FILE join( "", @newdata );
        flock( FILE, LOCK_UN );
        close(FILE);

        my $k = 1;
        foreach ( sort keys %closedShop::categories ) {
            my $catname = $query->param("category_$_");
            my ( $intro, $style, $numimages, $status, $name, $children, $parent ) = closedShop::GetCategoryInfo($_);
            if ( !$catname || $parent ) { next; }
            my $SQL = qq|update $closedShop::sql_tables{"cat"} set name="$catname"
	       where code="$_"|;
            my $sth = closedShop::DatabaseQuery($SQL);
            $k++;
        }
        for ( ; $k < $closedShop::MaxCats ; $k++ ) {
            my $catname = $query->param("category$k");
            if ($catname) {
                my $short = closedShop::GetNewCategoryCode($catname);
                my $SQL   = qq|insert into $closedShop::sql_tables{"cat"} set name="$catname",
	        code="$short",status="PUBLIC",style="text"|;
                my $sth = closedShop::DatabaseQuery($SQL);
            }
        }
        return AdminScreen( $closedShop::TEXT{'SUCC_VAR'} );
    }
    else {
        closedShop::Header();

        $closedShop::CompanyAddress =~ s/<BR>/\n/g;

        print qq|<br>

	
	
	<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><td colspan="2" >
	<font class="head"><b>$closedShop::TEXT{'INSTALL'} - <a href='#y' 
	onClick="javascript:toggle('compinfo');">$closedShop::TEXT{'COMP'} 
	$closedShop::TEXT{'NFO'}</a></b></font></td></tr>
	<form action="$program_name" method="POST" name='form1' enctype="multipart/form-data">
	<input type="hidden" name="action" value="ChangeVariables">
	<input type="hidden" name="selected" value="1">
	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='compinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='compinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'NAME'}</b>: </font></td><td><font class="main">
	<input type="text" name="CompanyName" value="$closedShop::CompanyName" size="35"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'EMAIL'}</b>: </font></td><td><font class="main">
	<input type="text" name="CompanyEmail" value="$closedShop::CompanyEmail" size="25"></td></tr>

	

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'ADDRESS'}</b>: </font></td><td><font class="main">
	<textarea name="CompanyAddress" cols="30" rows="3">$closedShop::CompanyAddress</textarea></td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'CITY'}</b>: </font></td><td><font class="main">
	<input type="text" name="CompanyCity" value="$closedShop::CompanyCity" size="30"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'STATE'}</b>: </font></td><td><font class="main">
	|;
        closedShop::ListStates( "CompanyState", $closedShop::CompanyState );
        print qq|</td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'ZIP'}</b>: </font></td><td><font class="main">
	<input type="text" name="CompanyZip" value="$closedShop::CompanyZip" size="10"></td></tr>
	
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'COUNTRY'}</b>: </font></td><td><font class="main">
	|;
        closedShop::ListCountries( "CompanyCountry", $closedShop::CompanyCountry );
        print qq|</td></tr>
	
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'PHONE'}</b>: </font></td><td><font class="main">
	<input type="text" name="CompanyPhone" value="$closedShop::CompanyPhone" size="14"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COMP'} $closedShop::TEXT{'FAX'}</b>: </font></td><td><font class="main">
	<input type="text" name="CompanyFax" value="$closedShop::CompanyFax" size="14"></td></tr>
	
	

	<tr class="mainrow"><td><font class="main"><b>Restrict Country List</b>: <BR>
	If yes, will only show your country</font></td><td><font class="main">|;
        my ( $yescheck, $nocheck ) = "";
        if ( $closedShop::RestrictCountry eq "Y" ) { $yescheck = "checked"; }
        else { $nocheck = "checked"; }

        print qq|
	 <input type="radio" name="RestrictCountry" value="Y" $yescheck> $closedShop::TEXT{'YES'}<BR>
	 <input type="radio" name="RestrictCountry" value="N" $nocheck > $closedShop::TEXT{'NO'}
	 </td></tr>
	 
	 <tr class="mainrow"><td><font class="main"><b>Restrict State List</b>: <BR>
	If yes, will not show state list</font></td><td><font class="main">|;
        ( $yescheck, $nocheck ) = "";
        if ( $closedShop::RestrictState eq "Y" ) { $yescheck = "checked"; }
        else { $nocheck = "checked"; }

        print qq|
	 <input type="radio" name="RestrictState" value="Y" $yescheck> $closedShop::TEXT{'YES'}<BR>
	 <input type="radio" name="RestrictState" value="N" $nocheck > $closedShop::TEXT{'NO'}
	 </td></tr>

	 <tr class="mainrow"><td><font class="main"><b>Number of Attributes (Max)</font></td><td><font class="main"> 
	<input type="text" name="MaxAttributes" value="$closedShop::MaxAttributes" size="5"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>Number of Attribute Options (Max)</font></td><td><font class="main">
	<input type="text" name="MaxAttributeOptions" value="$closedShop::MaxAttributeOptions" size="5"></td></tr>

	 
	
	
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'CURRMRK'}</b>: </font></td><td><font class="main">
	<select name="CurrencyMarker">
	<option value="$closedShop::CurrencyMarker">$closedShop::CurrencyMarker</option>
	<option value="\$">\$</option>
	<option value="&pound;">&pound;</option>
	<option value="&#8364;">&#8364;</option>
	<option value="&yen;">&yen;</option>
	<option value="&cent;">&cent;</option>
	<option value="&#8224;">&#8224;</option>
  </select>
	</td></tr>


	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'MINAM'} </font></td><td><font class="main"> $closedShop::CurrencyMarker
	<input type="text" name="MinimumOrder" value="$closedShop::MinimumOrder" size="5"></td></tr>


	 <tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'SELECT'} $closedShop::TEXT{'ACCEPT'} $closedShop::TEXT{'CCS'}</b>:</td><td><font class="main">
	  |;

        my ( $mcheck, $vcheck, $dcheck, $acheck, $dicheck, $ccheck, $pcheck, $codcheck );

        if ( $closedShop::AcceptedCards{"check"} )      { $ccheck   = "checked"; }
        if ( $closedShop::AcceptedCards{"mastercard"} ) { $mcheck   = "checked"; }
        if ( $closedShop::AcceptedCards{"visa"} )       { $vcheck   = "checked"; }
        if ( $closedShop::AcceptedCards{"discover"} )   { $dcheck   = "checked"; }
        if ( $closedShop::AcceptedCards{"amex"} )       { $acheck   = "checked"; }
        if ( $closedShop::AcceptedCards{"diner"} )      { $dicheck  = "checked"; }
        if ( $closedShop::AcceptedCards{"paypal"} )     { $pcheck   = "checked"; }
        if ( $closedShop::AcceptedCards{"cod"} )        { $codcheck = "checked"; }

        print qq|
	<input type="checkbox" $pcheck name="creditcard" value="paypal">PayPal<br>
	<input type="checkbox" $ccheck name="creditcard" value="check">Checks<br>
	<input type="checkbox" $mcheck name="creditcard" value="mastercard">Mastercard<br>
	<input type="checkbox" $vcheck name="creditcard" value="visa">Visa<br>
	<input type="checkbox" $dcheck name="creditcard" value="discover">Discover<br>
	<input type="checkbox" $acheck name="creditcard" value="amex">American Express<br>
	<input type="checkbox" $dicheck name="creditcard" value="diner">Diner\'s Club<br>
	<input type="checkbox" $codcheck name="creditcard" value="cod">Cash on Delivery<br>

	 </td></tr>

	 <tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'CRE_CAT'}
           </td><td><font class="main"> |;

        my $j = 1;
        foreach ( sort keys %closedShop::categories ) {
            my ( $intro, $style, $numimages, $status, $name, $children, $parent ) = closedShop::GetCategoryInfo($_);
            if ( !$parent ) {
                print qq|Category $j: <input size="20" type="text"
                value="$closedShop::categories{$_}" name="category_$_"><BR>\n|;
                $j++;
            }
        }

        for ( ; $j < $closedShop::MaxCats ; $j++ ) {
            print qq|$closedShop::TEXT{'CAT'} $j: <input type="text" name="category$j" value="" size="20"><br>\n|;
        }

        print qq|</td></tr>

	

	
	 
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'ENAB_INV'}
	 </font></td><td><font class="main">
	 |;
        ( $yescheck, $nocheck ) = ( "", "" );
        if ( $closedShop::EnableInventory eq "Y" ) { $yescheck = "checked"; }
        else { $nocheck = "checked"; }

        print qq|
	 <input type="radio" name="EnableInventory" value="Y" $yescheck> $closedShop::TEXT{'ENABLE'}<BR>
	 <input type="radio" name="EnableInventory" value="N" $nocheck > $closedShop::TEXT{'DISABLE'}
	 </td></tr>
	 <tr class="mainrow"><td><font class="main"><b>Show Stock Levels</b><BR>
	 # of items in stock will be 
	 displayed to customer
	 </font></td><td><font class="main">
	 |;
        ( $yescheck, $nocheck ) = ( "", "" );
        if ( $closedShop::ShowStock eq "Y" ) { $yescheck = "checked"; }
        else { $nocheck = "checked"; }

        print qq|
	 <input type="radio" name="ShowStock" value="Y" $yescheck> $closedShop::TEXT{'ENABLE'}<BR>
	 <input type="radio" name="ShowStock" value="N" $nocheck > $closedShop::TEXT{'DISABLE'}
	 </td></tr>
	
	<!--
	  <tr class="mainrow"><td><font class="main"><b>Force SSL connection<BR>for checkout</b><BR>
	 </font></td><td><font class="main">
	 |;
        ( $yescheck, $nocheck ) = ( "", "" );
        if ( $closedShop::ForceSSL eq "Y" ) { $yescheck = "checked"; }
        else { $nocheck = "checked"; }

        print qq|
	 <input type="radio" name="ForceSSL" value="Y" $yescheck> $closedShop::TEXT{'ENABLE'}<BR>
	 <input type="radio" name="ForceSSL" value="N" $nocheck > $closedShop::TEXT{'DISABLE'}
	 </td></tr>

	|;

        my $forceurl = $closedShop::ForceCartURL;
        if ( !$forceurl ) {
            $forceurl = "https://";
        }

        print qq|
	 <tr class="mainrow"><td><font class="main"><b>URL Checkout</b><BR>
	 If your checkout URL is on another server (<i>experimental</i>)
	  </font></td><td><font class="main"> 
	  <input type='text' name='ForceCartURL' value='$forceurl' size='35'>
	</td></tr>	
	 -->
	 
	 	
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'FIN_MESG'}
	  </font></td><td><font class="main"> 
	<textarea mce_editable='true' name="FinalMessage" rows="6" cols="50">$closedShop::FinalMessage</textarea></td></tr>	
	
	<tr class="mainrow"><td><font class="main"><b>Email Signature</b><br>
	Enter a message you want to appear at the bottom of sent emails.
	  </font></td><td><font class="main"> 
	<textarea mce_editable='true' name="EmailFooter" rows="6" cols="50">$closedShop::EmailFooter</textarea></td></tr>	
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
	</table>
	</div>
	</div>
	</td></tr>
	</table> <br><br>

	<table width="$closedShop::TableWidth"  class="main">
	 <tr class="headrow"><td colspan="2" >
	<font class="head"> <b>$closedShop::TEXT{'INSTALL'} - <A href='#y'
	onClick="javascript:toggle('shipinfo');"> $closedShop::TEXT{'SHIP'}</a></b></font></td></tr>
	
	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='shipinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='shipinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
	
	<tr class="mainrow"><td><font class="main"><b>Shipping:</b><BR>
	
	 </font></td><td><font class="main">
	 |;
	 
        ($nocheck) = "";
        my ($itemcheck, $weightcheck, $totalcheck  ) = "";
        my ($dno1, $ditem1, $dweight1, $dtotal1) = ("hidden","hidden","hidden","hidden");
        my ($dno2, $ditem2, $dweight2, $dtotal2) = ("none","none","none","none");
        if ( $closedShop::ShowShipping eq "N" ) { $nocheck = "checked"; $dno1 = "visible"; $dno2 = "block"; }
        elsif ($closedShop::ShowShipping eq "ITEM") { $itemcheck = "checked"; $ditem1 = "visible"; $ditem2 = "block"; }
        elsif ($closedShop::ShowShipping eq "WEIGHT") { $weightcheck = "checked"; $dweight1 = "visible"; $dweight2 = "block"; }
        elsif ($closedShop::ShowShipping eq "TOTAL") { $totalcheck = "checked"; $dtotal1 = "visible";  $dtotal2 = "block"; }
	    else { $itemcheck = "checked"; $ditem1 = "visible"; $ditem2 = "block"; }

        print qq|
	 <input type="radio" name="ShowShipping" onClick="show('itemdiv'); hide('nonediv'); hide('totaldiv'); hide('weightdiv');" value="ITEM" $itemcheck> Per Item<BR>
	 <input type="radio" name="ShowShipping" onClick="show('weightdiv'); hide('itemdiv'); hide('totaldiv'); hide('nonediv');" value="WEIGHT" $weightcheck > By Weight<BR>
	 <input type="radio" name="ShowShipping" onClick="show('totaldiv'); hide('itemdiv'); hide('nonediv'); hide('weightdiv');" value="TOTAL" $totalcheck > By Order Total<BR>
	 <input type="radio" name="ShowShipping" onClick="show('nonediv'); hide('itemdiv'); hide('totaldiv'); hide('weightdiv');" value="N" $nocheck > None
	 </td></tr>
	
	 <tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'WGHT'} Identifier </b>: </font></td><td><font class="main">
	<select name="WeightIdentifier">|;

        if ($closedShop::WeightIdentifier) {
            print qq|<option value="$closedShop::WeightIdentifier">$closedShop::WeightIdentifier</option>|;
        }

        print qq|
	<option value="lbs">lbs</option>
	<option value="kg">kg</option>
  </select>
  
	</td></tr>
	<script language='JavaScript'>
	<!--
	function ChangeWeightStart(j,k,name) {
		var eobject = eval(name + "End" + j);
	    var sobject = eval(name + "Start" + k);
	    var sbox = document.getElementById(sobject.id);
	    var ebox = document.getElementById(eobject.id);
	    
		sbox.value = eval(parseInt(ebox.value)+1);
		
	}
	
	function CheckWeight(j,name) {
		var k = parseInt(j) + 1;
		var eobject = eval(name + "End" + j);
	    var sobject = eval(name + "Start" + j);
	    var sbox = document.getElementById(sobject.id);
	    var ebox = document.getElementById(eobject.id);

	    var nextobject = eval(name + "Start" + k);
	    var nextbox = document.getElementById(nextobject.id);
	    
	    if (parseInt(ebox.value) < parseInt(sbox.value)) {
	    	ebox.value = eval(parseInt(sbox.value) + 1);
	    	nextbox.value = eval(parseInt(sbox.value) + 2);
	    }
	    if (ebox.value == "") {
	    	nextbox.value = "";
	    }
	}
	
	-->
	</script>
	
	<tr class="mainrow"><td colspan='2'>
	<div style="visibility: collapse;">
	<div id='nonediv' name='nonediv' style="position: relative;
		  display: $dno2; visibility: $dno1; z-index: 1;">
		 <font class='main'> No shipping prices will be displayed
		 or calculated for the customer</font>
	</div>
	
	<div id='totaldiv' name='totaldiv' style="position: relative;
		  display: $dtotal2; visibility: $dtotal1; z-index: 1;">
		 <font class='main'> Shipping will be calculated from
		 the total cost of the order by the classes defined below.<BR>
	|;
	
	    for ( my $t = 0 ; $t < 8 ; $t++ ) {
	        my @cursclass = ();
            if ( $closedShop::ShipTotalClasses{$t} ) {
                @cursclass = @{ $closedShop::ShipTotalClasses{$t} };
            }

            my ( $s, $e ) = ( "", "" );
            if ( $t == 0 ) { $s = qq| value="0"   |; }
            else { $s = qq| value='$cursclass[0]' |; }

            if ( $t == 7 ) { $e = qq| value="Max"  disabled |; }
            else { $e = qq| value='$cursclass[1]' |; }

            my $js = "";
            if ( $t != 7 ) { 
              $js = qq| onChange="javascript:CheckWeight($t,'ShipTotalClass');" 
              onKeyUp="javascript:ChangeWeightStart($t,$t+1,'ShipTotalClass');" |; 
              }

            print qq|	  
	  Start: $closedShop::CurrencyMarker<input type='text' size='4' $s 
	  id='ShipTotalClassStart$t' name='ShipTotalClassStart$t' > 
	  &nbsp;&nbsp;
	  End: $closedShop::CurrencyMarker<input $js type='text' $e size='4' 
	  id='ShipTotalClassEnd$t' name='ShipTotalClassEnd$t'>
	  &nbsp;&nbsp;
	  Price: $closedShop::CurrencyMarker<input type='text' size='6' 
	  name='ShipTotalClassPrice$t' value='$cursclass[2]'>
	  <BR>
	    |;
	  }
	  
	  print qq|
		 </font>
	</div>
	
	<div id='itemdiv' name='itemdiv' style="position: relative;
		  display: $ditem2; visibility: $ditem1;  z-index: 1;">
		 <font class='main'> Shipping will be calculated by the
		 per-item shipping cost</font>
	</div>
	
	<div id='weightdiv' name='weightdiv' style="position: relative;
		   display: $dweight2; visibility: $dweight1; z-index: 100;">
	<font class="main"><b>
	Weight Classes: (enter the price for weight ranges)</b><BR>
	If you don't need all the weight classes, the last entry that only 
	has a start range will be considered maximum class. 
	<BR>
	
	|;

        for ( my $t = 0 ; $t < 8 ; $t++ ) {
            my @curweight = ();
            if ( $closedShop::WeightClasses{$t} ) {
                @curweight = @{ $closedShop::WeightClasses{$t} };
            }

            my ( $s, $e ) = ( "", "" );
            if ( $t == 0 ) { $s = qq| value="0"   |; }
            else { $s = qq| value='$curweight[0]' |; }

            if ( $t == 7 ) { $e = qq| value="Max"  disabled |; }
            else { $e = qq| value='$curweight[1]' |; }

            my $js = "";
            if ( $t != 7 ) { 
              $js = qq| onChange="javascript:CheckWeight($t,'WeightClass');" 
              onKeyUp="javascript:ChangeWeightStart($t,$t+1,'WeightClass');" |; 
            }

            print qq|	  
	  Start: <input type='text' size='4' $s id='WeightClassStart$t' 
	  name='WeightClassStart$t' > 
	  &nbsp;($closedShop::WeightIdentifier)&nbsp;&nbsp;&nbsp;
	  End: <input $js type='text' $e size='4' 
	  id='WeightClassEnd$t' name='WeightClassEnd$t'>
	  &nbsp;($closedShop::WeightIdentifier) &nbsp;&nbsp;&nbsp;
	  Price: $closedShop::CurrencyMarker<input type='text' size='6' name='WeightClassPrice$t'
	  value='$curweight[2]'>
	  <BR>
	  |;

        }

        print qq|
        
        </div>
        </div>
	 </td></tr>
	</table>
	</div>
	</div>
	</td></tr>	
	 </table>
<BR><BR>
 <table width="$closedShop::TableWidth"  class="main">
	 <tr class="headrow"><td colspan="2" >
	<font class="head"> <b>$closedShop::TEXT{'INSTALL'} - 
	<a href='#y' onClick="javascript:toggle('taxinfo');">$closedShop::TEXT{'TAX'}</a></b>
	</font></td></tr>
	
	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='taxinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='taxinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
	
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'SHW_TAX'}:</b>
	 </font></td><td><font class="main">
	 |;
        ( $yescheck, $nocheck ) = ( "", "" );
        if ( $closedShop::ShowTax eq "N" ) { $nocheck = "checked"; }
        else { $yescheck = "checked"; }

        print qq|
	 <input type="radio" name="ShowTax" value="Y" $yescheck> $closedShop::TEXT{'YES'}<BR>
	 <input type="radio" name="ShowTax" value="N" $nocheck > $closedShop::TEXT{'NO'}
	 </td></tr>
	 
	 <tr class="mainrow"><td><font class="main"><b>Allow customers to use Tax Exempt IDs:</b>
	 </font></td><td><font class="main">
	 |;
        ( $yescheck, $nocheck ) = ( "", "" );
        if ( $closedShop::EnableTaxExempt eq "N" ) { $nocheck = "checked"; }
        else { $yescheck = "checked"; }

        print qq|
	 <input type="radio" name="EnableTaxExempt" value="Y" $yescheck> $closedShop::TEXT{'YES'}<BR>
	 <input type="radio" name="EnableTaxExempt" value="N" $nocheck > $closedShop::TEXT{'NO'}
	 </td></tr>

<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'STATE'} $closedShop::TEXT{'TXRATE'}</b>: </font></td><td><font class="main">
	<input type="text" name="TaxRate" value="$closedShop::TaxRate" size="5"> <i>Example: for 5.07\% enter .0507</i></td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'COUNTRY'} $closedShop::TEXT{'TXRATE'}</b>: <BR>
	If left blank, will not appear</font></td><td><font class="main">
	<input type="text" value="$closedShop::TaxRateCountry" name="TaxRateCountry" size="5"> 
	<i>Used if billee is in same country</i></td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'INTNATL'} $closedShop::TEXT{'TXRATE'}</b>: <BR>
	If left blank, will not appear</font></td><td><font class="main">
	<input type="text" value="$closedShop::TaxRateInt" name="TaxRateInt" size="5"> 
	<i>Used if billee is in different country</i></td></tr>
	</table>
	</div></div>
	</td></tr>
	</table><BR><BR>


	 <table width="$closedShop::TableWidth"  class="main">
	 <tr class="headrow"><td colspan="2" >
	<font class="head"> <b>$closedShop::TEXT{'INSTALL'} - 
	<a href='#y' onClick="javascript:toggle('thirdinfo');">
	3rd Party Modules</a></b></font></td></tr>

	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='thirdinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='thirdinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
	
	 <tr class="headrow"><td colspan="2" >
	<font class="head">TinyMCE textarea editor</font></td></tr>
	 <tr class="mainrow"><td><font class="main"><b>Use <A href='http://tinymce.moxiecode.com/'>TinyMCE</a> for Editing</b>: <BR>
	TinyMCE is an advanced editor for text areas</font></td><td><font class="main">|;
        ( $yescheck, $nocheck ) = "";
        if ( $closedShop::UseTinyMCE eq "Y" ) { $yescheck = "checked"; }
        else { $nocheck = "checked"; }

        print qq|
	 <input type="radio" name="UseTinyMCE" value="Y" $yescheck> $closedShop::TEXT{'YES'}<BR>
	 <input type="radio" name="UseTinyMCE" value="N" $nocheck > $closedShop::TEXT{'NO'}
	 </td></tr>
	
	<tr class="mainrow"><td><font class="main"><b>TinyMCE format</b>: 
	</font></td><td><font class="main">
	<select name="TinyMCEFormat">
	<option value="$closedShop::TinyMCEFormat">$closedShop::TinyMCEFormat</option>
	<option value="default">Default</option>
	<option value="simple">Simple</option>
	<option value="advanced">Advanced</option></select>
	 </td></tr>
	
	<tr class="headrow"><td colspan="2" >
	<font class="head">MaxMind Fraud Prevention</font></td></tr>
	|;
        my ( $maxcheckyes, $maxcheckno ) = ();
        if ( $closedShop::EnableMaxMind eq "Y" ) { $maxcheckyes = "checked"; }
        else {
            $maxcheckno = "checked";
        }
        my $testmax = "";
        eval("use LWP::UserAgent");
        if ($@) {
            $testmax = "Before you can use this MaxMind, your server must have
	the LWP::UserAgent module installed. Please contact your webhoster";
        }

        print qq|
	<tr class="mainrow"><td><font class="main">
 	<b>$closedShop::TEXT{'ENABLE'}  MaxMind Fraud Prevention</b><BR><a 
	href="http://www.maxmind.com" target="_blank">MaxMind Fraud Prevention is 3rd party</a>
	system for checking against fraudulent orders. It can be used free of
	charge or as a registered user. As a free service, you are limited
	to 20 fraud checks a day.<B>$testmax</b> </td>
	<td><font class="main">
	<input type="radio" name="EnableMaxMind" $maxcheckyes value="Y">$closedShop::TEXT{'ENABLE'}<BR>
	<input type="radio" $maxcheckno name="EnableMaxMind" value="N">$closedShop::TEXT{'DISABLE'}<BR>
	</td></tr>
	<tr class="mainrow"><td><font class="main"><b>
	MaxMind License Key</b><BR>If there is no license key and MaxMind is enabled,
	closedShop will use the free service</td>
	<td><font class="main"> <input type="text" name="MaxMindLicenseKey"
         value="$closedShop::MaxMindLicenseKey" size="20"></td></tr>
	
	
	 
	<tr class="headrow"><td colspan="2" >
	<font class="head">PreCharge Fraud Prevention</font></td></tr>
	|;
        my ( $precheckyes, $precheckno ) = ();
        if ( $closedShop::EnablePreCharge eq "Y" ) { $precheckyes = "checked"; }
        else {
            $precheckno = "checked";
        }
        my $testpre = "";
        eval("use Net::SSLeay");
        if ($@) {
            $testpre = "Before you can use this PreCharge, your server must have
	the Net::SSLeay module installed. Please contact your webhoster";
        }

        print qq|
	<tr class="mainrow"><td><font class="main">
 	<b>$closedShop::TEXT{'ENABLE'} PreCharge Fraud Prevention</b><BR><a 
	href="http://www.precharge.com" target="_blank">PreCharge is 3rd party</a>
	system for checking against fraudulent orders. <B>$testpre</b> </td>
	<td><font class="main">
	<input type="radio" name="EnablePreCharge" $precheckyes value="Y">$closedShop::TEXT{'ENABLE'}<BR>
	<input type="radio" $precheckno name="EnablePreCharge" value="N">$closedShop::TEXT{'DISABLE'}<BR>
	</td></tr>
	<tr class="mainrow"><td><font class="main"><b>
	Merchant ID</b></td>
	<td><font class="main"> <input type="text" name="PreChargeMerchantID"
         value="$closedShop::PreChargeMerchantID" size="20"></td></tr>
	 <tr class="mainrow"><td><font class="main"><b>
	Security Key #1</b></td>
	<td><font class="main"> <input type="text" name="PreChargeSecurityKey1"
         value="$closedShop::PreChargeSecurityKey1" size="20"></td></tr>
	 <tr class="mainrow"><td><font class="main"><b>
	Security Key #1</b></td>
	<td><font class="main"> <input type="text" name="PreChargeSecurityKey2"
         value="$closedShop::PreChargeSecurityKey2" size="20"></td></tr>
	
	 
	<tr class="headrow"><td colspan="2" >
	<font class="head">Authorize.NET Live Order Processing</font></td></tr>
	|;
        ( $yescheck, $nocheck ) = ();
        if ( $closedShop::EnableAuthorizeNET eq "Y" ) { $yescheck = "checked"; }
        else {
            $nocheck = "checked";
        }
        $testmax = "";
        eval("use LWP::UserAgent");
        if ($@) {
            $testmax = "Before you can use Authorize.NET, your server must have
	the LWP::UserAgent module installed. Please contact your webhoster";
        }

        print qq|
	<tr class="mainrow"><td><font class="main">
 	<b>$closedShop::TEXT{'ENABLE'}  Authorize.NET</b><BR><a 
	href="http://www.authorize.net" target="_blank">Authorize.NET</a> is a live
	credit card processing service that can automatically
	process orders.<B>$testmax</b> </td>
	<td><font class="main">
	<input type="radio" name="EnableAuthorizeNET" $yescheck value="Y">$closedShop::TEXT{'ENABLE'}<BR>
	<input type="radio" $nocheck name="EnableAuthorizeNET" value="N">$closedShop::TEXT{'DISABLE'}<BR>
	</td></tr>
	<tr class="mainrow"><td><font class="main"><b>
	Authorize.NET Login ID</b></td>
	<td><font class="main"> <input type="text" name="AuthorizeNETLoginID"
         value="$closedShop::AuthorizeNETLoginID" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>
	Authorize.NET Transaction Key</b></td>
	<td><font class="main"> <input type="text" name="AuthorizeNETTransactionKey"
         value="$closedShop::AuthorizeNETTransactionKey" size="20"></td></tr>
	 
<tr class="headrow"><td colspan="2" >
	<font class="head"> PayPal Payments</font></td></tr>
	<tr class="mainrow"><td><font class="main"><b>
	PayPal $closedShop::TEXT{'EMAIL'}</b><BR>$closedShop::TEXT{'PP_MESG2'}</td>
	<td><font class="main"> <input type="text" name="PayPalEmail"
         value="$closedShop::PayPalEmail" size="20"></td></tr>
	<tr class="headrow"><td colspan="2" >
	<font class="head">Ebay Auctions</font></td></tr>
	|;
        my ( $ebaycheckyes, $ebaycheckno ) = ();
        if ( $closedShop::EbayAuction eq "Y" ) { $ebaycheckyes = "checked"; }
        else { $ebaycheckno = "checked"; }

        print qq|
	<tr class="mainrow"><td><font class="main">
 	<b>$closedShop::TEXT{'ENABLE'} Ebay Auctions</b></td>
	<td><font class="main">
	<input type="radio" name="EbayAuction" $ebaycheckyes value="Y">$closedShop::TEXT{'ENABLE'}<BR>
	<input type="radio" $ebaycheckno name="EbayAuction" value="N">$closedShop::TEXT{'DISABLE'}<BR>
	</td></tr>
	<tr class="mainrow"><td><font class="main"><b>
	Ebay Username</b></td>
	<td><font class="main"> <input type="text" name="EbayUsername"
         value="$closedShop::EbayUsername" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>
	Ebay $closedShop::TEXT{'PASS'}</b></td>
	<td><font class="main"> <input type="text" name="EbayPassword"
         value="$closedShop::EbayPassword" size="20"></td></tr>
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>

	</table>
	</div></div></tr></tr>
	</table>

	|;

        print qq|
	<BR><BR>

	 <table width="$closedShop::TableWidth" class="main"><tr class="headrow"><td colspan="2">
	<font class="head"><b>$closedShop::TEXT{'INSTALL'}  - 
	<a href='#y' onClick="javascript:toggle('sqlinfo');">SQL 
	$closedShop::TEXT{'DB'} $closedShop::TEXT{'NFO'}</a></b></font></td></tr>
	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='sqlinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='sqlinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'DB'} $closedShop::TEXT{'NAME'}</b>: </font></td><td><font class="main">
	<input type="text" name="DatabaseName" value="$closedShop::DatabaseName" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'DB'} Username</b>: </font></td><td><font class="main">
	<input type="text" name="DatabaseUsername" value="$closedShop::DatabaseUsername" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'DB'} $closedShop::TEXT{'PASS'}</b>: </font></td><td><font class="main">
	<input type="password" name="DatabasePassword" value="$closedShop::DatabasePassword" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'DB'} Location</b><br>
	<i>Only change if you know what you are doing</i>: </font></td><td><font class="main">
	<input type="text" name="DatabaseLocation" value="$closedShop::DatabaseLocation" size="20"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'DB'} Table Names</b>: </font></td><td><font class="main">
	Admin $closedShop::TEXT{'TBL'}: <input type="text" name="admintable" value="$closedShop::sql_tables{admin}" size="20"><br>
	Item $closedShop::TEXT{'TBL'}: <input type="text" name="itemtable" value="$closedShop::sql_tables{item}" size="20"><br>
	Order $closedShop::TEXT{'TBL'}: <input type="text" name="ordertable" value="$closedShop::sql_tables{order}" size="20"><br>
	User $closedShop::TEXT{'TBL'}: <input type="text" name="usertable" value="$closedShop::sql_tables{user}" size="20"><br>
	Gift Certificate $closedShop::TEXT{'TBL'}: <input type="text" name="gctable" value="$closedShop::sql_tables{gc}" size="20"><br>
	Categories $closedShop::TEXT{'TBL'}: <input type="text" name="cattable" value="$closedShop::sql_tables{cat}" size="20"><br>
	Ebay $closedShop::TEXT{'TBL'}: <input type="text" name="ebaytable" value="$closedShop::sql_tables{ebay}" size="20"><br>
	Affiliates $closedShop::TEXT{'TBL'}: <input type="text" name="affiliatetable" value="$closedShop::sql_tables{affiliates}" size="20"><br>
	Affiliate Sales $closedShop::TEXT{'TBL'}: <input type="text" name="affiliate_salestable" value="$closedShop::sql_tables{affiliate_sales}" size="20"><br>
	Attributes Table: <input type="text" name="attributetable" value="$closedShop::sql_tables{attributes}" size="20"><BR>
	Attribute Values Table: <input type="text" name="attribute_valuestable" value="$closedShop::sql_tables{attribute_values}" size="20"><BR>
    Groups Table: <input type="text" name="groupstable" value="$closedShop::sql_tables{groups}" size="20"><BR>
	MaxMind Table: <input type="text" name="maxmindtable" value="$closedShop::sql_tables{maxmind}" size="20"><BR>
    
	</td></tr>
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
	</table>
	</div></div></td></tr>
	</table><br><br>

	 <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><td colspan="2">
	<font class="head"> <b>$closedShop::TEXT{'INSTALL'} - 
	<a href='#y' onClick="javascript:toggle('serverinfo');">Server Specifics</a>
	</b></font></td></tr>

	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='serverinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='serverinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
       <tr class="mainrow"><td><font class="main"><b>IP Access</b>:<BR>
        $closedShop::TEXT{'IP_MESG'}</font></td>
        <td><font class="main"> <input type="text" name="IPAccess" value="$closedShop::IPAccess" 
	size="20"><BR>
        Examples 192.168.0.1</td></tr>

        <tr class="mainrow"><td><font class="main">
	<b>$closedShop::TEXT{'DEF_LANG'}</b>:</font></td>
        <td><font class="main"> <select name="Language">
	<option value="$closedShop::Language">$closedShop::Language</option>
	<option value="$closedShop::Language">------</option>
	|;
        foreach ( keys %closedShop::Languages ) {
            print qq|<option value="$_">$_</option>\n|;
        }
        print qq|</select></td></tr>



	<tr class="mainrow"><td><font class="main"><b>Website URL</b>: <br>
	No Trailing Slash</font></td><td><font class="main">
	<input type="text" name="website" value="$closedShop::website" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>URL to closedShop Folder</b>: <br>
	No Trailing Slash</font></td><td><font class="main">
	<input type="text" name="urltofolder" value="$closedShop::urltofolder" size="30"></td></tr>

		<tr class="mainrow"><td><font class="main"><b>URL to Continue Shopping Link</b>:<br>
  (If blank, will return to search page) <br>
	To make it go to the last page put:
<BR><dd><code>javascript:history.go(-1);</code></dd>
	<br>No Trailing Slash</font></td><td><font class="main">
	<input type="text" name="ContinueShoppingURL" value="$closedShop::ContinueShoppingURL" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>URL to Final link after completed order</b>:<br>
  (If blank, will return to homepage of website) <br>
	No Trailing Slash</font></td><td><font class="main">
	<input type="text" name="CompletedOrderURL" value="$closedShop::CompletedOrderURL" size="30"></td></tr>


	<tr class="mainrow"><td><font class="main"><b>Server Path to Image Folder</b>: <br>
	No Trailing Slash</font></td><td><font class="main">
	<input type="text" name="IMAGEPATH" value="$closedShop::IMAGEPATH" size="30"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>SECURE Server Path to Image Folder</b>: <br>
	No Trailing Slash<BR>
	Optional</font></td><td><font class="main">
	<input type="text" name="SIMAGEPATH" value="$closedShop::SIMAGEPATH" size="30"></td></tr>


	<tr class="mainrow"><td><font class="main"><b>URL to Image Folder</b>: <br>
	No Trailing Slash</font></td><td><font class="main">
	<input type="text" name="IMAGEURL" value="$closedShop::IMAGEURL" size="30"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>URL to <b>SECURE</b> Image Folder</b>: <br>
	No Trailing Slash</font></td><td><font class="main">
	<input type="text" name="SecureImageURL" value="$closedShop::SecureImageURL" size="30"></td></tr>



	 <tr class="mainrow"><td><font class="main"><b>Show search results as
	 thumbnails: </font></td><td><font class="main">|;
        ( $yescheck, $nocheck ) = "";
        if ( $closedShop::SearchThumbImages eq "Y" ) { $yescheck = "checked"; }
        else { $nocheck = "checked"; }

        print qq|
	 <input type="radio" name="SearchThumbImages" value="Y" $yescheck> $closedShop::TEXT{'YES'}<BR>
	 <input type="radio" name="SearchThumbImages" value="N" $nocheck > $closedShop::TEXT{'NO'}
	 </td></tr>

	
	<tr class="mainrow"><td><font class="main"><b>Mail Program</b>: </font></td><td><font class="main">
	<input type="text" name="mailprog" value="$closedShop::mailprog" size="25F"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Email Flags</b>: </font></td><td><font class="main">
	<input type="text" name="emailflags" value="$closedShop::emailflags" size="7"></td></tr>
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
	</table>
	</div></div></td></tr>
	</table><br><br>
 
	 <table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><td colspan="2" >
	<font class="head"><b>$closedShop::TEXT{'INSTALL'} - 
	<a href='#y' onClick="javascript:toggle('styleinfo');">$closedShop::TEXT{'STYLE'}</a>
	</b></font></td></tr>
	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='styleinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='styleinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
	<tr class="mainrow"><td><font class="main"><b>Path to Header File</b>: </font></td><td><font class="main">
	<input type="text" name="header" value="$closedShop::header" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Path to Footer File</b>: </font></td><td><font class="main">
	<input type="text" name="footer" value="$closedShop::footer" size="30"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>SECURE Path to Header File</b><BR>Optional </font></td><td><font class="main">
	<input type="text" name="sheader" value="$closedShop::sheader" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>SECURE Path to Footer File</b><BR>Optional </font></td><td><font class="main">
	<input type="text" name="sfooter" value="$closedShop::sfooter" size="30"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>Item Template Filename</b></font></td><td><font class="main">
	<input type="text" name="item_template" value="$closedShop::item_template" size="30"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>Category Template Filename</b></font></td><td><font class="main">
	<input type="text" name="cat_template" value="$closedShop::cat_template" size="30"></td></tr>

	<tr class="mainrow"><td><font class="main"><b>All Category Template Filename</b></font></td><td><font class="main">
	<input type="text" name="allcat_template" value="$closedShop::allcat_template" size="30"></td></tr>
	
	<tr class="mainrow"><td colspan='2'>&nbsp;</td></tr> 

	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'TBL'} Top Colors</b>: </font></td><td><font class="main">
	<input type="text" name="headrow_color" value="$closedShop::headrow_color" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'TBL'} Main Colors</b>: </font></td><td><font class="main">
	<input type="text" name="mainrow_color" value="$closedShop::mainrow_color" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'TBL'} Main Color Hovering</b>: </font></td><td><font class="main">
	<input type="text" name="mainrow_hover" value="$closedShop::mainrow_hover" size="20"></td></tr>	
	<tr class="mainrow"><td><font class="main"><b>Button Color Hovering</b>: </font></td><td><font class="main">
	<input type="text" name="input_hover" value="$closedShop::input_hover" size="20"></td></tr>	
	
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'TBL'} Width</b>: </font></td><td><font class="main">
	<input type="text" name="TableWidth" value="$closedShop::TableWidth" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'TBL'} Border Style</b>: </font></td><td><font class="main">
	<input type="text" name="table_border" value="$closedShop::table_border" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'TBL'} Border Spacing</b>: </font></td><td><font class="main">
	<input type="text" name="table_borderspacing" value="$closedShop::table_borderspacing" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>$closedShop::TEXT{'TBL'} Background Image</b>: <BR><i>URL of image</i></font></td><td><font class="main">
	<input type="text" name="table_backgroundimage" value="$closedShop::table_backgroundimage" size="20"></td></tr>


	<tr class="mainrow"><td><font class="main"><b>Main $closedShop::TEXT{'FONT'} Face</b>: </font></td><td><font class="main">
	<input type="text" name="fontmain_family" value="$closedShop::fontmain_family" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Main $closedShop::TEXT{'FONT'} Color</b>: </font></td><td><font class="main">
	<input type="text" name="fontmain_color" value="$closedShop::fontmain_color" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Main $closedShop::TEXT{'FONT'} Size</b>: </font></td><td><font class="main">
	<input type="text" name="fontmain_size" value="$closedShop::fontmain_size" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Main $closedShop::TEXT{'FONT'} Style</b>: </font></td><td><font class="main">
	<select name="fontmain_style"><option value="$closedShop::fontmain_style">$closedShop::fontmain_style</option>
	<option value="normal">normal</option><option value="italic">italic</option>
	<option value="oblique">oblique</option></select></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Main $closedShop::TEXT{'FONT'} Variant</b>: </font></td><td><font class="main">
	<select name="fontmain_variant"><option value="$closedShop::fontmain_variant">$closedShop::fontmain_variant</option>
	<option value="normal">normal</option><option value="small-caps">small-caps</option>
	</select></td></tr>

	<tr class="mainrow"><td><font class="main"><b>Header $closedShop::TEXT{'FONT'} Face</b>: </font></td><td><font class="main">
	<input type="text" name="fonthead_family" value="$closedShop::fonthead_family" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Header $closedShop::TEXT{'FONT'} Color</b>: </font></td><td><font class="main">
	<input type="text" name="fonthead_color" value="$closedShop::fonthead_color" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Header $closedShop::TEXT{'FONT'} Size</b>: </font></td><td><font class="main">
	<input type="text" name="fonthead_size" value="$closedShop::fonthead_size" size="30"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Header $closedShop::TEXT{'FONT'} Style</b>: </font></td><td><font class="main">
	<select name="fonthead_style"><option value="$closedShop::fonthead_style">$closedShop::fonthead_style</option>
	<option value="normal">normal</option><option value="italic">italic</option>
	<option value="oblique">oblique</option></select></td></tr>
	<tr class="mainrow"><td><font class="main"><b>Header $closedShop::TEXT{'FONT'} Variant</b>: </font></td><td><font class="main">
	<select name="fonthead_variant"><option value="$closedShop::fonthead_variant">$closedShop::fonthead_variant</option>
	<option value="normal">normal</option><option value="small-caps">small-caps</option>
	</select></td></tr>
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
	</table></div></div>
	</td></tr>
	</table><br><br>
	
	|;

        if ( $uid == 1 ) {
            print qq|

	<table width="$closedShop::TableWidth"  class="main"><tr class="headrow"><td colspan="2" >
	<font class="head"> <b>$closedShop::TEXT{'INSTALL'} - 
	<a href='#y' onClick="javascript:toggle('fileinfo');">Filenames</a></b></font></td></tr>
	<tr ><td colspan='2'>

	<div style='visibility: collapse;'>
	<div id='fileinfo' style='visibility: hidden; 
	position: relative; z-index: 1; display: none;' name='fileinfo'>

	<table  width=100% border=0 cellspacing=1 cellpadding=0>
	<tr class="mainrow"><td><font class="main"><b>URL for Admin.pl</b>: </font></td><td><font class="main"> 
	<input type="text" name="FileName_admin" value="$closedShop::FileNames{admin}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>URL for Cart.pl</b>: </font></td><td><font class="main"> 
	<input type="text" name="FileName_cart" value="$closedShop::FileNames{cart}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>URL for User.pl</b>: </font></td><td><font class="main"> 
	<input type="text" name="FileName_user" value="$closedShop::FileNames{user}" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main"><b>URL for Item.pl</b>: </font></td><td><font class="main"> 
	<input type="text" name="FileName_item" value="$closedShop::FileNames{item}" size="20"></td></tr>
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
	</table></div></div></td></tr>

	</table><br><br>|;
        }
        else {
            print qq|
	<input type="hidden" name="FileName_admin" value="$closedShop::FileNames{admin}">
	<input type="hidden" name="FileName_cart" value="$closedShop::FileNames{cart}">
	<input type="hidden" name="FileName_user" value="$closedShop::FileNames{user}">
	<input type="hidden" name="FileName_item" value="$closedShop::FileNames{item}">
			|;
        }

        print qq|


	 <table width="$closedShop::TableWidth"  class="main"><tr><td colspan="2" class="headrow"><center>
	<font class="main"> <font color="blue"><b><input type="submit" value="$closedShop::TEXT{'ED_PROG'}"></b></font></td></tr>
	</table></form>

	</center></div>
	|;
        BottomAdmin();
        closedShop::Footer();
    }
}

###############################################################################
# FUNCTION:  Login
# DESCRIPTION: The user will log in, sending the username and password
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Login {
    my $username = closedShop::ValidateInputAdmin( $query->param('username') );
    my $password = closedShop::ValidateInputAdmin( $query->param('password') );

    my $SQL = qq| select id from $closedShop::sql_tables{"admin"} where username = "$username" and
	              password = "$password" |;
    my $sth = closedShop::DatabaseQuery($SQL);
    my ($uid) = $sth->fetchrow_array();

    if ( !$uid ) {
        return LoginScreen( $closedShop::TEXT{'BAD_EM'} );
    }
    else {
        LoginTheAdmin($uid);
    }
}

################################################################################
# FUNCTION:  AdminScreen
# DESCRIPTION: Displays the admin screen
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub AdminScreen {
    my $message = shift;
    closedShop::Header($message);
    $message = "" if ( $message eq "nohttp" );

    my $mtime = ( stat("ebay.status") )[9];
    my $now   = time();

    my $ebay = "";

    if ( ( $now - $mtime ) < 720 ) {
        open( FILE, "ebay.status" );
        my @ebay = <FILE>;
        $ebay = join( "", @ebay ) . "<BR><BR>";
        close(FILE);
    }

    if ($message) {
        $message = qq|<table style='border: 1px solid black;' width='80%'><tr>
      <td><font color="red">$message</td></tr></table>|;
    }

    my $date = localtime;
    print qq|<br>
    	</center>
	<font class="head">$closedShop::TEXT{'ADM_MESG'}<br>
	<br>$ebay
	$message<font class="main">
	<table width='$closedShop::TableWidth' border='0'><tr><td valign='top'>
	<table width="200"  class="main"><tr class="headrow"><th colspan="1">
	<font class="head">Administration Options</font></th></tr>
	|;

    foreach ( sort keys %actions ) {
        unless ( $_ eq "AdminScreen"
            || $_ eq "PrintEbaySubCategory"
            || $_ eq "PrintReceipt" )
        {
            print qq|<tr class="mainrow">
	    <form action="$program_name" METHOD="POST">
	    <input type="hidden" name="action" value="$_">
	    <td>
	    <center><font class="main">
		<input type="submit" style='width: 190' value="$actions{$_}">
	    </center></td></form></tr>|;
        }
    }

    my $w              = $closedShop::TableWidth - 200;
    my @recentorders   = GetRecentOrders(3);
    my $numorderstoday = CountOrders("day");
    my $numordersweek  = CountOrders("week");
    my $numordersmonth = CountOrders("month");
    my $numordersall   = CountOrders("");

    my $amounttoday = CountAmount("day");
    my $amountweek  = CountAmount("week");
    my $amountmonth = CountAmount("month");
    my $amountall   = CountAmount("all");
    print qq|
	<tr class="headrow"><td colspan="2">&nbsp;</td></tr>
        </table></td><td width='10'>&nbsp;</td>
	<td valign='top'>
	
	<table  class="main" width='$w'>
	<tr class='headrow'>
	<td colspan='2'><center>Sales Statistics</td>
	</tr>
	|;
    for ( my $k = 0 ; $k < 3 ; $k++ ) {
        if ( $recentorders[$k] ) {
            my @order = @{ $recentorders[$k] };

            my $sub   = $order[0];
            my $bdate = $order[1];
            my $bname = $order[2];
            my $items = $order[3];
            if ( $sub && $bdate && $bname ) {
                print qq|<tr class='mainrow'><td ><u>Most Recent Order: | . eval( $k + 1 ) . qq|)</u></td>
	      <td style='font-size: 10px'>
	      <B>$bdate</b> - by $bname ($closedShop::CurrencyMarker$sub)<BR>Items: $items</td></tr>|;
            }
        }
    }

    print qq|
	<tr><td class='mainrow' colspan='2'>&nbsp;</td></tr>
	<tr class='mainrow'><td ><u>Orders Today</u>: </td><td>$numorderstoday</td></tr>
	<tr class='mainrow'><td ><u>Amount Ordered Today</u>: </td><td>$closedShop::CurrencyMarker| . closedShop::CurrencyFormat($amounttoday) . qq|</td></tr>
	<tr class='mainrow'><td ><u>Orders This Week</u>: </td><td>$numordersweek</td></tr>
	<tr class='mainrow'><td <u>Amount Ordered This Week</u>: </td><td>$closedShop::CurrencyMarker| . closedShop::CurrencyFormat($amountweek) . qq|</td></tr>
	<tr class='mainrow'><td ><u>Orders This Month</u>: </td><td>$numordersmonth</td></tr>
	<tr class='mainrow'><td <u>Amount Ordered This Month</u>: </td><td>$closedShop::CurrencyMarker| . closedShop::CurrencyFormat($amountmonth) . qq|</td></tr>
	
	<tr><td class='mainrow' colspan='2'>&nbsp;</td></tr>
	<tr class='mainrow'><td ><u>Total Orders: </td><td>$numordersall</td></tr>
	<tr class='mainrow'><td ><u>Total Amount Ordered: </td><td>$closedShop::CurrencyMarker| . closedShop::CurrencyFormat($amountall) . qq|</td></tr>
	<tr class='mainrow'><td colspan='2'  class='mainrow'><center>(inc. processed and unprocessed orders)</td></tr>
	<tr><td colspan='2'>&nbsp;</td></tr>
	|;

    if ( $closedShop::EnableInventory eq "Y" ) {
        my @lowitems = GetLowItems(4);

        print qq|
	  <tr class='headrow'>
	  <td colspan='2'><center>Inventory Update (4 lowest items)</td>
	  </tr><tr><td colspan='2'><table border='0' width=100%>
	  <tr class='mainrow'><td ><b>Name (code)</b></td><td><b># in Stock</b></td>
	  <td><b>Action</b></td>
	  </tr>
	  |;

        foreach (@lowitems) {
            my @tmp = @{$_};
            print qq|
	  <tr class='mainrow'><td >$tmp[0] ($tmp[1])</td><td>$tmp[2]</td>
	  <td><A href='$program_name?action=EditProduct&id=$tmp[3]&selected=2'>Edit Item</a></td>
	  </tr>
	  |;

        }

        print qq|</table></td></tr>
	  <tr class='mainrow'><td ></td><td></td></tr>
	  |;
    }

    print qq|
	</table>
	
	
	</td>
	</tr></table>
	
	
	</center></div>|;

    closedShop::Footer();
}

################################################################################
# FUNCTION:  GetLowItems
# DESCRIPTION: gets low items in inventory
################################################################################
# MODIFICATION HISTORY
# 2005-04-04   cbf    Validated input, cleaned up sql statements
################################################################################
sub GetLowItems {
    my ($limit) = @_;
    my $SQL = qq|select name,product_code,quantity,id from 
    $closedShop::sql_tables{'item'}  order by quantity ASC limit $limit|;
    my $sth    = closedShop::DatabaseQuery($SQL);
    my @return = ();
    while ( my @array = $sth->fetchrow_array() ) {
        push @return, \@array;
    }
    return @return;
}

################################################################################
# FUNCTION:  GetRecentOrders
# DESCRIPTION: gets recent orders
################################################################################
# MODIFICATION HISTORY
# 2005-04-04   cbf    Validated input, cleaned up sql statements
################################################################################
sub GetRecentOrders {
    my ($num) = @_;
    my $SQL = qq|select total,DATE_FORMAT(date_bought,"%m/%e/%y"),
  	item_ids,b_lname from $closedShop::sql_tables{'order'} order by date_bought DESC
	limit $num|;
    my $sth    = closedShop::DatabaseQuery($SQL);
    my @return = ();
    while ( my @array = $sth->fetchrow_array() ) {
        my $sub   = $array[0];
        my $dateb = $array[1];
        my $bname = $array[3];
        my @items = split( /$closedShop::CookieItemDelimiter/, $array[2] );
        my @names = ();
        my $t     = 1;
        foreach (@items) {
            my $SQL2   = qq|select name from $closedShop::sql_tables{'item'} where id='$_'|;
            my $sth2   = closedShop::DatabaseQuery($SQL2);
            my ($name) = $sth2->fetchrow_array();
            if ($name) {
                if ( $t % 2 == 0 ) {
                    $name .= ", <BR>";
                }
                else {
                    $name .= ", ";
                }
                push @names, $name;
                $t++;
            }
        }
        my $names = join( "", @names );
        my @temp = ( $sub, $dateb, $bname, $names );
        push @return, \@temp;
    }

    return @return;
}

################################################################################
# FUNCTION:  CountAmount
# DESCRIPTION: counts orders for amount
################################################################################
# MODIFICATION HISTORY
# 2005-04-04   cbf    Validated input, cleaned up sql statements
################################################################################
sub CountAmount {
    my ($period) = @_;
    my $SQL = "select sum(sub_total) from $closedShop::sql_tables{'order'} ";
    if ( $period eq "day" ) {
        $SQL .= " where DATE_SUB(CURDATE(),INTERVAL 1 DAY) <= date_bought ";
    }
    elsif ( $period eq "week" ) {
        $SQL .= " where DATE_SUB(CURDATE(),INTERVAL 7 DAY) <= date_bought ";
    }
    elsif ( $period eq "month" ) {
        $SQL .= " where DATE_SUB(CURDATE(),INTERVAL 30 DAY) <= date_bought ";
    }
    my $sth = closedShop::DatabaseQuery($SQL);
    my ($total) = $sth->fetchrow_array();
    if ( !$total ) { $total = "0.00"; }
    return $total;
}

################################################################################
# FUNCTION:  CountOrders
# DESCRIPTION: counts orders for time
################################################################################
# MODIFICATION HISTORY
# 2005-04-04   cbf    Validated input, cleaned up sql statements
################################################################################
sub CountOrders {
    my ($period) = @_;
    my $SQL = "select count(id) from $closedShop::sql_tables{'order'} ";
    if ( $period eq "day" ) {
        $SQL .= " where DATE_SUB(CURDATE(),INTERVAL 1 DAY) <= date_bought ";
    }
    elsif ( $period eq "week" ) {
        $SQL .= " where DATE_SUB(CURDATE(),INTERVAL 7 DAY) <= date_bought ";
    }
    elsif ( $period eq "month" ) {
        $SQL .= " where DATE_SUB(CURDATE(),INTERVAL 30 DAY) <= date_bought ";
    }
    my $sth = closedShop::DatabaseQuery($SQL);
    my ($count) = $sth->fetchrow_array();
    return $count;
}

################################################################################
# FUNCTION:  LoginScreen
# DESCRIPTION: login screen for admin
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginScreen {
    my ( $mesg, $header ) = @_;
    closedShop::Header($header);
    print qq| <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="Login">
	<font class="main">
	<font color="red">$mesg</font>
	<table width='350' class="main"><tr class="headrow"><td colspan="2">
	<font class="head">
	<div align="center">$closedShop::TEXT{'LOGIN'}</div></td></tr><tr class="mainrow"><td><font class="main">
	$closedShop::TEXT{'USERNAME'}: </td><td><input type="text" name="username" size="20"></td></tr>
	<tr class="mainrow"><td><font class="main">
	$closedShop::TEXT{'PASS'}: </td><td><input type="password" name="password" size="20"></td></tr>
	<tr class="mainrow"><td  colspan="2">
	<div align="center"><input type="submit" name="admin_login" value="$closedShop::TEXT{'LOGIN'}"></div>
	</td></tr></table>
	<br></form>
	|;

    closedShop::Footer();
}

################################################################################
# FUNCTION:  LoginTheAdmin
# DESCRIPTION: process screen
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginTheAdmin {
    my $uid     = shift;
    my $session = closedShop::CreateSession();
    my $SQL     = qq| update $closedShop::sql_tables{"admin"} set session="$session" where id = "$uid" |;
    my $sth     = closedShop::DatabaseQuery($SQL);
    closedShop::SetSessionCookie( $session, $uid );
    AdminScreen("nohttp");
}
